package com.gic.enterprise.dto.market;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.*;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;

/**
 * @ClassName: MarketServiceDTO 
 * @Description:    基础服务订购
 * @author guojuxing 
 * @date 2019/9/30 9:24 AM 
 */
public class MarketServiceDTO implements Serializable{
    private static final long serialVersionUID = 2070761220969570610L;

    /**
     * 服务商id
     */
    private String isvId;
    /**服务ID*/
    private String appServeId;

    /**是否可以购买 0不能购买 1购买 2续费 3升级 4扩容*/
    private Integer canBuy = 1;
    /**
     * 优惠id
     */
    private String couponId;
    /**
     * 对公转账  验证码
     */
    private String verifyCode;
    /**
     * 回传前端的卡券校验金额
     */
    private BigDecimal couponCheckAmount;
    /**
     * 总优惠金额
     */
    private BigDecimal discountAmount;
    /**
     * 优惠详情{200:卡券}
     */
    private String discountDetail;
    private String discountFee;
    /**返回页面购买的金额*/
    private BigDecimal price;
    /**
     * 到期时间
     */
    private Date endDate;

    private Long auditTime;

    private List<SkuDTO> skuList;

    private List<Map<String, Object>> skuMapList;

    private Map<String, String> skuCodeMapId;

    private BasicServiceDTO basicService;

    private List<LicenseDTO> licenseList;

    public String getCouponId() {
        return couponId;
    }

    public void setCouponId(String couponId) {
        this.couponId = couponId;
    }

    public BigDecimal getCouponCheckAmount() {
        return couponCheckAmount;
    }

    public void setCouponCheckAmount(BigDecimal couponCheckAmount) {
        this.couponCheckAmount = couponCheckAmount;
    }

    public BasicServiceDTO getBasicService() {
        return basicService;
    }

    public void setBasicService(BasicServiceDTO basicService) {
        this.basicService = basicService;
    }

    public List<LicenseDTO> getLicenseList() {
        return licenseList;
    }

    public void setLicenseList(List<LicenseDTO> licenseList) {
        this.licenseList = licenseList;
    }

    public void setSkuList(List<SkuDTO> skuList) {
        this.skuList = skuList;
    }

    public Date getEndDate() {
        return endDate;
    }

    public void setEndDate(Date endDate) {
        this.endDate = endDate;
    }

    public Long getAuditTime() {
        if (endDate == null) {
            return null;
        }
        return endDate.getTime();
    }

    public List<Map<String, Object>> getSkuMapList() {
        Map<String, List<SkuDTO>> skuMap = group(skuList, "versionCode") ;
        List<Map<String, Object>> result = new ArrayList<>(skuMap.size());
        for(Map.Entry<String, List<SkuDTO>> entry : skuMap.entrySet()) {
            Map<String, Object> tempResult = new HashMap<>(16);
            List<SkuDTO> skuList = entry.getValue();
            tempResult.put("versionName", getVersionNameFromJson(skuList.get(0).getSkuDetailJson()));
            tempResult.put("period", skuList);
            result.add(tempResult);
        }
        return result;
    }

    public Map<String, String> getSkuCodeMapId() {
        Map<String, String> result = new HashMap<>(8);
        if (CollectionUtils.isNotEmpty(skuList)) {
            for (SkuDTO skuDTO : skuList) {
                result.put(skuDTO.getVersionCode(), skuDTO.getSkuId());
            }
        }
        return result;
    }

    private String getVersionNameFromJson(String jsonStr) {
        JSONObject jsonObject = JSONObject.parseObject(jsonStr);
        JSONObject version = jsonObject.getJSONObject("version");
        return version.getString("name");
    }

    public Integer getCanBuy() {
        return canBuy;
    }

    public void setCanBuy(Integer canBuy) {
        this.canBuy = canBuy;
    }

    public BigDecimal getPrice() {
        return price;
    }

    public void setPrice(BigDecimal price) {
        this.price = price;
    }

    public String getVerifyCode() {
        return verifyCode;
    }

    public void setVerifyCode(String verifyCode) {
        this.verifyCode = verifyCode;
    }

    public String getIsvId() {
        return isvId;
    }

    public void setIsvId(String isvId) {
        this.isvId = isvId;
    }

    public String getAppServeId() {
        return appServeId;
    }

    public void setAppServeId(String appServeId) {
        this.appServeId = appServeId;
    }

    public BigDecimal getDiscountAmount() {
        return discountAmount;
    }

    public void setDiscountAmount(BigDecimal discountAmount) {
        this.discountAmount = discountAmount;
    }

    public String getDiscountDetail() {
        return discountDetail;
    }

    public void setDiscountDetail(String discountDetail) {
        this.discountDetail = discountDetail;
    }

    public String getDiscountFee() {
        if (StringUtils.isBlank(discountDetail)) {
            return "0";
        }
        JSONObject json = (JSONObject) JSON.parse(discountDetail);
        return (String) json.get("200");
    }

    @SuppressWarnings("unchecked")
    public static <K, V> Map<K, List<V>> group(List<V> list, String field) {
        Map<K, List<V>> map = new HashMap<K, List<V>>();
        if (list == null || list.isEmpty()) {
            return map;
        }
        for (V v : list) {
            K p = null;
            try {
                p = (K) PropertyUtils.getProperty(v, field);
            } catch (Exception e) {
                throw new RuntimeException(e.getMessage(), e);
            }
            List<V> sublist = map.get(p);
            if (sublist == null) {
                sublist = new ArrayList<V>();
                map.put(p, sublist);
            }
            sublist.add(v);
        }
        return map;
    }

    @Override
    public String toString() {
        return "MarketServiceDTO{" +
                "isvId='" + isvId + '\'' +
                ", appServeId='" + appServeId + '\'' +
                ", canBuy=" + canBuy +
                ", couponId='" + couponId + '\'' +
                ", verifyCode='" + verifyCode + '\'' +
                ", couponCheckAmount=" + couponCheckAmount +
                ", discountAmount=" + discountAmount +
                ", discountDetail='" + discountDetail + '\'' +
                ", discountFee='" + discountFee + '\'' +
                ", price=" + price +
                ", endDate=" + endDate +
                ", auditTime=" + auditTime +
                ", skuList=" + skuList +
                ", skuMapList=" + skuMapList +
                ", skuCodeMapId=" + skuCodeMapId +
                ", basicService=" + basicService +
                ", licenseList=" + licenseList +
                '}';
    }
}
