package com.gic.enterprise.service;

import java.util.List;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.dto.AppletCustomPageDTO;
import com.gic.enterprise.qo.AppletPageQO;

/**
 * 小程序自定义页面装修接口
 * @ClassName: AppletCustomPageApiService 
 * @Description:  
 * @author guojuxing 
 * @date 2019/11/25 11:44 AM 
 */
public interface AppletCustomPageApiService {

    /**
     * 新增
     * @Title: save 
     * @Description:
      * @author guojuxing
     * @param param 页面信息和组件信息的json字符串
     * @param pageId 页面ID，如果为空，是新增；如果有值是编辑
     * @param enterpriseId 商户ID
     * @param saveType 1：保存并发布 2：保存
     * @param appType 1：小程序 2：服务号
     * @param appId  appKey/appId
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    ServiceResponse<Integer> saveOrUpdate(String param, Integer pageId, Integer enterpriseId, Integer saveType,
            Integer appType, String appId);

    /**
     * 页面查询分页
     * @Title: page 
     * @Description: 
      * @author guojuxing 
     * @param params 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.enterprise.dto.AppletCustomPageDTO>> 
      */
    ServiceResponse<Page<AppletCustomPageDTO>> page(AppletPageQO params);

    /**
     * 页面详情
     * @Title: getDetailByPageId 
     * @Description:
      * @author guojuxing
     * @param pageId 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.enterprise.dto.AppletCustomPageDTO> 
      */
    ServiceResponse<AppletCustomPageDTO> getDetailByPageId(Integer pageId);

    /**
     * 发布状态的自定义页面页面详情数据
     * @Title: getMemberCenterDetailByAppId 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @param appType 1：小程序 2：服务号 如果为空，默认查询1：小程序
     * @param appId  小程序/服务号的ID
     * @param pageType 页面类型，详情见枚举类AppletPageTypeEnum
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.enterprise.dto.AppletCustomPageDTO> 
      */
    ServiceResponse<AppletCustomPageDTO> getDetailByAppId(Integer enterpriseId, Integer appType,
            String appId, Integer pageType);

    /**
     * 会员中心装修配置，通用
     *
     * @param enterpriseId
     * @param appId 小程序的ID
     * @return
     */
    ServiceResponse<AppletCustomPageDTO> getCommonMemberInfo(Integer enterpriseId, String appId);

    /**
     * 查询积分服务页面。积分服务本来是一条开启，现在可以开启多个
     * @param enterpriseId
     * @param appType 1：小程序 2：服务号 如果为空，默认查询1：小程序
     * @param appId 小程序/服务号的ID
     * @return
     */
    ServiceResponse<List<AppletCustomPageDTO>> listIntegralServiceByAppId(Integer enterpriseId, Integer appType,
                                                               String appId);

    /**
     * 积分服务页面设置页面序号
     * 页面序号，如果开启页面有多个，根据序号取优先级（积分服务页面专用）。序号越大，优先级越大，如果相同，则编辑时间越晚，优先级越大
     * @param pageId
     * @param serialNumber
     * @return
     */
    ServiceResponse<Void> setSerialNumber(Integer pageId, Integer serialNumber);

    /**
     * 复制页面
     * @param pageId
     * @return
     */
    ServiceResponse<Void> copyPage(Integer pageId);

    /**
     * 开启
     * @Title: enable 
     * @Description:
      * @author guojuxing
     * @param pageId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.String> 
      */
    ServiceResponse<String> enable(Integer pageId);

    /**
     * 关闭
     * @Title: disable 
     * @Description:
      * @author guojuxing
     * @param pageId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.String> 
      */
    ServiceResponse<String> disable(Integer pageId);

    /**
     * 删除
     * @Title: delete 
     * @Description:
      * @author guojuxing
     * @param pageId  页面id
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.String> 
      */
    ServiceResponse<String> delete(Integer pageId);
    /**
     * 删除
     * @Title: delete 
     * @Description:
      * @author guojuxing
     * @param pageIds  页面id
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.String> 
      */
    ServiceResponse<String> deleteFetch(String pageIds);
}
