package com.gic.enterprise.service;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.dto.BalancePayReturnDTO;
import com.gic.enterprise.dto.OutPayDTO;
import com.gic.enterprise.dto.RefundDTO;
import com.gic.enterprise.dto.RefundReturnDTO;

/**
 * 余额支付接口
 * @ClassName: BillingBalancePayApiService
 * @Description:
 * @author zhiwj
 * @date 2019/8/27 9:23
 */
public interface BillingBalancePayApiService {

    /**
     * 查询账户余额
     * @Title: queryAccount
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Double>
     * @throws
     */
    ServiceResponse<Double> queryAccount(Integer enterpriseId);

    /**
     * 统一下单接口
     * @Title: savePrePay
     * @Description:  必填字段：
     *                  enterpriseId： 商户id
     *                  totalFeePaid： 订单金额（实付金额）
     *                  productDescription： 商品描述
     *                  phone： 超管手机号
     *                  productType： 产品类型   1:充值,2:短信套餐,3:服务,4:拓展包,5:套餐包
     *                  timeStart： 交易起始时间
     *                  orderSerialNumber： 订单号
     *                  nonceStr： 随机字符串
     *                  sign： 签名
     *                  requestCode： 渠道来源
     * @author zhiwj
     * @param outPayDTO
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.enterprise.dto.BalancePayReturnDTO>
     * @throws
     */
    ServiceResponse<BalancePayReturnDTO> savePrePay(OutPayDTO outPayDTO);

    /**
     * 确认支付
     * @Title: confirmPay
     * @Description:
     * @author zhiwj
     * @param outPayDTO
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.enterprise.dto.BalancePayReturnDTO>
     * @throws
     */
    ServiceResponse<BalancePayReturnDTO> confirmPay(OutPayDTO outPayDTO);

    /**
     *  确认支付
     * @Title: confirmPay
     * @Description:
     * @author zhiwj
     * @param outPayDTO
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.enterprise.dto.BalancePayReturnDTO>
     * @throws
     */
    ServiceResponse<BalancePayReturnDTO> savePayAndConfirmPay(OutPayDTO outPayDTO);
    /**
     * 订单查询接口
     * @Title: getPayInfo
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param orderSerialNumber
     * @param requestCode
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.enterprise.dto.BalancePayReturnDTO>
     * @throws
     */
    ServiceResponse<BalancePayReturnDTO> getPayInfo(Integer enterpriseId, String orderSerialNumber, String requestCode);

    /**
     * cancelPay 
     * @Title: cancelPay
     * @Description: 
     * @author taogs 
 * @param outPayDTO
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws 
     */
    ServiceResponse<Void> cancelPay(OutPayDTO outPayDTO);

    /**
     * 申请退款
     * @Title: refund
     * @Description:
     * @author zhiwj
     * @param refundDTO
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.enterprise.dto.RefundReturnDTO>
     * @throws
     */
    ServiceResponse<RefundReturnDTO> refund(RefundDTO refundDTO);
    
    /**
     * 查询退款接口
     * @Title: getRefund
     * @Description: 
     * @author zhiwj 
     * @param refundDTO
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.enterprise.dto.RefundReturnDTO>
     * @throws 
     */
    ServiceResponse<RefundReturnDTO> getRefund(RefundDTO refundDTO);

}
