package com.gic.enterprise.service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.dto.BillingPayInfoDTO;
import com.gic.enterprise.dto.InvoiceDTO;
import com.gic.enterprise.dto.OutPayDTO;
import com.gic.enterprise.qo.BillListQueryQO;

/**
 * 支付信息
 * @ClassName: BillingPayInfoApiService 
 * @Description:  
 * @author guojuxing 
 * @date 2019/8/15 1:53 PM 
 */
public interface BillingPayInfoApiService {

    /**
     * 分页查询为开票单据列表数据
     * @Title: listBill 
     * @Description:
      * @author guojuxing
     * @param params 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.enterprise.dto.BillingPayInfoDTO>> 
      */
    ServiceResponse<Page<BillingPayInfoDTO>> listBill(BillListQueryQO params);

    /**
     * 开票申请时，把开户相关信息存入到redis
     * @Title: saveInvoiceAccountToRedis 
     * @Description:
      * @author guojuxing
     * @param invoiceDTO
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> saveInvoiceAccountToRedis(InvoiceDTO invoiceDTO, Integer enterpriseId);

    /**
     * 从redis获取开票申请中的开户相关信息
     * @Title: getInvoiceInfoFromRedis 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.enterprise.dto.InvoiceDTO> 
      */
    ServiceResponse<InvoiceDTO> getInvoiceInfoFromRedis(Integer enterpriseId);

    /**
     * 预支付，生成二维码, 保存支付信息
     * @Title: savePrePayInfo
     * @Description:
     * @author zhiwj
     * @param enterpriseId 企业id
     * @param totalFee 充值金额
     * @param payType 支付方式
     * @param buyType 2:短信套餐包购买 1:商户余额充值
     * @param orderSerialNumber 订单或者充值流水号
     * @return com.gic.enterprise.dto.BillingPayInfoDTO  codeUrl:二维码url
     * @throws
     */
    ServiceResponse<BillingPayInfoDTO> savePrePayInfo(Integer enterpriseId, Double totalFee, Integer payType, Integer buyType, String orderSerialNumber);
    
    /**
     * 保存预支付信息
     * @Title: savePrePayInfo
     * @Description: 
     * @author zhiwj 
     * @param outPayDTO
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.enterprise.dto.BillingPayInfoDTO>
     * @throws 
     */
    ServiceResponse<BillingPayInfoDTO> savePrePayInfo(OutPayDTO outPayDTO);
    /**
     * confirmPay 
     * @Title: confirmPay
     * @Description: 
     * @author taogs 
     * @param outPayDTO
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.enterprise.dto.BillingPayInfoDTO>
     * @throws 
     */
    ServiceResponse<BillingPayInfoDTO> confirmPay(OutPayDTO outPayDTO);

    /**
     * 微信支付回调
     * @Title: wxCallBack
     * @Description:
     * @author zhiwj
     * @param scanPayResDataDTO
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws
     */
    ServiceResponse<Void> wxCallBack(String scanPayResDataDTO);

    /**
     * offlineCallBack 
     * @Title: offlineCallBack
     * @Description: 
     * @author taogs 
     * @param approvalCallBackDTO
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws 
     */
    ServiceResponse<Void> offlineCallBack(String approvalCallBackDTO);

    /**
     * 查询支付状态 并更新
     * @Title: queryAndUpdatePayStatus
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param orderSerialNumber
     * @param payType
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws
     */
    ServiceResponse<Boolean> queryAndUpdatePayStatus(Integer enterpriseId, String orderSerialNumber, Integer payType);

    /**
     * 查详情
     * @Title: getPayInfoDetail
     * @Description:
     * @author zhiwj
     * @param payInfoId
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.enterprise.dto.BillingPayInfoDTO>
     * @throws
     */
    ServiceResponse<BillingPayInfoDTO> getPayInfoDetail(Integer payInfoId);

    /**
     * 查详情
     * @Title: getPayInfo
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param orderSerialNumber
     * @param requestCode
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.enterprise.dto.BillingPayInfoDTO>
     * @throws
     */
    ServiceResponse<BillingPayInfoDTO> getPayInfo(Integer enterpriseId, String orderSerialNumber, String requestCode);

    /**
     * cancelPay
     * @Title: cancelPay
     * @Description: 
     * @author taogs 
 * @param outPayDTO
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws 
     */
    ServiceResponse<Void> cancelPay(OutPayDTO outPayDTO);

    /**
     * 开票更改状态为已开票
     * @Title: updateInvoiceStatus 
     * @Description:
      * @author guojuxing
     * @param serialNumbers 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> updateInvoiceStatus(String serialNumbers);

    /**
     * 根据支付流水号查询支付信息
     * @Title: getPayInfoBySerialNumber 
     * @Description:
      * @author guojuxing
     * @param serialNumber 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.enterprise.dto.BillingPayInfoDTO> 
      */
    ServiceResponse<BillingPayInfoDTO> getPayInfoBySerialNumber(String serialNumber);
}
