package com.gic.enterprise.service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.dto.rule.*;

import java.util.List;
import java.util.Map;

/**
 * 平台规则配置
 * @ClassName: ConfigRuleApiService 
 * @Description:  
 * @author guojuxing 
 * @date 2020/3/3 2:50 PM 
 */
public interface ConfigRuleApiService {

    /**
     * 新增规则
     * @Title: saveRule 
     * @Description:
     * @author guojuxing
     * @param dto 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
     */
    ServiceResponse<Integer> saveRule(RuleDTO dto);
    /**
     * editRule
     * @Title: editRule
     * @Description: 
     * @author taogs 
     * @param dto
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws 
     */
    ServiceResponse<Integer> editRule(RuleDTO dto);

    /**
     * 规则列表
     * @Title: listRule 
     * @Description:
     * @param search 规则名称
     * @author guojuxing  
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.enterprise.dto.rule.RuleDTO>> 
     */
    ServiceResponse<List<RuleDTO>> listRule(String search);

    /**
     * 分页查询分类列表
     * @Title: listRuleClassify 
     * @Description:
     * @author guojuxing
     * @param search 分类名称
     * @param ruleId 规则ID
     * @param pageNum
     * @param pageSize 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.enterprise.dto.rule.RuleClassifyDTO>> 
     */
    ServiceResponse<Page<RuleClassifyDTO>> listRuleClassify(String search, Integer ruleId, Integer pageNum,
            Integer pageSize);

    /**
     * 分页查询场景列表
     * @Title: listRuleClassifyScene 
     * @Description:
     * @author guojuxing
     * @param search 场景名称
     * @param ruleId 分类ID
     * @param pageNum
     * @param pageSize 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.enterprise.dto.rule.RuleClassifySceneDTO>> 
     */
    ServiceResponse<Page<RuleClassifySceneDTO>> listRuleClassifyScene(String search, Integer ruleId, Integer pageNum,
            Integer pageSize);

    /**
     * 新增分类
     * @Title: saveRuleClassify 
     * @Description:
     * @author guojuxing
     * @param dto 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
     */
    ServiceResponse<Integer> saveRuleClassify(RuleClassifyDTO dto);
    /**
     * editRuleClassify
     * @Title: editRuleClassify
     * @Description:
     * @author taogs
     * @param dto
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.String>
     * @throws
     */
    ServiceResponse<String> editRuleClassify(RuleClassifyDTO dto);

    /**
     * 新增场景
     * @Title: saveRuleClassifyScene 
     * @Description:
     * @author guojuxing
     * @param dto 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
     */
    ServiceResponse<Integer> saveRuleClassifyScene(RuleClassifySceneDTO dto);
    /**
     * editRuleClassifyScene
     * @Title: editRuleClassifyScene
     * @Description:
     * @author taogs
     * @param dto
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.String>
     * @throws
     */
    ServiceResponse<String> editRuleClassifyScene(RuleClassifySceneDTO dto);
    /**
     * deleteRule
     * @Title: deleteRule
     * @Description:
     * @author taogs
     * @param ruleId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.String>
     * @throws
     */
    ServiceResponse<String> deleteRule(Integer ruleId);
    /**
     * deleteRuleClassify
     * @Title: deleteRuleClassify
     * @Description:
     * @author taogs
     * @param ruleId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.String>
     * @throws
     */
    ServiceResponse<String> deleteRuleClassify(Integer ruleId);
    /**
     * deleteRuleClassifyScene
     * @Title: deleteRuleClassifyScene
     * @Description:
     * @author taogs
     * @param ruleId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.String>
     * @throws
     */
    ServiceResponse<String> deleteRuleClassifyScene(Integer ruleId);

    /**
     * 查询详情
     * @Title: getDetailByRuleId 
     * @Description:
     * @author guojuxing
     * @param ruleId 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.enterprise.dto.rule.ConfigRuleDTO> 
     */
    ServiceResponse<ConfigRuleDTO> getDetailByRuleId(Integer ruleId);

    /**
     * 复制规则，包括下面的分类和分类下的场景;应用code是空
     * @Title: copyRule 
     * @Description:
     * @author guojuxing
     * @param ruleId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Integer> copyRule(Integer ruleId);

    /**
     * 规则树
     * @Title: getRuleTree 
     * @Description:
     * @author guojuxing
     * @param search 应用名称/规则名称/分类名称/场景名称
     * @param enterpriseRuleId 商户规则ID 如果有值，查询的是有勾选的规则树数据
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.enterprise.dto.rule.RuleListDTO>> 
     */
    ServiceResponse<List<RuleListDTO>> getRuleTree(String search, Integer enterpriseRuleId);

    /**
     * 商户规则详情
     * @Title: getEnterpriseRuleDetail 
     * @Description:
     * @author guojuxing
     * @param enterpriseRuleId 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.enterprise.dto.rule.EnterpriseRuleDetailDTO> 
     */
    ServiceResponse<EnterpriseRuleDetailDTO> getEnterpriseRuleDetail(Integer enterpriseRuleId);

   /**
    * getEnterpriseList 
    * @Title: getEnterpriseList
    * @Description: 
    * @author taogs 
    * @param search
    * @param enterpriseIdList
    * @return com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.enterprise.dto.rule.EnterpriseListDTO>>
    * @throws 
    */
    ServiceResponse<List<EnterpriseListDTO>> getEnterpriseList(String search, List<Integer> enterpriseIdList);

    /**
     * 获取应用列表字典数据
     * @Title: getAppListByDict 
     * @Description:
     * @author guojuxing  
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.Map<java.lang.String,java.lang.String>> 
     */
    ServiceResponse<Map<String, String>> getAppListByDict();

    /***************对接其他组*******************/

    /**
     * listRuleClassifyByAppCode
     * @Title: listRuleClassifyByAppCode
     * @Description: 
     * @author taogs 
 * @param appCode
 * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.enterprise.dto.rule.RuleClassifyListDTO>>
     * @throws 
     */
    ServiceResponse<List<RuleClassifyListDTO>> listRuleClassifyByAppCode(String appCode, Integer enterpriseId);

    /**
     * 根据分类ID查询场景列表
     * @Title: listRuleSceneByRuleClassifyId 
     * @Description:
     * @author guojuxing
     * @param ruleClassifyId 分类ID
     * @param enterpriseId
     * @param appCode 应用code
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.enterprise.dto.rule.RuleClassifyListDTO>> 
     */
    ServiceResponse<List<RuleClassifySceneListDTO>> listRuleSceneByRuleClassifyId(Integer ruleClassifyId,
            String appCode, Integer enterpriseId);
}
