package com.gic.enterprise.service;

import com.gic.api.base.commons.ServiceResponse;

import javax.validation.constraints.NotNull;

/**
 * 联合商户资源详情
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/2/4 9:51 AM 
 */
public interface UnionEnterpriseAuthResDetailApiService {

    /**
     * 业务方数据授权回调
     * @return
     */
    ServiceResponse<Void> authDataCallback();

    /**
     * 门店资源联合进程
     * @param authStatus 资源联合状态 UnionEnterpriseAuthDetailStatusEnum
     * @param ownEnterpriseId 自有商户ID
     * @param unionEnterpriseId 联合商户ID
     * @param authKey 用于重新授权节点
     * @param remark 授权备注
     * @return
     */
    ServiceResponse<Void> unionForStore(@NotNull Integer authStatus, @NotNull Integer ownEnterpriseId, @NotNull Integer unionEnterpriseId,
                                        String authKey, String remark);

    /**
     * 门店资源解除联合进程
     * @param authStatus UnionEnterpriseAuthDetailStatusEnum 会有资源处理中，解除成功等状态
     * @param ownEnterpriseId
     * @param unionEnterpriseId
     * @return
     */
    ServiceResponse<Void> relieveForStore(@NotNull Integer authStatus, @NotNull Integer ownEnterpriseId, @NotNull Integer unionEnterpriseId);

    /**
     * 用户域资源授权回调
     * @param authStatus 资源联合状态 UnionEnterpriseAuthDetailStatusEnum
     * @param ownEnterpriseId 自有商户ID
     * @param unionEnterpriseId 联合商户ID
     * @param authKey 用于重新授权节点
     * @param remark 授权备注
     * @return
     */
    ServiceResponse<Void> unionForUser(@NotNull Integer authStatus, @NotNull Integer ownEnterpriseId, @NotNull Integer unionEnterpriseId,
                                        String authKey, String remark);

    /**
     * 用户域资源解除联合进程
     * @param authStatus UnionEnterpriseAuthDetailStatusEnum 会有资源处理中，解除成功等状态
     * @param ownEnterpriseId
     * @param unionEnterpriseId
     * @return
     */
    ServiceResponse<Void> relieveForUser(@NotNull Integer authStatus, @NotNull Integer ownEnterpriseId, @NotNull Integer unionEnterpriseId);

}
