package com.gic.enterprise.dao.mapper;

import com.gic.enterprise.dto.HomeStatisticsDTO;
import com.gic.enterprise.entity.TabBillingSmsRecord;
import com.gic.enterprise.qo.HomeStatisticsQO;
import com.gic.enterprise.qo.RecordQO;
import com.github.pagehelper.Page;
import org.apache.ibatis.annotations.Param;

import java.util.List;
/**
 *
 * @ClassName: TabBillingSmsRecordMapper
 * @Description:
 * @author taogs
 * @date 2020/9/11 15:39
 */
public interface TabBillingSmsRecordMapper {


    /**
     * 插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insert(TabBillingSmsRecord record);

    /**
     * 动态插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insertSelective(TabBillingSmsRecord record);

    /**
     * 根据主键查询
     *
     * @param smsRecordId 主键
     * @return 实体对象
     */
    TabBillingSmsRecord selectByPrimaryKey(Integer smsRecordId);

    /**
     * 根据主键动态更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKeySelective(TabBillingSmsRecord record);

    /**
     * 根据主键更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKey(TabBillingSmsRecord record);

    /**
     * 消费趋势分析图
     * @Title: listHomeStatistics 
     * @Description:
      * @author guojuxing
     * @param params 
     * @return  java.util.List<com.gic.enterprise.dto.HomeStatisticsDTO> 
      */
    List<HomeStatisticsDTO> listHomeStatistics(HomeStatisticsQO params);

    /**
     * 查询本月或者上月的消费金额
     * @Title: listFee 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @param dateType 
     * @return  double 
      */
    double getFee(@Param("enterpriseId") Integer enterpriseId, @Param("dateType") Integer dateType);
    /**
     * listSmsPackageRecord
     * @Title: listSmsPackageRecord
     * @Description:
     * @author taogs
     * @param search
     * @param startDate
     * @param endDate
     * @param smsPackageId
     * @return com.github.pagehelper.Page<com.gic.enterprise.entity.TabBillingSmsRecord>
     * @throws
     */
    Page<TabBillingSmsRecord> listSmsPackageRecord(@Param("search") String search,
                                                   @Param("startDate") String startDate,
                                                   @Param("endDate") String endDate,
                                                   @Param("smsPackageId") Integer smsPackageId);
    /**
     * listSmsRecord
     * @Title: listSmsRecord
     * @Description:
     * @author taogs
     * @param recordQO
     * @return com.github.pagehelper.Page<com.gic.enterprise.entity.TabBillingSmsRecord>
     * @throws
     */
    Page<TabBillingSmsRecord> listSmsRecord(RecordQO recordQO);
    /**
     * getTotalFee
     * @Title: getTotalFee
     * @Description:
     * @author taogs
     * @param enterpriseId
     * @param startTime
     * @param endTime
     * @return double
     * @throws
     */
    double getTotalFee(@Param("enterpriseId") Integer enterpriseId, @Param("startTime") String startTime, @Param("endTime") String endTime);
    /**
     * countSmsRecord
     * @Title: countSmsRecord
     * @Description:
     * @author taogs
     * @param recordQO
     * @return java.lang.Integer
     * @throws
     */
    Integer countSmsRecord(RecordQO recordQO);
}