package com.gic.enterprise.dao.mapper;

import java.util.List;

import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.qo.EnterpriseQO;
import org.apache.ibatis.annotations.Param;

import com.gic.enterprise.dto.EnterpriseListDTO;
import com.gic.enterprise.entity.TabEnterprise;
/**
 *
 * @ClassName: TabEnterpriseMapper
 * @Description:
 * @author taogs
 * @date 2020/9/11 15:43
 */
public interface TabEnterpriseMapper {

    /**
     * 插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insert(TabEnterprise record);

    /**
     * 动态插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insertSelective(TabEnterprise record);

    /**
     * 根据主键查询
     *
     * @param enterpriseId 主键
     * @return 实体对象
     */
    TabEnterprise selectByPrimaryKey(Integer enterpriseId);

    /**
     * 根据主键动态更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKeySelective(TabEnterprise record);

    /**
     * 根据主键更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKey(TabEnterprise record);

    /**
     * 根据商户名称相同的个数
     * @Title: countByName 
     * @Description:
      * @author guojuxing
     * @param enterpriseName
     * @param enterpriseId 
     * @return  int 
      */
    int countByEnterpriseName(@Param("enterpriseName") String enterpriseName,
                              @Param("enterpriseId") Integer enterpriseId);

    /**
     * 根据公司名称相同的个数
     * @Title: countByName 
     * @Description:
      * @author guojuxing
     * @param companyName
     * @param enterpriseId 
     * @return  int 
      */
    int countByCompanyName(@Param("companyName") String companyName,
                              @Param("enterpriseId") Integer enterpriseId);

    /**
     * listEnterpriseId
     * @Title: listEnterpriseId
     * @Description: 
     * @author taogs 
     * @param search
     * @param serviceVersion
     * @param serviceStatus
     * @param orderBy
     * @return java.util.List<com.gic.enterprise.dto.EnterpriseListDTO>
     * @throws 
     */
    List<EnterpriseListDTO> listEnterpriseId(@Param("search") String search,
                                             @Param("serviceVersion") Integer serviceVersion,
                                             @Param("serviceStatus") Integer serviceStatus,
                                             @Param("orderBy") String orderBy);


    /**
     * 查询列表最新
     * @Title: listEnterpriseIdOfNew 
     * @Description:
     * @author guojuxing
     * @param params 
     * @return  java.util.List<com.gic.enterprise.dto.EnterpriseListDTO> 
     */
    List<EnterpriseListDTO> listEnterpriseIdOfNew(EnterpriseQO params);

    /**
     * 商户名称/公司名称模糊查询商户
     * @Title: listEnterpriseByName 
     * @Description:
      * @author guojuxing
     * @param search 
     * @return  java.util.List<com.gic.enterprise.dto.EnterpriseDTO> 
      */
    List<TabEnterprise> listEnterpriseByName(@Param("search")String search);
    /**
     * listEnterpriseBySearch
     * @Title: listEnterpriseBySearch
     * @Description:
     * @author taogs
     * @param search
     * @return java.util.List<com.gic.enterprise.entity.TabEnterprise>
     * @throws
     */
    List<TabEnterprise> listEnterpriseBySearch(@Param("search")String search);


    /**
     * 查询商户列表数据
     * @Title: listEnterprsie 
     * @Description:
      * @author guojuxing  
     * @return  java.util.List<com.gic.enterprise.dto.EnterpriseListDTO> 
      */
    List<TabEnterprise> listEnterprise();
    /**
     * listEnterpriseByIds
     * @Title: listEnterpriseByIds
     * @Description:
     * @author taogs
     * @param enterpriseIdlist
     * @return java.util.List<com.gic.enterprise.entity.TabEnterprise>
     * @throws
     */
    List<TabEnterprise> listEnterpriseByIds(@Param("enterpriseIdList") List<Integer> enterpriseIdlist);
    /**
     * getEnterpriseByName
     * @Title: getEnterpriseByName
     * @Description:
     * @author taogs
     * @param enterpriseName
     * @return java.util.List<com.gic.enterprise.entity.TabEnterprise>
     * @throws
     */
    List<TabEnterprise> getEnterpriseByName(@Param("enterpriseName") String enterpriseName);
}