package com.gic.enterprise.dao.mapper;

import com.gic.enterprise.dto.union.OwnEnterpriseAuthListDTO;
import com.gic.enterprise.dto.union.UnionEnterpriseAuthListDTO;
import com.gic.enterprise.entity.TabSysUnionEnterpriseAuth;
import org.apache.ibatis.annotations.Param;

import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * 联合商户
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/2/3 7:56 PM 
 */
public interface TabSysUnionEnterpriseAuthMapper {

    /**
     * 插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insert(TabSysUnionEnterpriseAuth record);

    /**
     * 动态插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insertSelective(TabSysUnionEnterpriseAuth record);

    /**
     * 根据主键查询
     *
     * @param unionEnterpriseAuthId 主键
     * @return 实体对象
     */
    TabSysUnionEnterpriseAuth selectByPrimaryKey(Integer unionEnterpriseAuthId);

    /**
     * 根据主键动态更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKeySelective(TabSysUnionEnterpriseAuth record);

    /**
     * 根据主键更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKey(TabSysUnionEnterpriseAuth record);

    /**
     * 查询未解除联合的商户数据
     * @param ownEnterpriseId
     * @param unionEnterpriseId
     * @return
     */
    TabSysUnionEnterpriseAuth getUnionEnterprise(@Param("ownEnterpriseId") Integer ownEnterpriseId,
                                                 @Param("unionEnterpriseId") Integer unionEnterpriseId);

    /**
     * 联合商户列表接口
     * @param ownEnterpriseId
     * @param unionEnterpriseIdList
     * @param authStatus
     * @return
     */
    List<UnionEnterpriseAuthListDTO> listUnionEnterpriseAuth(@Param("ownEnterpriseId") Integer ownEnterpriseId,
                                                             @Param("list") List<Integer> unionEnterpriseIdList,
                                                             @Param("authStatus") Integer authStatus);

    /**
     * 自有商户列表
     * @param unionEnterpriseId
     * @param startTime
     * @param endTime
     * @return
     */
    List<OwnEnterpriseAuthListDTO> listOwnEnterpriseAuth(@Param("unionEnterpriseId") Integer unionEnterpriseId,
                                                         @Param("startTime") String startTime,
                                                         @Param("endTime") String endTime);

    /**
     * 模糊查询账号分组授权的记录
     * @param accountGroupId
     * @return
     */
    List<TabSysUnionEnterpriseAuth> listUnionEnterpriseAuthByAccountGroupId(@Param("accountGroupId") Integer accountGroupId);

    /**
     * 查询全部
     * @return
     */
    List<TabSysUnionEnterpriseAuth> listAll();

    /**
     * 查询
     * @param ownEnterpriseId
     * @return
     */
    List<TabSysUnionEnterpriseAuth> listUnionEnterprise(@Param("ownEnterpriseId") Integer ownEnterpriseId);

    /**
     * 联合授权数量
     * @param resourceGroupId
     * @return
     */
    int countUnionEnterpriseAuth(@Param("resourceGroupId") Integer resourceGroupId);

    /**
     * 资源组授权给的商户
     * @param resourceGroupId
     * @return
     */
    List<TabSysUnionEnterpriseAuth> listUnionEnterpriseAuthByResourceGroupId(@Param("resourceGroupId") Integer resourceGroupId);

    /**
     * 查询自有商户数据
     * @param unionEnterpriseId
     * @return
     */
    List<TabSysUnionEnterpriseAuth> listOwnEnterpriseAuthByOwnEnterpriseId(@Param("unionEnterpriseId") Integer unionEnterpriseId);
}