package com.gic.enterprise.service;

import com.gic.enterprise.dto.rule.RuleClassifyDTO;
import com.gic.enterprise.dto.rule.RuleClassifySceneDTO;
import com.gic.enterprise.dto.rule.RuleDTO;
import com.gic.enterprise.entity.TabConfigRule;
import com.github.pagehelper.Page;

import java.util.List;

/**
 * 平台规则配置
 * @ClassName: ConfigRuleService 
 * @Description:  
 * @author guojuxing 
 * @date 2020/3/3 2:51 PM 
 */
public interface ConfigRuleService {

    /**
     * 新增规则
     * @Title: saveRule 
     * @Description:
     * @author guojuxing
     * @param dto 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
     */
    Integer saveRule(RuleDTO dto);
    /**
     * 新增分类
     * @Title: saveRuleClassify 
     * @Description:
     * @author guojuxing
     * @param dto 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
     */
    Integer saveRuleClassify(RuleClassifyDTO dto);
    /**
     * 新增场景
     * @Title: saveRuleClassifyScene 
     * @Description:
     * @author guojuxing
     * @param dto 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
     */
    Integer saveRuleClassifyScene(RuleClassifySceneDTO dto);
    /**
     * update
     * @Title: update
     * @Description:
     * @author taogs
     * @param record
     * @return void
     * @throws
     */
    void update(TabConfigRule record);
    /**
     * getByRuleId
     * @Title: getByRuleId
     * @Description:
     * @author taogs
     * @param ruleId
     * @return com.gic.enterprise.entity.TabConfigRule
     * @throws
     */
    TabConfigRule getByRuleId(Integer ruleId);

    /**
     * 名称唯一性校验
     * @param ruleId
     * @param ruleLevel 层级1：规则 2：分类 3：场景
     * @param name 规则/分类/场景名称
     * @param parentRuleId 父级ID
     * @return
     */
    boolean isRepeatName(Integer ruleId, Integer ruleLevel, String name, Integer parentRuleId);
    /**
     * isRepeatAppCode
     * @Title: isRepeatAppCode
     * @Description:
     * @author taogs
     * @param ruleId
     * @param appCode
     * @return boolean
     * @throws
     */
    boolean isRepeatAppCode(Integer ruleId, String appCode);

    /**
     * 分页查询
     * @Title: listRule 
     * @Description:
     * @author guojuxing
     * @param search 规则/分类/场景名称
     * @param ruleLevel 1：规则 2：分类 3：场景
     * @param ruleId 如果是规则，则不需要；如果是分类，则需要规则ID；如果是场景，需要分类ID
     * @param pageNum
     * @param pageSize
     * @return  java.util.List<com.gic.enterprise.entity.TabConfigRule> 
     */
    Page<TabConfigRule> listRule(String search, Integer ruleLevel, Integer ruleId, Integer pageNum, Integer pageSize);
    /**
     * listRule
     * @Title: listRule
     * @Description:
     * @author taogs
     * @param search
     * @param ruleLevel
     * @param ruleId
     * @return java.util.List<com.gic.enterprise.entity.TabConfigRule>
     * @throws
     */
    List<TabConfigRule> listRule(String search, Integer ruleLevel, Integer ruleId);
    /**
     * listAll
     * @Title: listAll
     * @Description:
     * @author taogs
     * @param ruleId
     * @return java.util.List<com.gic.enterprise.entity.TabConfigRule>
     * @throws
     */
    List<TabConfigRule> listAll(Integer ruleId);

    /**
     * 应用code查询第一层规则
     * @Title: getByAppCode 
     * @Description:
     * @author guojuxing
     * @param appCode 
     * @return  com.gic.enterprise.entity.TabConfigRule 
     */
    TabConfigRule getByAppCode(String appCode);

    /**
     * 名称查询规则树
     * @Title: getRuleTreeByName 
     * @Description:
     * @author guojuxing
     * @param search 
     * @return  java.util.List<com.gic.enterprise.entity.TabConfigRule> 
     */
    List<TabConfigRule> getRuleTreeByName(String search);
}
