package com.gic.enterprise.service;

import com.gic.enterprise.dto.union.OwnEnterpriseAuthListDTO;
import com.gic.enterprise.dto.union.UnionEnterpriseAuthDTO;
import com.gic.enterprise.dto.union.UnionEnterpriseAuthListDTO;
import com.gic.enterprise.entity.TabSysUnionEnterpriseAuth;

import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * 联合商户
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/2/3 1:52 PM 
 */
public interface UnionEnterpriseAuthService {
    /**
     * 创建联合商户
     *
     * @param ownEnterpriseId   自有商户ID
     * @param unionEnterpriseId 联合商户ID
     * @return
     */
    int saveUnionEnterprise(Integer ownEnterpriseId, Integer unionEnterpriseId);

    /**
     * 获取联合信息
     *
     * @param ownEnterpriseId
     * @param unionEnterpriseId
     * @return
     */
    TabSysUnionEnterpriseAuth getUnionEnterprise(Integer ownEnterpriseId, Integer unionEnterpriseId);

    /**
     * 查询账号分组授权给了哪个联合商户,如果没有，返回null
     * @param accountGroupId
     * @return
     */
    Integer getEnterpriseIdByAuthAccountGroupId(Integer accountGroupId);

    /**
     * 更新数据
     *
     * @param dto
     * @return
     */
    int update(UnionEnterpriseAuthDTO dto);

    /**
     * 联合商户列表接口
     *
     * @param ownEnterpriseId
     * @param unionEnterpriseIdList 联合商户ID
     * @param authStatus            资源授权状态 UnionEnterpriseAuthDetailStatusEnum
     * @return
     */
    List<UnionEnterpriseAuthListDTO> listUnionEnterpriseAuth(Integer ownEnterpriseId, List<Integer> unionEnterpriseIdList, Integer authStatus);

    /**
     * 查询自有商户列表
     *
     * @param unionEnterpriseId
     * @param startTime
     * @param endTime
     * @return
     */
    List<OwnEnterpriseAuthListDTO> listOwnEnterpriseAuth(@NotNull Integer unionEnterpriseId, String startTime, String endTime);

    /**
     * 查询全部
     *
     * @return
     */
    List<TabSysUnionEnterpriseAuth> listAll();

    /**
     * 查询
     *
     * @param ownEnterpriseId
     * @return
     */
    List<TabSysUnionEnterpriseAuth> listUnionEnterpriseAuth(@NotNull Integer ownEnterpriseId);

    /**
     * 是否已经联合授权
     *
     * @param resourceGroupId
     * @return
     */
    Boolean hasUnionEnterpriseAuth(Integer resourceGroupId);

    /**
     * 资源组授权商户
     *
     * @param resourceGroupId
     * @return
     */
    List<TabSysUnionEnterpriseAuth> listUnionEnterpriseAuthByResourceGroupId(Integer resourceGroupId);

    /**
     * 查询自有商户数据
     *
     * @param unionEnterpriseId
     * @return
     */
    List<TabSysUnionEnterpriseAuth> listOwnEnterpriseAuth(@NotNull Integer unionEnterpriseId);

}
