package com.gic.enterprise.service.impl;

import com.gic.enterprise.constant.LinkSortEnum;
import com.gic.enterprise.constant.applet.AppletServiceConfigEnum;
import com.gic.enterprise.dao.mapper.TabAppletServiceConfigMapper;
import com.gic.enterprise.entity.TabAppletServiceConfig;
import com.gic.enterprise.qo.AppletServiceConfigQO;
import com.gic.enterprise.service.AppletServiceConfigService;
import com.gic.member.config.api.dto.AppletsConfigSimpleOutputDTO;
import com.gic.member.config.api.dto.ServiceConfigSimpleOutputDTO;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-11-27 17:04
 */
@Service
public class AppletServiceConfigServiceImpl implements AppletServiceConfigService {

    @Autowired
    private TabAppletServiceConfigMapper tabAppletServiceConfigMapper;

    @Override
    public Page<TabAppletServiceConfig> listApplet(AppletServiceConfigQO appletServiceConfigQO) {
        appletServiceConfigQO.setType(AppletServiceConfigEnum.APPLET.getCode());
        getPage(appletServiceConfigQO);
        return tabAppletServiceConfigMapper.list(appletServiceConfigQO);
    }

    @Override
    public Page<TabAppletServiceConfig> listService(AppletServiceConfigQO appletServiceConfigQO) {
        appletServiceConfigQO.setType(AppletServiceConfigEnum.SERVICE_CONFIG.getCode());
        getPage(appletServiceConfigQO);
        return tabAppletServiceConfigMapper.list(appletServiceConfigQO);
    }

    @Override
    public void syncApplet(List<AppletsConfigSimpleOutputDTO> responseResult, Integer enterpriseId) {
        // 先删除configId不在入参里的 如果入参是空的 就全部删除
        List<Long> configIdList = Optional.ofNullable(responseResult).orElse(Collections.emptyList())
                .stream().map(AppletsConfigSimpleOutputDTO::getAppletConfigId).collect(Collectors.toList());
        tabAppletServiceConfigMapper.delByConfigIdList(configIdList, enterpriseId, AppletServiceConfigEnum.APPLET.getCode());

        List<TabAppletServiceConfig> configList = Optional.ofNullable(responseResult).orElse(Collections.emptyList()).stream()
                .filter(e -> tabAppletServiceConfigMapper.getByConfigId(enterpriseId, e.getAppletConfigId()) == null)
        .map(e -> {
            TabAppletServiceConfig appletServiceConfig = new TabAppletServiceConfig();
            appletServiceConfig.setEnterpriseId(enterpriseId);
            appletServiceConfig.setAppId(e.getAppId());
            appletServiceConfig.setName(e.getAppletConfigName());
            appletServiceConfig.setType(AppletServiceConfigEnum.APPLET.getCode());
            appletServiceConfig.setConfigId(e.getAppletConfigId());
            appletServiceConfig.setCreateTime(new Date());
            appletServiceConfig.setUpdateTime(new Date());
            return appletServiceConfig;
        }).collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(configList)) {
            tabAppletServiceConfigMapper.insertList(configList);
        }
    }

    @Override
    public void syncServiceConfig(List<ServiceConfigSimpleOutputDTO> responseResult, Integer enterpriseId) {
        List<Long> configIdList = Optional.ofNullable(responseResult).orElse(Collections.emptyList())
                .stream().map(ServiceConfigSimpleOutputDTO::getServiceConfigId).collect(Collectors.toList());
        tabAppletServiceConfigMapper.delByConfigIdList(configIdList, enterpriseId, AppletServiceConfigEnum.SERVICE_CONFIG.getCode());

        List<TabAppletServiceConfig> configList = Optional.ofNullable(responseResult).orElse(Collections.emptyList()).stream()
            .filter(e -> tabAppletServiceConfigMapper.getByConfigId(enterpriseId, e.getServiceConfigId()) == null)
        .map(e -> {
            TabAppletServiceConfig appletServiceConfig = new TabAppletServiceConfig();
            appletServiceConfig.setEnterpriseId(enterpriseId);
            appletServiceConfig.setAppId(e.getAppId());
            appletServiceConfig.setName(e.getServiceConfigName());
            appletServiceConfig.setType(AppletServiceConfigEnum.SERVICE_CONFIG.getCode());
            appletServiceConfig.setConfigId(e.getServiceConfigId());
            appletServiceConfig.setCreateTime(new Date());
            appletServiceConfig.setUpdateTime(new Date());
            return appletServiceConfig;
        }).collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(configList)) {
            tabAppletServiceConfigMapper.insertList(configList);
        }
    }

    @Override
    public void updateAppletTime(String appId) {
        this.tabAppletServiceConfigMapper.updateTime(appId, AppletServiceConfigEnum.APPLET.getCode(), new Date());
    }
    @Override
    public void updateServiceTime(String appId) {
        this.tabAppletServiceConfigMapper.updateTime(appId, AppletServiceConfigEnum.SERVICE_CONFIG.getCode(),new Date());
    }

    @Override
    public void delAppletService(Integer appletServiceId) {
        TabAppletServiceConfig appletServiceConfig = new TabAppletServiceConfig();
        appletServiceConfig.setAppletServiceId(appletServiceId);
        appletServiceConfig.setDeleteFlag(1);
        this.tabAppletServiceConfigMapper.updateByPrimaryKeySelective(appletServiceConfig);
    }

    @Override
    public TabAppletServiceConfig getAppletService(Integer appletServiceId) {
        return this.tabAppletServiceConfigMapper.selectByPrimaryKey(appletServiceId);
    }

    private void getPage(AppletServiceConfigQO appletServiceConfigQO) {
        appletServiceConfigQO.setPageNum(appletServiceConfigQO.getCurrentPage());
        if (appletServiceConfigQO.getSortColumn() != null && appletServiceConfigQO.getSortType() != null) {
            String sortColumn = LinkSortEnum.LinkSortColumn.parse(appletServiceConfigQO.getSortColumn());
            String sortType = LinkSortEnum.LinkSortType.parse(appletServiceConfigQO.getSortType());
            if (StringUtils.isNotBlank(sortColumn) && StringUtils.isNotBlank(sortType)) {
                appletServiceConfigQO.setOrderBy(sortColumn + " " + sortType);
            }
        }
        PageHelper.startPage(appletServiceConfigQO);
    }
}
