package com.gic.enterprise.service.impl;

import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.constant.DataSecurityAuditEnum;
import com.gic.enterprise.dao.mapper.TabAuditLogMapper;
import com.gic.enterprise.dto.AuditLogDTO;
import com.gic.enterprise.entity.TabAuditLog;
import com.gic.enterprise.qo.AuditLogQO;
import com.gic.enterprise.service.AuditLogService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.List;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-10-25 15:32
 */
@Service
public class AuditLogServiceImpl implements AuditLogService {

    @Autowired
    private TabAuditLogMapper tabAuditLogMapper;

    @Override
    public TabAuditLog getLastWaitAuditByApplyUser(Integer enterpriseId, Integer applyUserId, Integer projectItemId) {
        AuditLogDTO auditLogDTO = new AuditLogDTO();
        auditLogDTO.setEnterpriseId(enterpriseId);
        auditLogDTO.setApplyUserId(applyUserId);
        auditLogDTO.setProjectItemId(projectItemId);
        return tabAuditLogMapper.getLastWaitAuditByApplyUser(auditLogDTO);
    }

    @Override
    public void update(TabAuditLog auditLog) {
        TabAuditLog result = new TabAuditLog();
        result.setAuditExpireTime(new Date());
        result.setAuditLogId(auditLog.getAuditLogId());
        tabAuditLogMapper.updateByPrimaryKeySelective(result);
    }

    @Override
    public Integer save(AuditLogDTO auditLogDTO) {
        TabAuditLog auditLog = EntityUtil.changeEntityByJSON(TabAuditLog.class, auditLogDTO);
        auditLog.setDeleteFlag(0);
        auditLog.setCreateTime(new Date());
        tabAuditLogMapper.insertSelective(auditLog);
        return auditLog.getAuditLogId();
    }

    @Override
    public TabAuditLog getAuditById(Integer auditLogId) {
        return tabAuditLogMapper.selectByPrimaryKey(auditLogId);
    }

    @Override
    public void updateAudit(AuditLogDTO auditLogDTO) {
        Date date = new Date();
        TabAuditLog auditLog = new TabAuditLog();
        auditLog.setAuditLogId(auditLogDTO.getAuditLogId());
        auditLog.setAuditorId(auditLogDTO.getAuditorId());
        auditLog.setAuditorName(auditLogDTO.getAuditorName());
        auditLog.setAuditorPhone(auditLogDTO.getAuditorPhone());
        auditLog.setAuditTime(date);
        auditLog.setAuditResult(auditLogDTO.getAuditResult());
        auditLog.setUpdateTime(date);
        auditLog.setAuditReason(auditLogDTO.getAuditReason());
        tabAuditLogMapper.updateByPrimaryKeySelective(auditLog);
    }

    @Override
    public int closeAudit(Integer auditLogId) {
        TabAuditLog auditLog = new TabAuditLog();
        auditLog.setUpdateTime(new Date());
        auditLog.setAuditResult(DataSecurityAuditEnum.CLOSE.getCode());
        auditLog.setAuditLogId(auditLogId);
        return tabAuditLogMapper.updateByPrimaryKeySelective(auditLog);
    }

    @Override
    public Page<TabAuditLog> listAuditLog(AuditLogQO auditLogQO) {
        PageHelper.startPage(auditLogQO.getCurrentPage(), auditLogQO.getPageSize());
        return tabAuditLogMapper.listAuditLog(auditLogQO);
    }

    @Override
    public boolean hasProjectItemInUse(Integer projectItemId) {
        return tabAuditLogMapper.getWaitAuditByProjectItem(projectItemId, new Date()) != null;
    }

    @Override
    public List<TabAuditLog> listBySendRandomId(Integer auditLogId) {
        TabAuditLog record = getAuditById(auditLogId);
        if (record != null) {
            List<TabAuditLog> list = tabAuditLogMapper.listBySendRandomId(record.getSendRandomId());
            if (CollectionUtils.isNotEmpty(list)) {
                return list;
            } else {
                return Arrays.asList(record);
            }
        }
        return Collections.EMPTY_LIST;
    }
}
