package com.gic.enterprise.service.impl;

import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.UserApiService;
import com.gic.enterprise.config.Config;
import com.gic.enterprise.constant.Constants;
import com.gic.enterprise.dao.mapper.TabBillingAccountMapper;
import com.gic.enterprise.entity.TabBillingAccount;
import com.gic.enterprise.service.BillingAccountService;
import com.gic.enterprise.service.TodoItemService;
import com.gic.redis.data.util.RedisUtil;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.redisson.api.RAtomicDouble;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
/**
 *
 * @ClassName: BillingAccountServiceImpl
 * @Description:
 * @author taogs
 * @date 2020/9/11 15:25
 */
@Service("billingAccountService")
public class BillingAccountServiceImpl implements BillingAccountService{
    private Logger logger = LogManager.getLogger(BillingAccountServiceImpl.class);
    @Autowired
    private TabBillingAccountMapper tabBillingAccountMapper;
    @Autowired
    private TodoItemService todoItemService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private Config config;

    @Override
    public TabBillingAccount getByEnterpriseId(Integer enterpriseId) {
        return tabBillingAccountMapper.selectByEnterpriseId(enterpriseId);
    }

    @Override
    public void updateBillingAccount(TabBillingAccount tabBillingAccount) {
        tabBillingAccount.setUpdateTime(new Date());
        tabBillingAccountMapper.updateByPrimaryKeySelective(tabBillingAccount);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int updateAccountAfterRechargeOrRefund(double totalFee, Integer enterpriseId) {

        RedissonClient redisClient = RedisUtil.getRedisClient();
        RAtomicDouble account = redisClient.getAtomicDouble(Constants.ENTERPRISE_BILLING_ACCOUNT + enterpriseId);
        account.addAndGet(totalFee);

        TabBillingAccount tabBillingAccount = tabBillingAccountMapper.selectByEnterpriseId(enterpriseId);
        double newTotal = tabBillingAccount.getAccountBalance() + totalFee;
        logger.info("商户：{}, 充值/退款前余额:{}, 充值/退款金额:{}, 充值/退款后余额:{}", enterpriseId, tabBillingAccount.getAccountBalance(), totalFee, newTotal);
        UserDTO user = userApiService.getUserByEnterpriseId(enterpriseId).getResult();
        todoItemService.finishTodoItem(enterpriseId, user.getUserId(), Constants.PRODUCT, config.getHost() + Constants.TODO_ITEM_ACCOUNT_BALANCE_URL);

        return tabBillingAccountMapper.updateAccountAfterRechargeOrRefund(totalFee, enterpriseId);
    }

    @Override
    public int deductAccount(double totalFee, Integer enterpriseId) {
        deductAccountByCache(totalFee, enterpriseId);
        return deductAccountByDatabase(totalFee, enterpriseId);
    }

    @Override
    public int deductAccountByDatabase(double totalFee, Integer enterpriseId) {
        return tabBillingAccountMapper.deductAccount(totalFee, enterpriseId);
    }


    @Override
    public void deductAccountByCache(double totalFee, Integer enterpriseId) {
        RedissonClient redisClient = RedisUtil.getRedisClient();
        RAtomicDouble account = redisClient.getAtomicDouble(Constants.ENTERPRISE_BILLING_ACCOUNT + enterpriseId);
        account.addAndGet(- totalFee);
    }

    @Override
    public void updateBillingAccountByEnterpriseId(TabBillingAccount tabBillingAccount) {
        tabBillingAccount.setUpdateTime(new Date());
        tabBillingAccountMapper.updateByEnterpriseIdSelective(tabBillingAccount);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public int createBillingAccount(Integer enterpriseId) {
        TabBillingAccount record = new TabBillingAccount();
        record.setCreateTime(new Date());
        record.setUpdateTime(new Date());
        record.setEnterpriseId(enterpriseId);
        record.setCreditLine(0);
        record.setAutoRecharge(0);
        record.setAccountBalance(0D);

        RedissonClient redisClient = RedisUtil.getRedisClient();
        RAtomicDouble account = redisClient.getAtomicDouble(Constants.ENTERPRISE_BILLING_ACCOUNT + enterpriseId);
        account.set(0D);

        return tabBillingAccountMapper.insert(record);
    }

    @Override
    public List<TabBillingAccount> listBillingAccount(List<Integer> enterpriseIdList) {
        return tabBillingAccountMapper.listBillingAccount(enterpriseIdList);
    }
}
