package com.gic.enterprise.service.impl;

import com.alibaba.fastjson.JSON;
import com.gic.enterprise.constant.BillingOrderStatusEnum;
import com.gic.enterprise.constant.EnterpriseStatusTypeEnum;
import com.gic.enterprise.constant.PayTypeEnum;
import com.gic.enterprise.dao.mapper.TabBillingOrderMapper;
import com.gic.enterprise.dto.BillingOrderDTO;
import com.gic.enterprise.dto.SmsPackageOrderItemDTO;
import com.gic.enterprise.entity.TabBillingEnterpriseSmsPackage;
import com.gic.enterprise.entity.TabBillingOrder;
import com.gic.enterprise.service.BillingEnterpriseSmsPackageService;
import com.gic.enterprise.service.BillingOrderService;
import com.gic.enterprise.utils.CreateRandomUtils;
import com.github.pagehelper.Page;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Calendar;
import java.util.Date;
/**
 * 
 * @ClassName: BillingOrderServiceImpl
 * @Description:  
 * @author taogs
 * @date 2020/9/11 15:27
 */
@Service("billingOrderService")
public class BillingOrderServiceImpl implements BillingOrderService {

    private Logger logger = LogManager.getLogger(BillingOrderServiceImpl.class);
    @Autowired
    private TabBillingOrderMapper tabBillingOrderMapper;
    @Autowired
    private BillingEnterpriseSmsPackageService billingEnterpriseSmsPackageService;

    @Override
    public Integer saveOrderBuyNow(BillingOrderDTO billingOrderDTO) {
        TabBillingOrder billingOrder = new TabBillingOrder();
        billingOrder.setEnterpriseId(billingOrderDTO.getEnterpriseId());
        // 统一流水号
        billingOrder.setSerialNumber(CreateRandomUtils.createSerialNumber());
        billingOrder.setOrderStatus(BillingOrderStatusEnum.WAIT_PAY.getCode());
        billingOrder.setTotalFee(billingOrderDTO.getTotalFee());
        billingOrder.setTotalFeePaid(billingOrderDTO.getTotalFeePaid());
        billingOrder.setDeleteFlag(0);
        billingOrder.setCreateTime(new Date());
        billingOrder.setItemJson(billingOrderDTO.getItemJson());
        billingOrder.setPayType(billingOrderDTO.getPayType());
        billingOrder.setOrderContent(billingOrderDTO.getOrderContent());
        billingOrder.setInitiatorUser(billingOrderDTO.getInitiatorUser());
        tabBillingOrderMapper.insertSelective(billingOrder);
        return billingOrder.getOrderId();
    }

    @Override
    public boolean updateOrderPayInfoId(Integer orderId, Integer payInfoId, Integer payType) {
        TabBillingOrder order = new TabBillingOrder();
        order.setOrderId(orderId);
        order.setPayInfoId(payInfoId);
        order.setPayType(payType);
        if(payType == PayTypeEnum.OFFLINE_PAY.getCode()){
            order.setOrderStatus(BillingOrderStatusEnum.AUDITING.getCode());
        }
        int result = this.tabBillingOrderMapper.updateByPrimaryKeySelective(order);
        if(result > 0){
            return true;
        }
        return false;
    }

    @Override
    public TabBillingOrder getOrderByPayInfoId(Integer payInfoId) {
        return this.tabBillingOrderMapper.getOrderByPayInfoId(payInfoId);
    }

    @Override
    public Page<TabBillingOrder> listOrder(Integer enterpriseId, Integer status, String search, String startDate, String endDate) {
        Page<TabBillingOrder> list = this.tabBillingOrderMapper.listOrder(enterpriseId, status, search, startDate, endDate);
        return list;
    }

    @Override
    public TabBillingOrder getOrderById(Integer id) {
        return this.tabBillingOrderMapper.selectByPrimaryKey(id);
    }

    @Override
    public Integer cancelOrderApproval(Integer orderId, Integer auditStatus) {
        TabBillingOrder billingOrder = new TabBillingOrder();
        billingOrder.setOrderId(orderId);
        billingOrder.setOrderStatus(BillingOrderStatusEnum.CLOSE.getCode());
        return this.tabBillingOrderMapper.updateByPrimaryKeySelective(billingOrder);
    }

    @Override
    @Transactional(rollbackFor=Exception.class)
    public void updateBySerialNumber(String orderSerialNumber, Double totalFeePaid, Integer auditStatus, String timeEnd) {
        //更新订单状态
        //更新企业套餐包
        TabBillingOrder order = this.getOrderBySerialNumber(orderSerialNumber);
        if(order != null){
            int i1 = this.tabBillingOrderMapper.updateOrderBySerialNumber(orderSerialNumber, totalFeePaid, auditStatus);
            logger.info("修改订单状态：{}", i1);
            SmsPackageOrderItemDTO itemDTO = JSON.parseObject(order.getItemJson(), SmsPackageOrderItemDTO.class);
            if(itemDTO != null){
                for(int i= 0; i< itemDTO.getBuyCount(); i++){
                    TabBillingEnterpriseSmsPackage smsPackage = new TabBillingEnterpriseSmsPackage();
                    smsPackage.setType(1);
                    smsPackage.setEnterpriseId(order.getEnterpriseId());
                    smsPackage.setStatus(EnterpriseStatusTypeEnum.EFFECTIVE.getCode());
                    smsPackage.setCreateTime(new Date());
                    smsPackage.setUpdateTime(new Date());
                    smsPackage.setSmsPackageCode(itemDTO.getCode());
                    smsPackage.setSmsPackageName(itemDTO.getSmsPackageName());
                    smsPackage.setBillingOrderId(order.getOrderId());
                    smsPackage.setTotalAcount(itemDTO.getSmsAmount());
                    smsPackage.setRemainAccount(itemDTO.getSmsAmount());
                    Calendar now = Calendar.getInstance();
                    now.setTime(new Date());
                    now.add(Calendar.MONTH, itemDTO.getPackageLength());
                    now.set(Calendar.HOUR, 23);
                    now.set(Calendar.MINUTE, 59);
                    now.set(Calendar.SECOND, 59);

                    smsPackage.setEffectiveDate(now.getTime());
                    this.billingEnterpriseSmsPackageService.saveEnterpriseSmsPackage(smsPackage);
                }
            }
        }

    }

    @Override
    public TabBillingOrder getOrderBySerialNumber(String orderSerialNumber) {
        return tabBillingOrderMapper.getOrderBySerialNumber(orderSerialNumber);
    }

}
