package com.gic.enterprise.service.impl;

import com.gic.commons.util.DateUtil;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.enterprise.dao.mapper.TabBillingPayInfoMapper;
import com.gic.enterprise.dto.ApprovalCallBackDTO;
import com.gic.enterprise.dto.BillingPayInfoDTO;
import com.gic.enterprise.entity.TabBillingPayInfo;
import com.gic.enterprise.qo.BillListQueryQO;
import com.gic.enterprise.service.BillingPayInfoService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
/**
 * 
 * @ClassName: BillingPayInfoServiceImpl
 * @Description:  
 * @author taogs
 * @date 2020/9/11 15:27
 */
@Service("billingPayInfoService")
public class BillingPayInfoServiceImpl implements BillingPayInfoService{
    @Autowired
    private TabBillingPayInfoMapper tabBillingPayInfoMapper;
    @Override
    public TabBillingPayInfo getById(Integer billingPayInfoId) {
        return tabBillingPayInfoMapper.selectByPrimaryKey(billingPayInfoId);
    }

    @Override
    public Page<TabBillingPayInfo> listBilling(BillListQueryQO params) {
        PageHelper.startPage(params.getCurrentPage(), params.getPageSize());
        List<TabBillingPayInfo> list = tabBillingPayInfoMapper.listBill(params);
        return (Page<TabBillingPayInfo>) list;
    }

    @Override
    public Integer savePayInfo(BillingPayInfoDTO billingPayInfoDTO) {
        billingPayInfoDTO.setInvoiceStatus(GlobalInfo.DATA_STATUS_DELETE);
        TabBillingPayInfo tabBillingPayInfo = EntityUtil.changeEntityByJSON(TabBillingPayInfo.class, billingPayInfoDTO);
        tabBillingPayInfo.setCreateTime(new Date());
        tabBillingPayInfoMapper.insertSelective(tabBillingPayInfo);
        return tabBillingPayInfo.getPayInfoId();
    }

    @Override
    public Integer updatePayInfoBySerialNumber(ApprovalCallBackDTO resData) {
        TabBillingPayInfo tabBillingPayInfo = new TabBillingPayInfo();
        tabBillingPayInfo.setEnterpriseId(resData.getEnterpriseId());
        tabBillingPayInfo.setTimeEnd(resData.getTimeEnd());
        tabBillingPayInfo.setBuyType(resData.getBuyType());
        tabBillingPayInfo.setSerialNumber(resData.getPaySerialNumber());
        tabBillingPayInfo.setPrepayId(resData.getPrepayId());
        tabBillingPayInfo.setTotalFeePaid(resData.getTotalFeePaid());
        tabBillingPayInfo.setResultCode(resData.getResultCode());
        tabBillingPayInfo.setErrCode(resData.getErrorCode());
        tabBillingPayInfo.setErrCodeDes(resData.getErrorCodeRes());
        return tabBillingPayInfoMapper.updateBySerialNumber(tabBillingPayInfo);
    }

    @Override
    public TabBillingPayInfo getBySerialNumber(String serialNumber) {
        return tabBillingPayInfoMapper.getBySerialNumber(serialNumber);
    }

    @Override
    public void updateAudit(Double accountAmount, Date approvalTime, Integer payInfoId) {
        TabBillingPayInfo billingPayInfo = new TabBillingPayInfo();
        billingPayInfo.setTotalFeePaid(accountAmount);
        billingPayInfo.setTimeEnd(DateUtil.formatString(approvalTime, DateUtil.FORMAT_DATETIME_14));
        billingPayInfo.setPayInfoId(payInfoId);
        tabBillingPayInfoMapper.updateByPrimaryKeySelective(billingPayInfo);
    }

    @Override
    public TabBillingPayInfo getByOrderSerialNumber(String orderSerialNumber, Integer payType) {
        TabBillingPayInfo billingPayInfo = new TabBillingPayInfo();
        billingPayInfo.setOrderSerialNumber(orderSerialNumber);
        billingPayInfo.setPayType(payType);
        return tabBillingPayInfoMapper.getByOrderSerialNumber(billingPayInfo);
    }

    @Override
    public TabBillingPayInfo getByOrderSerialNumber(Integer enterpriseId, String requestCode, String orderSerialNumber) {
        TabBillingPayInfo billingPayInfo = new TabBillingPayInfo();
        billingPayInfo.setEnterpriseId(enterpriseId);
        billingPayInfo.setRequestCode(requestCode);
        billingPayInfo.setOrderSerialNumber(orderSerialNumber);
        return tabBillingPayInfoMapper.getByOrderSerialNumber(billingPayInfo);
    }

    @Override
    public Integer cancelPay(BillingPayInfoDTO payInfoDTO) {
        return this.tabBillingPayInfoMapper.cancelPay(payInfoDTO);
    }

    @Override
    public void delOtherPayInfoExceptId(Integer payInfoId, Integer enterpriseId, String orderSerialNumber) {
        this.tabBillingPayInfoMapper.delOtherPayInfoExceptId(payInfoId, enterpriseId, orderSerialNumber);
    }

    @Override
    public void updateInvoiceStatus(String serialNumbers) {
        tabBillingPayInfoMapper.updateInvoiceStatus(serialNumbers);
    }

    @Override
    public TabBillingPayInfo getPayInfoBySerialNumber(String serialNumber) {
        return tabBillingPayInfoMapper.getPayInfoBySerialNumber(serialNumber);
    }

    @Override
    public void updatePayType(Integer payInfoId, Integer payType) {
        TabBillingPayInfo info = new TabBillingPayInfo();
        info.setPayInfoId(payInfoId);
        info.setPayType(payType);
        tabBillingPayInfoMapper.updateByPrimaryKeySelective(info);
    }

}
