package com.gic.enterprise.service.impl;

import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.enterprise.constant.DataSecurityAuditEnum;
import com.gic.enterprise.constant.DownloadReportStatusEnum;
import com.gic.enterprise.dao.mapper.TabDownloadReportMapper;
import com.gic.enterprise.dto.DownloadReportDTO;
import com.gic.enterprise.entity.TabDownloadReport;
import com.gic.enterprise.qo.DownloadReportQO;
import com.gic.enterprise.service.DownloadReportService;
import com.gic.enterprise.utils.CreateRandomUtils;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-10-28 15:52
 */
@Service
public class DownloadReportServiceImpl implements DownloadReportService {

    @Autowired
    private TabDownloadReportMapper tabDownloadReportMapper;

    @Override
    public Integer save(DownloadReportDTO downloadReportDTO) {
        TabDownloadReport tabDownloadReport = EntityUtil.changeEntityByJSON(TabDownloadReport.class, downloadReportDTO);
        tabDownloadReport.setEnterpriseId(downloadReportDTO.getEnterpriseId());
        tabDownloadReport.setReportId(getReportId());
        tabDownloadReport.setStatus(GlobalInfo.DATA_STATUS_NORMAL);
        tabDownloadReport.setCreateTime(new Date());
        tabDownloadReportMapper.insertSelective(tabDownloadReport);
        return tabDownloadReport.getDownloadReportId();
    }

    @Override
    public Integer updateDownloadUrl(Integer downloadReportId, String downloadUrl) {
        return updateDownloadUrl(downloadReportId, downloadUrl, null);
    }

    @Override
    public Integer updateDownloadUrl(Integer downloadReportId, String downloadUrl, Integer count) {
        TabDownloadReport downloadReport = new TabDownloadReport();
        downloadReport.setDownloadReportId(downloadReportId);
        downloadReport.setDownloadUrl(downloadUrl);
        downloadReport.setStatus(DownloadReportStatusEnum.GENERATED.getCode());
        if(count != null){
            downloadReport.setDataCount(count);
        }
        return tabDownloadReportMapper.updateByPrimaryKeySelective(downloadReport);
    }

    @Override
    public Page<TabDownloadReport> listDownloadReport(DownloadReportQO downloadReportQO) {
        PageHelper.startPage(downloadReportQO.getCurrentPage(), downloadReportQO.getPageSize());
        if (downloadReportQO.getDownloadReportStatus() != null) {
            Integer downloadReportStatus = downloadReportQO.getDownloadReportStatus();
            downloadReportQO.setDownloadReportStatus(null);
            switch (downloadReportStatus) {
                case 0:
                    downloadReportQO.setAuditResult(DataSecurityAuditEnum.WAIT.getCode());
                    downloadReportQO.setLtAuditExpireTime(new Date());
                    downloadReportQO.setHasDownloadUrl(null);
                    break;
                case 2:
                    downloadReportQO.setAuditResult(DataSecurityAuditEnum.FAIL.getCode());
                    downloadReportQO.setHasDownloadUrl(null);
                    break;
                case 3:
                    downloadReportQO.setAuditResult(DataSecurityAuditEnum.WAIT.getCode());
                    downloadReportQO.setAuditExpireTime(new Date());
                    downloadReportQO.setHasDownloadUrl(null);
                    break;
                case 4:
                    downloadReportQO.setDownloadReportStatus(4);
                    break;
                case 5:
                    downloadReportQO.setDownloadReportStatus(5);
                    break;
                case 6:
                    downloadReportQO.setDownloadReportStatus(6);
                    break;
                case 7:
                    downloadReportQO.setDownloadReportStatus(7);
                default:
                    break;
            }
        } else {
            downloadReportQO.setAuditResult(null);
            downloadReportQO.setHasDownloadUrl(null);
        }
        return tabDownloadReportMapper.listDownloadReport(downloadReportQO);
    }

    @Override
    public int updateAudit(Integer downloadReportId, Integer auditResult) {
        TabDownloadReport downloadReport = new TabDownloadReport();
        downloadReport.setDownloadReportId(downloadReportId);
        downloadReport.setAuditResult(auditResult);
        return tabDownloadReportMapper.updateByPrimaryKeySelective(downloadReport);
    }

    @Override
    public TabDownloadReport getById(Integer downloadReportId) {
        return tabDownloadReportMapper.selectByPrimaryKey(downloadReportId);
    }

    @Override
    public void updateAudit(TabDownloadReport downloadReport) {
        tabDownloadReportMapper.updateByPrimaryKeySelective(downloadReport);
    }

    @Override
    public void updateDownloadStatus(Integer downloadReportId, DownloadReportStatusEnum downloadReportStatusEnum) {
        TabDownloadReport downloadReport = new TabDownloadReport();
        downloadReport.setDownloadReportId(downloadReportId);
        downloadReport.setStatus(downloadReportStatusEnum.getCode());
        tabDownloadReportMapper.updateByPrimaryKeySelective(downloadReport);
    }

    @Override
    public void updateDataCount(Integer reportId, Integer count) {
        TabDownloadReport downloadReport = new TabDownloadReport();
        downloadReport.setReportId(reportId.toString());
        downloadReport.setDataCount(count);
        tabDownloadReportMapper.updateByPrimaryKeySelective(downloadReport);
    }

    @Override
    public List<TabDownloadReport> listUnfinishedTask() {
        return tabDownloadReportMapper.listUnfinishedTask();
    }

    private String getReportId() {
        String reportId = CreateRandomUtils.generateNumberStr(8);
        TabDownloadReport downloadReport = new TabDownloadReport();
        downloadReport.setReportId(reportId);
        TabDownloadReport result = tabDownloadReportMapper.getBySelective(downloadReport);
        if (result == null) {
            return reportId;
        } else {
            return getReportId();
        }
    }
}
