package com.gic.enterprise.service.impl;

import com.gic.commons.util.GlobalInfo;
import com.gic.enterprise.constant.DeleteFlagConstants;
import com.gic.enterprise.dao.mapper.TabProjectItemMapper;
import com.gic.enterprise.dao.mapper.TabProjectMapper;
import com.gic.enterprise.dto.ProjectDTO;
import com.gic.enterprise.entity.TabProject;
import com.gic.enterprise.entity.TabProjectItem;
import com.gic.enterprise.service.ProjectService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * 
 * @ClassName: ProjectServiceImpl
 * @Description:  
 * @author zhiwj
 * @date 2019-09-17 9:49
 */
@Service
public class ProjectServiceImpl implements ProjectService {

    @Autowired
    private TabProjectMapper tabProjectMapper;
    @Autowired
    private TabProjectItemMapper tabProjectItemMapper;

    @Override
    public Integer save(ProjectDTO copy) {
        TabProject tabProject = new TabProject();
        tabProject.setProjectName(copy.getProjectName());
        tabProject.setDeleteFlag(DeleteFlagConstants.NORMAL_STATUS);
        tabProject.setCreateTime(new Date());

        Double sort = tabProjectMapper.getMinSort();
        tabProject.setSort(sort == null ? 1d : sort - 1d);

        tabProjectMapper.insertSelective(tabProject);
        return tabProject.getProjectId();
    }

    @Override
    public Integer update(ProjectDTO copy) {
        TabProject tabProject = new TabProject();
        tabProject.setProjectId(copy.getProjectId());
        tabProject.setProjectName(copy.getProjectName());
        return tabProjectMapper.updateByPrimaryKeySelective(tabProject);
    }

    @Override
    public Integer delProject(Integer projectId) {
        TabProject tabProject = new TabProject();
        tabProject.setProjectId(projectId);
        tabProject.setDeleteFlag(DeleteFlagConstants.DELETE_STATUS);
        return tabProjectMapper.updateByPrimaryKeySelective(tabProject);
    }

    @Override
    public Page<TabProject> listProject(ProjectDTO projectDTO, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        return tabProjectMapper.listProject(projectDTO);
    }

    @Override
    public boolean validProjectNameIsRepeat(String projectName, Integer projectId) {
        ProjectDTO projectDTO = new ProjectDTO();
        projectDTO.setProjectName(projectName);
        projectDTO.setProjectId(projectId);
        TabProject project = tabProjectMapper.getFirstProject(projectDTO);
        return project != null;
    }

    @Override
    public TabProject getById(Integer projectId) {
        TabProject project = tabProjectMapper.selectByPrimaryKey(projectId);
        if (project != null && project.getDeleteFlag() == DeleteFlagConstants.NORMAL_STATUS) {
            return project;
        }
        return null;
    }

    @Override
    public List<TabProject> listAllProject() {
        return tabProjectMapper.listProject(new ProjectDTO());
    }

    @Override
    public TabProject getProjectByProjectItem(Integer projectItemId) {
        TabProjectItem projectItem = tabProjectItemMapper.selectByPrimaryKey(projectItemId);
        return tabProjectMapper.selectByPrimaryKey(projectItem.getProjectId());
    }

    @Override
    public void setProjectSort(Integer projectId, Integer seq) {
        if (seq <= 1) {
            // 相当于置顶
            sortTopping(projectId);
        } else {
            int start = seq - 2;
            int size = 2;

            List<TabProject> list = tabProjectMapper.listTwoElementExceptId(start, size, projectId);
            if (CollectionUtils.isEmpty(list)) {
                // 置底
                sortBottom(projectId);
            } else {
                Double sort;
                if (list.size() == 1) {
                    // 置底
                    sort = list.get(0).getSort() + 1d;
                } else {
                    sort = (list.get(0).getSort() + list.get(1).getSort()) / 2d;
                }
                TabProject project = new TabProject();
                project.setProjectId(projectId);
                project.setSort(sort);
                tabProjectMapper.updateByPrimaryKeySelective(project);
            }
        }
    }

    private void sortTopping(Integer projectId) {
        Double sort = tabProjectMapper.getMinSort();
        TabProject project = new TabProject();
        project.setProjectId(projectId);
        project.setSort(sort - 1d);
        tabProjectMapper.updateByPrimaryKeySelective(project);
    }

    private void sortBottom(Integer projectId) {
        Double sort = tabProjectMapper.getMaxSort();
        if (sort != null) {
            TabProject project = new TabProject();
            project.setProjectId(projectId);
            project.setSort(sort + 1d);
            tabProjectMapper.updateByPrimaryKeySelective(project);
        }
    }

}
