package com.gic.enterprise.service.impl;

import com.gic.enterprise.dao.mapper.TabTableSettingMapper;
import com.gic.enterprise.dto.TableSettingFieldDTO;
import com.gic.enterprise.entity.TabTableSetting;
import com.gic.enterprise.service.TableSettingFieldService;
import com.gic.enterprise.service.TableSettingService;
import com.github.pagehelper.Page;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

@Service
public class TableSettingServiceImpl implements TableSettingService {
    private static final Logger log = LogManager.getLogger(TableSettingServiceImpl.class);
    @Autowired
    private TabTableSettingMapper tabTableSettingMapper;
    @Autowired
    private TableSettingFieldService tableSettingFieldService;

    @Override
    public String saveTableSetting(TabTableSetting tabTableSetting) {
        tabTableSetting.setUpdateTime(new Date());
        if(tabTableSetting.getTableId() == null){
            //校验是否重复
            TabTableSetting existTable = tabTableSettingMapper.getTableSettingByTableName(tabTableSetting.getTableName());
            if(existTable != null){
                return "表名已经存在";
            }
            tabTableSetting.setCreateTime(new Date());
            tabTableSettingMapper.insertSelective(tabTableSetting);
            Integer tableId = tabTableSetting.getTableId();
            saveDefaultField(tableId);
        } else {
            tabTableSettingMapper.updateByPrimaryKeySelective(tabTableSetting);
        }
        return null;
    }

    @Override
    public Page<TabTableSetting> pageTableSetting(String search) {
        Page<TabTableSetting> page = tabTableSettingMapper.pageTableSetting(search);
        return page;
    }

    @Override
    public TabTableSetting getTableSetting(Integer tableId) {
        return tabTableSettingMapper.selectByPrimaryKey(tableId);
    }

    @Override
    public void delTabSetting(Integer tableId) {
        TabTableSetting tabTableSetting = new TabTableSetting();
        tabTableSetting.setTableId(tableId);
        tabTableSetting.setDeleteFlag(1);
        tabTableSettingMapper.updateByPrimaryKeySelective(tabTableSetting);
    }

    @Override
    public TabTableSetting getTableSetting(String tableName) {
        return tabTableSettingMapper.getTableSettingByTableName(tableName);
    }

    private void saveDefaultField(Integer tableId){
        TableSettingFieldDTO defaultField = new TableSettingFieldDTO();
        defaultField.setFieldKey("id");
        defaultField.setFieldName("主键");
        defaultField.setFieldType("string");
        defaultField.setTableId(tableId);
        tableSettingFieldService.saveTableField(defaultField);
        defaultField.setFieldKey("create_time");
        defaultField.setFieldName("创建时间");
        defaultField.setFieldType("date");
        tableSettingFieldService.saveTableField(defaultField);
        defaultField.setFieldKey("update_time");
        defaultField.setFieldName("更新时间");
        defaultField.setFieldType("date");
        tableSettingFieldService.saveTableField(defaultField);
    }
}
