package com.gic.enterprise.service.impl;

import com.gic.enterprise.constant.union.UnionEnterpriseAuthDetailStatusEnum;
import com.gic.enterprise.constant.union.UnionEnterpriseAuthResOperateEnum;
import com.gic.enterprise.constant.union.UnionEnterpriseAuthResTypeEnum;
import com.gic.enterprise.dao.mapper.TabSysUnionEnterpriseAuthResDetailMapper;
import com.gic.enterprise.entity.TabSysUnionEnterpriseAuthResDetail;
import com.gic.enterprise.service.UnionEnterpriseAuthResDetailService;
import com.gic.enterprise.service.UnionEnterpriseAuthResLogService;
import com.gic.enterprise.service.UnionEnterpriseAuthService;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.validation.constraints.NotNull;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Optional;

@Service("unionEnterpriseAuthResDetailService")
public class UnionEnterpriseAuthResDetailServiceImpl implements UnionEnterpriseAuthResDetailService{
    @Autowired
    private TabSysUnionEnterpriseAuthResDetailMapper tabSysUnionEnterpriseAuthResDetailMapper;
    @Autowired
    private UnionEnterpriseAuthResLogService unionEnterpriseAuthResLogService;
    @Autowired
    private UnionEnterpriseAuthService unionEnterpriseAuthService;

    @Override
    public List<TabSysUnionEnterpriseAuthResDetail> listUnionEnterpriseAuthResDetail(Integer unionEnterpriseAuthId, Integer authStatus) {
        return Optional.ofNullable(tabSysUnionEnterpriseAuthResDetailMapper
                .listUnionEnterpriseAuthResDetail(unionEnterpriseAuthId, authStatus))
                .orElse(Collections.emptyList());
    }

    @Override
    public TabSysUnionEnterpriseAuthResDetail getUnionEnterpriseAuthResDetail(@NotNull Integer unionEnterpriseAuthId, @NotNull Integer resourceType) {
        List<TabSysUnionEnterpriseAuthResDetail> list = listUnionEnterpriseAuthResDetail(unionEnterpriseAuthId, null);
        if (CollectionUtils.isNotEmpty(list)) {
            for (TabSysUnionEnterpriseAuthResDetail obj : list) {
                if (obj.getResourceType().equals(resourceType)) {
                    return obj;
                }
            }
        }
        return null;
    }

    @Override
    public void initUnionEnterpriseAuthResDetail(@NotNull Integer unionEnterpriseAuthId) {
        List<TabSysUnionEnterpriseAuthResDetail> list = listUnionEnterpriseAuthResDetail(unionEnterpriseAuthId, null);
        if (CollectionUtils.isNotEmpty(list)) {
            //重置状态
            for (TabSysUnionEnterpriseAuthResDetail old : list) {
                old.setAuthStatus(UnionEnterpriseAuthDetailStatusEnum.NO_AUTH.getCode())
                        .setUpdateTime(new Date())
                        .setRemark("")
                        .setAuthKey("");
                tabSysUnionEnterpriseAuthResDetailMapper.updateByPrimaryKeySelective(old);
                //日志
                unionEnterpriseAuthResLogService.saveUnionEnterpriseAuthResLog(unionEnterpriseAuthId,
                        old.getResourceType(), UnionEnterpriseAuthResOperateEnum.EDIT_RESOURCE_AREA.getMessage());
            }
            return;
        }
        Date now = new Date();
        //因为资源种类不多，循环单条创建
        for (UnionEnterpriseAuthResTypeEnum typeEnum : UnionEnterpriseAuthResTypeEnum.values()) {
            TabSysUnionEnterpriseAuthResDetail record = new TabSysUnionEnterpriseAuthResDetail();
            record.setUpdateTime(now)
                    .setCreateTime(now)
                    .setDeleteFlag(0)
                    .setUnionEnterpriseAuthId(unionEnterpriseAuthId)
                    .setResourceType(typeEnum.getCode())
                    .setAuthStatus(UnionEnterpriseAuthDetailStatusEnum.NO_AUTH.getCode());
            tabSysUnionEnterpriseAuthResDetailMapper.insertSelective(record);
            //日志
            unionEnterpriseAuthResLogService.saveUnionEnterpriseAuthResLog(unionEnterpriseAuthId,
                    typeEnum.getCode(), UnionEnterpriseAuthResOperateEnum.UNION.getMessage());
        }
    }

    @Override
    public void relieveUnion(@NotNull Integer unionEnterpriseAuthId) {
        List<TabSysUnionEnterpriseAuthResDetail> list = listUnionEnterpriseAuthResDetail(unionEnterpriseAuthId, null);
        if (CollectionUtils.isNotEmpty(list)) {
            for (TabSysUnionEnterpriseAuthResDetail temp : list) {
                temp.setUpdateTime(new Date())
                .setAuthStatus(UnionEnterpriseAuthDetailStatusEnum.TO_RELIEVE.getCode());

                tabSysUnionEnterpriseAuthResDetailMapper.updateByPrimaryKeySelective(temp);
                //日志
                unionEnterpriseAuthResLogService.saveUnionEnterpriseAuthResLog(unionEnterpriseAuthId,
                        temp.getResourceType(), UnionEnterpriseAuthResOperateEnum.START_RELIEVE.getMessage());
            }
        }
    }

    @Override
    public List<TabSysUnionEnterpriseAuthResDetail> listByUnionEnterpriseAuthIdList(List<Integer> unionEnterpriseAuthIdList) {
        return tabSysUnionEnterpriseAuthResDetailMapper.listByUnionEnterpriseAuthIdList(unionEnterpriseAuthIdList);
    }

    @Override
    public int union(@NotNull Integer unionEnterpriseAuthResDetailId, @NotNull Integer authStatus, String authKey, String remark) {
        TabSysUnionEnterpriseAuthResDetail record = new TabSysUnionEnterpriseAuthResDetail();
        record.setUpdateTime(new Date());
        record.setUnionEnterpriseAuthResDetailId(unionEnterpriseAuthResDetailId);
        record.setAuthKey(authKey);
        record.setRemark(remark);
        record.setAuthStatus(authStatus);
        return tabSysUnionEnterpriseAuthResDetailMapper.updateByPrimaryKeySelective(record);
    }
}
