package com.gic.enterprise.service.impl;

import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.constant.union.UnionEnterpriseAuthStatusEnum;
import com.gic.enterprise.dao.mapper.TabSysUnionEnterpriseAuthMapper;
import com.gic.enterprise.dto.union.OwnEnterpriseAuthListDTO;
import com.gic.enterprise.dto.union.UnionEnterpriseAuthDTO;
import com.gic.enterprise.dto.union.UnionEnterpriseAuthListDTO;
import com.gic.enterprise.entity.TabSysUnionEnterpriseAuth;
import com.gic.enterprise.service.UnionEnterpriseAuthService;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.validation.constraints.NotNull;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * 联合商户
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/2/3 1:57 PM 
 */
@Service("unionEnterpriseAuthService")
public class UnionEnterpriseAuthServiceImpl implements UnionEnterpriseAuthService{
    @Autowired
    private TabSysUnionEnterpriseAuthMapper tabSysUnionEnterpriseAuthMapper;

    @Override
    public int saveUnionEnterprise(Integer ownEnterpriseId, Integer unionEnterpriseId) {
        TabSysUnionEnterpriseAuth record = new TabSysUnionEnterpriseAuth();
        Date now = new Date();
        record.setCreateTime(now)
                .setUpdateTime(now)
                .setDeleteFlag(0)
                .setOwnEnterpriseId(ownEnterpriseId)
                .setUnionEnterpriseId(unionEnterpriseId)
                .setStatusFlag(UnionEnterpriseAuthStatusEnum.NORMAL.getCode());
        return tabSysUnionEnterpriseAuthMapper.insertSelective(record);
    }

    @Override
    public TabSysUnionEnterpriseAuth getUnionEnterprise(Integer ownEnterpriseId, Integer unionEnterpriseId) {
        return tabSysUnionEnterpriseAuthMapper.getUnionEnterprise(ownEnterpriseId, unionEnterpriseId);
    }

    @Override
    public Integer getEnterpriseIdByAuthAccountGroupId(Integer accountGroupId) {
        List<TabSysUnionEnterpriseAuth> list = tabSysUnionEnterpriseAuthMapper.listUnionEnterpriseAuthByAccountGroupId(accountGroupId);
        if (CollectionUtils.isNotEmpty(list)) {
            //账号分组不能同时授权多个商户
            for (TabSysUnionEnterpriseAuth temp : list) {
                String accountGroup = temp.getAccountGroup();
                if (Arrays.stream(accountGroup.split(",")).anyMatch(e -> Objects.equals(accountGroupId.toString(), e))) {
                    return temp.getUnionEnterpriseId();
                }
            }
        }
        return null;
    }

    @Override
    public int update(UnionEnterpriseAuthDTO dto) {
        TabSysUnionEnterpriseAuth record = EntityUtil.changeEntityNew(TabSysUnionEnterpriseAuth.class, dto);
        record.setUpdateTime(new Date());
        return tabSysUnionEnterpriseAuthMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public List<UnionEnterpriseAuthListDTO> listUnionEnterpriseAuth(Integer ownEnterpriseId, List<Integer> unionEnterpriseIdList, Integer authStatus) {
        return tabSysUnionEnterpriseAuthMapper.listUnionEnterpriseAuth(ownEnterpriseId, unionEnterpriseIdList, authStatus);
    }

    @Override
    public List<OwnEnterpriseAuthListDTO> listOwnEnterpriseAuth(@NotNull Integer unionEnterpriseId, String startTime, String endTime) {
        return tabSysUnionEnterpriseAuthMapper.listOwnEnterpriseAuth(unionEnterpriseId, startTime, endTime);
    }

    @Override
    public List<TabSysUnionEnterpriseAuth> listAll() {
        return tabSysUnionEnterpriseAuthMapper.listAll();
    }

    @Override
    public List<TabSysUnionEnterpriseAuth> listUnionEnterpriseAuth(@NotNull Integer ownEnterpriseId) {
        return tabSysUnionEnterpriseAuthMapper.listUnionEnterprise(ownEnterpriseId);
    }

    @Override
    public Boolean hasUnionEnterpriseAuth(Integer resourceGroupId) {
        int count = tabSysUnionEnterpriseAuthMapper.countUnionEnterpriseAuth(resourceGroupId);
        if (count > 0) {
            return true;
        }
        return false;
    }

    @Override
    public List<TabSysUnionEnterpriseAuth> listUnionEnterpriseAuthByResourceGroupId(Integer resourceGroupId) {
        return tabSysUnionEnterpriseAuthMapper.listUnionEnterpriseAuthByResourceGroupId(resourceGroupId);
    }

    @Override
    public List<TabSysUnionEnterpriseAuth> listOwnEnterpriseAuth(@NotNull Integer unionEnterpriseId) {
        return tabSysUnionEnterpriseAuthMapper.listOwnEnterpriseAuthByOwnEnterpriseId(unionEnterpriseId);
    }

}
