package com.gic.enterprise.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.dto.BlackListDTO;
import com.gic.enterprise.entity.TabBlackList;
import com.gic.enterprise.entity.TabEnterprise;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.exception.CommonException;
import com.gic.enterprise.service.BlackListApiService;
import com.gic.enterprise.service.BlackListService;
import com.gic.enterprise.service.EnterpriseService;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 达摩情报销商户黑名单
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/5/18 3:47 PM 
 */
@Service("blackListApiService")
public class BlackListApiServiceImpl implements BlackListApiService{
    @Autowired
    private BlackListService blackListService;
    @Autowired
    private EnterpriseService enterpriseService;

    @Override
    public ServiceResponse<Void> saveBlackList(Integer enterpriseId) {
        TabEnterprise enterprise = getEnterprise(enterpriseId);
        blackListService.saveBlackList(enterprise);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> deleteBlackList(Integer blackListId) {
        blackListService.deleteBlackList(blackListId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page<BlackListDTO>> pageBlackList(String search, Integer pageNum, Integer pageSize) {
        com.github.pagehelper.Page<TabBlackList> page =  blackListService.pageBlackList(search, pageNum, pageSize);
        Page<BlackListDTO> result = PageHelperUtils.changePageHelperToCurrentPage(page, BlackListDTO.class);
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<List<Integer>> getBlankListEnterprise() {
        List<TabBlackList> list = blackListService.getBlankListEnterprise();
        if (CollectionUtils.isNotEmpty(list)) {
            return ServiceResponse.success(list.stream().map(e -> e.getEnterpriseId()).collect(Collectors.toList()));
        }
        return ServiceResponse.success(Collections.EMPTY_LIST);
    }

    private TabEnterprise getEnterprise(Integer enterpriseId) {
        TabEnterprise record = enterpriseService.getEnterpriseById(enterpriseId);
        if (record == null) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "商户不存在");
        }
        return record;
    }
}
