package com.gic.enterprise.service.outer.impl;

import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.dto.CustomStoreDTO;
import com.gic.enterprise.entity.TabCustomGuide;
import com.gic.enterprise.entity.TabCustomStore;
import com.gic.enterprise.service.*;
import com.github.pagehelper.PageHelper;
/**
 *
 * @ClassName: CustomStoreApiServiceImpl
 * @Description:
 * @author taogs
 * @date 2020/9/11 15:28
 */
@Service("customStoreApiService")
public class CustomStoreApiServiceImpl implements CustomStoreApiService {
    @Autowired
    private CustomStoreService customStoreService;
    @Autowired
    private AppletCustomPageService appletCustomPageService;
    @Autowired
    private CustomGuideService customGuideService;
    @Autowired
    private EnterpriseInitApiService enterpriseInitApiService;

    @Override
    public ServiceResponse<Integer> saveCustomStore(CustomStoreDTO customStoreDTO) {
        int i = this.customStoreService.saveCustomStore(EntityUtil.changeEntityByJSON(TabCustomStore.class, customStoreDTO));
        this.customSettingHasCommpleted(customStoreDTO.getEnterpriseId());
        return ServiceResponse.success(i);
    }

    @Override
    public ServiceResponse<Page<CustomStoreDTO>> pageCustomStore(Integer enterpriseId, String search, String appid, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        com.github.pagehelper.Page<TabCustomStore> page = this.customStoreService.pageCustomStore(search, enterpriseId, appid);
        Page<CustomStoreDTO> storePage = PageHelperUtils.changePageHelperToCurrentPage(page, CustomStoreDTO.class);
        return ServiceResponse.success(storePage);

    }

    @Override
    public ServiceResponse<Integer> deleteCustomStore(Integer customStoreId) {
        int i = this.customStoreService.deleteCustomStore(customStoreId);
        return ServiceResponse.success(i);
    }

    @Override
    public ServiceResponse<CustomStoreDTO> getCustomStore(Integer customStoreId) {
        TabCustomStore customStore = this.customStoreService.getCustomStore(customStoreId);
        return ServiceResponse.success(EntityUtil.changeEntityByJSON(CustomStoreDTO.class, customStore));
    }

    @Override
    @Transactional(rollbackFor=Exception.class)
    public ServiceResponse<Integer> openCustomStore(Integer enterpriseId, String appid, Integer customStoreId) {
        this.customStoreService.closeCustomStore(enterpriseId, appid);
        this.customStoreService.openCustomStore(customStoreId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Integer> closeCustomStore(Integer enterpriseId, String appid) {
        int i = this.customStoreService.closeCustomStore(enterpriseId, appid);
        return ServiceResponse.success(i);
    }

    @Override
    public ServiceResponse<Integer> closeCustomStore(Integer customStoreId) {
        int i = this.customStoreService.closeCustomStore(customStoreId);
        return ServiceResponse.success(i);
    }

    @Override
    public ServiceResponse<CustomStoreDTO> getCustomStore(Integer enterpriseId, String appid) {
        TabCustomStore customStore = this.customStoreService.getCustomStore(enterpriseId, appid);
        return ServiceResponse.success(EntityUtil.changeEntityByJSON(CustomStoreDTO.class, customStore));
    }

    @Override
    public ServiceResponse<Void> customSettingHasCommpleted(Integer enterpriseId) {
        boolean customStore = false;
        boolean customGuide = false;
        boolean customMemeberCenter = false;
        ServiceResponse<Page<CustomStoreDTO>> serviceResponse = this.pageCustomStore(enterpriseId, null, null, 1, 1);
        if(serviceResponse.isSuccess() && serviceResponse.getResult() != null){
            if(serviceResponse.getResult().getPages()>0){
                customStore = true;
            }
        }
        customMemeberCenter = this.appletCustomPageService.hasCreateMemberCenterPage(enterpriseId);
        List<TabCustomGuide> guides = this.customGuideService.listGuide(enterpriseId);
        if(CollectionUtils.isNotEmpty(guides)){
            customGuide = true;
        }
        if(customGuide && customStore && customMemeberCenter){
            this.enterpriseInitApiService.perfectWeChatPage(enterpriseId);
        }
        return ServiceResponse.success();
    }
}
