package com.gic.enterprise.service.outer.impl;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

import com.gic.enterprise.exception.CommonException;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.member.config.api.dto.AreaSearchDTO;
import com.gic.member.config.api.dto.ChannelAreaInfoDTO;
import com.gic.member.config.api.dto.OwnerMemberCardDTO;
import com.gic.member.config.api.enums.ChannelChangeCodeEnum;
import com.gic.member.config.api.service.MemberCardApiService;
import com.gic.member.config.api.service.UserListFieldApiService;
import com.gic.platform.member.constant.enums.CuTypeEnum;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.dto.data.*;
import com.gic.enterprise.entity.*;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.*;
import com.gic.member.config.api.dto.ExecParamsDTO;
import org.springframework.transaction.annotation.Transactional;

/**
 *
 * @ClassName: DataConfigApiServiceImpl
 * @Description:
 * @author taogs
 * @date 2020/9/11 15:29
 */
@Service("dataConfigApiService")
public class DataConfigApiServiceImpl implements DataConfigApiService {
    private static final Logger LOGGER = LogManager.getLogger(DataConfigApiServiceImpl.class);
    @Autowired
    private DataMemberConsumeConfigService dataMemberConsumeConfigService;
    @Autowired
    private DataWechatMemberConfigService dataWechatMemberConfigService;
    @Autowired
    private DataActuallyPaidConfigService dataActuallyPaidConfigService;
    @Autowired
    private DataSingleEffectConfigService dataSingleEffectConfigService;
    @Autowired
    private DataPerformanceConfigService dataPerformanceConfigService;
    @Autowired
    private DataGenerationConfigService dataGenerationConfigService;
    @Autowired
    private MemberCardApiService memberCardApiService;
    @Autowired
    private UserListFieldApiService userListFieldApiService;

    @Override
    public ServiceResponse<Void> monitorMemberCardNew(String params) {
        LOGGER.info("会员卡新增监听：{}", params);
        ExecParamsDTO execParamsDTO = (ExecParamsDTO) JSON.parse(params);
        if (ChannelChangeCodeEnum.CARD_CREAYE == execParamsDTO.getChannelChangeCodeEnum()) {
            configWechatMember(new DataWechatMemberConfigDTO()
                    .setEnterpriseId(execParamsDTO.getEnterpriseId())
                    .setMemberCardArea(execParamsDTO.getCardConfigId().toString())
                    .setValidRefresh(1));
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> configWechatMember(DataWechatMemberConfigDTO dto) {
        String memberCardId = dto.getMemberCardArea();
        if (StringUtils.isBlank(memberCardId)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "会员卡域不能为空");
        }
        TabDataWechatMemberConfig result = dataWechatMemberConfigService.getByMemberCardId(dto.getEnterpriseId(), memberCardId);

        //是否是刷新通道的保存
        boolean isRefresh = dto.getValidRefresh() == 1;
        if (isRefresh) {
            dto.setValidSave(0);
        } else {
            dto.setValidSave(1);
        }
        if (result == null) {
            dataWechatMemberConfigService.saveWechatMemberConfig(dto);
        } else {
            dto.setWechatMemberConfigId(result.getWechatMemberConfigId());
            dataWechatMemberConfigService.editWechatMemberConfig(dto);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> newWechatMember(Integer enterpriseId, Long memberCardArea) {
        return configWechatMember(new DataWechatMemberConfigDTO().setEnterpriseId(enterpriseId).setMemberCardArea(memberCardArea.toString()).setValidRefresh(1));
    }

    @Override
    public ServiceResponse<Void> deleteWechatMember(Integer enterpriseId, Long memberCardArea) {
        if (memberCardArea == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "会员卡域不能为空");
        }
        TabDataWechatMemberConfig result = dataWechatMemberConfigService.getByMemberCardId(enterpriseId, memberCardArea.toString());
        if (result != null) {
            dataWechatMemberConfigService.editWechatMemberConfig(new DataWechatMemberConfigDTO()
                    .setWechatMemberConfigId(result.getWechatMemberConfigId())
                    .setStatus(0));
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<DataWechatMemberConfigDTO>> listWechatMemberConfig(Integer enterpriseId) {
        return ServiceResponse.success(Optional
                .ofNullable(EntityUtil.changeEntityListNew(DataWechatMemberConfigDTO.class, dataWechatMemberConfigService
                        .listByEnterpriseId(enterpriseId))).orElse(Collections.EMPTY_LIST));
    }

    @Override
    public ServiceResponse<Integer> configMemberConsume(DataMemberConsumeConfigDTO dto) {
        if (dto.getEnterpriseId() == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID不能为空");
        }
        TabDataMemberConsumeConfig record = dataMemberConsumeConfigService.getByEnterpriseId(dto.getEnterpriseId());
        if (record == null) {
            //还未配置，新增
            return ServiceResponse.success(dataMemberConsumeConfigService.saveMemberConsumeConfig(dto));
        }
        dto.setMemberConsumeConfigId(record.getMemberConsumeConfigId());
        dataMemberConsumeConfigService.editMemberConsumeConfig(dto);
        return ServiceResponse.success(record.getMemberConsumeConfigId());
    }

    @Override
    public ServiceResponse<DataMemberConsumeConfigDTO> getMemberConsumeConfigByEnterpriseId(Integer enterpriseId) {
        TabDataMemberConsumeConfig record = dataMemberConsumeConfigService.getByEnterpriseId(enterpriseId);
        if (record == null) {
            TabDataMemberConsumeConfig defaultRecord = dataMemberConsumeConfigService.getByEnterpriseId(-1);
            return ServiceResponse.success(EntityUtil.changeEntityNew(DataMemberConsumeConfigDTO.class, defaultRecord));
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(DataMemberConsumeConfigDTO.class, record));
    }

    @Override
    public ServiceResponse<Void> initMemberConsumeConfig(Integer enterpriseId) {
        TabDataMemberConsumeConfig defaultRecord = dataMemberConsumeConfigService.getByEnterpriseId(-1);
        DataMemberConsumeConfigDTO dto = EntityUtil.changeEntityNew(DataMemberConsumeConfigDTO.class, defaultRecord);
        dto.setMemberConsumeConfigId(null);
        dto.setEnterpriseId(enterpriseId);
        configMemberConsume(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> configActuallyPaid(List<DataActuallyPaidConfigDTO> dtoList) {
        if (CollectionUtils.isEmpty(dtoList)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "实付配置内容为空");
        }
        for (DataActuallyPaidConfigDTO dto : dtoList) {
            if (dto.getClassify() == null) {
                dto.setClassify(1);
            }
            TabDataActuallyPaidConfig temp = dataActuallyPaidConfigService.get(dto.getEnterpriseId(), dto.getClassify());
            if (temp != null && temp.getHasConfig() == 1) {
                continue;
            }
            if (temp == null) {
                //如果是第一次生成，则
                dto.setHasConfig(0);
            } else {
                dto.setHasConfig(1);
            }
            dataActuallyPaidConfigService.delete(dto.getEnterpriseId(), dto.getClassify());
            dto.setConfigStatus(Optional.ofNullable(dto.getConfigStatus()).orElse(0));
            dataActuallyPaidConfigService.saveActuallyPaidConfig(dto);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> configActuallyPaid(Integer enterpriseId, Integer memberBusiness, Integer performanceCount, Integer memberBusinessOfGoods) {
        List<DataActuallyPaidConfigDTO> list = new ArrayList<>(5);
        boolean isRight = enterpriseId != null && memberBusiness != null && performanceCount != null && memberBusinessOfGoods != null;
        if (!isRight) {
            ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数不能为空");
        }
        if (memberBusiness != null) {
            list.add(new DataActuallyPaidConfigDTO()
                    .setEnterpriseId(enterpriseId)
                    .setClassify(1)
                    .setConfigStatus(memberBusiness));
            list.add(new DataActuallyPaidConfigDTO()
                    .setEnterpriseId(enterpriseId)
                    .setClassify(2)
                    .setConfigStatus(memberBusiness));
        }
        if (performanceCount != null) {
            list.add(new DataActuallyPaidConfigDTO()
                    .setEnterpriseId(enterpriseId)
                    .setClassify(3)
                    .setConfigStatus(performanceCount));
            list.add(new DataActuallyPaidConfigDTO()
                    .setEnterpriseId(enterpriseId)
                    .setClassify(4)
                    .setConfigStatus(performanceCount));
            list.add(new DataActuallyPaidConfigDTO()
                    .setEnterpriseId(enterpriseId)
                    .setClassify(5)
                    .setConfigStatus(performanceCount));
        }
        if (memberBusinessOfGoods != null) {
            list.add(new DataActuallyPaidConfigDTO()
                    .setEnterpriseId(enterpriseId)
                    .setClassify(6)
                    .setConfigStatus(memberBusinessOfGoods));
        }

        return configActuallyPaid(list);
    }

    @Override
    public ServiceResponse<List<DataActuallyPaidConfigDTO>> listActuallyPaidConfig(Integer enterpriseId) {
        List<TabDataActuallyPaidConfig> list = dataActuallyPaidConfigService.listByEnterpriseId(enterpriseId);
        if (CollectionUtils.isEmpty(list)) {
            //如果没有配置，默认数据
            return ServiceResponse.success(EntityUtil.changeEntityListNew(DataActuallyPaidConfigDTO.class,
                    dataActuallyPaidConfigService.listByEnterpriseId(-1)));
        }
        return ServiceResponse.success(EntityUtil.changeEntityListNew(DataActuallyPaidConfigDTO.class, list));
    }

    @Override
    public ServiceResponse<DataActuallyPaidConfig> getDataActuallyPaidConfig(Integer enterpriseId) {
        List<TabDataActuallyPaidConfig> list = dataActuallyPaidConfigService.listByEnterpriseId(enterpriseId);
        if (CollectionUtils.isNotEmpty(list)) {
            Map<String, TabDataActuallyPaidConfig> map = list.stream().collect(Collectors.toMap(e -> e.getClassify().toString(), e -> e));
            //如果没有配置，默认数据
            Integer memberBusiness = map.get("1").getConfigStatus();
            Integer performanceCount = map.get("3").getConfigStatus();
            Integer memberBusinessOfGoods = map.get("6").getConfigStatus();
            return ServiceResponse.success(new DataActuallyPaidConfig()
                    .setEnterpriseId(enterpriseId)
                    .setMemberBusiness(memberBusiness)
                    .setPerformanceCount(performanceCount)
                    .setHasConfigMemberBusiness(map.get("1").getHasConfig() == 1)
                    .setHasConfigPerformanceCount(map.get("3").getHasConfig() == 1)
                    .setHasConfigMemberBusinessOfGoods(map.get("6").getHasConfig() == 1)
                    .setMemberBusinessOfGoods(memberBusinessOfGoods));
        }
        return ServiceResponse.success(new DataActuallyPaidConfig().setEnterpriseId(enterpriseId).setMemberBusiness(0).setPerformanceCount(0).setMemberBusinessOfGoods(0));
    }

    @Override
    public ServiceResponse<Void> initActuallyPaidData(Integer enterpriseId) {
        List<TabDataActuallyPaidConfig> list = dataActuallyPaidConfigService.listByEnterpriseId(-1);
        list = list.stream().map(e -> e.setEnterpriseId(enterpriseId)).map(e -> e.setActuallyPaidConfigId(null))
                .collect(Collectors.toList());
        configActuallyPaid(EntityUtil.changeEntityListNew(DataActuallyPaidConfigDTO.class, list));
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> configSingleEffect(List<DataSingleEffectConfigDTO> dtoList) {
        if (CollectionUtils.isEmpty(dtoList)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "单效配置内容为空");
        }
        for (DataSingleEffectConfigDTO dto : dtoList) {
            dataSingleEffectConfigService.deleteSingleEffectConfig(dto.getEnterpriseId(), dto.getOrderGrain());
            dataSingleEffectConfigService.saveSingleEffectConfig(dto);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<DataSingleEffectConfigDTO>> getSingleEffectConfig(Integer enterpriseId) {
        List<TabDataSingleEffectConfig> record = dataSingleEffectConfigService.getByEnterpriseId(enterpriseId);
        if (CollectionUtils.isEmpty(record)) {
            return ServiceResponse.success(EntityUtil.changeEntityListNew(DataSingleEffectConfigDTO.class,
                    dataSingleEffectConfigService.getByEnterpriseId(-1)));
        }
        return ServiceResponse.success(EntityUtil.changeEntityListNew(DataSingleEffectConfigDTO.class, record));
    }

    @Override
    public ServiceResponse<Void> initSingleEffectConfig(Integer enterpriseId) {
        List<DataSingleEffectConfigDTO> dtoList = EntityUtil.changeEntityListNew(DataSingleEffectConfigDTO.class,
                dataSingleEffectConfigService.getByEnterpriseId(-1));
        dtoList = dtoList.stream().map(e -> e.setEnterpriseId(enterpriseId))
                .map(e -> e.setSingleEffectConfigId(null)).collect(Collectors.toList());
        configSingleEffect(dtoList);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> configPerformance(DataPerformanceConfigDTO dto) {
        TabDataPerformanceConfig record = dataPerformanceConfigService.getByEnterpriseId(dto.getEnterpriseId());
        if (record == null) {
            dataPerformanceConfigService.savePerformanceConfig(dto);
            return ServiceResponse.success();
        }
        dto.setPerformanceConfigId(record.getPerformanceConfigId());
        dataPerformanceConfigService.editPerformanceConfig(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<DataPerformanceConfigDTO> getPerformanceConfig(Integer enterpriseId) {
        TabDataPerformanceConfig record = dataPerformanceConfigService.getByEnterpriseId(enterpriseId);
        if (record == null) {
            return ServiceResponse.success(EntityUtil.changeEntityNew(DataPerformanceConfigDTO.class,
                    dataPerformanceConfigService.getByEnterpriseId(-1)));
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(DataPerformanceConfigDTO.class, record));
    }

    @Override
    public ServiceResponse<Void> initPerformanceConfig(Integer enterpriseId) {
        DataPerformanceConfigDTO dto = EntityUtil.changeEntityNew(DataPerformanceConfigDTO.class,
                dataPerformanceConfigService.getByEnterpriseId(-1));
        dto.setPerformanceConfigId(null);
        dto.setEnterpriseId(enterpriseId);
        configPerformance(dto);
        return ServiceResponse.success();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<Void> saveGenerationConfig(List<DataGenerationConfigDTO> dtoList) {
        if (CollectionUtils.isEmpty(dtoList)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "年龄段配置内容为空");
        }
        Integer enterpriseId = dtoList.get(0).getEnterpriseId();
        Long memberCardId = dtoList.get(0).getMemberCardId();

        if (-1L == memberCardId) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "会员卡数据错误");
        }

        checkShareMemberCard(enterpriseId, memberCardId);
        checkUpdateTime(memberCardId);

        dataGenerationConfigService.deleteGenerationConfig(enterpriseId, memberCardId);
        dtoList.stream()
                .forEach(e -> dataGenerationConfigService.saveGenerationConfig(e));
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<DataGenerationConfigDTO>> listGenerationConfig(Long memberCardId) {
        List<TabDataGenerationConfig> list = dataGenerationConfigService.listGenerationByMemberCardId(memberCardId);
        return ServiceResponse.success(CollectionUtils.isEmpty(list) ?
                EntityUtil.changeEntityListNew(DataGenerationConfigDTO.class, dataGenerationConfigService.listGenerationByMemberCardId(-1L)) :
        EntityUtil.changeEntityListNew(DataGenerationConfigDTO.class, list));
    }

    @Override
    public ServiceResponse<List<DataGenerationConfigDTO>> listGenerationConfig(Integer enterpriseId) {
        List<ChannelAreaInfoDTO> list = listMemberCard(enterpriseId);
        return listGenerationConfig(getOriginalMemberCard(list));
    }

    @Override
    public ServiceResponse<Object> listAllMemberCard(Integer enterpriseId) {
        return ServiceResponse.success(listMemberCard(enterpriseId));
    }

    @Override
    public ServiceResponse<Void> initGenerationConfig(Integer enterpriseId) {
        List<DataGenerationConfigDTO> dtoList = EntityUtil.changeEntityListNew(DataGenerationConfigDTO.class,
                dataGenerationConfigService.listGenerationByMemberCardId(-1L));
        saveGenerationConfig(dtoList.stream().map(e -> e.setEnterpriseId(enterpriseId))
                .map(e -> e.setGenerationConfigId(null)).collect(Collectors.toList()));
        return ServiceResponse.success();
    }


    private void checkShareMemberCard(Integer enterpriseId, Long memberCardId) {
        List<ChannelAreaInfoDTO> list = listMemberCard(enterpriseId);
        if (!list.stream().anyMatch(e -> Objects.equals(e.getOwnerEntId(), e.getEnterpriseId()) && Objects.equals(memberCardId, e.getAreaId()))) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "共享卡不能设置");
        }
    }

    private void checkUpdateTime(Long memberCardId) {
        List<TabDataGenerationConfig> list = dataGenerationConfigService.listGenerationByMemberCardId(memberCardId);
        if (CollectionUtils.isNotEmpty(list)) {
            //如果查询到，则查看创建时间，是否是当天，如果不是，则需要间隔30天才能update
            Date createTime = list.get(0).getCreateTime();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            if (!sdf.format(new Date()).equals(sdf.format(createTime))) {
                //如果不是当天
                Calendar time = Calendar.getInstance();
                time.setTime(createTime);
                time.add(Calendar.DAY_OF_YEAR, 30);
                if (sdf.format(new Date()).compareTo(sdf.format(time.getTime())) > 0) {
                    throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "30天内不能调整");
                }
            }
        }
    }


    private List<ChannelAreaInfoDTO> listMemberCard(Integer enterpriseId) {
        AreaSearchDTO param = new AreaSearchDTO();
        param.setEnterpriseId(enterpriseId);
        List<CuTypeEnum> cuTypes = new ArrayList<>();
        cuTypes.add(CuTypeEnum.MCU);
        param.setCuTypes(cuTypes);
        ServiceResponse<List<ChannelAreaInfoDTO>> response = userListFieldApiService.queryCustomAreaInfo(param);
        if (!response.isSuccess() || CollectionUtils.isEmpty(response.getResult())) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "没有会员卡数据");
        }
        return response.getResult();
    }


    private Long getOriginalMemberCard(List<ChannelAreaInfoDTO> list) {
        List<Long> memberCardIdList = list.stream().filter(e -> Objects.equals(e.getOwnerEntId(), e.getEnterpriseId()))
                .mapToLong(e -> e.getAreaId()).boxed().collect(Collectors.toList());
        return Collections.min(memberCardIdList);
    }
}
