package com.gic.enterprise.service.outer.impl;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.bizdict.api.dto.BizdictDTO;
import com.gic.bizdict.api.service.BizdictService;
import com.gic.enterprise.constant.Constants;
import com.gic.enterprise.dto.DictDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.DictApiService;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
/**
 *
 * @ClassName: DictApiServiceImpl
 * @Description:
 * @author taogs
 * @date 2020/9/11 15:31
 */
@Service("dictApiService")
public class DictApiServiceImpl implements DictApiService {

    @Autowired
    private BizdictService bizdictService;

    @Override
    public ServiceResponse<List<DictDTO>> listBusinessCategory() {
        List<DictDTO> result = new ArrayList<>();
        List<BizdictDTO> list = bizdictService.treeByCategoryCode(Constants.BUSINESS_CATEGORY);
        if(CollectionUtils.isNotEmpty(list)){
            result = changeListToTree(list);
        }
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<List<DictDTO>> listRiskModeDuration() {
        List<DictDTO> result = new ArrayList<>();
        List<BizdictDTO> list = bizdictService.treeByCategoryCode(Constants.RISK_MODE_DURATION_CATEGORY);
        if(CollectionUtils.isNotEmpty(list)){
            result = changeListToTree(list);
        }
        return ServiceResponse.success(result);
    }

    private List<DictDTO> changeListToTree(List<BizdictDTO> list) {
        List<DictDTO> result = new ArrayList<>();
        for(BizdictDTO bizdictDTO : list){
            DictDTO dictDTO = new DictDTO();
            dictDTO.setKey(bizdictDTO.getName());
            dictDTO.setValue(bizdictDTO.getCode());
            dictDTO.setContent(bizdictDTO.getValueContent());
            if (CollectionUtils.isNotEmpty(bizdictDTO.getNodeChildren())) {
                List nodeChildren = bizdictDTO.getNodeChildren();
                List<DictDTO> dtoList = changeListToTree((List<BizdictDTO>) nodeChildren);
                dictDTO.setDictList(dtoList);
            }
            result.add(dictDTO);
        }
        return result;
    }

    @Override
    public ServiceResponse<DictDTO> getDict(String categoryCode, String code) {
        BizdictDTO bizdictDTO = bizdictService.getByCode(categoryCode, code);
        if (bizdictDTO != null) {
            DictDTO dictDTO = new DictDTO();
            dictDTO.setKey(bizdictDTO.getName());
            dictDTO.setValue(bizdictDTO.getCode());
            dictDTO.setContent(bizdictDTO.getValueContent());
            return ServiceResponse.success(dictDTO);
        } else {
            return ServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), ErrorCode.NOTEXISTS.getMsg());
        }
    }

    @Override
    public ServiceResponse<DictDTO> getDictOriginal(String categoryCode, String code) {
        BizdictDTO bizdictDTO = bizdictService.getByCode(categoryCode, code);
        if (bizdictDTO != null) {
            DictDTO dictDTO = new DictDTO();
            dictDTO.setKey(bizdictDTO.getName());
            dictDTO.setValue(bizdictDTO.getCode());
            dictDTO.setContent(bizdictDTO.getValueContent());
            return ServiceResponse.success(dictDTO);
        } else {
            return ServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), ErrorCode.NOTEXISTS.getMsg());
        }
    }

    @Override
    public ServiceResponse<List<DictDTO>> listModuleTemplateCode() {
        List<DictDTO> result = new ArrayList<>();
        List<BizdictDTO> list = bizdictService.treeByCategoryCode(Constants.MODULE_TEMPLATE_CODE);
        if(CollectionUtils.isNotEmpty(list)){
            result = changeListToTree(list);
        }
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<List<DictDTO>> listSmsAppType() {
        List<DictDTO> result = new ArrayList<>();
        List<BizdictDTO> list = bizdictService.treeByCategoryCode(Constants.SMS_APP_TYPE);
        if(CollectionUtils.isNotEmpty(list)){
            result = changeListToTree(list);
        }
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<List<DictDTO>> listTodoItemType() {
        List<DictDTO> result = new ArrayList<>();
        List<BizdictDTO> list = bizdictService.treeByCategoryCode(Constants.TODO_ITEM_TYPE);
        if(CollectionUtils.isNotEmpty(list)){
            result = changeListToTree(list);
        }
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<List<DictDTO>> listRequestCode() {
        List<DictDTO> result = new ArrayList<>();
        List<BizdictDTO> list = bizdictService.treeByCategoryCode(Constants.BALANCE_PAY_REQUEST_CODE);
        if(CollectionUtils.isNotEmpty(list)){
            result = changeListToTree(list);
        }
        return ServiceResponse.success(result);
    }
}
