package com.gic.enterprise.service.outer.impl;

import java.util.Date;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.enterprise.constant.AccountStandardEnum;
import com.gic.enterprise.constant.Constants;
import com.gic.enterprise.dto.DictDTO;
import com.gic.enterprise.dto.DoubleCallingRecordDTO;
import com.gic.enterprise.dto.PstnCallBillDTO;
import com.gic.enterprise.service.*;
import com.gic.enterprise.utils.CreateRandomUtils;
import com.gic.mq.sdk.GicMQClient;
import com.gic.store.utils.valid.ValidUtil;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-12-05 13:37
 */
@Service("doubleCallingApiService")
public class DoubleCallingApiServiceImpl implements DoubleCallingApiService {

    private Logger logger = LogManager.getLogger(DoubleCallingApiServiceImpl.class);

    @Autowired
    private BillingDoubleCallingRecordService billingDoubleCallingRecordService;
    @Autowired
    private PstnCallBillService pstnCallBillService;
    @Autowired
    private AccountStandardService accountStandardService;
    @Autowired
    private BillingAccountService billingAccountService;
    @Autowired
    private DictApiService dictApiService;

    @Override
    public ServiceResponse<Void> saveDoubleCalling(Integer enterpriseId, PstnCallBillDTO pstnCallBillDTO) {
        GicMQClient client = GICMQClientUtil.getClientInstance();
        try {
            pstnCallBillDTO.setEnterpriseId(enterpriseId);
            String json = JSON.toJSONString(pstnCallBillDTO);
            logger.info("收到双向呼叫计费信息：{}", json);
            client.sendMessage("doubleCallMq", json);
        } catch (Exception e) {
            logger.warn("调用mq失败", e);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> saveDoubleCalling(DoubleCallingRecordDTO doubleCallingRecordDTO) {
        ServiceResponse paramResult = ValidUtil.allCheckValidate(doubleCallingRecordDTO, DoubleCallingRecordDTO.DoubleCallingRecordValid.class);
        if (!paramResult.isSuccess()) {
            return paramResult;
        }
        GicMQClient client = GICMQClientUtil.getClientInstance();
        try {
            String json = JSON.toJSONString(doubleCallingRecordDTO);
            logger.info("收到双向呼叫计费信息：{}", json);
            this.run2(json);
        } catch (Exception e) {
            logger.warn("调用mq失败", e);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> run(String arg) {

        GicMQClient client = GICMQClientUtil.getClientInstance();
        try {
            logger.info("双向呼叫缓存扣费：{}", arg);
            PstnCallBillDTO pstnCallBillDTO = JSON.parseObject(arg, PstnCallBillDTO.class);
            Integer duration = pstnCallBillDTO.getDuration();
            Integer unitFee = accountStandardService.queryUnitFee(pstnCallBillDTO.getEnterpriseId(), AccountStandardEnum.CALL.getCode());
            double minute = Math.ceil(duration.doubleValue() / 60d);
            double fee = unitFee * minute / 1000;
            pstnCallBillDTO.setFee(fee);

            // 缓存扣费
            billingAccountService.deductAccountByCache(fee, pstnCallBillDTO.getEnterpriseId());
            // 队列
            client.sendMessage("doubleCallDealDbMq", JSON.toJSONString(pstnCallBillDTO));
        } catch (Exception e) {
            logger.warn("调用mq失败", e);
        }
        return ServiceResponse.success();
    }

    public ServiceResponse<Void> run2(String arg) {
        GicMQClient client = GICMQClientUtil.getClientInstance();
        try {
            logger.info("双向呼叫缓存扣费：{}", arg);
            DoubleCallingRecordDTO doubleCallingRecordDTO = JSON.parseObject(arg, DoubleCallingRecordDTO.class);
            // 缓存扣费
            billingAccountService.deductAccountByCache(doubleCallingRecordDTO.getFee(), doubleCallingRecordDTO.getEnterpriseId());
            this.executeDatabaseTwo(arg);
        } catch (Exception e) {
            logger.warn("调用mq失败", e);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> executeDatabase(String arg) {
        logger.info("执行双向呼叫计费：{}", arg);
        PstnCallBillDTO pstnCallBillDTO = JSON.parseObject(arg, PstnCallBillDTO.class);
        Integer duration = pstnCallBillDTO.getDuration();
        if (duration != null) {
            // 记录日志
            pstnCallBillService.save(pstnCallBillDTO);
            DoubleCallingRecordDTO doubleCallingRecordDTO = new DoubleCallingRecordDTO();
            doubleCallingRecordDTO.setCallerPhone(pstnCallBillDTO.getSrc());

            doubleCallingRecordDTO.setDeductionTime(new Date());
            doubleCallingRecordDTO.setCalledPhone(pstnCallBillDTO.getDst());
            doubleCallingRecordDTO.setFee(pstnCallBillDTO.getFee());
            doubleCallingRecordDTO.setVoiceTime(duration.toString());
            doubleCallingRecordDTO.setEnterpriseId(pstnCallBillDTO.getEnterpriseId());

            doubleCallingRecordDTO.setStoreId(pstnCallBillDTO.getStoreId());
            doubleCallingRecordDTO.setStoreName(pstnCallBillDTO.getStoreName());
            doubleCallingRecordDTO.setStoreGroupId(pstnCallBillDTO.getStoreGroupId());
            doubleCallingRecordDTO.setStoreGroupName(pstnCallBillDTO.getStoreGroupName());
            // 明细
            billingDoubleCallingRecordService.save(doubleCallingRecordDTO);
            // 扣费
            billingAccountService.deductAccountByDatabase(doubleCallingRecordDTO.getFee(), doubleCallingRecordDTO.getEnterpriseId());
        }
        return ServiceResponse.success();
    }

    public ServiceResponse<Void> executeDatabaseTwo(String arg) {
        logger.info("执行双向呼叫计费：{}", arg);
        DoubleCallingRecordDTO doubleCallingRecordDTO = JSON.parseObject(arg, DoubleCallingRecordDTO.class);
        doubleCallingRecordDTO.setPaySerialNumber(CreateRandomUtils.createSerialNumber());
        ServiceResponse<DictDTO> serviceDict = dictApiService.getDict(Constants.VOICE_APP_TYPE, doubleCallingRecordDTO.getAppCode());
        if (serviceDict.isSuccess()) {
            DictDTO dictDTO = serviceDict.getResult();
            if (dictDTO != null) {
                String key = dictDTO.getKey();
                doubleCallingRecordDTO.setAppName(key);
            }
        }
        // 记录日志
        // 明细
        billingDoubleCallingRecordService.save(doubleCallingRecordDTO);
        // 扣费
        billingAccountService.deductAccountByDatabase(doubleCallingRecordDTO.getFee(), doubleCallingRecordDTO.getEnterpriseId());
        return ServiceResponse.success();
    }


}
