package com.gic.enterprise.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.UserApiService;
import com.gic.commons.util.DateUtil;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.config.Config;
import com.gic.enterprise.constant.*;
import com.gic.enterprise.dto.AuditLogDTO;
import com.gic.enterprise.dto.DownloadReportDTO;
import com.gic.enterprise.entity.TabDownloadReport;
import com.gic.enterprise.entity.TabProjectItem;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.qo.DownloadReportQO;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.enterprise.service.*;
import com.gic.redis.data.util.RedisUtil;
import com.gic.store.utils.valid.ValidUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.concurrent.BasicThreadFactory;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.redisson.api.RMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-10-28 15:49
 */
@Service("downloadReportApiService")
public class DownloadReportApiServiceImpl implements DownloadReportApiService {
    private static final Logger log = LogManager.getLogger(DownloadReportApiServiceImpl.class);
    @Autowired
    private DownloadReportService downloadReportService;
    @Autowired
    private DownloadLogService downloadLogService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private AuditLogApiService auditLogApiService;
    @Autowired
    private ProjectItemService projectItemService;
    @Autowired
    private PushMessageApiService pushMessageApiService;
    @Autowired
    private Config config;

    private static final String reportKey = "enterprise:report";
    ScheduledExecutorService checkReportHeart = new ScheduledThreadPoolExecutor(1,
            new BasicThreadFactory.Builder().namingPattern("reportHeartTimer-%d").daemon(true).build());

    @PostConstruct
    public void init(){
        checkReportHeart.scheduleAtFixedRate(new Runnable() {
            @Override
            public void run() {
                try {
                    RMap<Object, Object> map = RedisUtil.getRedisClient().getMap(reportKey);
                    log.info("监测心跳数据:{}", JSON.toJSONString(map));
                    if(MapUtils.isNotEmpty(map)){
                        Iterator<Object> iterator = map.keySet().iterator();
                        while (iterator.hasNext()){
                            Object next = iterator.next();
                            long putTime = (long) map.get(next);
                            if((System.currentTimeMillis() - putTime) > 10*60*1000){
                                log.info("报告超时:{}", next);
                                //超过十分钟 如果报告还在生成中则设置为失败
                                TabDownloadReport report = downloadReportService.getById((Integer) next);
                                if(report != null && !report.getStatus().equals(DownloadReportStatusEnum.GENERATED.getCode()) && report.getAuditResult().equals(DownloadReportStatusEnum.PASS.getCode())){
                                    downloadReportService.updateDownloadStatus((Integer)next, DownloadReportStatusEnum.ERROR);
                                }
                                map.remove(next);
                            }
                        }
                    }
                }catch (Throwable e){
                    log.warn(e);
                }
            }
        }, 3, 3, TimeUnit.SECONDS);
    }

    @Override
    public ServiceResponse<Integer> saveDownloadReport(DownloadReportDTO downloadReportDTO) {
        ServiceResponse paramResult = ValidUtil.allCheckValidate(downloadReportDTO, DownloadReportDTO.SaveValid.class);
        if (!paramResult.isSuccess()) {
            return paramResult;
        }
        if (StringUtils.isBlank(downloadReportDTO.getProjectCode())) {
            downloadReportDTO.setProjectCode("gic");
        }
        ServiceResponse<UserDTO> serviceResponse = userApiService.getUserInfoById(downloadReportDTO.getApplyUserId());
        if (!serviceResponse.isSuccess()) {
            return EnterpriseServiceResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
        UserDTO applyUser = serviceResponse.getResult();
        downloadReportDTO.setApplyUserName(applyUser.getUserName());

        UserDTO superAdmin = userApiService.getUserByEnterpriseId(downloadReportDTO.getEnterpriseId()).getResult();

        if (DownloadReportDataTypeEnum.COMPLETE_DATA.getCode().equals(downloadReportDTO.getDataType()) && !superAdmin.getUserId().equals(applyUser.getUserId())) {
            TabProjectItem projectItem = projectItemService.getByProjectItemCode(Constants.PROJECT_ITEM_COMPLETE_DATA_DOWNLOAD);
            downloadReportDTO.setAuditExpireTime(DateUtil.addDay(new Date(), projectItem.getAuditExpireDuration()));
            downloadReportDTO.setAuditResult(DataSecurityAuditEnum.WAIT.getCode());
        } else {
            downloadReportDTO.setAuditResult(DataSecurityAuditEnum.PASS.getCode());
        }
        // 重新定义文件名
        if (downloadReportDTO.getReportStartTime() != null && downloadReportDTO.getReportEndTime() != null) {
            String fileName = downloadReportDTO.getFileName() +
                    "-" +
                    DateUtil.dateToStr(downloadReportDTO.getReportStartTime(), DateUtil.FORMAT_DATE_10) +
                    "至" +
                    DateUtil.dateToStr(downloadReportDTO.getReportEndTime(), DateUtil.FORMAT_DATE_10);
            downloadReportDTO.setFileName(fileName);
        }
        if("hive".equals(downloadReportDTO.getDataSource())){
            downloadReportDTO.setStatus(DownloadReportStatusEnum.WAITLINE.getCode());
        } else {
            //sdk下载没有任务队列  直接生成中状态
            downloadReportDTO.setStatus(DownloadReportStatusEnum.BUILD.getCode());
        }
        Integer downloadReportId = downloadReportService.save(downloadReportDTO);
        return EnterpriseServiceResponse.success(downloadReportId);
    }

    @Override
    public ServiceResponse<Integer> updateDownloadUrl(Integer downloadReportId, String downloadUrl) {
        Integer line = downloadReportService.updateDownloadUrl(downloadReportId, downloadUrl);
        TabDownloadReport downloadReport = this.downloadReportService.getById(downloadReportId);
        if(downloadReport != null){
            String url = config.getHost() + "damo-system/report-list";
            String content = "您的"+ downloadReport.getDataContent() +"已生成，请前往<a href='"+ url +"' style='color:#2f54ed'>下载中心</a>查看";
            this.pushMessageApiService.pushUserMessage(downloadReport.getEnterpriseId(), downloadReport.getApplyUserId(), "系统消息", "下载通知", content);
        }
        return EnterpriseServiceResponse.success(line);
    }

    @Override
    public ServiceResponse<Page<DownloadReportDTO>> listDownloadReport(DownloadReportQO downloadReportQO) {
        com.github.pagehelper.Page<TabDownloadReport> tabPage = downloadReportService.listDownloadReport(downloadReportQO);
        Date currDate = new Date();
        Page<DownloadReportDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(tabPage, DownloadReportDTO.class);
        if (CollectionUtils.isNotEmpty(tabPage)) {
            List<DownloadReportDTO> result = resultPage.getResult();
            for (DownloadReportDTO downloadReportDTO : result) {
                // 判断是否超时
                if (DataSecurityAuditEnum.WAIT.getCode().equals(downloadReportDTO.getAuditResult()) && DateUtil.compareDate(currDate, downloadReportDTO.getAuditExpireTime())) {
                    downloadReportDTO.setDownloadReportStatus(DownloadReportStatusEnum.OVER_TIME.getCode());
                    downloadReportDTO.setAuditReason("审核超时");
                } else {
                    downloadReportDTO.setDownloadReportStatus(downloadReportDTO.getStatus());
                }
                downloadReportDTO.setDownloadUrl(null);
            }
        }
        return EnterpriseServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<Void> applyDownloadReport(Integer downloadReportId) {
        return applyDownloadReportCommon(downloadReportId, null);
    }

    @Override
    public ServiceResponse<Void> applyDownloadReport(Integer downloadReportId, String remark) {
        return applyDownloadReportCommon(downloadReportId, remark);
    }

    private ServiceResponse<Void> applyDownloadReportCommon(Integer downloadReportId, String remark) {
        TabDownloadReport downloadReport = this.downloadReportService.getById(downloadReportId);
        if (downloadReport == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
        if (!DownloadReportDataTypeEnum.COMPLETE_DATA.getCode().equals(downloadReport.getDataType())) {
            return EnterpriseServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "该下载不需要申请");
        }

        AuditLogDTO auditLogDTO = new AuditLogDTO();
        auditLogDTO.setEnterpriseId(downloadReport.getEnterpriseId());
        auditLogDTO.setApplyUserId(downloadReport.getApplyUserId());
        auditLogDTO.setApplyUserName(downloadReport.getApplyUserName());
        auditLogDTO.setApplyTime(new Date());
        auditLogDTO.setProjectItemCode(Constants.PROJECT_ITEM_COMPLETE_DATA_DOWNLOAD);
        auditLogDTO.setAuditExpireTime(downloadReport.getAuditExpireTime());
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("downloadReportId", downloadReportId);
        auditLogDTO.setExtraInfo(jsonObject.toJSONString());
        auditLogDTO.setAuditType(AuditTypeEnum.DOWNLOAD_REPORT.getCode());
        auditLogDTO.setRemark(remark);
        return auditLogApiService.apply(auditLogDTO);
    }

    @Override
    public ServiceResponse<DownloadReportDTO> getDownloadReport(Integer downloadReportId) {
        TabDownloadReport downloadReport = this.downloadReportService.getById(downloadReportId);
        if (downloadReport != null) {
            DownloadReportDTO downloadReportDTO = EntityUtil.changeEntityByJSON(DownloadReportDTO.class, downloadReport);
            return EnterpriseServiceResponse.success(downloadReportDTO);
        } else {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
    }

    @Override
    public ServiceResponse<Void> auditCallBack(String json) {
        JSONObject jsonObject = JSON.parseObject(json);
        // 1：审核通过  2：审核失败
        Integer auditResult = jsonObject.getInteger("auditResult");
        Integer auditorId = jsonObject.getInteger("auditorId");
        String auditorPhone = jsonObject.getString("auditorPhone");
        String auditorName = jsonObject.getString("auditorName");
        String auditReason = jsonObject.getString("auditReason");
        Integer downloadReportId = jsonObject.getInteger("downloadReportId");
        TabDownloadReport downloadReport = new TabDownloadReport();
        downloadReport.setAuditResult(auditResult);
        downloadReport.setAuditorId(auditorId);
        downloadReport.setAuditorPhone(auditorPhone);
        downloadReport.setAuditorName(auditorName);
        downloadReport.setAuditReason(auditReason);
        downloadReport.setDownloadReportId(downloadReportId);
        downloadReportService.updateAudit(downloadReport);
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<String> saveDownloadLogReturnDownloadUrl(Integer downloadReportId, Integer loginUserId) {
        TabDownloadReport downloadReport = downloadReportService.getById(downloadReportId);
        if (downloadReport == null || StringUtils.isBlank(downloadReport.getDownloadUrl())) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
        UserDTO userDTO = userApiService.getUserInfoById(loginUserId).getResult();
        downloadLogService.save(downloadReport, loginUserId, userDTO.getUserName());
        return EnterpriseServiceResponse.success(downloadReport.getDownloadUrl());
    }

    @Override
    public ServiceResponse<Void> receiveHeart(Integer reportId) {
        log.info("接受心跳包：{}", reportId);
        RMap<Object, Object> map = RedisUtil.getRedisClient().getMap(reportKey);
        map.put(reportId, System.currentTimeMillis());
        return ServiceResponse.success();
    }
}
