package com.gic.enterprise.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.ResourceGroupCustomUserDTO;
import com.gic.auth.dto.ResourceGroupDTO;
import com.gic.auth.service.ResourceGroupApiService;
import com.gic.commons.util.CollectionUtil;
import com.gic.enterprise.entity.TabSysUnionEnterpriseAuth;
import com.gic.enterprise.service.EnterpriseLicensePatternApiService;
import com.gic.enterprise.service.UnionEnterpriseAuthService;
import com.gic.enterprise.utils.LicenseUtils;
import com.gic.member.config.api.dto.AreaSearchDTO;
import com.gic.member.config.api.dto.ChannelAreaInfoDTO;
import com.gic.member.config.api.service.UserListFieldApiService;
import com.gic.open.api.dto.ApplicationLicenseDTO;
import com.gic.platform.member.constant.enums.CuTypeEnum;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * license查询接口
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/6/22 2:02 PM 
 */
@Service("enterpriseLicensePatternApiService")
public class EnterpriseLicensePatternApiServiceImpl implements EnterpriseLicensePatternApiService {
    private static final Logger LOGGER = LogManager.getLogger(EnterpriseLicensePatternApiServiceImpl.class);
    @Autowired
    private com.gic.open.api.service.EnterpriseLicenseApiService enterpriseLicenseApiService;
    @Autowired
    private UserListFieldApiService userListFieldApiService;
    @Autowired
    private UnionEnterpriseAuthService unionEnterpriseAuthService;
    @Autowired
    private ResourceGroupApiService resourceGroupApiService;


    @Override
    public ServiceResponse<Boolean> validUserLicenseEnough(Integer resourceGroupId, Integer unionEnterpriseId) {
        ServiceResponse<List<ApplicationLicenseDTO>> hasBuyLicense = enterpriseLicenseApiService.listEnterpriseBuyLicense(Arrays.asList(unionEnterpriseId));
        if (hasBuyLicense.isSuccess()) {
            List<ApplicationLicenseDTO> licenseList = hasBuyLicense.getResult();
            if (CollectionUtils.isNotEmpty(licenseList)) {
                Map<String, List<ApplicationLicenseDTO>> licenseMap = CollectionUtil.group(licenseList, "enterpriseId");
                List<ApplicationLicenseDTO> tempLicense = licenseMap.get(Long.valueOf(unionEnterpriseId));

                Integer memberCardNum = 0;
                Integer serviceNum = 0;
                Integer appNum = 0;
                //查询会员卡、服务号、小程序license数量
                if (CollectionUtils.isNotEmpty(tempLicense)) {
                    for (ApplicationLicenseDTO licenseDTO : tempLicense) {
                        if (LicenseUtils.MEMBER_CARD_LICENSE_CODE.equals(licenseDTO.getLicenseCode())) {
                            memberCardNum = licenseDTO.getQuantity();
                        }
                        if (LicenseUtils.SERVICE_CARD_LICENSE_CODE.equals(licenseDTO.getLicenseCode())) {
                            serviceNum = licenseDTO.getQuantity();
                        }
                        if (LicenseUtils.WECHAT_LICENSE_CODE.equals(licenseDTO.getLicenseCode())) {
                            appNum = licenseDTO.getQuantity();
                        }
                    }
                }
                //查询该商户已存在的用户域数量
                //1：本商户的用户域数据
                Integer memberCardNumOfCurrentEnterprise = getAreaIdList(unionEnterpriseId, CuTypeEnum.MCU).size();
                Integer serviceNumOfCurrentEnterprise = getAreaIdList(unionEnterpriseId, CuTypeEnum.SCU).size();
                Integer appNumOfCurrentEnterprise = getAreaIdList(unionEnterpriseId, CuTypeEnum.ACU).size();
                //自有商户用户域数量
                Map<Integer, Integer> areaValueOfOwnEnterprise = new HashMap<>();

                //2：已经授权过来的用户域数据
                List<TabSysUnionEnterpriseAuth> ownEnterpriseList = unionEnterpriseAuthService.listOwnEnterpriseAuth(unionEnterpriseId);
                if (CollectionUtils.isNotEmpty(ownEnterpriseList)) {
                    List<Integer> resourceGroupIdList = ownEnterpriseList.stream().filter(e -> e.getResourceGroupId() != null)
                            .map(e -> e.getResourceGroupId()).collect(Collectors.toList());
                    areaValueOfOwnEnterprise = getAreaValueMap(resourceGroupIdList);
                }
                //自有商户用户域数量
                Integer memberCardNumOfOwnEnterprise = areaValueOfOwnEnterprise.getOrDefault(CuTypeEnum.MCU.getType(), 0);
                Integer serviceNumOfOwnEnterprise = areaValueOfOwnEnterprise.getOrDefault(CuTypeEnum.SCU.getType(), 0);
                Integer appNumOfOwnEnterprise = areaValueOfOwnEnterprise.getOrDefault(CuTypeEnum.ACU.getType(), 0);
                //3:当前授权ID用户域数量
                Map<Integer, Integer> areaValueOfAuth = getAreaValueMap(Arrays.asList(resourceGroupId));

                Integer memberCardNumOfAuth = areaValueOfAuth.getOrDefault(CuTypeEnum.MCU.getType(), 0);
                Integer serviceNumOfAuth = areaValueOfAuth.getOrDefault(CuTypeEnum.SCU.getType(), 0);
                Integer appNumOfAuth = areaValueOfAuth.getOrDefault(CuTypeEnum.ACU.getType(), 0);

                LOGGER.info("会员卡域--本商户license：{}——本商户:{}——自有商户:{}——当前授权:{}",
                        memberCardNum, memberCardNumOfCurrentEnterprise, memberCardNumOfOwnEnterprise, memberCardNumOfAuth);
                LOGGER.info("服务号域--本商户license：{}——本商户:{}——自有商户:{}——当前授权:{}",
                        serviceNum, serviceNumOfCurrentEnterprise, serviceNumOfOwnEnterprise, serviceNumOfAuth);
                LOGGER.info("小程序域--本商户license：{}——本商户:{}——自有商户:{}——当前授权:{}",
                        appNum, appNumOfCurrentEnterprise, appNumOfOwnEnterprise, appNumOfAuth);
                //判断是否足够
                boolean result = true;
                if (memberCardNum < (memberCardNumOfCurrentEnterprise + memberCardNumOfOwnEnterprise + memberCardNumOfAuth)) {
                    result = false;
                }
                if (serviceNum < (serviceNumOfCurrentEnterprise + serviceNumOfOwnEnterprise + serviceNumOfAuth)) {
                    result = false;
                }
                if (appNum < (appNumOfCurrentEnterprise + appNumOfOwnEnterprise + appNumOfAuth)) {
                    result = false;
                }
                return ServiceResponse.success(result);
            }
        }
        return ServiceResponse.success(false);
    }

    /**
     * 获取用户域数据
     * @param enterpriseId
     * @param cuTypeEnum
     * @return
     */
    private List<Long> getAreaIdList(Integer enterpriseId, CuTypeEnum cuTypeEnum) {
        AreaSearchDTO areaSearchDTO = new AreaSearchDTO();
        areaSearchDTO.setCuTypes(Arrays.asList(cuTypeEnum));
        areaSearchDTO.setEnterpriseId(enterpriseId);
        List<ChannelAreaInfoDTO> listCard = userListFieldApiService.queryCustomAreaInfo(areaSearchDTO).getResult();
        if(CollectionUtils.isNotEmpty(listCard)){
            return listCard.stream().map(e -> e.getAreaId()).collect(Collectors.toList());
        }
        return new ArrayList<>();
    }

    private Integer getAreaValue(String areaValue) {
        if (StringUtils.isNotBlank(areaValue)) {
            return areaValue.split(",").length;
        }
        return 0;
    }

    private Map<Integer, Integer> getAreaValueMap(List<Integer> resourceGroupIdList) {
        Integer memberCardNumOfOwnEnterprise = 0;
        Integer serviceNumOfOwnEnterprise = 0;
        Integer appNumOfOwnEnterprise = 0;

        ServiceResponse<List<ResourceGroupDTO>> resourceResponse = resourceGroupApiService.listResourceGroup(resourceGroupIdList);
        if (resourceResponse.isSuccess()) {
            List<ResourceGroupDTO> resourceGroupList = resourceResponse.getResult();
            if (CollectionUtils.isNotEmpty(resourceGroupList)) {
                for (ResourceGroupDTO resourceGroupDTO : resourceGroupList) {
                    Integer customUser = resourceGroupDTO.getCustomUser();
                    if (customUser != 1) {
                        continue;
                    }
                    String customUserData = resourceGroupDTO.getCustomUserData();
                    List<ResourceGroupCustomUserDTO> customUserList = JSON.parseArray(customUserData, ResourceGroupCustomUserDTO.class);
                    if (CollectionUtils.isNotEmpty(customUserList)) {
                        for (ResourceGroupCustomUserDTO customUserDTO : customUserList) {
                            if (customUserDTO.getType() == CuTypeEnum.MCU.getType()) {
                                memberCardNumOfOwnEnterprise += getAreaValue(customUserDTO.getAreaValue());
                            }
                            if (customUserDTO.getType() == CuTypeEnum.SCU.getType()) {
                                serviceNumOfOwnEnterprise += getAreaValue(customUserDTO.getAreaValue());
                            }
                            if (customUserDTO.getType() == CuTypeEnum.ACU.getType()) {
                                appNumOfOwnEnterprise += getAreaValue(customUserDTO.getAreaValue());
                            }
                        }
                    }
                }
            }
        }
        Map<Integer, Integer> areaValueMap = new HashMap<>(8);
        areaValueMap.put(CuTypeEnum.ACU.getType(), appNumOfOwnEnterprise);
        areaValueMap.put(CuTypeEnum.SCU.getType(), serviceNumOfOwnEnterprise);
        areaValueMap.put(CuTypeEnum.MCU.getType(), memberCardNumOfOwnEnterprise);
        return areaValueMap;
    }

}
