package com.gic.enterprise.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.dto.EnterpriseWhiteListDTO;
import com.gic.enterprise.entity.TabBillingAccount;
import com.gic.enterprise.entity.TabEnterprise;
import com.gic.enterprise.entity.TabEnterpriseWhiteList;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.BillingAccountService;
import com.gic.enterprise.service.EnterpriseService;
import com.gic.enterprise.service.EnterpriseWhiteListApiService;
import com.gic.enterprise.service.EnterpriseWhiteListService;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
/**
 *
 * @ClassName: EnterpriseWhiteListApiServiceImpl
 * @Description:
 * @author taogs
 * @date 2020/9/11 15:32
 */
@Service("enterpriseWhiteListApiService")
public class EnterpriseWhiteListApiServiceImpl implements EnterpriseWhiteListApiService{
    @Autowired
    private EnterpriseWhiteListService enterpriseWhiteListService;
    @Autowired
    private EnterpriseService enterpriseService;
    @Autowired
    private BillingAccountService billingAccountService;

    @Override
    public ServiceResponse<Void> saveEnterpriseWhiteList(Integer enterpriseId) {
        TabEnterpriseWhiteList record = enterpriseWhiteListService.getEnterpriseWhiteList(enterpriseId);
        if (record != null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "重复配置");
        }
        TabEnterprise enterprise = enterpriseService.getEnterpriseById(enterpriseId);
        enterpriseWhiteListService.saveEnterpriseWhiteList(new EnterpriseWhiteListDTO()
                .setCompanyName(enterprise.getCompanyName())
                .setEnterpriseName(enterprise.getEnterpriseName())
                .setEnterpriseId(enterpriseId));
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> removeEnterpriseWhiteList(Integer enterpriseId) {
        TabEnterpriseWhiteList record = enterpriseWhiteListService.getEnterpriseWhiteList(enterpriseId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "未配置");
        }
        enterpriseWhiteListService.removeEnterpriseWhiteList(enterpriseId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page<EnterpriseWhiteListDTO>> pageEnterpriseWhiteList(String search, Integer currentPage, Integer pageSize) {
        com.github.pagehelper.Page<TabEnterpriseWhiteList> page = enterpriseWhiteListService
                .pageEnterpriseWhiteList(search, currentPage, pageSize);
        Page<EnterpriseWhiteListDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, EnterpriseWhiteListDTO.class);
        if (resultPage != null) {
            List<EnterpriseWhiteListDTO> list = resultPage.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                List<Integer> enterpriseIdList = list.stream().map(e -> e.getEnterpriseId()).collect(Collectors.toList());
                List<TabBillingAccount> accountList = billingAccountService.listBillingAccount(enterpriseIdList);
                Map<String, Double> accountMap = accountList
                        .stream()
                        .collect(Collectors.toMap(e -> e.getEnterpriseId().toString(), e -> e.getAccountBalance()));
                List<TabEnterprise> enterpriseList = enterpriseService.listEnterpriseByIds(enterpriseIdList);
                Map<String, Date> enterpriseMap = enterpriseList
                        .stream()
                        .collect(Collectors.toMap(e -> e.getEnterpriseId().toString(), e -> e.getAuditTime()));
                list.forEach(e -> {
                    e.setAccountBalance(accountMap.get(e.getEnterpriseId().toString()));
                    e.setAuditTime(enterpriseMap.get(e.getEnterpriseId().toString()));
                });
            }
        }
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<Boolean> existEnterpriseWhiteList(Integer enterpriseId) {
        TabEnterpriseWhiteList record = enterpriseWhiteListService.getEnterpriseWhiteList(enterpriseId);
        if (record == null) {
            return ServiceResponse.success(false);
        }
        return ServiceResponse.success(true);
    }

    @Override
    public ServiceResponse<Double> getEnterpriseBalance(Integer enterpriseId) {
        TabBillingAccount obj = billingAccountService.getByEnterpriseId(enterpriseId);
        if (obj != null) {
            return ServiceResponse.success(obj.getAccountBalance());
        }
        return ServiceResponse.success(0D);
    }
}
