package com.gic.enterprise.service.outer.impl;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.bizdict.api.dto.BizdictDTO;
import com.gic.bizdict.sdk.service.impl.RedisCacheBizdictService;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.dto.*;
import com.gic.enterprise.service.BillingOrderService;
import com.gic.enterprise.service.PackageApiService;
import com.gic.enterprise.service.PackageService;
/**
 *
 * @ClassName: PackageApiServiceImpl
 * @Description:
 * @author taogs
 * @date 2020/9/11 15:33
 */
@Service("packageApiService")
public class PackageApiServiceImpl implements PackageApiService {
    @Autowired
    private RedisCacheBizdictService bizdictService;
    @Autowired
    private PackageService packageService;
    @Autowired
    private BillingOrderService orderService;

    @Override
    public ServiceResponse<List<PackageDTO>> listAllPackage() {
        List<BizdictDTO> list = this.bizdictService.treeByCategoryCode("package_service_type");
        List<PackageDTO> result = new ArrayList<>();
        for(BizdictDTO dto : list){
            PackageDTO packageDTO = new PackageDTO();
            packageDTO.setPackageName(dto.getName());
            packageDTO.setIcon(dto.getAliasName());
            packageDTO.setDesc(dto.getValueContent());
            packageDTO.setType(Integer.valueOf(dto.getCode()));
            result.add(packageDTO);
        }
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<List<SmsPackageDTO>> listAllSmsPackage() {
        List<BizdictDTO> list = this.bizdictService.treeByCategoryCode("sms_package");
        List<SmsPackageDTO> result = new ArrayList<>();
        for(BizdictDTO dto : list){
            SmsPackageDTO smsPackageDTO = new SmsPackageDTO();
            smsPackageDTO.setSmsPackageName(dto.getAliasName());
            smsPackageDTO.setSmsPackageCode(dto.getCode());
            smsPackageDTO.setItemCount(Integer.valueOf(dto.getName()));
            smsPackageDTO.setItemFee(Double.valueOf(dto.getValueContent()));
            result.add(smsPackageDTO);
        }
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<Page<EnterpriseSmsPackageDTO>> pageEnterpriseSmsPackage(Integer enterpriseId, String search,
                                                                                   Integer status, String startDate,
                                                                                   String endDate, int currentPage,
                                                                                   int pageSize) {
        com.github.pagehelper.Page page = this.packageService.pageEnterpriseSmsPackage(enterpriseId, search, status,
                startDate, endDate, currentPage, pageSize);
        Page<EnterpriseSmsPackageDTO> result = PageHelperUtils.changePageHelperToCurrentPage(page, EnterpriseSmsPackageDTO.class);
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<List<AutoRechargeSmsPackageDTO>> listAutoRechargeSmsPackage() {
        List<BizdictDTO> list = this.bizdictService.treeByCategoryCode("auto_recharge_sms_package");
        List<AutoRechargeSmsPackageDTO> result = new ArrayList<>();
        for(BizdictDTO dto : list){
            AutoRechargeSmsPackageDTO dict = new AutoRechargeSmsPackageDTO();
            dict.setDictName(dto.getCategoryName());
            dict.setDictCode(dto.getCode());
            dict.setItemCount(Integer.valueOf(dto.getName()));
            result.add(dict);
        }
        return ServiceResponse.success(result);
    }

}
