package com.gic.enterprise.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.constant.DeleteFlagConstants;
import com.gic.enterprise.dto.ProductMarketDTO;
import com.gic.enterprise.entity.TabProductMarket;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.exception.CommonException;
import com.gic.enterprise.qo.ProductMarketQO;
import com.gic.enterprise.service.ProductMarketApiService;
import com.gic.enterprise.service.ProductMarketService;
import com.gic.redis.data.util.RedisUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.validation.constraints.NotNull;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.concurrent.TimeUnit;

/**
 * 达摩产品情报站
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/5/18 9:26 AM 
 */
@Service("productMarketApiService")
public class ProductMarketApiServiceImpl implements ProductMarketApiService{
    @Autowired
    private ProductMarketService productMarketService;

    @Override
    public ServiceResponse<Void> saveProductMarket(ProductMarketDTO productMarketDTO) {
        validParam(productMarketDTO);
        productMarketService.saveProductMarket(productMarketDTO);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> updateProductMarket(ProductMarketDTO productMarketDTO) {
        validParam(productMarketDTO);
        productMarketService.updateProductMarket(productMarketDTO);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> deleteProductMarket(Integer productMarketId) {
        exist(productMarketId);
        productMarketService.updateProductMarket(new ProductMarketDTO()
                .setProductMarketId(productMarketId)
                .setDeleteFlag(DeleteFlagConstants.DELETE_STATUS));
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<ProductMarketDTO> getProductMarket(@NotNull Integer productMarketId) {
        TabProductMarket record = exist(productMarketId);
        return ServiceResponse.success(EntityUtil.changeEntityNew(ProductMarketDTO.class, record));
    }

    @Override
    public ServiceResponse<Page<ProductMarketDTO>> pageProductMarket(ProductMarketQO params) {
        com.github.pagehelper.Page<TabProductMarket> page = productMarketService.pageProductMarket(params);
        Page<ProductMarketDTO> result = PageHelperUtils.changePageHelperToCurrentPage(page, ProductMarketDTO.class);
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<Boolean> isNeedToPopup(Integer enterpriseId, Integer userId) {
        Boolean hasNewInfo = productMarketService.hasNewInfo();

        if (hasNewInfo) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            String today = sdf.format(new Date());
            //是否已经弹窗提示过，一天只提示一次
            String redisKey = "enterprise:product:market:" + today + ":" + userId;
            Object obj = RedisUtil.getCache(redisKey);
            if(obj != null){
                //已经提示过，不需要再次提示
                return ServiceResponse.success(false);
            } else {
                RedisUtil.setCache(redisKey, redisKey, 24L, TimeUnit.HOURS);
                return ServiceResponse.success(true);
            }
        }
        //没有新发布的数据，不需要提示
        return ServiceResponse.success(false);
    }

    private void validParam(ProductMarketDTO productMarketDTO) {
        Integer productMarketId = productMarketDTO.getProductMarketId();
        boolean isEdit = productMarketId != null;
        if (isEdit) {
            exist(productMarketId);
        }
    }

    private TabProductMarket exist(Integer productMarketId) {
        TabProductMarket record = productMarketService.getProductMarket(productMarketId);
        if (record == null) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "情报不存在");
        }
        return record;
    }
}
