package com.gic.enterprise.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.constant.Constants;
import com.gic.enterprise.dto.*;
import com.gic.enterprise.entity.TabQrcodeDownload;
import com.gic.enterprise.entity.TabQrcodeScenes;
import com.gic.enterprise.entity.TabQrcodeScenesClassify;
import com.gic.enterprise.entity.TabQrcodeScenesValue;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.QrCodeApiService;
import com.gic.enterprise.service.QrCodeService;
import com.github.pagehelper.PageHelper;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
/**
 *
 * @ClassName: QrCodeApiServiceImpl
 * @Description:
 * @author taogs
 * @date 2020/9/11 15:34
 */
@Service("qrCodeApiService")
public class QrCodeApiServiceImpl implements QrCodeApiService {
    @Autowired
    private QrCodeService qrCodeService;

    @Override
    public ServiceResponse<List<QrcodeScenesClassifyDTO>> listQrCodeScenesClassify(Integer enterpriseId) {
        List<TabQrcodeScenesClassify> list = this.qrCodeService.listQrcodeClassify(enterpriseId, null);
        return ServiceResponse.success(EntityUtil.changeEntityListByJSON(QrcodeScenesClassifyDTO.class, list));
    }

    @Override
    public ServiceResponse<Integer> saveQrCodeScenesClassify(QrcodeScenesClassifyDTO classifyDTO) {
        if(classifyDTO.getId() != null){
            TabQrcodeScenesClassify qrCodeScenesClassify = this.qrCodeService.getQrCodeScenesClassifyById(classifyDTO.getId());
            if(qrCodeScenesClassify == null){
                return ServiceResponse.success(0);
            }
            if(qrCodeScenesClassify.getEnterpriseId() == -1){
                return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "系统分类无法删除");
            }
        }
        List<TabQrcodeScenesClassify> list = this.qrCodeService.listQrcodeScenesClassifyByName(classifyDTO.getEnterpriseId(), classifyDTO.getClassifyName());
        for(TabQrcodeScenesClassify tab : list){
            if(tab.getId().intValue() != classifyDTO.getId().intValue()){
                return ServiceResponse.failure(ErrorCode.DATA_EXISTS.getCode(), "分类名称已经存在");
            }
        }
        int i = this.qrCodeService.saveQrCodeClassify(EntityUtil.changeEntityByJSON(TabQrcodeScenesClassify.class, classifyDTO));
        return ServiceResponse.success(i);
    }

    @Override
    public ServiceResponse<QrcodeScenesClassifyDTO> getQrcodeScenesClassify(Integer id) {
        TabQrcodeScenesClassify classify = this.qrCodeService.getQrCodeScenesClassifyById(id);
        return ServiceResponse.success(EntityUtil.changeEntityByJSON(QrcodeScenesClassifyDTO.class, classify));
    }

    @Override
    public ServiceResponse<Integer> deleteQrcodeScenesClassify(Integer id) {
        return ServiceResponse.success(this.qrCodeService.deleteQrcodeScenesClassify(id));
    }

    @Override
    public ServiceResponse<Integer> saveQrCodeScenes(QrcodeScenesDTO qrcodeScenesDTO) {
        List<TabQrcodeScenes> scenes = this.qrCodeService.listQrcodeScenesByName(qrcodeScenesDTO.getEnterpriseId(), qrcodeScenesDTO.getScenesName());
        if(CollectionUtils.isNotEmpty(scenes)){
            for(TabQrcodeScenes tab : scenes){
                if(tab.getScenesId().intValue() != qrcodeScenesDTO.getScenesId().intValue()){
                    return ServiceResponse.failure(ErrorCode.DATA_EXISTS.getCode(), "场景名称已重复");
                }
            }
        }
        int i = this.qrCodeService.saveQrCodeScenes(EntityUtil.changeEntityByJSON(TabQrcodeScenes.class, qrcodeScenesDTO));
        return ServiceResponse.success(i);
    }

    @Override
    public ServiceResponse<Page<QrcodeScenesDTO>> pageQrCodeScenes(Integer enterpriseId, String search,
                                                                   Integer classifyId, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        com.github.pagehelper.Page<QrcodeScenesDTO> page = this.qrCodeService.listQrCodeScenes(search, classifyId, enterpriseId);
        return ServiceResponse.success(PageHelperUtils.changePageHelperToCurrentPage(page));
    }

    @Override
    public ServiceResponse<QrcodeScenesDTO> getQrcodeScenesById(Integer scenedId) {
        TabQrcodeScenes qrcodeScenes = this.qrCodeService.getQrcodeScenesById(scenedId);
        return ServiceResponse.success(EntityUtil.changeEntityByJSON(QrcodeScenesDTO.class, qrcodeScenes));
    }

    @Override
    public ServiceResponse<Integer> openScenesStatus(Integer scenesId) {
        int i = this.qrCodeService.updateQrCodeScenesStatus(scenesId, 1);
        return ServiceResponse.success(i);
    }

    @Override
    public ServiceResponse<Integer> closeScenesStatus(Integer scenesId) {
        int i = this.qrCodeService.updateQrCodeScenesStatus(scenesId, 2);
        return ServiceResponse.success(i);
    }


    @Override
    public ServiceResponse<Integer> deleteScenes(Integer scenesId) {
        int i = this.qrCodeService.updateQrCodeScenesStatus(scenesId, 0);
        return ServiceResponse.success(i);
    }

    @Override
    public ServiceResponse<Integer> saveScenesValue(QrcodeScenesValueDTO qrcodeScenesValueDTO) {
        com.github.pagehelper.Page<TabQrcodeScenesValue> page = this.qrCodeService.listQrCodeScenesValue
                (qrcodeScenesValueDTO.getEnterpriseId(), qrcodeScenesValueDTO.getScenesId(), null);
        if(CollectionUtils.isNotEmpty(page.getResult())){
            for(TabQrcodeScenesValue tab : page.getResult()){
                if(tab.getValueId().intValue() != qrcodeScenesValueDTO.getValueId().intValue()){
                    if(tab.getCode().equals(qrcodeScenesValueDTO.getCode()) || tab.getName().equals(qrcodeScenesValueDTO.getName())){
                        return ServiceResponse.failure(ErrorCode.DATA_EXISTS.getCode(), "场景code或者名称已经存在");
                    }
                }
            }
        }
        int i = this.qrCodeService.saveQrcodeScenesValue(EntityUtil.changeEntityByJSON(TabQrcodeScenesValue.class, qrcodeScenesValueDTO));
        return ServiceResponse.success(i);
    }

    @Override
    public ServiceResponse<Page<QrcodeScenesValueDTO>> pageQrCodeScenesValue(String search, Integer scenesId,
                                                                             Integer enterpriseId, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        com.github.pagehelper.Page<TabQrcodeScenesValue> page = this.qrCodeService.listQrCodeScenesValue(enterpriseId, scenesId, search);
        return ServiceResponse.success(PageHelperUtils.changePageHelperToCurrentPage(page, QrcodeScenesValueDTO.class));
    }

    @Override
    public ServiceResponse<Integer> deleteQrcodeScenesValue(Integer valueId) {
        int i = this.qrCodeService.deleteQrcodeScenesValue(valueId);
        return ServiceResponse.success(i);
    }

    @Override
    public ServiceResponse<Integer> saveQrcodeDownload(QrcodeDTO qrcodeDTO) {
        int i = this.qrCodeService.saveQrcodeDownload(EntityUtil.changeEntityByJSON(TabQrcodeDownload.class, qrcodeDTO));
        return ServiceResponse.success(i);
    }

    @Override
    public ServiceResponse<QrcodeDTO> getQrcodeDownload(Integer id) {
        TabQrcodeDownload qrcodeDownload = this.qrCodeService.getQrcodeDownload(id);
        return ServiceResponse.success(EntityUtil.changeEntityByJSON(QrcodeDTO.class, qrcodeDownload));
    }

    @Override
    public ServiceResponse<QrCodeValueDTO> getQrCodeValueById(String id) {
        TabQrcodeDownload qrcodeDownload = this.qrCodeService.getQrcodeDownload(Integer.valueOf(id));
        if(qrcodeDownload != null){
            QrCodeValueDTO qrCodeValueDTO = new QrCodeValueDTO();
            qrCodeValueDTO.setQrCodeType(qrcodeDownload.getQrCodeType());
            qrCodeValueDTO.setValue(qrcodeDownload.getCustomParams());
            return ServiceResponse.success(qrCodeValueDTO);
        }
        return ServiceResponse.success();
    }
}
