package com.gic.enterprise.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.dto.TableSettingDTO;
import com.gic.enterprise.dto.TableSettingFieldDTO;
import com.gic.enterprise.entity.TabTableSetting;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.TableSettingApiService;
import com.gic.enterprise.service.TableSettingFieldService;
import com.gic.enterprise.service.TableSettingService;
import com.github.pagehelper.PageHelper;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service("tableSettingApiService")
public class TableSettingApiServiceImpl implements TableSettingApiService {
    @Autowired
    private TableSettingService tableSettingService;
    @Autowired
    private TableSettingFieldService tableSettingFieldService;

    @Override
    public ServiceResponse<String> saveTableSetting(TableSettingDTO tableSettingDTO) {
        String result = tableSettingService.saveTableSetting(EntityUtil.changeEntityByJSON(TabTableSetting.class, tableSettingDTO));
        if(StringUtils.isBlank(result)){
            return ServiceResponse.success();
        }
        return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), result);
    }

    @Override
    public ServiceResponse<Page<TableSettingDTO>> pageTableSetting(String search, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        com.github.pagehelper.Page<TabTableSetting> page = tableSettingService.pageTableSetting(search);
        return ServiceResponse.success(PageHelperUtils.changePageHelperToCurrentPage(page, TableSettingDTO.class));
    }

    @Override
    public ServiceResponse<TableSettingDTO> getTableSetting(Integer tableId) {
        return ServiceResponse.success(EntityUtil.changeEntityByJSON(TableSettingDTO.class, tableSettingService.getTableSetting(tableId)));
    }

    @Override
    public ServiceResponse<Void> delTableSetting(Integer tableId) {
        tableSettingService.delTabSetting(tableId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> saveTableField(TableSettingFieldDTO fieldDTO) {
        String result = tableSettingFieldService.saveTableField(fieldDTO);
        if(StringUtils.isBlank(result)){
            return ServiceResponse.success();
        }
        return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), result);
    }

    @Override
    public ServiceResponse<List<TableSettingFieldDTO>> listTableField(Integer tableId) {
        List<TableSettingFieldDTO> list = tableSettingFieldService.listTableField(tableId);
        return ServiceResponse.success(EntityUtil.changeEntityListByJSON(TableSettingFieldDTO.class, list));
    }

    @Override
    public ServiceResponse<Void> delField(Integer fieldId) {
        tableSettingFieldService.delTableField(fieldId);
        return ServiceResponse.success();
    }
}
