package com.gic.enterprise.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.enterprise.constant.Constants;
import com.gic.enterprise.dto.DictDTO;
import com.gic.enterprise.dto.VoiceCodeRecordDTO;
import com.gic.enterprise.service.BillingAccountService;
import com.gic.enterprise.service.BillingVoiceCodeRecordService;
import com.gic.enterprise.service.DictApiService;
import com.gic.enterprise.service.VoiceCodeRecordApiService;
import com.gic.mq.sdk.GicMQClient;
import com.gic.store.utils.valid.ValidUtil;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-12-09 11:08
 */
@Service("voiceCodeRecordApiService")
public class VoiceCodeRecordApiServiceImpl implements VoiceCodeRecordApiService {

    private Logger logger = LogManager.getLogger(VoiceCodeRecordApiServiceImpl.class);

    @Autowired
    private BillingAccountService billingAccountService;
    @Autowired
    private BillingVoiceCodeRecordService billingVoiceCodeRecordService;
    @Autowired
    private DictApiService dictApiService;

    @Override
    public ServiceResponse<Void> saveVoiceCodeRecord(VoiceCodeRecordDTO voiceCodeRecordDTO) {
        ServiceResponse paramResult = ValidUtil.allCheckValidate(voiceCodeRecordDTO, VoiceCodeRecordDTO.SaveValid.class);
        if (!paramResult.isSuccess()) {
            return paramResult;
        }
        GicMQClient client = GICMQClientUtil.getClientInstance();
        try {
            String json = JSON.toJSONString(voiceCodeRecordDTO);
            client.sendMessage("VoiceCodeMq", json);
            this.run(json);
        } catch (Exception e) {
            logger.warn("调用mq失败", e);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> run(String arg) {
        // 缓存中计费
        GicMQClient client = GICMQClientUtil.getClientInstance();
        try {
            logger.info("语音验证码缓存扣费：{}", arg);
            VoiceCodeRecordDTO voiceCodeRecordDTO = JSON.parseObject(arg, VoiceCodeRecordDTO.class);
            // 判断是扣套餐包还是扣余额
            billingAccountService.deductAccountByCache(voiceCodeRecordDTO.getFee(), voiceCodeRecordDTO.getEnterpriseId());

            // 队列
            client.sendMessage("VoiceCodeDbMq", arg);
        } catch (Exception e) {
            logger.warn("调用mq失败", e);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> executeDatabase(String arg) {
        logger.info("语音验证码数据库扣费：{}", arg);
        VoiceCodeRecordDTO voiceCodeRecordDTO = JSON.parseObject(arg, VoiceCodeRecordDTO.class);
        ServiceResponse<DictDTO> serviceDict = dictApiService.getDict(Constants.VOICE_APP_TYPE, voiceCodeRecordDTO.getAppCode());
        if (serviceDict.isSuccess()) {
            DictDTO dictDTO = serviceDict.getResult();
            if (dictDTO != null) {
                String key = dictDTO.getKey();
                voiceCodeRecordDTO.setAppName(key);
            }
        }
        billingAccountService.deductAccountByDatabase(voiceCodeRecordDTO.getFee(), voiceCodeRecordDTO.getEnterpriseId());
        voiceCodeRecordDTO.setDeductionTime(new Date());
        billingVoiceCodeRecordService.save(voiceCodeRecordDTO);
        return ServiceResponse.success();
    }
}
