package com.gic.enterprise.service.outer.impl.pay;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.UserDTO;
import com.gic.commons.util.DateUtil;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.enterprise.constant.BuyTypeEnum;
import com.gic.enterprise.constant.Constants;
import com.gic.enterprise.constant.PayTypeEnum;
import com.gic.enterprise.dto.ApprovalCallBackDTO;
import com.gic.enterprise.dto.BillingPayInfoDTO;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.dto.OutPayDTO;
import com.gic.enterprise.entity.TabBillingPayInfo;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.BillingPayInfoService;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.enterprise.service.outer.impl.paycomplete.PayCompleteHandler;
import com.gic.enterprise.utils.CreateRandomUtils;
import com.gic.log.api.dto.SystemSetLogDTO;
import com.gic.log.api.service.LogApiService;
import com.gic.marketing.process.api.service.sms.SmsSendApiService;
import com.gic.redis.data.util.RedisUtil;
import com.gic.thirdparty.api.dto.ScanPayReqDataDTO;
import com.gic.thirdparty.api.service.Pay4WXService;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.Date;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019/8/20 15:35
 */
@Component("1")
public class WxPayStrategy implements PayStrategy {
    private Logger logger = LogManager.getLogger(WxPayStrategy.class);

    @Autowired
    private Pay4WXService pay4WxService;
    @Autowired
    private BillingPayInfoService billingPayInfoService;
    @Autowired
    private EnterpriseApiService enterpriseApiService;
    @Autowired
    SmsSendApiService smsSendApiService;
    @Autowired
    private LogApiService logApiService;

    private Map<String, PayCompleteHandler> payCompleteHandlerMap = new ConcurrentHashMap<>();
    @Autowired
    public WxPayStrategy(Map<String, PayCompleteHandler> payCompleteHandlerMap) {
        this.payCompleteHandlerMap.putAll(payCompleteHandlerMap);
    }


    @Override
    public ServiceResponse<BillingPayInfoDTO> savePrePayInfo(OutPayDTO outPayDTO) {
        try {
            RedisUtil.lock(Constants.PAY_BACK_LOCK + outPayDTO.getEnterpriseId(), 5L);
            if (StringUtils.isNotBlank(outPayDTO.getOrderSerialNumber())) {
                TabBillingPayInfo payInfo = billingPayInfoService.getByOrderSerialNumber(outPayDTO.getOrderSerialNumber(), outPayDTO.getPayType());
                if (payInfo != null) {
                    BillingPayInfoDTO payInfoDTO = EntityUtil.changeEntityByOrika(BillingPayInfoDTO.class, payInfo);
                    return ServiceResponse.success(payInfoDTO);
                }
            }

            String serialNumber = CreateRandomUtils.createSerialNumber();
            BillingPayInfoDTO billingPayInfoDTO = saveWxPrePay(outPayDTO.getEnterpriseId(), serialNumber, outPayDTO.getTotalFee(), outPayDTO.getBuyType(), outPayDTO.getOrderSerialNumber());
            // 微信支付
            if (billingPayInfoDTO != null) {
                return ServiceResponse.success(billingPayInfoDTO);
            } else {
                return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "调用微信支付失败");
            }
        } catch (Exception e) {
            logger.info("预支付错误", e);
            throw new RuntimeException(e);
        } finally {
            RedisUtil.unlock(Constants.PAY_BACK_LOCK);
        }
    }

    @Override
    public ServiceResponse<BillingPayInfoDTO> confirmPay(OutPayDTO outPayDTO) {
        logger.info("微信没有该方法 confirmPay");
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Boolean> queryAndUpdatePayStatus(Integer enterpriseId, String serialNumber) {
        TabBillingPayInfo payInfo = billingPayInfoService.getBySerialNumber(serialNumber);
        return ServiceResponse.success(payInfo != null && payInfo.getResultCode() == 1);
//        // 微信支付
//        ServiceResponse<ScanPayQueryResData> serviceResponse = pay4WXService.scanPayOrderQuery(serialNumber);
//        if (serviceResponse.isSuccess()) {
//            ScanPayQueryResData resData = serviceResponse.getResult();
//            if (StringUtils.equalsIgnoreCase(resData.getReturn_code(), "SUCCESS")) {
//                Integer i = billingPayInfoService.updatePayInfoBySerialNumber(
//                        enterpriseId,
//                        serialNumber,
//                        StringUtils.equalsIgnoreCase(resData.getResult_code(), "SUCCESS") ? 1 : 0,
//                        resData.getErr_code(),
//                        resData.getErr_code_des()
//                );
//            }
//            return ServiceResponse.success(StringUtils.equalsIgnoreCase(resData.getResult_code(), "SUCCESS"));
//        } else {
//            return ServiceResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
//        }

    }


    @Override
    public ServiceResponse<Void> callBack(ApprovalCallBackDTO resData) {
        try {
            RedisUtil.lock(Constants.PAY_BACK_LOCK + resData.getEnterpriseId(), 5L);
            logger.info("微信支付回调结果:{}", JSON.toJSONString(resData));
            // 写入支付结果

            TabBillingPayInfo billingPayInfo = billingPayInfoService.getBySerialNumber(resData.getPaySerialNumber());
            // 防止微信重复推信息
            if (StringUtils.isNotBlank(billingPayInfo.getTimeEnd())) {
                logger.info("微信重复回调 paySerialNumber:{}", resData.getPaySerialNumber());
                return ServiceResponse.success();
            }
            Integer i = billingPayInfoService.updatePayInfoBySerialNumber(resData);
            resData.setOrderSerialNumber(billingPayInfo.getOrderSerialNumber());
            payCompleteHandlerMap.get(BuyTypeEnum.getTypeByCode(resData.getBuyType())).payComplete(resData, billingPayInfo);
            // 删除其他的支付方式
            billingPayInfoService.delOtherPayInfoExceptId(billingPayInfo.getPayInfoId(), billingPayInfo.getEnterpriseId(), billingPayInfo.getOrderSerialNumber());
            return ServiceResponse.success();
        } catch (Exception e) {
            logger.info("微信回调错误", e);
            throw new RuntimeException(e);
        } finally {
            RedisUtil.unlock(Constants.PAY_BACK_LOCK);
        }
    }

    private BillingPayInfoDTO saveWxPrePay(Integer enterpriseId, String serialNumber, Double totalFee, Integer buyType, String orderSerialNumber) {
        Date startDate = new Date();
        String startDateFormat = DateUtil.formatString(startDate, DateUtil.FORMAT_DATETIME_14);
        // 生成付款码
        ScanPayReqDataDTO scanPayReqDataDTO = new ScanPayReqDataDTO(
                "充值",
                "gic:" + enterpriseId.toString(),
                serialNumber,
                new BigDecimal(totalFee.toString()).multiply(new BigDecimal("100")).intValue(),
                startDateFormat,
                "",
                "NATIVE"
        );
        ServiceResponse<String> serviceResponse = pay4WxService.createPrePaymentOrderForWX(scanPayReqDataDTO);
        if (serviceResponse.isSuccess()) {
            // 生成支付记录
            BillingPayInfoDTO billingPayInfoDTO = new BillingPayInfoDTO();
            billingPayInfoDTO.setEnterpriseId(enterpriseId);
            billingPayInfoDTO.setSerialNumber(serialNumber);
            billingPayInfoDTO.setOrderSerialNumber(orderSerialNumber);
            billingPayInfoDTO.setPayType(PayTypeEnum.WE_CHAT_PAY.getCode());
            billingPayInfoDTO.setTimeStart(startDateFormat);
            billingPayInfoDTO.setCodeUrl(serviceResponse.getResult());
            billingPayInfoDTO.setTotalFeePaid(totalFee);
            billingPayInfoDTO.setInvoiceStatus(GlobalInfo.DATA_STATUS_DELETE);
            billingPayInfoDTO.setDeleteFlag(0);
            billingPayInfoDTO.setCreateTime(new Date());
            billingPayInfoDTO.setBuyType(buyType);
            Integer payInfoId = billingPayInfoService.savePayInfo(billingPayInfoDTO);
            billingPayInfoDTO.setPayInfoId(payInfoId);
            return billingPayInfoDTO;
        } else {
            return null;
        }
    }

    private void doLog(String content, String operationObject, UserDTO userDTO) {
        final SystemSetLogDTO logDTO = new SystemSetLogDTO();
        //操作时间
        logDTO.setLogTime(new Date());
        logDTO.setCreateTime(new Date());
        //操作人
        logDTO.setUserId(Long.valueOf(userDTO.getUserId()));
        logDTO.setUserName(userDTO.getUserName());
        //操作内容
        logDTO.setContent(content);
        //为了search查询，暂时用这个字段存
        logDTO.setInterfaceName(content);
        ////操作对象
        logDTO.setBusinessName(operationObject);
        //操作模块
        logDTO.setRelationId(45000L);
        logDTO.setProject("企业-计费中心");
//            }
        //账号（手机号）
        logDTO.setRemark1(userDTO.getPhoneAreaCode());
        logDTO.setRemark2(userDTO.getPhoneNumber());

        logDTO.setEnterpriseId(userDTO.getEnterpriseId().longValue());
        EnterpriseDTO enterpriseDTO = enterpriseApiService.getEnterpriseById(userDTO.getEnterpriseId()).getResult();
        logDTO.setEnterpriseName(enterpriseDTO.getEnterpriseName());
        logApiService.saveSystemSetLog(logDTO);
    }

}
