package com.gic.enterprise.service.outer.impl.paycomplete;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.UserApiService;
import com.gic.enterprise.constant.BillingAuditStatusEnum;
import com.gic.enterprise.dto.ApprovalCallBackDTO;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.entity.TabBillingOrder;
import com.gic.enterprise.entity.TabBillingPayInfo;
import com.gic.enterprise.service.BillingOrderService;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.log.api.dto.SystemSetLogDTO;
import com.gic.log.api.service.LogApiService;
import com.gic.marketing.process.api.service.sms.SmsSendApiService;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2020-11-24 16:01
 */
@Component("BUY_SHORT_MESSAGE_PACKAGE")
public class SmsPackageHandler implements PayCompleteHandler {

    private Logger logger = LogManager.getLogger(SmsPackageHandler.class);

    @Autowired
    private BillingOrderService billingOrderService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private EnterpriseApiService enterpriseApiService;
    @Autowired
    private SmsSendApiService smsSendApiService;
    @Autowired
    private LogApiService logApiService;

    @Override
    public void payComplete(ApprovalCallBackDTO resData, TabBillingPayInfo billingPayInfo) {
        billingOrderService.updateBySerialNumber(resData.getOrderSerialNumber(), resData.getTotalFeePaid(), resData.getAuditStatus(), resData.getTimeEnd());
        TabBillingOrder order = this.billingOrderService.getOrderBySerialNumber(resData.getOrderSerialNumber());
        UserDTO user = userApiService.getUserInfoById(order.getInitiatorUser()).getResult();
        doLog("支付", "订单内容", user);
        sendOrderApprovalSms(resData, "GICJFZX006");
    }

    private void sendOrderApprovalSms(ApprovalCallBackDTO resData, String code) {
        TabBillingOrder order = billingOrderService.getOrderBySerialNumber(resData.getOrderSerialNumber());
        Integer initiator = order.getInitiatorUser();
        if (initiator != null) {
            UserDTO userDTO = userApiService.getUserInfoById(initiator).getResult();
            sendSms(resData, userDTO, code);
        }
    }

    private void sendSms(ApprovalCallBackDTO resData, UserDTO userDTO, String code) {
        try {
            if (StringUtils.isBlank(userDTO.getPhoneAreaCode())) {
                userDTO.setPhoneAreaCode("+86");
            }
            EnterpriseDTO enterpriseDTO = enterpriseApiService.getEnterpriseById(userDTO.getEnterpriseId()).getResult();
            String[] smsArr = new String[]{enterpriseDTO.getEnterpriseName(), resData.getOrderSerialNumber(), BillingAuditStatusEnum.PASS.getCode().equals(resData.getAuditStatus()) ? "成功" : "失败"};
            ServiceResponse<Void> smsSendResult = smsSendApiService.sendPlatformSms(code,
                    userDTO.getEnterpriseId(), userDTO.getPhoneAreaCode(), userDTO.getPhoneNumber(), smsArr);
            if (!smsSendResult.isSuccess()) {
                logger.warn(smsSendResult.getMessage());
            }
        } catch (Exception e) {
            logger.info("发送短信失败", e);
        }
    }

    private void doLog(String content, String operationObject, UserDTO userDTO) {
        final SystemSetLogDTO logDTO = new SystemSetLogDTO();
        //操作时间
        logDTO.setLogTime(new Date());
        logDTO.setCreateTime(new Date());
        //操作人
        logDTO.setUserId(Long.valueOf(userDTO.getUserId()));
        logDTO.setUserName(userDTO.getUserName());
        //操作内容
        logDTO.setContent(content);
        //为了search查询，暂时用这个字段存
        logDTO.setInterfaceName(content);
        ////操作对象
        logDTO.setBusinessName(operationObject);
        //操作模块
        logDTO.setRelationId(45000L);
        logDTO.setProject("企业-计费中心");
        //            }
        //账号（手机号）
        logDTO.setRemark1(userDTO.getPhoneAreaCode());
        logDTO.setRemark2(userDTO.getPhoneNumber());

        logDTO.setEnterpriseId(userDTO.getEnterpriseId().longValue());
        EnterpriseDTO enterpriseDTO = enterpriseApiService.getEnterpriseById(userDTO.getEnterpriseId()).getResult();
        logDTO.setEnterpriseName(enterpriseDTO.getEnterpriseName());
        logApiService.saveSystemSetLog(logDTO);
    }
}
