package com.gic.enterprise.utils;

import com.gic.enterprise.constant.DownloadReportDataTypeEnum;
import com.opencsv.ResultSetHelper;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.SimpleDateFormat;
import java.util.List;

public class CsvResultSetHelper implements ResultSetHelper {
	private static final Logger log = LogManager.getLogger(CsvResultSetHelper.class);
	/** 数据过滤模式 */
	private Integer dataType = DownloadReportDataTypeEnum.DESENSITIZATION_DATA.getCode();

	/** 数据过滤用关键字集合 */
	private List<String> filters = null;

	public CsvResultSetHelper(Integer dataType, List<String> filters) {
		this.dataType = dataType;
		this.filters = filters;
	}

	@Override
	public String[] getColumnNames(ResultSet resultSet) throws SQLException {
		String[] result = new String[0];
		int columnCount = resultSet.getMetaData().getColumnCount();
		if (columnCount > 0) {
			result = new String[columnCount];
			for (int i=0; i<columnCount; i++) {
				String rsColumnName = resultSet.getMetaData().getColumnLabel(i + 1);
				result[i] = rsColumnName;
			} // FOR OVER
		} // IF OVER
		return result;
	}

	@Override
	public String[] getColumnValues(ResultSet resultSet) throws SQLException, IOException {
		return this.getColumnValues(resultSet, false);
	}

	@Override
	public String[] getColumnValues(ResultSet resultSet, boolean b) throws SQLException, IOException {
		return this.getColumnValues(resultSet, false, "yyyy-MM-dd HH:mm:ss", "HH:mm:ss");
	}


	private static SimpleDateFormat datetimeFormatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

	private static SimpleDateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd");

	private static SimpleDateFormat timeFormatter = new SimpleDateFormat("hh:mm:ss");

	@Override
	public String[] getColumnValues(ResultSet resultSet, boolean b, String s, String s1) throws SQLException, IOException {
		String[] result = new String[0];
		int columnCount = resultSet.getMetaData().getColumnCount();
		if (columnCount > 0) {
			result = new String[columnCount];
			for (int i=0; i<columnCount; i++) {
				String columnName = resultSet.getMetaData().getColumnLabel(i + 1);
				if(dataType == DownloadReportDataTypeEnum.DESENSITIZATION_DATA.getCode()){
					//如果是脱敏数据
					if(filters.contains(columnName)){
						result[i] = "******";
					} else {
						getValue(resultSet, result, i, columnName);
					}
				} else {
					//完整数据不处理
					getValue(resultSet, result, i, columnName);
				}
			}
		}
		return result;
	}

	private void getValue(ResultSet resultSet, String[] result, int i, String columnName){
		try {
			int columnType = resultSet.getMetaData().getColumnType(i + 1);
			switch (columnType) {
				case Types.TIMESTAMP:
					result[i] = resultSet.getTimestamp(columnName) != null ? datetimeFormatter.format(resultSet.getTimestamp(columnName)) : "";
					break;
				case Types.DATE:
					//result[i] = resultSet.getTimestamp(columnName) != null ? dateFormatter.format(resultSet.getTimestamp(columnName)) : "";
					result[i] = resultSet.getDate(columnName) != null ? dateFormatter.format(resultSet.getDate(columnName)) : "";
					break;
				case Types.TIME:
					result[i] = resultSet.getTimestamp(columnName) != null ? timeFormatter.format(resultSet.getTimestamp(columnName)) : "";
					break;
				default:
					result[i] = String.valueOf(resultSet.getObject(columnName));
					break;
			}
		}catch (Exception e){
			log.warn(e);
		}

	}
}
