package com.gic.enterprise.utils;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.SynchronousQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import javax.inject.Singleton;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 *  线程池单例类
 * 用于导出数据异步使用
 * @ClassName: ExecutorPoolSingleton 
 * @Description:  
 * @author guojuxing 
 * @date 2019/11/5 7:56 PM 
 */
public class ExecutorPoolSingleton {
    private Logger logger = LogManager.getLogger(ExecutorPoolSingleton.class);
    private static final int availableProcessor = Runtime.getRuntime().availableProcessors();
    private static ExecutorService executorService;

    private ExecutorPoolSingleton() {
        if (executorService == null) {
            int coreNum = availableProcessor / 2;
            // 用单例模式创建线程池，保留2个核心线程，最多线程为CPU个数的2n+1的两倍.
            int maxProcessor = (availableProcessor * 2 + 1) * 2 ;

            executorService = new ThreadPoolExecutor(coreNum > 2 ? 2 : coreNum, maxProcessor,
                    60L, TimeUnit.SECONDS,
                    new SynchronousQueue<Runnable>());
        }
    }

    private static ExecutorPoolSingleton instance;

    public static ExecutorPoolSingleton getInstance() {
        if (instance == null) {
            synchronized (Singleton.class) {
                if (instance == null) {
                    instance = new ExecutorPoolSingleton();
                }
            }
        }
        return instance;
    }

    int executeThreadNum = 1;

    /**
     * 执行任务
     * @param runnable
     */
    public void executeTask(Runnable runnable) {
        executorService.execute(runnable);
        logger.info("异步线程执行了%d次:{} ", executeThreadNum++);
    }
}
