package com.gic.enterprise.utils;

import com.gic.enterprise.constant.DownloadReportDataTypeEnum;
import com.gic.enterprise.constant.ExcelExtensionEnum;
import com.gic.enterprise.pojo.DownloadTaskPOJO;
import com.gic.thirdparty.BucketNameEnum;
import com.gic.thirdparty.FileOperateUtils;
import com.opencsv.CSVWriter;
import com.opencsv.ResultSetHelper;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;

import java.io.*;
import java.nio.charset.Charset;
import java.sql.ResultSet;
import java.sql.Types;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

public class FileUtils {
    /** csv / xls 下载目录 */
    public static final String SAVE_FOLDER = "/usr/local/data-hook-file";
    private static final Logger log = LogManager.getLogger(FileUtils.class);
    private static SimpleDateFormat datetimeFormatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    private static SimpleDateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd");
    private static SimpleDateFormat timeFormatter = new SimpleDateFormat("HH:mm:ss");

    public static String createFile(ResultSet rs, Integer dataType, List<String> desenField, String fileName, Integer fileExt){
        String filePath = getFilePath(fileName, fileExt);
        File tmpFile = createTmpFile(filePath);
        if (fileExt.equals(ExcelExtensionEnum.CSV.getCode())) {
            createCsvFile(rs, dataType, desenField, tmpFile, fileName);
        } else {
            createXls(rs, dataType, desenField, tmpFile, fileName, fileExt);
        }
        String url = uploadFile(tmpFile, fileName, fileExt);
        tmpFile.deleteOnExit();
        return url;
    }

    private static void createXls(ResultSet rs, Integer dataType, List<String> desenField, File tempFile, String fileName, Integer fileExt){
        log.info("开始生成文件 " + fileName + ".xlsx");
        try{
            Workbook wb = null;
            if(fileExt.equals(ExcelExtensionEnum.EXCEL_2003.getCode())){
                wb = new SXSSFWorkbook(100); // 内存中保留 100 行
            } else {
                wb = new HSSFWorkbook();
            }
            Sheet sheet = wb.createSheet();
            Row row = sheet.createRow(0);
            Cell cell;
            for (int j = 0; j < rs.getMetaData().getColumnCount(); ++j) { // 遍历创建表头
                String colName = rs.getMetaData().getColumnLabel(j + 1);
                cell = row.createCell(j);
                cell.setCellValue(colName);
            }
            // 遍历输出行
            int rowCount = 0;
            while (rs.next()) {
                rowCount++;
                row = sheet.createRow(rowCount);
                for (int j = 0; j < rs.getMetaData().getColumnCount(); ++j) {
                    String cName = rs.getMetaData().getColumnName(j+1);
                    if (dataType.equals(DownloadReportDataTypeEnum.DESENSITIZATION_DATA.getCode())) {
                        if(desenField.contains(cName)){
                            row.createCell(j).setCellValue("******");
                        }else {
                            getValue(rs, row, j);
                        }
                    } else {
                        getValue(rs, row, j);
                    }
                }
            }
            FileOutputStream fileOut = new FileOutputStream(tempFile);
            wb.write(fileOut);
            fileOut.close();
            log.info("临时文件已生成， " + fileName + ".xlsx");
        }catch (Exception e){
            log.warn(e);
        }
    }

    private static void createCsvFile(ResultSet rs, Integer dataType, List<String> desenField, File tempFile, String fileName){
        try {
            OutputStreamWriter out = new OutputStreamWriter(new FileOutputStream(tempFile), Charset.forName("GBK"));
            ResultSetHelper helper = new CsvResultSetHelper(dataType, desenField);
            CSVWriter writer = new CSVWriter(out, ',');
            writer.setResultService(helper);
            writer.writeAll(rs, true);
            writer.close();
            out.close();//记得关闭资源
            log.info("临时文件已生成 " + fileName + ".csv");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private static File createTmpFile(String path){
        File dir = new File(SAVE_FOLDER);
        if(!dir.exists()){
            dir.mkdirs();
        }
        File tmp = new File(path);
        if (tmp.exists()) { // 删除可能存在的文件
            tmp.delete();
        }
        if(!tmp.exists()){
            try {
                tmp.createNewFile();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return tmp;
    }

    private static String getFilePath(String fileName, Integer fileExt){
        String originalFilePath = SAVE_FOLDER + File.separator + fileName ;
        if(ExcelExtensionEnum.CSV.getCode() == fileExt){
            originalFilePath += ExcelExtensionEnum.CSV.getExtension();
        } else if(ExcelExtensionEnum.EXCEL_2003.getCode() == fileExt){
            originalFilePath += ExcelExtensionEnum.EXCEL_2003.getExtension();
        } else if(ExcelExtensionEnum.EXCEL_2007.getCode() == fileExt){
            originalFilePath += ExcelExtensionEnum.EXCEL_2007.getExtension();
        }
        return originalFilePath;
    }

    private static String uploadFile(File tmp, String fileName, Integer fileExt){
        //上传文件
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        String fieldCode = "/" + fileName + "_" + sdf.format(new Date()) + ExcelExtensionEnum.getExtensionByCode(fileExt);
        log.info("上传文件到腾讯云，文件名：" + fieldCode);
        //文件存进腾讯云
        String url = FileOperateUtils.simpleUploadFileFromLocal(tmp, fieldCode, BucketNameEnum.REPORT_50000.getName());
        if (org.apache.commons.lang.StringUtils.isNotBlank(url)) {
            url = "https://" + url;
        }
        log.info("上传文件到腾讯云，路径：" + url);
        return url;
    }


    private static void getValue(ResultSet rs, Row row, int j){
        try{
            int cType = rs.getMetaData().getColumnType(j + 1);
            switch (cType) {
                case Types.TIMESTAMP:
                    row.createCell(j).setCellValue(rs.getTimestamp(j + 1) != null ? datetimeFormatter.format(rs.getTimestamp(j + 1)) : "");
                    break;
                case Types.DATE:
                    row.createCell(j).setCellValue(rs.getDate(j + 1) != null ? dateFormatter.format(rs.getDate(j + 1)) : "");
                    break;
                case Types.TIME:
                    row.createCell(j).setCellValue(rs.getTimestamp(j + 1) != null ? timeFormatter.format(rs.getTimestamp(j + 1)) : "");
                    break;
                default:
                    row.createCell(j).setCellValue(rs.getString(j + 1));
                    break;
            }
        }catch (Exception e){
            log.warn(e);
        }
    }
}
