package com.gic.enterprise.utils;

import com.ctrip.framework.apollo.Config;
import com.ctrip.framework.apollo.ConfigService;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

public class HiveConnUtils {
    private static final Logger log = LogManager.getLogger(HiveConnUtils.class);
    private static String driverName = "org.apache.hive.jdbc.HiveDriver";
    private static Map<String, Connection> conCache = new ConcurrentHashMap<>();
    private static boolean start = false;

    /**hive由于长时间跑会出现稳定性问题，所以需要每天重启，使用连接池连接没法释放，导致报错，所以采用jdbc直接获取连接并缓存，定时检测连接的有效性**/
    public static Connection getConnection(){
        Config config = ConfigService.getConfig("BUZ.COMMON.HIVE");
        String url = config.getProperty("hive.download.url", "");
        String user = config.getProperty("hive.username", "");
        String password = config.getProperty("hive.password", "");
        return getConnnection(url, user, password);
    }

    public static Connection getConnection(String hiveHost, String port){
        String hiveUrl = getHiveUrl(hiveHost, port);
        Config config = ConfigService.getConfig("BUZ.COMMON.HIVE");
        String user = config.getProperty("hive.username", "");
        String password = config.getProperty("hive.password", "");
        return getConnnection(hiveUrl, user, password);
    }

    public static Connection getConnnection(String url, String user, String password){
        try {
            String key = url + ":" + user;
            Connection connection = conCache.get(key);
            if(connection != null){
                if(connection.isValid(10)){
                    return connection;
                }
            }
            Class.forName(driverName);
            connection = DriverManager.getConnection(url, user, password);
            conCache.put(key, connection);
            return connection;
        } catch (ClassNotFoundException | SQLException e) {
            log.warn("获取hive连接异常：", e);
        }
        return null;
    }

    private static String getHiveUrl(String hiveHost, String port){
        String url = "jdbc:hive2://"+ hiveHost +":"+ port +"/gicads?thriftSocketTimeout=10000;auth=noSasl";
        return url;
    }
}
