package com.gic.enterprise.web.controller;

import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.web.vo.AppletServiceConfigVO;
import com.gic.member.config.api.dto.AppletsConfigSimpleOutputDTO;
import com.gic.member.config.api.dto.ServiceConfigSimpleOutputDTO;
import com.gic.member.config.api.service.AppletsConfigApiService;
import com.gic.member.config.api.service.ServiceConfigApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.log.LogUtils;
import com.gic.enterprise.constant.applet.AppletServiceConfigEnum;
import com.gic.enterprise.dto.AppletServiceConfigDTO;
import com.gic.enterprise.qo.AppletServiceConfigQO;
import com.gic.enterprise.service.AppletServiceConfigApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-11-27 17:46
 */
@RestController
public class AppletServiceConfigController {

    @Autowired
    private AppletServiceConfigApiService appletServiceConfigApiService;
    @Autowired
    private AppletsConfigApiService appletsConfigApiService;

    @RequestMapping("list-applet")
    public RestResponse listApplet(AppletServiceConfigQO appletServiceConfigQO) {
        ServiceResponse<Page<AppletServiceConfigDTO>> serviceResponse = appletServiceConfigApiService.listApplet(UserDetailUtils.getUserDetail().getEnterpriseId(), appletServiceConfigQO);

        Map<String, String> appIdMapImage = getAppletAppIdMapImage();
        if (serviceResponse.isSuccess()) {
            Page<AppletServiceConfigDTO> page = serviceResponse.getResult();
            if (page != null && CollectionUtils.isNotEmpty(page.getResult())) {
                Page<AppletServiceConfigVO> resultPage = EntityUtil.changeEntityNew(Page.class, page);

                List<AppletServiceConfigDTO> list = page.getResult();

                resultPage.setResult(list.stream().map(e -> {
                    AppletServiceConfigVO vo = EntityUtil.changeEntityNew(AppletServiceConfigVO.class, e);
                    vo.setImage(appIdMapImage.get(e.getAppId()));
                    return vo;
                }).collect(Collectors.toList()));
                return RestResponse.success(resultPage);
            }
            return RestResponse.success(page);
        }
        return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
    }

    @RequestMapping("list-service")
    public RestResponse listService(AppletServiceConfigQO appletServiceConfigQO) {
        ServiceResponse<Page<AppletServiceConfigDTO>> serviceResponse = appletServiceConfigApiService.listService(UserDetailUtils.getUserDetail().getEnterpriseId(), appletServiceConfigQO);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("sync-applet")
    public RestResponse listApplet() {
        ServiceResponse<Void> serviceResponse = appletServiceConfigApiService.syncApplet(UserDetailUtils.getUserDetail().getEnterpriseId());
        LogUtils.createLog("同步数据", "小程序");
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("sync-service")
    public RestResponse syncServiceConfig() {
        ServiceResponse<Void> serviceResponse = appletServiceConfigApiService.syncServiceConfig(UserDetailUtils.getUserDetail().getEnterpriseId());
        LogUtils.createLog("同步数据", "服务号");
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("del-applet-service")
    public RestResponse delAppletService(Integer appletServiceId) {
        ServiceResponse<AppletServiceConfigDTO> response = appletServiceConfigApiService.getAppletService(appletServiceId);
        AppletServiceConfigDTO appletService = response.getResult();
        if (appletService != null) {
            ServiceResponse<Void> serviceResponse = appletServiceConfigApiService.delAppletService(appletServiceId);
            if (serviceResponse.isSuccess()) {
                LogUtils.createLog("删除" + (AppletServiceConfigEnum.parseCode(appletService.getType())), appletService.getName());
                return ResultControllerUtils.commonResult(serviceResponse);
            }
        }
        return ResultControllerUtils.commonResult(response);
    }

    private Map<String, String> getAppletAppIdMapImage() {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<List<AppletsConfigSimpleOutputDTO>> serviceResponse = appletsConfigApiService.listValidAppletConfigAppId(enterpriseId);
        if (serviceResponse.isSuccess() && CollectionUtils.isNotEmpty(serviceResponse.getResult())) {
            return serviceResponse.getResult().stream().filter(e -> StringUtils.isNotBlank(e.getAppId()) && StringUtils.isNotBlank(e.getImg()))
                    .collect(Collectors.toMap(AppletsConfigSimpleOutputDTO::getAppId, AppletsConfigSimpleOutputDTO::getImg));
        }
        return new HashMap<>();
    }

}
