package com.gic.enterprise.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.constant.WechatServiceFieldEnum;
import com.gic.enterprise.constant.applet.AppletPageTypeEnum;
import com.gic.enterprise.dto.AppletCustomPageDTO;
import com.gic.enterprise.dto.CustomChatFiledDTO;
import com.gic.enterprise.dto.CustomUdeskFieldDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.qo.AppletPageQO;
import com.gic.enterprise.service.AppletCustomPageApiService;
import com.gic.enterprise.service.CustomUdeskFieldApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import org.apache.catalina.User;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/** @Description: 小程序配置客户服务
 * @author taogs
 * @Date 14:04 2019/12/3
 * @Param 
 * @return 
*/
@RestController
public class CustomChatController {
    @Autowired
    private CustomUdeskFieldApiService customUdeskFieldApiService;
    @Autowired
    private AppletCustomPageApiService appletCustomPageApiService;

    @RequestMapping("get-custom-chat-detail")
    public RestResponse getDetail(String appId){
        AppletPageQO qo = new AppletPageQO();
        qo.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        qo.setAppId(appId);
        qo.setPageType(AppletPageTypeEnum.CUSTOMER_SERVICE.getCode());
        ServiceResponse<Page<AppletCustomPageDTO>> page = this.appletCustomPageApiService.page(qo);
        if(page.isSuccess() && page.getResult() != null){
            if(page.getResult().getTotalCount()>0){
                ServiceResponse<AppletCustomPageDTO> detail = this.appletCustomPageApiService.getDetailByPageId(page.getResult().getResult().get(0).getPageId());
                return RestResponse.success(detail.getResult());
            }
        }
        return RestResponse.success();
    }

    @RequestMapping("list-all-field")
    public RestResponse listAllField(){
        ServiceResponse<List<CustomChatFiledDTO>> response = this.customUdeskFieldApiService.listAllField();
        if(response.isSuccess()){
            return RestResponse.success(response.getResult());
        }
        return RestResponse.success();
    }

    @RequestMapping("list-enterprise-field")
    public RestResponse listEnterpriseField(String appId) {
        if (StringUtils.isBlank(appId)) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "appId不能为空");
        }
        //初始化
        for (WechatServiceFieldEnum fieldEnum : WechatServiceFieldEnum.values()) {
            customUdeskFieldApiService.saveEnterpriseField(new CustomUdeskFieldDTO()
                    .setAppId(appId)
                    .setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId())
                    .setGicFieldId(fieldEnum.getCode())
                    .setGicFieldName(fieldEnum.getMessage())
                    .setStatus(2)
                    .setFieldType(fieldEnum.getFieldType()));
        }

        ServiceResponse<List<CustomUdeskFieldDTO>> response = this.customUdeskFieldApiService
                .listEnterpriseField(UserDetailUtils.getUserDetail().getEnterpriseId(), appId);
        return ResultControllerUtils.commonResult(response);
    }

    @RequestMapping("get-enterprise-field-detail")
    public RestResponse getEnterpriseFieldDetail(Integer id){
        ServiceResponse<CustomUdeskFieldDTO> response = this.customUdeskFieldApiService.getDetail(id);
        return RestResponse.success(response.getResult());
    }

    @RequestMapping("edit-enterprise-field-detail")
    public RestResponse editEnterpriseFieldDetail(CustomUdeskFieldDTO dto){
        return ResultControllerUtils.commonResult(this.customUdeskFieldApiService.editEnterpriseField(dto));
    }

    @RequestMapping("delete-enterprise-field")
    public RestResponse deleteEnterpriseField(Integer id){
        ServiceResponse<Integer> response = this.customUdeskFieldApiService.deleteField(id);
        return RestResponse.success(response.getResult());
    }

    @RequestMapping("update-enterprise-field-status")
    public RestResponse updateEnterpriseFieldStatus(Integer id, Integer status){
        ServiceResponse<Integer> response = this.customUdeskFieldApiService.updateStatus(id, status);
        return RestResponse.success(response.getResult());
    }
}
