package com.gic.enterprise.web.controller;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.StringUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.OperationResultUtils;
import com.gic.enterprise.ano.IgnoreLogin;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.dto.data.*;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.DataConfigApiService;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.enterprise.service.EnterpriseInitApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.enterprise.web.vo.MemberCardVO;
import com.gic.enterprise.web.vo.data.*;
import com.gic.member.config.api.dto.*;
import com.gic.member.config.api.service.AppletsConfigApiService;
import com.gic.member.config.api.service.ConfigApiService;
import com.gic.member.config.api.service.MemberCardApiService;
import com.gic.member.config.api.service.ServiceConfigApiService;
import com.gic.spark.api.service.SparkJobApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;
import java.util.stream.Collectors;
/**
 *
 * @ClassName: DataConfigController
 * @Description:
 * @author taogs
 * @date 2020/9/11 15:30
 */
@RestController
@RequestMapping("/data-config")
public class DataConfigController {
    private final static Logger LOGGER = LogManager.getLogger(DataConfigController.class);

    @Autowired
    private DataConfigApiService dataConfigApiService;
    @Autowired
    private AppletsConfigApiService appletsConfigApiService;
    @Autowired
    private ServiceConfigApiService serviceConfigApiService;
    @Autowired
    private ConfigApiService configApiService;
    @Autowired
    private MemberCardApiService memberCardApiService;
    @Autowired
    private EnterpriseInitApiService enterpriseInitApiService;
    @Autowired
    private EnterpriseApiService enterpriseApiService;
    @Autowired
    private SparkJobApiService sparkJobApiService;


    @RequestMapping("list-all-applet")
    public RestResponse listAllApplet() {
        ServiceResponse<List<AppletsConfigSimpleOutputDTO>> result = appletsConfigApiService
                .listAppletConfigAppIdContainDel(UserDetailUtils.getUserDetail().getEnterpriseId());
        if (result.isSuccess()) {
            List<AppletsConfigSimpleOutputDTO> list = result.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                return RestResponse.success(list.stream()
                        ///3：正常 5：异常
                        .filter(e -> e.getStatusFlag() != null && (e.getStatusFlag() == 3 || e.getStatusFlag() == 5))
                        .collect(Collectors.toList()));
            }
        }
        return ResultControllerUtils.commonResult(result);
    }

    @RequestMapping("list-all-service")
    public RestResponse listAllService() {
        ServiceResponse<List<ServiceConfigSimpleOutputDTO>> result = serviceConfigApiService
                .listServiceConfigAppIdContainDel(UserDetailUtils.getUserDetail().getEnterpriseId());
        if (result.isSuccess()) {
            List<ServiceConfigSimpleOutputDTO> list = result.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                return RestResponse.success(list.stream()
                        //3：正常 5：异常
                        .filter(e -> e.getStatusFlag() != null && (e.getStatusFlag() == 3 || e.getStatusFlag() == 5))
                        .collect(Collectors.toList()));
            }
        }
        return ResultControllerUtils.commonResult(result);
    }

    @RequestMapping("/list-all-member-card")
    public RestResponse listAllMemberCard() {
        ServiceResponse<List<OwnerMemberCardDTO>> result = memberCardApiService.getAll(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(result, MemberCardVO.class);
    }

    @IgnoreLogin
    @RequestMapping("/init-actually-data-config")
    public RestResponse initActuallyDataConfig(Integer enterpriseId) {
        if (enterpriseId == null) {
            ServiceResponse<List<EnterpriseDTO>> result = enterpriseApiService.listEnterpriseByName(null);
            if (result.isSuccess()) {
                List<EnterpriseDTO> list = result.getResult();
                if (CollectionUtils.isNotEmpty(list)) {
                    list.forEach(e -> dataConfigApiService.initActuallyPaidData(e.getEnterpriseId()));
                }
            }
        }
        return RestResponse.success();
    }

    @RequestMapping("config-wechat-member")
    public RestResponse configWechatMember(DataWechatMemberConfigDTO dto) {
        dto.setValidRefresh(2);
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        initMemberWechat(enterpriseId);
        dto.setEnterpriseId(enterpriseId);

        ServiceResponse<Void> response = dataConfigApiService
                .configWechatMember(dto);
        if (response.isSuccess()) {
            configWechatMember(enterpriseId);
            sparkJobApiService.enterMemberUpdateIndex(enterpriseId);
        }
        return ResultControllerUtils.commonResult(response);
    }

    private Set<String> getMemberAreaIdSet(Integer enterpriseId) {
        ServiceResponse<List<OwnerMemberCardDTO>> result = memberCardApiService.getAll(enterpriseId);
        if (result.isSuccess()) {
            List<OwnerMemberCardDTO> list = result.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                return list.stream().map(e -> e.getCardConfigId().toString()).collect(Collectors.toSet());
            }
        }
        return new HashSet<>(2);
    }

    private void initMemberWechat(Integer enterpriseId) {
        ServiceResponse<List<OwnerMemberCardDTO>> result = memberCardApiService.getAll(enterpriseId);
        if (result.isSuccess()) {
            List<OwnerMemberCardDTO> list = result.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                list.forEach(e -> {
                    dataConfigApiService.configWechatMember(new DataWechatMemberConfigDTO()
                            .setEnterpriseId(enterpriseId)
                            .setMemberCardArea(e.getCardConfigId().toString())
                            .setValidRefresh(1));
                });
            }
        }
    }

    private void configWechatMember(Integer enterpriseId) {
        ServiceResponse<List<DataWechatMemberConfigDTO>> all = dataConfigApiService
                .listWechatMemberConfig(UserDetailUtils.getUserDetail().getEnterpriseId());
        if (all.isSuccess()) {
            List<DataWechatMemberConfigDTO> allList = all.getResult();
            if (CollectionUtils.isNotEmpty(allList)) {
                if (allList.stream().allMatch(e -> (StringUtils.isNotBlank(e.getRelateService()) || StringUtils.isNotBlank(e.getRelateWechat())))) {
                    //所有的卡都配置，则初始化
                    enterpriseInitApiService.configWechatMember(enterpriseId);
                }
            }
        }
    }

    @RequestMapping("refresh-wechat-member-config")
    public RestResponse refreshWechatMemberConfig() {
        //
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        initMemberWechat(enterpriseId);

        ServiceResponse<List<DigitalConfigDTO>> response = configApiService.getDigitalConfigByEnterpriseId(enterpriseId);
        LOGGER.info("刷新数字会员配置:{}", JSON.toJSONString(response));
        if (response.isSuccess()) {
            List<DigitalConfigDTO> list = response.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                Set<String> memberCardIdSet = getMemberAreaIdSet(enterpriseId);

                list.forEach(e -> {
                    if (memberCardIdSet.contains(e.getCardConfigId().toString())) {

                        dataConfigApiService.configWechatMember(new DataWechatMemberConfigDTO()
                                .setEnterpriseId(enterpriseId)
                                .setMemberCardArea(e.getCardConfigId().toString())
                                .setRelateService(listToString(e.getServiceIds()))
                                .setRelateWechat(listToString(e.getAppletsIds()))
                                .setValidRefresh(1));
                    }
                });
                configWechatMember(enterpriseId);

            }
        }
        sparkJobApiService.enterMemberUpdateIndex(enterpriseId);
        return RestResponse.success();
    }

    private static String listToString(List<Long> ids) {
        StringBuilder str = new StringBuilder();
        if (CollectionUtils.isNotEmpty(ids)) {
            for (Long temp : ids) {
                str.append(temp.toString()).append(",");
            }
        }
        int length = str.toString().length();
        if (StringUtils.isNotBlank(str.toString())) {
            return str.substring(0, length - 1);
        }
        return str.toString();
    }

    @RequestMapping("list-wechat-member-config")
    public RestResponse listWechatMemberConfig() {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        initMemberWechat(enterpriseId);
        ServiceResponse<List<DataWechatMemberConfigDTO>> response = dataConfigApiService
                .listWechatMemberConfig(UserDetailUtils.getUserDetail().getEnterpriseId());

        Map<String, String> memberCardMap = new HashMap<>(2);
        ServiceResponse<List<OwnerMemberCardDTO>> result = memberCardApiService.getAll(UserDetailUtils.getUserDetail().getEnterpriseId());
        if (result.isSuccess()) {
            List<OwnerMemberCardDTO> memberCardList = result.getResult();
            if (CollectionUtils.isNotEmpty(memberCardList)) {
                memberCardMap = memberCardList.stream().collect(Collectors.toMap(e -> e.getCardConfigId().toString(), e -> e.getCardName()));
            }
        }
        if (response.isSuccess()) {
            List<DataWechatMemberConfigDTO> list = response.getResult();
            if (CollectionUtils.isNotEmpty(list)) {

                Set<String> memberCardIdSet = getMemberAreaIdSet(enterpriseId);
                for (DataWechatMemberConfigDTO temp : list) {
                    if (!memberCardIdSet.contains(temp.getMemberCardArea())) {
                        //如果不包含已存在的会员卡，则删除
                        dataConfigApiService.deleteWechatMember(enterpriseId, Long.valueOf(temp.getMemberCardArea()));
                    }
                    temp.setMemberCardAreaName(memberCardMap.get(temp.getMemberCardArea()));
                }
            }
        }
        return ResultControllerUtils.commonResult(response);
    }

    @RequestMapping("/config-member-consume")
    public RestResponse configMemberConsume(DataMemberConsumeConfigDTO dto) {
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return OperationResultUtils.operationResult(dataConfigApiService.configMemberConsume(dto),
                OperationResultUtils.LOG_EDIT + "数据统计配置-会员消费配置",
                UserDetailUtils.getUserDetail().getEnterpriseInfo().getEnterpriseName());
    }

    @RequestMapping("/get-member-consume-config")
    public RestResponse getMemberConsumeConfig() {
        return ResultControllerUtils.commonResultOne(dataConfigApiService
                .getMemberConsumeConfigByEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId()), MemberConsumeConfigVO.class);
    }

    @Deprecated
    @RequestMapping("/config-actually-paid")
    public RestResponse configActuallyPaid(String jsonArr) {
        if (StringUtil.isBlank(jsonArr)) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "实付配置内容为空");
        }
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        List<DataActuallyPaidConfigDTO> dtoList = JSON.parseArray(jsonArr, DataActuallyPaidConfigDTO.class);
        dtoList = dtoList.stream().map(e -> e.setEnterpriseId(enterpriseId)).collect(Collectors.toList());
        return OperationResultUtils.operationResult(dataConfigApiService.configActuallyPaid(dtoList),
                OperationResultUtils.LOG_EDIT + "数据统计配置-实付配置",
                UserDetailUtils.getUserDetail().getEnterpriseInfo().getEnterpriseName());
    }

    @Deprecated
    @RequestMapping("/config-actually-paid-new")
    public RestResponse configActuallyPaidNew(Integer memberBusiness, Integer performanceCount, Integer memberBusinessOfGoods) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        return OperationResultUtils.operationResult(dataConfigApiService.configActuallyPaid(enterpriseId, memberBusiness, performanceCount, memberBusinessOfGoods),
                OperationResultUtils.LOG_EDIT + "数据统计配置-实付配置",
                UserDetailUtils.getUserDetail().getEnterpriseInfo().getEnterpriseName());
    }

    @RequestMapping("/get-actually-paid-config")
    public RestResponse getActuallyPaidConfig() {
        return ResultControllerUtils.commonResult(
                dataConfigApiService.listActuallyPaidConfig(UserDetailUtils.getUserDetail().getEnterpriseId()),
                ActuallyPaidConfigVO.class);
    }

    @RequestMapping("/get-actually-paid-config-new")
    public RestResponse getActuallyPaidConfigNew() {
        return ResultControllerUtils.commonResult(
                dataConfigApiService.getDataActuallyPaidConfig(UserDetailUtils.getUserDetail().getEnterpriseId()));
    }

    @RequestMapping("/config-single-effect")
    public RestResponse configSingleEffect(String jsonArr) {
        if (StringUtil.isBlank(jsonArr)) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "单效配置内容为空");
        }
        List<DataSingleEffectConfigDTO> dtoList = JSON.parseArray(jsonArr, DataSingleEffectConfigDTO.class);
        dtoList = dtoList.stream().map(e -> e.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId())).collect(Collectors.toList());
        return OperationResultUtils.operationResult(dataConfigApiService.configSingleEffect(dtoList),
                OperationResultUtils.LOG_EDIT + "数据统计配置-单效取值配置",
                UserDetailUtils.getUserDetail().getEnterpriseInfo().getEnterpriseName());
    }

    @RequestMapping("/get-single-effect-config")
    public RestResponse getSingleEffectConfig() {
        return ResultControllerUtils.commonResult(
                dataConfigApiService.getSingleEffectConfig(UserDetailUtils.getUserDetail().getEnterpriseId()),
                DataSingleEffectConfigVO.class);
    }

    @RequestMapping("/config-performance")
    public RestResponse configPerformance(DataPerformanceConfigDTO dto) {
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return OperationResultUtils.operationResult(dataConfigApiService.configPerformance(dto),
                OperationResultUtils.LOG_EDIT + "数据统计配置-业绩指标配置",
                UserDetailUtils.getUserDetail().getEnterpriseInfo().getEnterpriseName());
    }

    @RequestMapping("/get-performance-config")
    public RestResponse getPerformanceConfig() {
        return ResultControllerUtils.commonResultOne(
                dataConfigApiService.getPerformanceConfig(UserDetailUtils.getUserDetail().getEnterpriseId()),
                PerformanceVO.class);
    }

    @RequestMapping("/list-all-member-card-new")
    public RestResponse listAllMemberCardNew() {
        ServiceResponse<Object> result = dataConfigApiService.listAllMemberCard(UserDetailUtils.getUserDetail().getEnterpriseId());
        if (result.isSuccess()) {
            List<ChannelAreaInfoDTO> list = (List<ChannelAreaInfoDTO>) result.getResult();
            Long minMemberCardId = list.stream().filter(e -> Objects.equals(e.getOwnerEntId(), e.getEnterpriseId())).mapToLong(e -> e.getAreaId()).min().getAsLong();
            List<MemberCardVO> voList = new ArrayList<>();
            for (ChannelAreaInfoDTO temp : list) {
                MemberCardVO vo = new MemberCardVO();
                vo.setCardConfigId(temp.getAreaId());
                vo.setCardName(temp.getName());
                vo.setEnterpriseId(temp.getEnterpriseId());
                vo.setOwnerEntId(temp.getOwnerEntId());
                vo.setMinMemberCardId(minMemberCardId);

                voList.add(vo);
            }

            return RestResponse.success(voList);
        }
        return RestResponse.failure(result.getCode(), result.getMessage());
    }

    @RequestMapping("/config-generation")
    public RestResponse configGeneration(@RequestBody List<DataGenerationConfigDTO> dtoList) {
        return OperationResultUtils.operationResult(dataConfigApiService.saveGenerationConfig(dtoList),
                OperationResultUtils.LOG_EDIT + "数据统计配置-年龄段配置:" + JSON.toJSONString(dtoList),
                UserDetailUtils.getUserDetail().getEnterpriseInfo().getEnterpriseName());
    }

    @RequestMapping("/get-generation-config")
    public RestResponse getGenerationConfig(Long memberCardId) {
        return ResultControllerUtils.commonResult(
                dataConfigApiService.listGenerationConfig(memberCardId),
                GenerationConfigVO.class);
    }
}
