package com.gic.enterprise.web.controller;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.annotation.HeaderSignIgnore;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.log.LogUtils;
import com.gic.enterprise.ano.IgnoreLogin;
import com.gic.enterprise.constant.DownloadReportDataTypeEnum;
import com.gic.enterprise.constant.ExcelExtensionEnum;
import com.gic.enterprise.constant.FieldFilterOptEnum;
import com.gic.enterprise.constant.FieldTypeEnum;
import com.gic.enterprise.dto.DownloadReportDTO;
import com.gic.enterprise.dto.HiveDataDownloadDTO;
import com.gic.enterprise.dto.HiveQueryCondition;
import com.gic.enterprise.qo.DownloadReportQO;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.service.DownloadReportApiService;
import com.gic.enterprise.service.HiveDataDownloadApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.enterprise.web.vo.DownloadReportVO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-10-29 20:28
 */
@RestController
public class DownloadReportController {

    @Autowired
    private DownloadReportApiService downloadReportApiService;
    @Autowired
    private HiveDataDownloadApiService hiveDataDownloadApiService;

    @RequestMapping("/list-download-report")
    public RestResponse listDownloadReport(@Validated({DownloadReportQO.SelectValidView.class}) DownloadReportQO downloadReportQO) {
        downloadReportQO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        if (StringUtils.isNotBlank(downloadReportQO.getStartTime())) {
            downloadReportQO.setStartTime(downloadReportQO.getStartTime() + " 00:00:00");
        }
        if (StringUtils.isNotBlank(downloadReportQO.getEndTime())) {
            downloadReportQO.setEndTime(downloadReportQO.getEndTime() + " 23:59:59");
        }
        ServiceResponse<Page<DownloadReportDTO>> serviceResponse = downloadReportApiService.listDownloadReport(downloadReportQO);
        return ResultControllerUtils.commonPageResult(serviceResponse, DownloadReportVO.class);
    }

    @RequestMapping("/get-download-report-url")
    public RestResponse saveDownloadLogReturnDownloadUrl(Integer downloadReportId) {
        ServiceResponse<String> serviceResponse = this.downloadReportApiService.saveDownloadLogReturnDownloadUrl(downloadReportId, UserDetailUtils.getUserDetail().getUserId());
        ServiceResponse<DownloadReportDTO> downloadReport = this.downloadReportApiService.getDownloadReport(downloadReportId);
        if (serviceResponse.isSuccess()) {
            LogUtils.createLog("下载报表", downloadReport.getResult().getFileName());
        }
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/apply-download-report")
    public RestResponse applyDownloadReport(Integer downloadReportId, String remark) {
        ServiceResponse<Void> serviceResponse = this.downloadReportApiService.applyDownloadReport(downloadReportId, remark);
        ServiceResponse<DownloadReportDTO> downloadReport = this.downloadReportApiService.getDownloadReport(downloadReportId);
        if (serviceResponse.isSuccess()) {
            LogUtils.createLog("下载报表", downloadReport.getResult().getFileName());
        }
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("receive-heart")
    @ResponseBody
    @IgnoreLogin
    @HeaderSignIgnore
    public RestResponse receiveHeart(Integer reportId){
        ServiceResponse<Void> response = downloadReportApiService.receiveHeart(reportId);
        return RestResponse.success();
    }

    @RequestMapping("download-hive-data")
    @ResponseBody
    @IgnoreLogin
    @HeaderSignIgnore
    public RestResponse downloadHiveData(){
        HiveDataDownloadDTO dataDownloadDTO = new HiveDataDownloadDTO();
        dataDownloadDTO.setEnterpriseId(1166);
        dataDownloadDTO.setApplyUserId(13805);
        dataDownloadDTO.setDataContent("用户-用户列表");
        dataDownloadDTO.setDataType(DownloadReportDataTypeEnum.DESENSITIZATION_DATA.getCode());
        dataDownloadDTO.setDataUrl("https://four.gicdev.com/member-tag/member-tag/tag-val-edit?id=264137629747888139&name=111111");
        //dataDownloadDTO.setDesenField();
        dataDownloadDTO.setExcelExtension(ExcelExtensionEnum.CSV.getCode());
        dataDownloadDTO.setFileName("boomhivetest");
        //dataDownloadDTO.setSql("SELECT order_id AS `订单号` FROM extract_order_information;");
        dataDownloadDTO.setTableName("extract_order_information");
        List<HiveQueryCondition> list = new ArrayList<>();
        HiveQueryCondition queryCondition = new HiveQueryCondition();
        queryCondition.setFieldName("order_id");
        queryCondition.setFieldMark("订单号");
        list.add(queryCondition);
        HiveQueryCondition queryCondition1 = new HiveQueryCondition();
        queryCondition1.setFieldName("enterprise_id");
        queryCondition1.setFieldMark("企业id");
        queryCondition1.setFieldFilterOptEnum(FieldFilterOptEnum.EQ);
        queryCondition1.setFieldTypeEnum(FieldTypeEnum.STRING);
        queryCondition1.setFilterValue(Arrays.asList("ff8080816a36326c016a53380d8b5f52"));
        list.add(queryCondition1);
        dataDownloadDTO.setQueryConditionList(list);
        ServiceResponse<Integer> downloadTask = hiveDataDownloadApiService.createDownloadTask(dataDownloadDTO);
        if(downloadTask.isSuccess()){
            return RestResponse.success();
        }
        return EnterpriseRestResponse.failure(downloadTask);
    }
}
