package com.gic.enterprise.web.controller;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

import com.gic.data.api.dto.index.*;
import com.gic.data.api.qo.index.*;
import com.gic.enterprise.web.vo.index.ChannelUserRateVO;
import com.gic.enterprise.web.vo.index.ChannelUserVO;
import com.gic.enterprise.web.vo.index.MemberCardInfoVO;
import com.gic.member.config.api.dto.*;
import com.gic.member.config.api.service.UserListFieldApiService;
import com.gic.platform.member.constant.enums.CuTypeEnum;
import com.gic.store.constant.StoreESFieldsEnum;
import com.gic.store.service.*;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.data.api.dto.*;
import com.gic.data.api.service.MemberDataApiService;
import com.gic.data.api.service.StoreStatisicsApiService;
import com.gic.enterprise.constant.Constants;
import com.gic.enterprise.dto.HelpCenterDTO;
import com.gic.enterprise.dto.IndexModuleDTO;
import com.gic.enterprise.dto.PushTypeDTO;
import com.gic.enterprise.dto.UserMessageDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.qo.PageQO;
import com.gic.enterprise.service.HelpCenterApiService;
import com.gic.enterprise.service.IndexModuleApiService;
import com.gic.enterprise.service.PushMessageApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.enterprise.web.vo.MemberDataRightVO;
import com.gic.enterprise.web.vo.MemberDataVO;
import com.gic.enterprise.web.vo.ResouceVO;
import com.gic.member.config.api.service.AppletsConfigApiService;
import com.gic.member.config.api.service.MemberCardApiService;
import com.gic.member.config.api.service.ServiceConfigApiService;
import com.gic.store.dto.CityDTO;
import com.gic.store.dto.ProvinceDTO;
import com.gic.store.dto.StoreSearchDTO;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-11-14 9:11
 */
@RestController
public class IndexModuleController {
    private static final Logger log = LogManager.getLogger(IndexModuleController.class);
    @Autowired
    private IndexModuleApiService indexModuleApiService;
    @Autowired
    private HelpCenterApiService helpCenterApiService;
    @Autowired
    private PushMessageApiService pushMessageApiService;
    @Autowired
    private StoreStatisicsApiService storeStatisicsApiService;
    @Autowired
    private ProvincesApiService provincesApiService;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;
    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private MemberDataApiService memberDataApiService;
    @Autowired
    private MemberCardApiService memberCardApiService;
    @Autowired
    private AppletsConfigApiService appletsConfigApiService;
    @Autowired
    private ServiceConfigApiService serviceConfigApiService;
    @Autowired
    private UserListFieldApiService userListFieldApiService;
    @Autowired
    private StoreResourceApiService storeResourceApiService;

    @RequestMapping("/list-index-module")
    public RestResponse listIndexModule() {
        ServiceResponse<List<IndexModuleDTO>> serviceResponse = indexModuleApiService.listIndexModuleEnterprise(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    /**
     * 首页会员卡列表数据，包括共享
     * @return
     */
    @RequestMapping("/list-member-card-of-index")
    public RestResponse listMemberCardOfIndex() {
        return RestResponse.success(getAreaData(UserDetailUtils.getUserDetail().getUserId(),
                UserDetailUtils.getUserDetail().getEnterpriseId(), CuTypeEnum.MCU));
    }

    @RequestMapping("list-help-center")
    public RestResponse listHelpCenter(){
        ServiceResponse<List<HelpCenterDTO>> response = this.helpCenterApiService.listHelpCenter();
        return RestResponse.success(response.getResult());
    }

    @RequestMapping("my-message")
    public RestResponse myMessage(String search, Integer status, String startTime, String endTime, Integer pushClassifyId, PageQO pageQO){
        Integer userId = UserDetailUtils.getUserDetail().getUserId();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        try {
            ServiceResponse<Page<UserMessageDTO>> response = this.pushMessageApiService.pageUserMessage(userId, search, status, StringUtils.isBlank(startTime) ? null : sdf.parse(startTime+" 00:00:00"),
                    StringUtils.isBlank(endTime) ? null : sdf.parse(endTime+" 23:59:59"), pushClassifyId, pageQO.getCurrentPage(), pageQO.getPageSize());
            ServiceResponse<Integer> unReadResponse = this.pushMessageApiService.totalUserMessage(userId, search, 1, StringUtils.isBlank(startTime) ? null : sdf.parse(startTime+" 00:00:00"),
                    StringUtils.isBlank(endTime) ? null : sdf.parse(endTime+" 23:59:59"), pushClassifyId);
            Map<String, Object> result = new HashMap<>();
            result.put("page", response.getResult());
            result.put("unReadCount", unReadResponse.getResult());
            return RestResponse.success(result);
        } catch (ParseException e) {
            log.warn("异常", e);
            return RestResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "日期转换错误");
        }
    }

    @RequestMapping("total-message")
    public RestResponse totalMessage(){
        ServiceResponse<Integer> response = this.pushMessageApiService.totalUserMessage(UserDetailUtils.getUserDetail().getUserId(), null, null, null,
                null, null);
        return RestResponse.success(response.getResult());
    }

    @RequestMapping("read-all")
    public RestResponse readAll(){
        ServiceResponse<Integer> response = this.pushMessageApiService.readAll(UserDetailUtils.getUserDetail().getUserId());
        return RestResponse.success(response.getResult());
    }

    @RequestMapping("message-detail")
    public RestResponse messageDetail(Integer messageId){
        ServiceResponse<UserMessageDTO> userMessageDetail = this.pushMessageApiService.getUserMessageDetail(messageId);
        this.pushMessageApiService.readMessage(messageId);
        return RestResponse.success(userMessageDetail.getResult());
    }

    @RequestMapping("message-classify")
    public RestResponse listMessageClassify(){
        ServiceResponse<Page<PushTypeDTO>> response = this.pushMessageApiService.pagePushType(null, 1, 10000);
        return RestResponse.success(response.getResult().getResult());
    }

    /**
     * 门店数据概览-省
     * @param regionId 会员卡域ID
     * @param brandId 品牌ID
     * @return
     */
    @RequestMapping("store-data")
    public RestResponse storeData(Long regionId, Integer brandId){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        Integer userId = UserDetailUtils.getUserDetail().getUserId();
        ServiceResponse<List<ProvinceDTO>> serviceResponse = this.provincesApiService.selectAllProvince();

        IndexStoreOverviewQO param = new IndexStoreOverviewQO();
        param.setEntId(enterpriseId);
        param.setAuthStoreIdList(getAuthStore());
        param.setMemberCardIdList(getAuthAreaList(regionId, enterpriseId, userId, CuTypeEnum.MCU));
        //开卡门店
        param.setMemberCardCountType(0);
        ServiceResponse<List<IndexStoreAreaDTO>> dataOpenStore = this.storeStatisicsApiService.storeProvinceData(param);
        //归属品牌门店
        param.setMemberCardCountType(1);
        ServiceResponse<List<IndexStoreAreaDTO>> belongBrandResponse = this.storeStatisicsApiService.storeProvinceData(param);
        //品牌服务门店
        param.setMemberCardCountType(2);
        param.setBrandId(brandId);
        ServiceResponse<List<IndexStoreAreaDTO>> brandResponse = this.storeStatisicsApiService.storeProvinceData(param);

        List<Map<String, Object>> result = new ArrayList<>();
        if(serviceResponse.isSuccess() && serviceResponse.getResult() != null){
            for(ProvinceDTO provinceDTO : serviceResponse.getResult()){
                Map<String, Object> map = new HashMap<>();
                map.put("key", provinceDTO.getProvinceName());
                map.put("mainStoreMemberCount", getProvince(belongBrandResponse, provinceDTO));
                map.put("openStoreMemberCount", getProvince(dataOpenStore, provinceDTO));
                map.put("belongBrandMemberCount", getProvince(belongBrandResponse, provinceDTO));
                map.put("brandMemberCount", getProvince(brandResponse, provinceDTO));
                //查询门店数量
                map.put("storeCount", this.getStoreNum(enterpriseId, null, provinceDTO.getProvinceId()));
                map.put("onlineStoreCount", this.getStoreNum(enterpriseId, 1, provinceDTO.getProvinceId()));
                result.add(map);
            }
        }
        Map<String, Object> data = new HashMap<>();
        data.put("data", result);
        data.put("legend", this.getLegend(enterpriseId, null));
        return RestResponse.success(data);
    }

    private Integer getProvince(ServiceResponse<List<IndexStoreAreaDTO>> response, Object obj) {
        if (response.isSuccess() && CollectionUtils.isNotEmpty(response.getResult())) {
            List<IndexStoreAreaDTO> list = response.getResult();
            for (IndexStoreAreaDTO temp : list) {
                if (obj instanceof ProvinceDTO) {
                    if (((ProvinceDTO) obj).getProvinceName().equals(temp.getAreaName())) {
                        return temp.getMemberCount();
                    }
                } else if (obj instanceof CityDTO) {
                    if (((CityDTO) obj).getCityName().equals(temp.getAreaName())) {
                        return temp.getMemberCount();
                    }
                }
            }
        }
        return 0;
    }

    /**
     * 门店数据概览-城市
     * @param province
     * @param regionId
     * @param brandId
     * @return
     */
    @RequestMapping("store-city-data")
    public RestResponse storeCityData(String province, Long regionId, Integer brandId){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        Integer userId = UserDetailUtils.getUserDetail().getUserId();

        ServiceResponse<List<CityDTO>> serviceResponse = this.provincesApiService.selectAllCity();
        ServiceResponse<List<ProvinceDTO>> provinceResponse = this.provincesApiService.selectAllProvince();
        String provinceId = null;
        for(ProvinceDTO provinceDTO : provinceResponse.getResult()){
            if(provinceDTO.getProvinceName().equals(province)){
                provinceId = provinceDTO.getProvinceId();
            }
        }
        if(StringUtils.isBlank(provinceId)){
            return RestResponse.success();
        }
        IndexStoreOverviewQO param = new IndexStoreOverviewQO();
        param.setEntId(enterpriseId);
        param.setAuthStoreIdList(getAuthStore());
        param.setMemberCardIdList(getAuthAreaList(regionId, enterpriseId, userId, CuTypeEnum.MCU));
        //开卡门店
        param.setMemberCardCountType(0);
        ServiceResponse<List<IndexStoreAreaDTO>> dataOpenStore = this.storeStatisicsApiService.storeCityData(param);
        //归属品牌门店
        param.setMemberCardCountType(1);
        ServiceResponse<List<IndexStoreAreaDTO>> belongBrandResponse = this.storeStatisicsApiService.storeCityData(param);
        //品牌服务门店
        param.setMemberCardCountType(2);
        param.setBrandId(brandId);
        ServiceResponse<List<IndexStoreAreaDTO>> brandResponse = this.storeStatisicsApiService.storeCityData(param);

        List<Map<String, Object>> result = new ArrayList<>();
        if(serviceResponse.isSuccess() && serviceResponse.getResult() != null){
            for(CityDTO cityDTO : serviceResponse.getResult()){
                if(cityDTO.getProvinceId().equals(provinceId)){
                    Map<String, Object> map = new HashMap<>();
                    map.put("key", cityDTO.getCityName());
                    map.put("mainStoreMemberCount", getProvince(belongBrandResponse, cityDTO));
                    map.put("openStoreMemberCount", getProvince(dataOpenStore, cityDTO));
                    map.put("belongBrandMemberCount", getProvince(belongBrandResponse, cityDTO));
                    map.put("brandMemberCount", getProvince(brandResponse, cityDTO));
                    //查询门店数量
                    map.put("storeCount", this.getStoreNum(enterpriseId, null, provinceId, cityDTO.getCityId()));
                    map.put("onlineStoreCount", this.getStoreNum(enterpriseId, 1, provinceId, cityDTO.getCityId()));
                    result.add(map);
                }
            }
        }
        Map<String, Object> data = new HashMap<>();
        data.put("data", result);
        data.put("legend", this.getLegend(enterpriseId, provinceId));
        return RestResponse.success(data);
    }

    /**
     * 首页-会员概览-左侧趋势
     * @param type
     * @param cardAreaId
     * @param brandId
     * @return
     */
    @RequestMapping("list-member-data")
    public RestResponse listMember(Integer type, Long cardAreaId, Integer brandId){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        Integer userId = UserDetailUtils.getUserDetail().getUserId();
        IndexMemberOverviewQO param = new IndexMemberOverviewQO();
        param.setEntId(enterpriseId);
        param.setAuthStoreIdList(this.getAuthStore());
        param.setType(type);
        param.setBrandId(brandId);
        param.setMemberCardIdList(getAuthAreaList(cardAreaId, enterpriseId, userId, CuTypeEnum.MCU));

        ServiceResponse<List<IndexMemberOverviewDTO>> response = this.memberDataApiService.listMemberData(param);
        return RestResponse.success(EntityUtil.changeEntityListByJSON(MemberDataVO.class, response.getResult()));
    }

    /**
     * 首页-会员概览-右侧
     * @param cardAreaId
     * @param brandId
     * @return
     */
    @RequestMapping("get-member-data")
    public RestResponse getMemberData(Long cardAreaId, Integer brandId){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        Integer userId = UserDetailUtils.getUserDetail().getUserId();
        IndexMemberOverviewQO param = new IndexMemberOverviewQO();
        param.setEntId(enterpriseId);
        param.setAuthStoreIdList(this.getAuthStore());
        param.setBrandId(brandId);
        param.setMemberCardIdList(getAuthAreaList(cardAreaId, enterpriseId, userId, CuTypeEnum.MCU));
        ServiceResponse<IndexMemberOverviewTotalDTO> memberData = this.memberDataApiService.getMemberData(param);

        IndexMemberOverviewTotalDTO data = memberData.getResult();
        MemberDataRightVO vo = new MemberDataRightVO();
        vo.setNewDayMemberCount(data.getNewMemberOfDay());
        vo.setNewDayMemberRate(data.getNewMemberRateOfDay());
        vo.setNewMonthMemberCount(data.getNewMemberOfMonth());
        vo.setNewMonthMemberRate(data.getNewMemberRateOfMonth());

        vo.setOpenCardTotal(data.getOpenCardTotal());
        vo.setBrandServiceTotal(data.getBrandServiceTotal());
        vo.setBelongBrandServiceTotal(data.getBelongBrandServiceTotal());
        //开卡门店招募总数
        vo.setOpenStoreMemberAllCount(data.getOpenCardTotal());
        return RestResponse.success(vo);
    }

    /**
     * 首页-渠道用户概览
     * @param type
     * @param cardConfigId
     * @param appletConfigId
     * @param serviceConfigId
     * @return
     */
    @RequestMapping("list-member-channel-data")
    public RestResponse listMemberChannelData(Integer type, Long cardConfigId, Long appletConfigId, Long serviceConfigId){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        Integer userId = UserDetailUtils.getUserDetail().getUserId();

        ChannelUserOverviewQO param = new ChannelUserOverviewQO();
        param.setEntId(enterpriseId);
        param.setAppletIdList(getAuthAreaList(appletConfigId, enterpriseId, userId, CuTypeEnum.ACU));
        param.setMemberCardIdList(getAuthAreaList(cardConfigId, enterpriseId, userId, CuTypeEnum.MCU));
        param.setServiceIdList(getAuthAreaList(serviceConfigId, enterpriseId, userId, CuTypeEnum.SCU));
        ServiceResponse<List<IndexChannelUserOverviewDTO>> response = this.memberDataApiService.listMemberChannelData(param);
        if (response.isSuccess()) {
            List<ChannelUserVO> result = response.getResult().stream().map(e ->
                new ChannelUserVO().setDate(e.getDateData())
                        .setAcuCount(e.getAppletUserTotal())
                        .setEcuCount(e.getEnterpriseWeChatUserTotal())
                        .setScuCount(e.getServiceUserTotal())
                        .setMcuCount(e.getMemberCardUserTotal())).collect(Collectors.toList());
            return RestResponse.success(result);
        }
        return RestResponse.failure(response.getCode(), response.getMessage());
    }



    @RequestMapping("get-member-channel-data")
    public RestResponse getMemberChannelData(Integer type){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<AdsGicMbrVennAllDTO> memberChannelData = this.memberDataApiService.getMemberChannelData(enterpriseId);
        return RestResponse.success(memberChannelData.getResult());
    }

    /**
     * 用户数量占比
     * @param type 会员卡统计类型 0：开卡门店 1：归属品牌服务门店 2：品牌服务门店
     * @param cuType 1小程序用户  2服务号用户  3会员卡用户
     * @return
     */
    @RequestMapping("list-channel-data")
    public RestResponse listChannelData(Integer type, Integer cuType){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        Integer userId = UserDetailUtils.getUserDetail().getUserId();

        IndexChannelUserRateQO param = new IndexChannelUserRateQO();
        if (cuType == null) {
            cuType = 3;
        }
        CuTypeEnum cuTypeEnum = CuTypeEnum.getEnum(cuType);

        List<MemberCardInfoVO> areaDataList;
        switch (cuTypeEnum) {
            case ACU:
                areaDataList = getAreaData(userId, enterpriseId, CuTypeEnum.ACU);
                param.setAppletIdList(areaDataList.stream().mapToLong(e -> Long.valueOf(e.getMbrAreaId())).boxed().collect(Collectors.toList()));
            case MCU:
                areaDataList = getAreaData(userId, enterpriseId, CuTypeEnum.MCU);
                param.setMemberCardIdList(areaDataList.stream().mapToLong(e -> Long.valueOf(e.getMbrAreaId())).boxed().collect(Collectors.toList()));
            case SCU:
                areaDataList = getAreaData(userId, enterpriseId, CuTypeEnum.SCU);
                param.setServiceIdList(areaDataList.stream().mapToLong(e -> Long.valueOf(e.getMbrAreaId())).boxed().collect(Collectors.toList()));
            default:
                areaDataList = getAreaData(userId, enterpriseId, CuTypeEnum.MCU);
                param.setMemberCardIdList(areaDataList.stream().mapToLong(e -> Long.valueOf(e.getMbrAreaId())).boxed().collect(Collectors.toList()));
        }

        param.setMemberCardCountType(type);
        param.setCuType(cuType);
        param.setEntId(enterpriseId);
        ServiceResponse<List<IndexChannelUserRateDTO>> listServiceResponse = this.memberDataApiService.listChannelData(param);

        Map<Long, Integer> map = new HashMap<>();
        if (listServiceResponse.isSuccess()) {
            List<IndexChannelUserRateDTO> list = listServiceResponse.getResult();
            map = list.stream().collect(Collectors.toMap(IndexChannelUserRateDTO::getAreaId, IndexChannelUserRateDTO::getMemberCount));

        }
        Map<Long, Integer> finalMap = map;
        Integer finalCuType = cuType;
        return RestResponse.success(areaDataList.stream().map(e -> {
            ChannelUserRateVO vo = new ChannelUserRateVO();
            vo.setAreaId(Long.parseLong(e.getMbrAreaId()));
            vo.setAreaName(e.getMbrAreaName());
            vo.setOwnType(e.getOwnType());
            vo.setCuType(finalCuType);
            vo.setMemberCount(finalMap.getOrDefault(Long.parseLong(e.getMbrAreaId()), 0));
            return vo;
        }).collect(Collectors.toList()));
    }


    /**
     * 首页-消费分析
     * @param consumptionType
     * @param cardConfigId
     * @param brandId
     * @return
     */
    @RequestMapping("consumption-analysis")
    public RestResponse consumptionAnalysis(Integer consumptionType, Long cardConfigId, Integer brandId){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        Integer userId = UserDetailUtils.getUserDetail().getUserId();

        IndexConsumedAnalysisQO param = new IndexConsumedAnalysisQO();
        param.setEntId(enterpriseId);
        param.setMemberCardIdList(getAuthAreaList(cardConfigId, enterpriseId, userId, CuTypeEnum.MCU));
        param.setBrandId(brandId);
        param.setAuthStoreIdList(getAuthStore());
        return ResultControllerUtils.commonResult(this.memberDataApiService.consumptionAnalysis(param));
    }

    /**
     * 首页-会员卡/小程序/服务号
     * @return
     */
    @RequestMapping("list-resource")
    public RestResponse listResource(){
        List<MemberCardInfoVO> memberCardList = getAreaData(UserDetailUtils.getUserDetail().getUserId(),
                UserDetailUtils.getUserDetail().getEnterpriseId(), CuTypeEnum.MCU);
        List<MemberCardInfoVO> appletList = getAreaData(UserDetailUtils.getUserDetail().getUserId(),
                UserDetailUtils.getUserDetail().getEnterpriseId(), CuTypeEnum.ACU);
        List<MemberCardInfoVO> serviceList = getAreaData(UserDetailUtils.getUserDetail().getUserId(),
                UserDetailUtils.getUserDetail().getEnterpriseId(), CuTypeEnum.SCU);

        Map<String, Object> result = new HashMap<>();
        List<ResouceVO> applet = getResource(appletList);
        List<ResouceVO> service = getResource(serviceList);
        List<ResouceVO> card = getResource(memberCardList);

        result.put("card", card);
        result.put("service", service);
        result.put("applet", applet);
        result.put("isSingleArea", isSingleArea());
        
        //
        List<MemberCardInfoVO> memberCardListAll = getAreaData(null,
                UserDetailUtils.getUserDetail().getEnterpriseId(), CuTypeEnum.MCU);
        List<MemberCardInfoVO> appletListAll = getAreaData(null,
                UserDetailUtils.getUserDetail().getEnterpriseId(), CuTypeEnum.ACU);
        List<MemberCardInfoVO> serviceListAll = getAreaData(null,
                UserDetailUtils.getUserDetail().getEnterpriseId(), CuTypeEnum.SCU);

        result.put("memberCardAll", memberCardListAll.size() == memberCardList.size() ? 1 : 0);
        result.put("appletAll", appletListAll.size() == appletList.size() ? 1 : 0);
        result.put("serviceAll", serviceListAll.size() == serviceList.size() ? 1 : 0);
        return RestResponse.success(result);
    }

    private boolean isSingleArea() {
        List<MemberCardInfoVO> memberCardList = getAreaData(null,
                UserDetailUtils.getUserDetail().getEnterpriseId(), CuTypeEnum.MCU);
        List<MemberCardInfoVO> appletList = getAreaData(null,
                UserDetailUtils.getUserDetail().getEnterpriseId(), CuTypeEnum.ACU);
        List<MemberCardInfoVO> serviceList = getAreaData(null,
                UserDetailUtils.getUserDetail().getEnterpriseId(), CuTypeEnum.SCU);
        return memberCardList.size()<=1 && serviceList.size()<=1 && appletList.size()<=1;
    }

    /**
     * 首页-品牌列表
     * @return
     */
    @RequestMapping("list-brand-of-index")
    public RestResponse listBrand() {
        return ResultControllerUtils.commonResult(storeResourceApiService
                .getAuthDataFromStoreWidget(UserDetailUtils.getUserDetail().getUserId(), StoreESFieldsEnum.STOREBRANDIDLIST.getField()));
    }

    private List<ResouceVO> getResource(List<MemberCardInfoVO> list) {
        List<ResouceVO> result = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(list)) {
            for(MemberCardInfoVO dto : list){
                ResouceVO vo = new ResouceVO();
                vo.setId(Long.valueOf(dto.getMbrAreaId()));
                vo.setName(dto.getMbrAreaName());
                vo.setOwnType(dto.getOwnType());
                result.add(vo);
            }
        }
        return result;
    }

    private List<Integer> getLegend(Integer enterpriseId, String provinceId){
        Integer storeNum = this.getStoreNum(enterpriseId, null, provinceId);
        List<Integer> legend = new ArrayList<>();
        Integer max = 15;
        if(storeNum <=max){
            int index = 1;
            while (storeNum>0){
                legend.add(index*1);
                storeNum -= index*1;
                index++;
            }
        }else {
            int sig = storeNum%15 == 0 ? storeNum/15 : storeNum/15+1;
            for(int i = 1; i<= Constants.FIVE; i++){
                legend.add(i*sig);
            }
        }
        return legend;
    }

    private Integer getStoreNum(Integer enterpriseId, Integer status, String provinceId){
        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
        storeSearchDTO.setEnterpriseId(enterpriseId);
        storeSearchDTO.setStatus(status);
        storeSearchDTO.setProvinceId(provinceId);
        if(UserDetailUtils.getUserDetail().getUserInfo().getSuperAdmin() != 1){
            Long storeResource = UserDetailUtils.getUserDetail().getStoreResourceId();
            if(storeResource != null){
                storeSearchDTO.setStoreResource(storeResource);
            }
        }
        ServiceResponse<Long> countFromES = this.storeApiService.queryStoreCountFromEs(storeSearchDTO);
        return countFromES.getResult().intValue();
    }

    private Integer getStoreNum(Integer enterpriseId, Integer status, String provinceId, String cityId){
        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
        storeSearchDTO.setEnterpriseId(enterpriseId);
        storeSearchDTO.setStatus(status);
        storeSearchDTO.setProvinceId(provinceId);
        storeSearchDTO.setCityId(cityId);
        if(UserDetailUtils.getUserDetail().getUserInfo().getSuperAdmin() != 1){
            Long storeResource = UserDetailUtils.getUserDetail().getStoreResourceId();
            if(storeResource != null){
                storeSearchDTO.setStoreResource(storeResource);
            }
        }
        ServiceResponse<Long> countFromES = this.storeApiService.queryStoreCountFromEs(storeSearchDTO);
        return countFromES.getResult().intValue();
    }

    private List<Integer> getAuthStore(){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        List<Integer> authStoreIdList = new ArrayList<>();
        if(UserDetailUtils.getUserDetail().getUserInfo().getSuperAdmin() != 1){
            if (UserDetailUtils.getUserDetail().getStoreResourceInfo().getStoreResourceId() == null) {
                authStoreIdList.add(-1);
            } else {
                ServiceResponse<List<Integer>> authResponse = this.storeWidgetApiService.listStoreInfoIdByStoreWidgetId(enterpriseId, UserDetailUtils.getUserDetail().getStoreResourceInfo().getStoreResourceId());
                if(authResponse.isSuccess()){
                    authStoreIdList = authResponse.getResult();
                }
                if(CollectionUtils.isEmpty(authStoreIdList)){
                    authStoreIdList.add(-1);
                }
            }
        }
        return authStoreIdList;
    }

    /**
     * 查询管理员用户域权限数据
     * @param userId
     * @param enterpriseId
     * @param cuTypeEnum CuTypeEnum枚举
     * @return
     */
    private List<MemberCardInfoVO> getAreaData(Integer userId, Integer enterpriseId, CuTypeEnum cuTypeEnum){
        AreaSearchDTO areaSearchDTO = new AreaSearchDTO();
        areaSearchDTO.setCuTypes(Arrays.asList(cuTypeEnum));
        areaSearchDTO.setEnterpriseId(enterpriseId);
        areaSearchDTO.setUserId(userId);
        List<ChannelAreaInfoDTO> listCard = userListFieldApiService.queryCustomAreaInfo(areaSearchDTO).getResult();
        if(CollectionUtils.isNotEmpty(listCard)){
            return listCard.stream().map(dto -> {
                MemberCardInfoVO vo = new MemberCardInfoVO();
                vo.setMbrAreaId(dto.getAreaId().toString());
                vo.setMbrAreaName(dto.getName());
                vo.setOwnType(dto.getOwnerFlag());
                return vo;
            }).collect(Collectors.toList());
        }
        return new ArrayList<>();
    }

    /**
     * 获取用户域数据
     * @param userId
     * @param enterpriseId
     * @param cuTypeEnum
     * @return
     */
    private List<Long> getAreaIdList(Integer userId, Integer enterpriseId, CuTypeEnum cuTypeEnum) {
        AreaSearchDTO areaSearchDTO = new AreaSearchDTO();
        areaSearchDTO.setCuTypes(Arrays.asList(cuTypeEnum));
        areaSearchDTO.setEnterpriseId(enterpriseId);
        areaSearchDTO.setUserId(userId);
        List<ChannelAreaInfoDTO> listCard = userListFieldApiService.queryCustomAreaInfo(areaSearchDTO).getResult();
        if(CollectionUtils.isNotEmpty(listCard)){
            return listCard.stream().map(e -> e.getAreaId()).collect(Collectors.toList());
        }
        return new ArrayList<>();
    }


    private List<Long> getAuthAreaList(Long cardAreaId, Integer enterpriseId, Integer userId, CuTypeEnum cuTypeEnum) {
        if (cardAreaId != null) {
            return Arrays.asList(cardAreaId);
        } else {
            List<Long> areaIdList = getAreaIdList(userId, enterpriseId, cuTypeEnum);
            if (CollectionUtils.isNotEmpty(areaIdList)) {
                return areaIdList;
            }
        }
        return null;
    }
}