package com.gic.enterprise.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.constant.ProductMarketTypeEnum;
import com.gic.enterprise.dto.ProductMarketDTO;
import com.gic.enterprise.qo.ProductMarketQO;
import com.gic.enterprise.service.BlackListApiService;
import com.gic.enterprise.service.ProductMarketApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 达摩产品情报站
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/5/18 2:35 PM 
 */
@RestController
@RequestMapping("/product-market")
public class ProductMarketController {
    @Autowired
    private ProductMarketApiService productMarketApiService;
    @Autowired
    private BlackListApiService blackListApiService;


    @RequestMapping("query-product-market")
    public RestResponse queryProductMarket(Integer showCount) {
        Map<String, List<ProductMarketDTO>> result = new HashMap<>(8);
        //功能更新
        ProductMarketQO params = new ProductMarketQO();
        if (showCount == null) {
            showCount = 5;
        }
        params.setShowGic(1);
        params.setPageNum(1);
        params.setPageSize(showCount);
        params.setInfoType(ProductMarketTypeEnum.FUNCTION_UPDATE.getCode());
        ServiceResponse<Page<ProductMarketDTO>> functionUpdate = productMarketApiService.pageProductMarket(params);
        if (isNotEmptyList(functionUpdate)) {
            result.put("functionUpdate", functionUpdate.getResult().getResult());
        } else {
            result.put("functionUpdate", Collections.EMPTY_LIST);
        }
        //
        params.setInfoType(ProductMarketTypeEnum.PRODUCT_PROMOTION.getCode());
        ServiceResponse<Page<ProductMarketDTO>> productPromotion = productMarketApiService.pageProductMarket(params);
        if (isNotEmptyList(productPromotion)) {
            result.put("productPromotion", productPromotion.getResult().getResult());
        } else {
            result.put("productPromotion", Collections.EMPTY_LIST);
        }
        //
        params.setInfoType(ProductMarketTypeEnum.SUCCESS_STORIES.getCode());
        ServiceResponse<Page<ProductMarketDTO>> successStories = productMarketApiService.pageProductMarket(params);
        if (isNotEmptyList(successStories)) {
            result.put("successStories", successStories.getResult().getResult());
        } else {
            result.put("successStories", Collections.EMPTY_LIST);
        }
        return RestResponse.success(result);
    }

    /**
     * 首页前端是否需要弹窗
     * @return
     */
    @RequestMapping("is-need-to-popup")
    public RestResponse isNeedToPopup() {
        return ResultControllerUtils.commonResult(productMarketApiService
                .isNeedToPopup(UserDetailUtils.getUserDetail().getEnterpriseId(),
                        UserDetailUtils.getUserDetail().getUserId()));
    }

    /**
     * 是否在黑名单
     * @return
     */
    @RequestMapping("is-in-black-list")
    public RestResponse isInBlackList() {
        List<Integer> list = blackListApiService.getBlankListEnterprise().getResult();
        return RestResponse.success(list.contains(UserDetailUtils.getUserDetail().getEnterpriseId()));
    }

    private boolean isNotEmptyList(ServiceResponse<Page<ProductMarketDTO>> response) {
        return response.isSuccess() && response.getResult() != null && CollectionUtils.isNotEmpty(response.getResult().getResult());
    }
}
