package com.gic.enterprise.web.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.ctrip.framework.apollo.Config;
import com.ctrip.framework.apollo.ConfigService;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PropertiesVal;
import com.gic.commons.util.ToolUtil;
import com.gic.commons.util.ZipFileUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.ExecutorPoolSingleton;
import com.gic.enterprise.ano.IgnoreLogin;
import com.gic.enterprise.constant.applet.AppletPageTypeEnum;
import com.gic.enterprise.context.RequestContext;
import com.gic.enterprise.dto.*;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.qo.PageQO;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.service.DownloadReportApiService;
import com.gic.enterprise.service.QrCodeApiService;
import com.gic.enterprise.utils.UserDetail;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.enterprise.web.qo.QrcodeContent;
import com.gic.enterprise.web.qo.QrcodeQO;
import com.gic.enterprise.web.utils.QrcodeUtils;
import com.gic.enterprise.web.vo.QrcodeScenesClassifyVO;
import com.gic.thirdparty.BucketNameEnum;
import com.gic.thirdparty.FileOperateUtils;
import com.gic.thirdparty.pic.QqCloudPicUtils;
import com.gic.thirdparty.pic.UploadResult;
import com.gic.wechat.business.api.service.fwh.WeixinBaseFunService;
import com.gic.wechat.business.api.service.wxa.WeixinWxaFunService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.io.*;
import java.util.*;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;
/**
 *
 * @ClassName: QrcodeController
 * @Description:
 * @author taogs
 * @date 2020/9/11 15:34
 */
@RestController
public class QrcodeController {
    private static Logger logger = LogManager.getLogger(QrcodeController.class);
    @Autowired
    private QrCodeApiService qrCodeApiService;
    @Autowired
    private WeixinBaseFunService weixinBaseFunService;
    @Autowired
    private WeixinWxaFunService weixinWxaFunService;
    @Autowired
    private DownloadReportApiService downloadReportApiService;


    @RequestMapping("list-scenes-classify")
    public RestResponse listScenesClassify(){
        UserDetail userDetail = UserDetailUtils.getUserDetail();
        ServiceResponse<List<QrcodeScenesClassifyDTO>> list = this.qrCodeApiService.
                listQrCodeScenesClassify(userDetail.getEnterpriseId());
        return RestResponse.success(EntityUtil.changeEntityListByJSON(QrcodeScenesClassifyVO.class, list.getResult()));
    }

    @RequestMapping("save-scenes-classify")
    public RestResponse saveScenesClassify(String name, Integer id){
        if(id != null){
            ServiceResponse<QrcodeScenesClassifyDTO> classify = this.qrCodeApiService.getQrcodeScenesClassify(id);
            QrcodeScenesClassifyDTO result = classify.getResult();
            if(result != null){
                if(result.getEnterpriseId() == -1){
                    return RestResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "系统分类无法编辑");
                }
            }
        }
        QrcodeScenesClassifyDTO dto = new QrcodeScenesClassifyDTO();
        dto.setClassifyName(name);
        dto.setId(id);
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        ServiceResponse<Integer> response = this.qrCodeApiService.saveQrCodeScenesClassify(dto);
        if(response.isSuccess()){
            return RestResponse.success();
        }
        return EnterpriseRestResponse.failure(response);
    }

    @RequestMapping("delete-classify")
    public RestResponse deleteClassify(Integer classifyId){
        ServiceResponse<QrcodeScenesClassifyDTO> classify = this.qrCodeApiService.getQrcodeScenesClassify(classifyId);
        QrcodeScenesClassifyDTO result = classify.getResult();
        if(result != null){
            if(result.getEnterpriseId() == -1){
                return RestResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "系统分类无法编辑");
            }
        }
        ServiceResponse<Integer> response = this.qrCodeApiService.deleteQrcodeScenesClassify(classifyId);
        return RestResponse.success();
    }

    @RequestMapping("page-qrcode-scenes")
    public RestResponse pageQrcodeScenes(String search, Integer classifyId, PageQO pageQO){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<Page<QrcodeScenesDTO>> response = this.qrCodeApiService.pageQrCodeScenes
                (enterpriseId, search, classifyId, pageQO.getCurrentPage(), pageQO.getPageSize());
        return RestResponse.success(response.getResult());
    }

    @RequestMapping("qrcode-scenes-detail")
    public RestResponse qrCodeScenesDetail(Integer scenesId){
        ServiceResponse<QrcodeScenesDTO> response = this.qrCodeApiService.getQrcodeScenesById(scenesId);
        QrcodeScenesDTO result = response.getResult();
        return RestResponse.success(result);
    }

    @RequestMapping("save-qrcode-scenes")
    public RestResponse saveQrcodeScenes(String scenesName, Integer classifyId, String desc, Integer status, Integer scenesId){
        QrcodeScenesDTO dto = new QrcodeScenesDTO();
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        dto.setScenesClassifyId(classifyId);
        dto.setScenesDesc(desc);
        dto.setScenesId(scenesId);
        dto.setStatus(status);
        dto.setScenesName(scenesName);
        ServiceResponse<Integer> response = this.qrCodeApiService.saveQrCodeScenes(dto);
        if(response.isSuccess()){
            return RestResponse.success();
        }
        return EnterpriseRestResponse.failure(response);
    }

    @RequestMapping("delete-qrcode-scenes")
    public RestResponse deleteQrcodeScenes(Integer scenesId){
        this.qrCodeApiService.deleteScenes(scenesId);
        return RestResponse.success();
    }

    @RequestMapping("update-qrcode-scenes-status")
    public RestResponse updateStatus(Integer scenesId, Integer type){
        if(type == 1){
            this.qrCodeApiService.openScenesStatus(scenesId);
        }else {
            this.qrCodeApiService.closeScenesStatus(scenesId);
        }
        return RestResponse.success();
    }

    @RequestMapping("page-qrcode-scenes-value")
    public RestResponse pageQrcodeScensValue(Integer scenesId, String search, PageQO pageQO){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<Page<QrcodeScenesValueDTO>> response = this.qrCodeApiService.pageQrCodeScenesValue
                (search, scenesId, enterpriseId, pageQO.getCurrentPage(), pageQO.getPageSize());
        return RestResponse.success(response.getResult());
    }

    @RequestMapping("save-qrcode-scenes-value")
    public RestResponse saveQrcodeScenesValue(String code, String name, Integer scenesId, Integer valueId){
        QrcodeScenesValueDTO dto = new QrcodeScenesValueDTO();
        dto.setCode(code);
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        dto.setName(name);
        dto.setScenesId(scenesId);
        dto.setValueId(valueId);
        ServiceResponse<Integer> response = this.qrCodeApiService.saveScenesValue(dto);
        if(response.isSuccess()){
            return RestResponse.success();
        }
        return EnterpriseRestResponse.failure(response);
    }

    @RequestMapping("delete-qrcode-scenes-value")
    public RestResponse deleteQrcodeScenesValue(Integer valueId){
        ServiceResponse<Integer> response = this.qrCodeApiService.deleteQrcodeScenesValue(valueId);
        if(response.isSuccess() && response.getResult()>0){
            return RestResponse.success();
        }
        return RestResponse.failure(ErrorCode.NOTEXISTS.getCode(), ErrorCode.NOTEXISTS.getErrorMsg());
    }

    @RequestMapping("scenes-tree")
    public RestResponse scenesTree(){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<List<QrcodeScenesClassifyDTO>> response = this.qrCodeApiService.listQrCodeScenesClassify(enterpriseId);
        ServiceResponse<Page<QrcodeScenesDTO>> scenes = this.qrCodeApiService.pageQrCodeScenes(enterpriseId, null, null, 1, Integer.MAX_VALUE);
        ServiceResponse<Page<QrcodeScenesValueDTO>> valueResponse = this.qrCodeApiService.pageQrCodeScenesValue(null, null, enterpriseId, 1, Integer.MAX_VALUE);
        JSONArray array = new JSONArray();
        if(response.isSuccess()){
            List<QrcodeScenesClassifyDTO> list = response.getResult();
            if(CollectionUtils.isNotEmpty(list)){
                for(QrcodeScenesClassifyDTO dto : list){
                    JSONObject json = new JSONObject();
                    json.put("id", dto.getId());
                    json.put("name", dto.getClassifyName());
                    if(dto.getDeleteFlag() != null && dto.getDeleteFlag() == 0){
                        json.put("children", this.getScenesData(dto, scenes, valueResponse));
                    }else {
                        json.put("children", new JSONArray());
                    }
                    array.add(json);
                }
            }
        }
        return RestResponse.success(array);
    }

    @RequestMapping("take-qrcode")
    public RestResponse takeQrcode(@RequestBody QrcodeQO qrcodeQO) throws Exception{
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        String filePath = RequestContext.getContext().getRequest().getSession().getServletContext().getRealPath("/qrcode");
        String fileId = ToolUtil.randomUUID();
        if(CollectionUtils.isNotEmpty(qrcodeQO.getContent())){
            DownloadReportDTO reportDTO = new DownloadReportDTO();
            reportDTO.setEnterpriseId(enterpriseId);
            reportDTO.setApplyUserId(UserDetailUtils.getUserDetail().getUserId());
            reportDTO.setReportStartTime(null);
            reportDTO.setReportEndTime(null);
            reportDTO.setFileName(qrcodeQO.getFileName());
            reportDTO.setDataContent("二维码");
            reportDTO.setDataUrl("-");
            reportDTO.setDataType(3);
            reportDTO.setDataCount(qrcodeQO.getContent().size());
            ServiceResponse<Integer> response = downloadReportApiService.saveDownloadReport(reportDTO);
            filePath += File.separator + enterpriseId + File.separator + ToolUtil.randomUUID();
            File path = new File(filePath);
            if(!path.exists()){
                path.mkdirs();
            }
            this.createQrcode(qrcodeQO, enterpriseId, filePath, fileId, response.getResult(), UserDetailUtils.getUserDetail(),
                    qrcodeQO.getFileName());
        }
        return RestResponse.success();
    }

    /**
     * 装修页面预览二维码
     * @param appkey
     * @param pageId
     * @param pageType 页面类型
     * @return
     */
    @RequestMapping("take-applet-qrcode")
    public RestResponse takeAppletQrcode(String appkey, Integer pageId, Integer pageType){
        String path = "pages/customView/customView";
        //客服服务页面
        if (Objects.equals(AppletPageTypeEnum.CUSTOMER_SERVICE.getCode(), pageType)) {
            path = "pages/customer-service/customer-service";
        }
        byte[] appletQrcode = this.createAppletQrcode(appkey, path, pageId+"");
        UploadResult uploadResult = QqCloudPicUtils.uploadLocalPicStream("pic01", "", appletQrcode);
        Map<String, Object> pic = new HashMap<>();
        pic.put("fileId", uploadResult.getFileId());
        pic.put("url", uploadResult.getUrl());
        pic.put("downloadUrl", uploadResult.getDownloadUrl());
        pic.put("path", path+"?pageId="+pageId);
        return RestResponse.success(pic);
    }

    @RequestMapping("take-gyt-qrcode")
    public RestResponse takeGytQrcode(){
        Config config = ConfigService.getConfig("COMMON.4.0-gic-properties");
        logger.info(config.getPropertyNames());
        return RestResponse.success(config.getProperty("gyt-qrcode", ""));
    }

    @RequestMapping("take-store-qrcode")
    @IgnoreLogin
    public RestResponse takeStoreQrcode(String appkey, Integer id, Integer qrcodeType){
        String path = "pages/index/index?qrcodeParam="+id+"&qrcodeType="+qrcodeType;
        byte[] appletQrcode = this.createAppletQrcode(appkey, path, id+"");
        UploadResult uploadResult = QqCloudPicUtils.uploadLocalPicStream("pic01", "", appletQrcode);
        Map<String, Object> pic = new HashMap<>();
        pic.put("fileId", uploadResult.getFileId());
        pic.put("url", uploadResult.getUrl());
        pic.put("downloadUrl", uploadResult.getDownloadUrl());
        pic.put("path", path);
        return RestResponse.success(pic);
    }

    private void createQrcode(QrcodeQO qrcodeQO, Integer enterpriseId, final String filePath, String fileId,
                              Integer downLoadId, UserDetail userDetail, String fileName) throws Exception{
        ExecutorPoolSingleton.getInstance().executeTask(new Runnable() {
            @Override
            public void run() {
                try {
                    final String path = filePath + File.separator + fileName+".zip";
                    File file = new File(path);
                    if (!file.exists()) {
                        file.getParentFile().mkdirs();
                        file.createNewFile();
                    }
                    ZipOutputStream zipOutputStream = null;
                    try {
                        zipOutputStream = new ZipOutputStream(new FileOutputStream(file));
                    } catch (FileNotFoundException e) {
                        logger.warn("e", e);
                    }
                    for (QrcodeContent content : qrcodeQO.getContent()) {
                        QrcodeDTO dto = EntityUtil.changeEntityByJSON(QrcodeDTO.class, qrcodeQO);
                        dto.setEnterpriseId(enterpriseId);
                        dto.setCustomParams(content.getCustomParams());
                        dto.setContent(content.getContent());
                        dto.setFileId(fileId);
                        ServiceResponse<Integer> response = qrCodeApiService.saveQrcodeDownload(dto);
                        if (response.isSuccess()) {
                            Integer qrcodeId = response.getResult();
                            if (qrcodeQO.getH5() == 1) {
                                byte[] h5Qrcode = createH5Qrcode(content.getContent(), content.getTitle(), userDetail);
                                ZipEntry e = new ZipEntry(content.getTitle() + File.separator + content.getTitle() + ".jpg");
                                zipOutputStream.putNextEntry(e);
                                zipOutputStream.write(h5Qrcode);
                                zipOutputStream.flush();
                            }
                            if (StringUtils.isNotBlank(dto.getFwhIds())) {
                                String[] fwhArr = dto.getFwhIds().split(",");
                                for (String s : fwhArr) {
                                    byte[] fwhQrcode = createFwhQrcode(qrcodeId, dto.getType(), s, dto.getEndTime(), content.getTitle(), userDetail);
                                    if(fwhQrcode != null){
                                        ZipEntry e = new ZipEntry(content.getTitle() + File.separator + s + File.separator + content.getTitle() + ".jpg");
                                        zipOutputStream.putNextEntry(e);
                                        zipOutputStream.write(fwhQrcode);
                                        zipOutputStream.flush();
                                    }
                                }
                            }
                            if (StringUtils.isNotBlank(dto.getAppletIds())) {
                                String appletPath = content.getContent();
                                if(StringUtils.isBlank(appletPath)){
                                    appletPath = "pages/index/index";
                                }
                                appletPath += "?qrcodeParam="+qrcodeId;
                                String[] appletArr = dto.getAppletIds().split(",");
                                for (String s : appletArr) {
                                    byte[] appletQrcode = createAppletQrcode(s, appletPath, "");
                                    if(appletQrcode != null){
                                        ZipEntry e = new ZipEntry(content.getTitle() + File.separator + s + File.separator + content.getTitle() + ".jpg");
                                        zipOutputStream.putNextEntry(e);
                                        zipOutputStream.write(appletQrcode);
                                        zipOutputStream.flush();
                                    }
                                }
                            }
                        }
                    }
                    try {
                        zipOutputStream.flush();
                        zipOutputStream.closeEntry();
                        zipOutputStream.close();
                    } catch (IOException e) {
                        logger.warn("e", e);
                    }
                    String url = FileOperateUtils.simpleUploadFileFromLocal(file, fileName+".zip", BucketNameEnum.COMPRESS_60000.getName());

                    downloadReportApiService.updateDownloadUrl(downLoadId, "https://"+url);
                } catch (IOException e1) {
                    logger.warn("e", e1);
                }
            }
        });

    }

    private byte[] createFwhQrcode(Integer qrCodeId, int type, String appkey, Date endTime, String title, UserDetail userDetail){
        ServiceResponse<String> qRcodeUrl = null;
        if(type == 1){
            int expireSecond = Integer.valueOf((endTime.getTime() - System.currentTimeMillis())/1000+"");
            qRcodeUrl = this.weixinBaseFunService.getTemporaryQRcodeURL(appkey, qrCodeId + "", expireSecond);
        }else {
            qRcodeUrl = this.weixinBaseFunService.getQRcodeURL(appkey, qrCodeId + "");
        }
        if(qRcodeUrl.isSuccess()){
            return QrcodeUtils.createQrcode(qRcodeUrl.getResult(), userDetail.getEnterpriseInfo().getLogo(), title);
        }
        return null;
    }

    private byte[] createAppletQrcode(String appkey, String path, String scene){
        ServiceResponse<Map<String, Object>> wxaQrcode = this.weixinWxaFunService.getWxaCodeUnlimit(appkey, path, scene);
        if(wxaQrcode.isSuccess()){
            Map<String, Object> result = wxaQrcode.getResult();
            return (byte[]) result.get("content");
        }
        return null;
    }
    private byte[] createH5Qrcode(String content, String title, UserDetail userDetail){
        return QrcodeUtils.createQrcode(content, userDetail.getEnterpriseInfo().getLogo(), title);
    }

    private JSONArray getScenesData(QrcodeScenesClassifyDTO dto, ServiceResponse<Page<QrcodeScenesDTO>> scenes, ServiceResponse<Page<QrcodeScenesValueDTO>> valueResponse){
        JSONArray scenesArr = new JSONArray();
        if(scenes.isSuccess() && scenes.getResult() != null){
            List<QrcodeScenesDTO> result = scenes.getResult().getResult();
            if(CollectionUtils.isNotEmpty(result)){
                for(QrcodeScenesDTO scenesDTO : result){
                    if(dto.getId().intValue() == scenesDTO.getScenesClassifyId().intValue()){
                        JSONObject scenesJson = new JSONObject();
                        scenesJson.put("id", scenesDTO.getScenesId());
                        scenesJson.put("name", scenesDTO.getScenesName());
                        scenesJson.put("children", this.getScenesValueData(scenesDTO, valueResponse));
                        scenesArr.add(scenesJson);
                    }
                }
            }
        }
        return scenesArr;
    }

    private JSONArray getScenesValueData(QrcodeScenesDTO dto1, ServiceResponse<Page<QrcodeScenesValueDTO>> valueResponse){
        JSONArray valueArr = new JSONArray();
        if(valueResponse.isSuccess() && valueResponse.getResult() != null){
            List<QrcodeScenesValueDTO> valueList = valueResponse.getResult().getResult();
            if(CollectionUtils.isNotEmpty(valueList)){
                for(QrcodeScenesValueDTO valueDTO : valueList){
                    if(valueDTO.getScenesId().intValue() == dto1.getScenesId().intValue()){
                        JSONObject valueJson = new JSONObject();
                        valueJson.put("id", valueDTO.getValueId());
                        valueJson.put("name", valueDTO.getName());
                        valueJson.put("children", new JSONArray());
                        valueArr.add(valueJson);
                    }
                }
            }
        }
        return valueArr;
    }
}
