package com.gic.enterprise.web.controller;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.AuditorDTO;
import com.gic.auth.service.AuditorApiService;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.log.LogUtils;
import com.gic.enterprise.constant.Constants;
import com.gic.enterprise.dto.DictDTO;
import com.gic.enterprise.service.DictApiService;
import com.gic.enterprise.service.RiskModeApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetail;
import com.gic.enterprise.utils.UserDetailUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-10-29 14:18
 */
@RestController
public class RiskModeController {

    @Autowired
    private RiskModeApiService riskModeApiService;
    @Autowired
    private DictApiService dictApiService;
    @Autowired
    private AuditorApiService auditorApiService;

    @RequestMapping("/risk-mode-remaining-time")
    public RestResponse getRiskModeRemainingTime() {
        UserDetail userDetail = UserDetailUtils.getUserDetail();
        ServiceResponse<Long> serviceResponse = riskModeApiService.getRiskModeRemainingTime(userDetail.getEnterpriseId(), userDetail.getUserId());
        if (serviceResponse.isSuccess()) {
            return RestResponse.success(serviceResponse.getResult() != null ? serviceResponse.getResult().intValue() : null);
        }
        return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
    }

    @RequestMapping("/check-risk-mode")
    public RestResponse checkRiskMode() {
        UserDetail userDetail = UserDetailUtils.getUserDetail();
        ServiceResponse<Void> serviceResponse = riskModeApiService.checkRiskMode(userDetail.getEnterpriseId(), userDetail.getUserId());
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/risk-mode-list-duration")
    public RestResponse riskModeListDuration() {
        ServiceResponse<List<DictDTO>> serviceResponse = dictApiService.listRiskModeDuration();
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/apply-risk-mode")
    public RestResponse applyRiskMode(Integer duration, String applyReason) {
        UserDetail userDetail = UserDetailUtils.getUserDetail();
        ServiceResponse<Void> serviceResponse = riskModeApiService.applyRiskMode(userDetail.getEnterpriseId(), userDetail.getUserId(), duration, applyReason);
        if (serviceResponse.isSuccess()) {
            ServiceResponse<DictDTO> serviceDict = dictApiService.getDict(Constants.RISK_MODE_DURATION_CATEGORY, duration.toString());
            if (serviceDict.isSuccess() && serviceDict.getResult() != null) {
                LogUtils.createLog("申请" + serviceDict.getResult().getKey(), "风险模式");
            }
        }
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/list-all-auditor")
    public RestResponse listAllAuditor(String projectItemCode) {
        UserDetail userDetail = UserDetailUtils.getUserDetail();
        ServiceResponse<List<AuditorDTO>> serviceResponse = this.auditorApiService.listAuditorByProject(projectItemCode, userDetail.getUserId(), userDetail.getEnterpriseId());
        return ResultControllerUtils.commonResult(serviceResponse);
    }
}
