package com.gic.enterprise.web.controller;

import java.util.*;

import com.alibaba.fastjson.JSON;
import com.gic.download.utils.log.LogUtils;
import com.gic.enterprise.constant.Constants;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.EnterpriseInitApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.wechat.business.api.service.fwh.WeixinMenuFunService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.OperationResultUtils;
import com.gic.enterprise.constant.applet.WeChatMenuTypeEnum;
import com.gic.enterprise.dto.AppletWechatMenuDTO;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.service.AppletWechatMenuApiService;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.enterprise.web.vo.WechatMenuVO;
/**
 *
 * @ClassName: WechatMenuController
 * @Description:
 * @author taogs
 * @date 2020/9/11 16:29
 */
@RestController
@RequestMapping("/wechat-menu")
public class WechatMenuController {
    private static final Logger LOGGER = LogManager.getLogger(WechatMenuController.class);

    @Autowired
    private AppletWechatMenuApiService appletWechatMenuApiService;
    @Autowired
    private WeixinMenuFunService weixinMenuFunService;
    @Autowired
    private EnterpriseInitApiService enterpriseInitApiService;

    @Deprecated
    @RequestMapping("/save")
    public RestResponse saveMenu(AppletWechatMenuDTO dto) throws Exception {
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return OperationResultUtils.operationResult(appletWechatMenuApiService.save(dto),
                OperationResultUtils.LOG_NEW + "自定义菜单", dto.getMenuName());
    }

    @RequestMapping("/save-all")
    public RestResponse saveAllMenu(String jsonArr) throws Exception {
        LOGGER.info("公众号菜单保存参数:{}", jsonArr);
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        if (StringUtils.isBlank(jsonArr)) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数不能为空");
        }
        List<AppletWechatMenuDTO> list = JSON.parseArray(jsonArr, AppletWechatMenuDTO.class);

        List<AppletWechatMenuDTO> listOfHasValid = new ArrayList<>(list.size());

        //主菜单不能超过3个
        if (list.size() > Constants.THREE) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主菜单不能超过3个");
        }

        //菜单名称重复
        Set<String> menuNameSet = new HashSet<>();
        int saveItemCount = list.size();

        //校验
        for (AppletWechatMenuDTO first : list) {
            first.setEnterpriseId(enterpriseId);
            first.setFirstLevel(true);

            menuNameSet.add(first.getMenuName());

            RestResponse response = validParams(first);
            if (!"0000".equals(response.getCode())) {
                return RestResponse.failure(response.getCode(), response.getMessage());
            }
            //校验填充后可以保存的实体类
            AppletWechatMenuDTO hasValidOfFirst = (AppletWechatMenuDTO) response.getResult();

            List<AppletWechatMenuDTO> secondList = first.getChild();
            if (CollectionUtils.isNotEmpty(secondList)) {
                saveItemCount += secondList.size();

                List<AppletWechatMenuDTO> listSecondOfHasValid = new ArrayList<>(secondList.size());

                //子菜单不超过5个
                if (secondList.size() > 5) {
                    return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "子菜单不能超过5个");
                }
                for (AppletWechatMenuDTO second : secondList) {
                    second.setFirstLevel(false);
                    second.setEnterpriseId(enterpriseId);

                    menuNameSet.add(second.getMenuName());

                    RestResponse responseOfSecond = validParams(second);
                    if (!"0000".equals(responseOfSecond.getCode())) {
                        return RestResponse.failure(responseOfSecond.getCode(), responseOfSecond.getMessage());
                    }
                    //校验填充后可以保存的实体类
                    listSecondOfHasValid.add((AppletWechatMenuDTO) responseOfSecond.getResult());
                }
                hasValidOfFirst.setChild(listSecondOfHasValid);
            }
            listOfHasValid.add(hasValidOfFirst);
        }
        if (saveItemCount != menuNameSet.size()) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "菜单名称重复");
        }


        //校验通过，批量新增
        //先删除
        String appKey = listOfHasValid.get(0).getAppKey();
        appletWechatMenuApiService.deleteWechatMenu(enterpriseId, appKey);
        for (int i = 0, len = listOfHasValid.size(); i < len; i ++) {
            AppletWechatMenuDTO temp = listOfHasValid.get(i);
            temp.setSort(i + 1);
            ServiceResponse<Integer> response = appletWechatMenuApiService.saveOfHasValid(temp);
            Integer parentId = response.getResult();
            List<AppletWechatMenuDTO> tempSecond = temp.getChild();
            if (CollectionUtils.isNotEmpty(tempSecond)) {
                for (int j = 0, length = tempSecond.size(); j < length; j ++) {
                    AppletWechatMenuDTO tempOfSecond = tempSecond.get(j);
                    tempOfSecond.setParentMenuId(parentId);
                    tempOfSecond.setSort(j + 1);
                    appletWechatMenuApiService.saveOfHasValid(tempOfSecond);
                }
            }
        }
        LogUtils.createLog(OperationResultUtils.LOG_NEW + "自定义菜单", appKey);
        return RestResponse.success();
    }



    @Deprecated
    @RequestMapping("/edit")
    public RestResponse edit(AppletWechatMenuDTO dto) throws Exception {
        return OperationResultUtils.operationResult(appletWechatMenuApiService.edit(dto),
                OperationResultUtils.LOG_EDIT + "自定义菜单", dto.getMenuName());
    }

    @Deprecated
    @RequestMapping("/delete")
    public RestResponse delete(Integer menuId) throws Exception {
        ServiceResponse<String> result = appletWechatMenuApiService.deleteByMenuId(menuId);
        return OperationResultUtils.operationResult(result, OperationResultUtils.LOG_DELETE + "自定义菜单",
                OperationResultUtils.getOperationObject(result));
    }

    @Deprecated
    @RequestMapping("/sort")
    public RestResponse sort(Integer menuId, Integer sort) throws Exception {
        ServiceResponse<String> result = appletWechatMenuApiService.sort(menuId, sort);
        return OperationResultUtils.operationResult(result, OperationResultUtils.LOG_SORT + "自定义菜单",
                OperationResultUtils.getOperationObject(result));
    }

    @RequestMapping("/get-detail")
    public RestResponse sort(Integer menuId) throws Exception {
        return ResultControllerUtils.commonResult(appletWechatMenuApiService.getDetail(menuId));
    }

    @RequestMapping("/list")
    public RestResponse list(String appKey) {
        ServiceResponse<List<AppletWechatMenuDTO>> result = appletWechatMenuApiService
                .listAll(UserDetailUtils.getUserDetail().getEnterpriseId(), appKey);
        if (result.isSuccess()) {
            List<WechatMenuVO> list = EntityUtil.changeEntityListNew(WechatMenuVO.class, result.getResult());
            return RestResponse.success(listToTree(0, list));
        }

        return EnterpriseRestResponse.failure(result);
    }

    @RequestMapping("/synchronize-menu")
    public RestResponse synchronizeMenu(String appKey) {
        ServiceResponse<String> result = appletWechatMenuApiService
                .getWeChatMenuJson(UserDetailUtils.getUserDetail().getEnterpriseId(), appKey);
        if (result.isSuccess()) {
            String json = result.getResult();
            LOGGER.info("自定义菜单组装json:{}", json);
            //delete
            //create
            weixinMenuFunService.deleteMenu(appKey);
            ServiceResponse<String> response = weixinMenuFunService.createMenu(appKey, json);
            LOGGER.info("自定义菜单同步创建结果:{}", JSON.toJSONString(response));

            if (!response.isSuccess()) {
                return RestResponse.failure(response.getCode(), response.getMessage());
            }
            //初始化回调
            enterpriseInitApiService.perfectMenuConfig(UserDetailUtils.getUserDetail().getEnterpriseId());
            //日志
            LogUtils.createLog("自定义菜单同步", appKey);
        }
        return EnterpriseRestResponse.failure(result);
    }

    @RequestMapping("/synchronize-menu-test")
    public RestResponse synchronizeMenu() {
        weixinMenuFunService.deleteMenu("wxc6fd9beaf9a9dbe9");
        ServiceResponse<String> response = weixinMenuFunService.createMenu("wxc6fd9beaf9a9dbe9", "{\"button\":[{\"appid\":\"\",\"name\":\"会员中心\",\"pagepath\":\"test\",\"type\":\"miniprogram\",\"url\":\"www.baidu.com\"}]}");
        return RestResponse.success(response);
    }

    @RequestMapping("/list-menu-type")
    public RestResponse listMenuType() {
        List<Map<String, Object>> list = new ArrayList<>();
        Map<String, Object> result;
        for (WeChatMenuTypeEnum type : WeChatMenuTypeEnum.values()) {
            result = new HashMap<>(4);
            result.put("key", type.getCode());
            result.put("value", type.getMessage());
            list.add(result);
        }
        return RestResponse.success(list);
    }

    private static List<WechatMenuVO> listToTree(int parentMenuId, List<WechatMenuVO> list) {
        List<WechatMenuVO> result = null;
        if (CollectionUtils.isNotEmpty(list)) {
            for (WechatMenuVO vo : list) {
                if (vo.getParentMenuId().intValue() == parentMenuId) {
                    vo.setChild(listToTree(vo.getMenuId(), list));
                    if (result == null) {
                        result = new ArrayList<>();
                    }
                    result.add(vo);
                }
            }
        }
        return result;
    }

    private RestResponse validParams(AppletWechatMenuDTO dto) throws Exception{
        //参数校验
        if (dto.getEnterpriseId() == null) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID不能为空");
        }
        //是否是主菜单
        boolean isMainMenu = dto.isFirstLevel();
        String menuName = dto.getMenuName();
        boolean isNotRightMenuName;
        //菜单标题，不超过16个字节，子菜单不超过60个字节
        if (isMainMenu) {
            isNotRightMenuName = StringUtils.isBlank(menuName) || menuName.getBytes("UTF-8").length > 16;
        } else {
            isNotRightMenuName = StringUtils.isBlank(menuName) || menuName.getBytes("UTF-8").length > 60;
        }
        if (isNotRightMenuName) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "菜单名称为空或者过长");
        }
        if (!WeChatMenuTypeEnum.isRightCode(dto.getMenuType())) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "菜单内容错误");
        }
        if (WeChatMenuTypeEnum.isNeedMenuCode(dto.getMenuType()) && StringUtils.isBlank(dto.getMenuCode())) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "链接配置不能为空");
        }
        if (WeChatMenuTypeEnum.OUT_LINK.getCode() == dto.getMenuType().intValue()) {
            if (StringUtils.isBlank(dto.getMenuUrl())) {
                return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "外链网址不能为空");
            }
        }
        if (WeChatMenuTypeEnum.APPLET_LINK.getCode() == dto.getMenuType().intValue()
                && StringUtils.isBlank(dto.getMenuUrlPara())) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "小程序appId不能为空");
        }
        //todo edit
        //暂时只能外链和小程序
        if (dto.getMenuType().intValue() != WeChatMenuTypeEnum.OUT_LINK.getCode()
                && dto.getMenuType().intValue() != WeChatMenuTypeEnum.APPLET_LINK.getCode()) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "一期只能选择"
                    + WeChatMenuTypeEnum.OUT_LINK.getMessage() + "和" + WeChatMenuTypeEnum.APPLET_LINK.getMessage());
        }
        if (StringUtils.isBlank(dto.getAppKey())) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "公众号key不能为空");
        }
        //填充字段值
        if (isMainMenu) {
            //主菜单父级ID统一0
            dto.setParentMenuId(0);
        }
        return RestResponse.success(dto);
    }
}
