package com.gic.enterprise.web.controller.union;

import java.util.*;
import java.util.stream.Collectors;

import com.gic.enterprise.dto.WmStoreDTO;
import com.gic.enterprise.dto.union.*;
import com.gic.enterprise.qo.union.UnionEnterpriseAuthResLogQO;
import com.gic.enterprise.service.WmStoreApiService;
import com.gic.marketing.api.dto.sms.EnterpriseSmsSignDTO;
import com.gic.marketing.api.service.sms.SmsApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.*;
import com.gic.auth.service.*;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.OperationResultUtils;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.exception.CommonException;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.enterprise.service.UnionEnterpriseAuthApiService;
import com.gic.enterprise.utils.CreateRandomUtils;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetail;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.enterprise.web.utils.AuthCodeExpireTimeUtils;
import com.gic.enterprise.web.utils.union.UserAeaUtils;
import com.gic.enterprise.web.vo.union.*;
import com.gic.marketing.process.api.service.sms.SmsSendApiService;
import com.gic.member.config.api.dto.AppletsConfigSimpleOutputDTO;
import com.gic.member.config.api.dto.OwnerMemberCardDTO;
import com.gic.member.config.api.dto.ServiceConfigSimpleOutputDTO;
import com.gic.member.config.api.service.AppletsConfigApiService;
import com.gic.member.config.api.service.MemberCardApiService;
import com.gic.member.config.api.service.ServiceConfigApiService;
import com.gic.open.api.dto.ApplicationDTO;
import com.gic.open.api.service.ApplicationApiService;
import com.gic.platform.member.constant.enums.CuTypeEnum;

@RestController
@RequestMapping("union-enterprise")
public class UnionEnterpriseAuthController {
    private static Logger logger = LogManager.getLogger(UnionEnterpriseAuthController.class);

    @Autowired
    private EnterpriseApiService enterpriseApiService;
    @Autowired
    private SmsSendApiService smsSendApiService;
    @Autowired
    private AuthCodeApiService authCodeApiService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private ResourceGroupApiService resourceGroupApiService;
    @Autowired
    private AuditedGroupApiService auditedGroupApiService;
    @Autowired
    private AccountGroupApiService accountGroupApiService;
    @Autowired
    private ApplicationApiService applicationApiService;
    @Autowired
    private UnionEnterpriseAuthApiService unionEnterpriseAuthApiService;
    @Autowired
    private MemberCardApiService memberCardApiService;
    @Autowired
    private AppletsConfigApiService appletsConfigApiService;
    @Autowired
    private ServiceConfigApiService serviceConfigApiService;
    @Autowired
    private WmStoreApiService wmStoreApiService;
    @Autowired
    private SmsApiService smsApiService;

    private static final String UNION_ENTERPRISE_AUTH_SMS_CODE = "GICSHHT001";

    /**
     * 商户名称查询商户是否存在
     * @param enterpriseName
     * @return
     */
    @RequestMapping("check-enterprise-by-name")
    public RestResponse checkEnterpriseName(String enterpriseName, Boolean isAdd) {
        EnterpriseDTO enterpriseDTO = checkEnterpriseByEnterpriseName(enterpriseName);
        if (enterpriseDTO == null) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "联合商户不存在或者是自有商户");
        }
        if (isAdd != null && isAdd) {
            UnionEnterpriseAuthDTO old = getUnionEnterpriseAuth(UserDetailUtils.getUserDetail().getEnterpriseId(), enterpriseDTO.getEnterpriseId());
            if (old != null) {
                return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户已经联合");
            }
        }
        return RestResponse.success(EntityUtil.changeEntityNew(EnterpriseInfoVO.class, enterpriseDTO));
    }

    @RequestMapping("send-auth-code-to-union-enterprise")
    public RestResponse sendAuthCode(String enterpriseName) {
        EnterpriseDTO unionEnterprise = checkEnterpriseByEnterpriseName(enterpriseName);
        if (unionEnterprise == null) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "联合商户不存在或者是自有商户");
        }
        ServiceResponse<UserDTO> unionUserRes = userApiService.getUserByEnterpriseId(unionEnterprise.getEnterpriseId());
        UserDTO unionUser = unionUserRes.getResult();

        UserDetail userDetail = UserDetailUtils.getUserDetail();

        //验证码生成
        AuthCodeDTO authCodeDTO = new AuthCodeDTO();
        authCodeDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        //生成6位数字验证码
        String authCode = CreateRandomUtils.generateNumberStr(6);
        authCodeDTO.setAuthCode(authCode);
        authCodeDTO.setExpirationTime(AuthCodeExpireTimeUtils.setAuthCodeExpireTime(2));
        ServiceResponse<Integer> result = authCodeApiService.saveAuth(authCodeDTO);
        if (result.isSuccess()) {
            Integer authCodeId = result.getResult();
            //发送手机验证码
            String[] smsArr = new String[] { unionEnterprise.getEnterpriseName(),
                    userDetail.getEnterpriseInfo().getEnterpriseName(),
                    authCode,
                    userDetail.getEnterpriseInfo().getEnterpriseName()};
            ServiceResponse<Void> smsSendResult = smsSendApiService.sendPlatformSms(UNION_ENTERPRISE_AUTH_SMS_CODE,
                    unionEnterprise.getEnterpriseId(),
                    unionUser.getPhoneAreaCode(),
                    unionUser.getPhoneNumber(),
                    smsArr);
            if (!smsSendResult.isSuccess()) {
                return EnterpriseRestResponse.failure(smsSendResult);
            } else {
                return RestResponse.success(authCodeId);
            }
        }
        return EnterpriseRestResponse.failure(result);
    }

    /**
     * 发送验证码校验联合商户，如果已存在则进入资源授权页面；如果不存在，则新建联合商户
     * @param authCode
     * @param authCodeId
     * @param enterpriseName
     * @param isAdd 是否是新增入口
     * @return
     */
    @RequestMapping("create-union-enterprise")
    public RestResponse createUnionEnterprise(String authCode, Integer authCodeId, String enterpriseName) {
        ServiceResponse authCodeResult = authCodeApiService.validateAuthCode(authCodeId, authCode);
        if (!authCodeResult.isSuccess()) {
            return RestResponse.failure(authCodeResult.getCode(), authCodeResult.getMessage());
        }
        EnterpriseDTO unionEnterprise = checkEnterpriseByEnterpriseName(enterpriseName);
        if (unionEnterprise == null) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "联合商户不存在或者是自有商户");
        }

        Integer ownEnterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        Integer unionEnterpriseId = unionEnterprise.getEnterpriseId();
        //查询是否已存在联合商户
        UnionEnterpriseAuthDTO unionEnterpriseAuthDTO = getUnionEnterpriseAuth(ownEnterpriseId, unionEnterpriseId);
        if (unionEnterpriseAuthDTO != null) {
            //已存在，则不需要记录操作日志
            return RestResponse.success();
        } else {
            return OperationResultUtils.operationResult(
                    unionEnterpriseAuthApiService.saveUnionEnterprise(ownEnterpriseId, unionEnterpriseId),
                    OperationResultUtils.LOG_NEW + "联合商户",
                    unionEnterprise.getEnterpriseName());
        }
    }

    /**
     * 资源组
     * @param enterpriseId 联合商户ID
     * @return
     */
    @RequestMapping("list-resource-group")
    public RestResponse listResourceGroup(Integer enterpriseId) {
        ServiceResponse<List<ResourceGroupDTO>> response = resourceGroupApiService.listResourceGroup(null,
                UserDetailUtils.getUserDetail().getEnterpriseId());
        if (response.isSuccess()) {
            List<ResourceGroupDTO> list = response.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                ServiceResponse<List<UnionEnterpriseAuthDTO>> unionEnterpriseResponse = unionEnterpriseAuthApiService
                        .listUnionEnterpriseAuth(UserDetailUtils.getUserDetail().getEnterpriseId());
                Set<String> authResourceGroupSet = new HashSet<>();

                if (unionEnterpriseResponse.isSuccess()) {
                    List<UnionEnterpriseAuthDTO> unionEnterprise = unionEnterpriseResponse.getResult();
                    if (CollectionUtils.isNotEmpty(unionEnterprise)) {
                        authResourceGroupSet = unionEnterprise.stream()
                                .filter(e -> e.getResourceGroupId() != null)
                                .filter(e -> {
                                    if (enterpriseId != null) {
                                        if (e.getUnionEnterpriseId().equals(enterpriseId)) {
                                            return false;
                                        }
                                    }
                                    return true;
                                })
                                .map(e -> e.getResourceGroupId().toString())
                                .collect(Collectors.toSet());
                    }
                }
                List<UnionResourceGroupVO> result = new ArrayList<>();
                Set<String> finalAuthResourceGroupSet = authResourceGroupSet;
                list.forEach(e -> {
                    UnionResourceGroupVO vo = EntityUtil.changeEntityNew(UnionResourceGroupVO.class, e);
                    if (finalAuthResourceGroupSet.contains(e.getResourceGroupId().toString())) {
                        vo.setHasAuth(true);
                    } else {
                        vo.setHasAuth(false);
                    }
                    result.add(vo);
                });

                return RestResponse.success(result);
            }
        }
        return RestResponse.failure(response.getCode(), response.getMessage());
    }

    /**
     * 受审组
     * @return
     */
    @RequestMapping("list-audited-group")
    public RestResponse listAuditedGroup() {
        ServiceResponse<List<AuditedGroupDTO>> response = auditedGroupApiService.listAllAuditedGroup(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(response, UnionAuditedGroupVO.class);
    }

    /**
     * 账号分组
     * @return
     */
    @RequestMapping("list-account-group")
    public RestResponse listAccountGroup() {
        ServiceResponse<List<AccountGroupDTO>> response = accountGroupApiService.listNoMemberByEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(response, UnionAccountGroupVO.class);
    }

    /**
     * 资源授权
     * @param unionEnterpriseAuthResQO
     * @return
     */
    @RequestMapping("auth-resource")
    public RestResponse authResource(UnionEnterpriseAuthResQO unionEnterpriseAuthResQO) {
        //资源授权
        UnionEnterpriseAuthResDTO dto = EntityUtil.changeEntityNew(UnionEnterpriseAuthResDTO.class, unionEnterpriseAuthResQO);
        dto.setOwnEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        dto.setUnionEnterpriseId(unionEnterpriseAuthResQO.getEnterpriseId());
        return OperationResultUtils.operationResult(
                unionEnterpriseAuthApiService.authResource(dto),
                "资源授权联合",
                getEnterpriseById(dto.getUnionEnterpriseId()).getEnterpriseName());
    }

    @RequestMapping("get-resource-detail")
    public RestResponse getResourceDetail(Integer enterpriseId) {
        ServiceResponse<UnionEnterpriseAuthDTO> response = unionEnterpriseAuthApiService
                .getUnionEnterpriseAuth(UserDetailUtils.getUserDetail().getEnterpriseId(), enterpriseId);
        if (response.isSuccess()) {
            if (response.getResult().getResourceGroupId() == null) {
                return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "还未资源联合");
            }
        }
        return ResultControllerUtils.commonResultOne(response, UnionEnterpriseResourceDetailVO.class);
    }

    /**
     * 解除资源
     * @param enterpriseId 联合商户
     * @param relieveRemark 解除理由
     * @return
     */
    @RequestMapping("relieve-resource")
    public RestResponse relieveResource(Integer enterpriseId, String relieveRemark) {
        //解除资源
        return OperationResultUtils.operationResult(
                unionEnterpriseAuthApiService.relieveUnion(UserDetailUtils.getUserDetail().getEnterpriseId(), enterpriseId, relieveRemark),
                "解除联合",
                getEnterpriseById(enterpriseId).getEnterpriseName());
    }

    /**
     * 联合商户列表
     * @param enterpriseName
     * @param authStatus 资源授权状态
     * @return
     */
    @RequestMapping("list-union-enterprise")
    public RestResponse listUnionEnterprise(String enterpriseName, Integer authStatus) {
        return ResultControllerUtils.commonResult(unionEnterpriseAuthApiService
                        .listUnionEnterpriseAuth(UserDetailUtils.getUserDetail().getEnterpriseId(), enterpriseName, authStatus),
                UnionEnterpriseListVO.class);
    }

    @RequestMapping("resource-reset-auth")
    public RestResponse resourceResetAuth(Integer enterpriseId, Integer resourceType) {
        //重新授权
        return ResultControllerUtils.commonResult(unionEnterpriseAuthApiService
                .resetAuthResource(UserDetailUtils.getUserDetail().getEnterpriseId(), enterpriseId, resourceType));
    }

    @RequestMapping("resource-reset-relieve")
    public RestResponse resourceResetRelieve(Integer enterpriseId, Integer resourceType) {
        //重新授权
        return ResultControllerUtils.commonResult(unionEnterpriseAuthApiService
                .resetRelieveResource(UserDetailUtils.getUserDetail().getEnterpriseId(), enterpriseId, resourceType));
    }

    /**
     * 应用策略的用户域范围数据
     * @param resourceGroupId
     * @param enterpriseId
     * @return
     */
    @RequestMapping("/user-area")
    public RestResponse channelResource(Integer resourceGroupId, Integer enterpriseId, Integer ownEnterpriseId) {
        if (ownEnterpriseId == null) {
            ownEnterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        }
        UnionEnterpriseAuthDTO old = checkUnionEnterpriseAuth(ownEnterpriseId, enterpriseId);

        if (old.getResourceGroupId() != null && !old.getResourceGroupId().equals(resourceGroupId)) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "资源组数据错误，与联合商户授权资源不相符");
        }
        ServiceResponse<ResourceGroupDTO> resourceResponse = resourceGroupApiService.getResourceGroup(resourceGroupId);
        if (!resourceResponse.isSuccess()) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "不存在资源组");
        }
        ResourceGroupDTO resourceGroupDTO = resourceResponse.getResult();
        //不授予用户域
        if (resourceGroupDTO.getCustomUser() != 1) {
            return RestResponse.success();
        }
        List<ResourceGroupCustomUserDTO> customUserList = JSON.parseArray(resourceGroupDTO.getCustomUserData(), ResourceGroupCustomUserDTO.class);
        //资源组中的域数据
        Set<Long> memberCardAreaSet = new HashSet<>();
        Set<Long> serviceAreaSet = new HashSet<>();
        Set<Long> appletAreaSet = new HashSet<>();

        for (ResourceGroupCustomUserDTO resourceGroupCustomUserDTO : customUserList) {
            CuTypeEnum cuTypeEnum = CuTypeEnum.getEnum(resourceGroupCustomUserDTO.getType());

            String[] temp = resourceGroupCustomUserDTO.getAreaValue().split(",");
            for (String area : temp) {
                if (StringUtils.isBlank(area)) {
                    continue;
                }
                switch (cuTypeEnum) {
                    case MCU:
                        memberCardAreaSet.add(Long.valueOf(area));
                        break;
                    case ACU:
                        appletAreaSet.add(Long.valueOf(area));
                        break;
                    case SCU:
                        serviceAreaSet.add(Long.valueOf(area));
                        break;
                    default:
                }
            }
        }

        List<OwnerMemberCardDTO> memberCardList = memberCardApiService.getAll(ownEnterpriseId).getResult();
        List<AppletsConfigSimpleOutputDTO> appletList = appletsConfigApiService.listValidAppletConfigAppId(ownEnterpriseId)
                .getResult();
        List<ServiceConfigSimpleOutputDTO> serviceList = serviceConfigApiService
                .listValidServiceConfigAppId(ownEnterpriseId).getResult();
        Map<Integer, Object> map = new HashMap<>(4);
        //todo 过滤共享域
        map.put(CuTypeEnum.MCU.getType(), EntityUtil.changeEntityListByJSON(OwnerMemberCardVO.class,
                memberCardList.stream().filter(e -> memberCardAreaSet.contains(e.getCardConfigId())).collect(Collectors.toList())));
        map.put(CuTypeEnum.ACU.getType(), EntityUtil.changeEntityListByJSON(AppletsConfigSimpleOutputVO.class,
                appletList.stream().filter(e -> appletAreaSet.contains(e.getAppletConfigId())).collect(Collectors.toList())));
        map.put(CuTypeEnum.SCU.getType(), EntityUtil.changeEntityListByJSON(ServiceConfigSimpleOutputVO.class,
                serviceList.stream().filter(e -> serviceAreaSet.contains(e.getServiceConfigId())).collect(Collectors.toList())));
        return RestResponse.success(map);
    }

    @RequestMapping("list-app")
    public RestResponse listApp() {
        ServiceResponse<List<ApplicationDTO>> appListResponse = applicationApiService.listApplicationAndServeProp();
        return ResultControllerUtils.commonResult(appListResponse, UnionAppVO.class);
    }

    @RequestMapping("auth-app-resource")
    public RestResponse authAppResource(UnionEnterpriseAuthAppQO unionEnterpriseAuthAppQO) {
        //app资源授权
        UnionEnterpriseAuthDTO old = checkUnionEnterpriseAuth(UserDetailUtils.getUserDetail().getEnterpriseId(), unionEnterpriseAuthAppQO.getEnterpriseId());
        UnionEnterpriseAuthAppDTO dto = EntityUtil.changeEntityNew(UnionEnterpriseAuthAppDTO.class, unionEnterpriseAuthAppQO);
        dto.setUnionEnterpriseAuthId(old.getUnionEnterpriseAuthId());
        return OperationResultUtils.operationResult(
                unionEnterpriseAuthApiService.saveOrUpdateUnionEnterpriseAuthApp(dto),
                "应用资源策略",
                unionEnterpriseAuthAppQO.getStrategyName());
    }


    @RequestMapping("delete-app-resource")
    public RestResponse deleteAppResource(Integer unionEnterpriseAuthAppId) {
        return OperationResultUtils.operationResult(
                unionEnterpriseAuthApiService.deleteUnionEnterpriseAuthApp(unionEnterpriseAuthAppId),
                "删除应用资源策略");
    }

    @RequestMapping("get-app-resource-detail")
    public RestResponse getAppResourceDetail(Integer unionEnterpriseAuthAppId) {
        return ResultControllerUtils.commonResultOne(unionEnterpriseAuthApiService
                .getUnionEnterpriseAuthApp(unionEnterpriseAuthAppId), UnionEnterpriseAppResourceDetailVO.class);
    }

    @RequestMapping("list-app-resource")
    public RestResponse listAppResource(Integer enterpriseId) {
        return getAppStrategy(UserDetailUtils.getUserDetail().getEnterpriseId(), enterpriseId);
    }

    @RequestMapping("delete-all-app-resource")
    public RestResponse deleteAllAppResource(Integer enterpriseId) {
        UnionEnterpriseAuthDTO old = checkUnionEnterpriseAuth(UserDetailUtils.getUserDetail().getEnterpriseId(), enterpriseId);
        return ResultControllerUtils.commonResult(unionEnterpriseAuthApiService
                .deleteUnionEnterpriseAuthAppByUnionEnterpriseAuthId(old.getUnionEnterpriseAuthId()));
    }


    /**
     * 自有商户列表
     * @return
     */
    @RequestMapping("list-own-enterprise")
    public RestResponse listOwnEnterprise(String startTime, String endTime) {
        return ResultControllerUtils.commonResult(unionEnterpriseAuthApiService
                .listOwnEnterpriseAuth(UserDetailUtils.getUserDetail().getEnterpriseId(), startTime, endTime), OwnEnterpriseListVO.class);
    }

    @RequestMapping("get-own-enterprise-resource-detail")
    public RestResponse getOwnEnterpriseResourceDetail(Integer ownEnterpriseId) {
        UnionEnterpriseAuthDTO unionEnterpriseAuthDTO = checkUnionEnterpriseAuth(ownEnterpriseId, UserDetailUtils.getUserDetail().getEnterpriseId());

        if (unionEnterpriseAuthDTO.getResourceGroupId() == null) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "还未资源联合");
        }
        ServiceResponse<ResourceGroupDTO> resourceResponse = resourceGroupApiService.getResourceGroup(unionEnterpriseAuthDTO.getResourceGroupId());
        if (!resourceResponse.isSuccess()) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "不存在资源组");
        }
        ResourceGroupDTO resourceGroupDTO = resourceResponse.getResult();
        OwnEnterpriseResourceDetailVO vo = EntityUtil.changeEntityNew(OwnEnterpriseResourceDetailVO.class, resourceGroupDTO);

        setUserAreaData(ownEnterpriseId, vo);
        setWeimobName(ownEnterpriseId, vo);
        setSmsName(ownEnterpriseId, vo);

        vo.setWhiteListAuth(unionEnterpriseAuthDTO.getWhiteListAuth());
        vo.setDataEncryField(unionEnterpriseAuthDTO.getDataEncryField());
        vo.setUserEditAuth(unionEnterpriseAuthDTO.getUserEditAuth());
        //账号分组信息组长
        String accountGroup = unionEnterpriseAuthDTO.getAccountGroup();
        List<OwnEnterpriseAccountGroupVO> accountGroupVoList = new ArrayList<>();
        if (StringUtils.isNotBlank(accountGroup)) {
            ServiceResponse<List<AccountGroupDTO>> response = accountGroupApiService.listAccountGroupByEnterpriseId(ownEnterpriseId);
            Map<String, AccountGroupDTO> accountGroupMap = new HashMap<>();
            if (response.isSuccess()) {
                List<AccountGroupDTO> accountGroupDTOList = response.getResult();
                accountGroupMap = accountGroupDTOList.stream().filter(e -> e != null).collect(Collectors.toMap(e -> e.getAccountGroupId().toString(), e -> e));
            }
            String[] accountGroupArr = accountGroup.split(",");
            for (String accountGroupTemp : accountGroupArr) {
                if (StringUtils.isBlank(accountGroupTemp)) {
                    continue;
                }
                AccountGroupDTO accountGroupDTO = accountGroupMap.get(accountGroupTemp);
                if (accountGroupDTO != null) {
                    accountGroupVoList.add(EntityUtil.changeEntityNew(OwnEnterpriseAccountGroupVO.class, accountGroupDTO));
                }
            }
        }
        vo.setAccountGroup(accountGroupVoList);
        //受审组
        String auditedGroup = unionEnterpriseAuthDTO.getAuditeeGroup();
        List<OwnEnterpriseAuditedGroupVO> auditedGroupVoList = new ArrayList<>();
        if (StringUtils.isNotBlank(auditedGroup)) {
            ServiceResponse<List<AuditedGroupDTO>> auditedResponse = auditedGroupApiService.listAllAuditedGroup(ownEnterpriseId);
            Map<String, AuditedGroupDTO> auditedGroupMap = new HashMap<>();
            if (auditedResponse.isSuccess()) {
                List<AuditedGroupDTO> auditedGroupList = auditedResponse.getResult();
                if (CollectionUtils.isNotEmpty(auditedGroupList)) {
                    auditedGroupMap = auditedGroupList.stream().collect(Collectors.toMap(e -> e.getAuditedGroupId().toString(), e -> e));
                }
            }
            String[] auditedGroupArr = auditedGroup.split(",");
            for (String auditedGroupTemp : auditedGroupArr) {
                if (StringUtils.isBlank(auditedGroupTemp)) {
                    continue;
                }
                AuditedGroupDTO auditedGroupDTO = auditedGroupMap.get(auditedGroupTemp);
                if (auditedGroupDTO != null) {
                    auditedGroupVoList.add(EntityUtil.changeEntityNew(OwnEnterpriseAuditedGroupVO.class, auditedGroupDTO));
                }
            }
        }
        vo.setAuditedGroup(auditedGroupVoList);

        return RestResponse.success(vo);
    }

    @RequestMapping("get-own-enterprise-app-resource-detail")
    public RestResponse getOwnEnterpriseAppResourceDetail(Integer ownEnterpriseId) {
        RestResponse result = getAppStrategy(ownEnterpriseId, UserDetailUtils.getUserDetail().getEnterpriseId());
        if ("0000".equals(result.getCode())) {
            List<UnionAppResourceListVO> appStrategyList = (List<UnionAppResourceListVO>) result.getResult();

            List<OwnEnterpriseAuthAppDetailVO> voList = new ArrayList<>();

            if (CollectionUtils.isNotEmpty(appStrategyList)) {
                for (UnionAppResourceListVO strategy : appStrategyList) {
                    OwnEnterpriseAuthAppDetailVO vo = new OwnEnterpriseAuthAppDetailVO();
                    vo.setAppAuthStatus(strategy.getAppAuthStatus());
                    vo.setAppletArea(strategy.getAppletArea());
                    vo.setMemberArea(strategy.getMemberArea());
                    vo.setServiceArea(strategy.getServiceArea());

                    List<UnionAppVO> appList = strategy.getAppList();
                    for (UnionAppVO app : appList) {
                        vo.setApplicationId(app.getApplicationId());
                        vo.setName(app.getName());
                        vo.setIconUrl(app.getIconUrl());

                        voList.add(vo);
                    }
                }
            }

            return RestResponse.success(voList);
        }
        return result;
    }

    @RequestMapping("list-operation-log")
    public RestResponse listOperationLog(Integer ownEnterpriseId, UnionEnterpriseAuthResLogQO param) {
        UnionEnterpriseAuthDTO old = getUnionEnterpriseAuth(ownEnterpriseId, UserDetailUtils.getUserDetail().getEnterpriseId());
        if (old == null) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "联合商户不存在");
        }
        param.setUnionEnterpriseAuthId(old.getUnionEnterpriseAuthId());
        return ResultControllerUtils.commonPageResult(unionEnterpriseAuthApiService.listUnionEnterpriseAuthResLog(param), OwnEnterpriseOperationLogVO.class);
    }

    @RequestMapping("get-last-union-info")
    public RestResponse getLastUnionInfo() {
        ServiceResponse<List<OwnEnterpriseAuthListDTO>> response = unionEnterpriseAuthApiService
                .listOwnEnterpriseAuth(UserDetailUtils.getUserDetail().getEnterpriseId(), null, null);
        if (response.isSuccess()) {
            List<UnionEnterpriseIndexVO> voList = response.getResult().stream()
                    .filter(e -> e != null)
                    .filter(e -> e.getLastUnionTime() != null)
                    .map(e -> EntityUtil.changeEntityNew(UnionEnterpriseIndexVO.class, e))
                    .collect(Collectors.toList());
            voList.sort(Comparator.comparing(UnionEnterpriseIndexVO::getLastUnionTime).reversed());
            return RestResponse.success(voList);
        }
        return RestResponse.failure(response.getCode(), response.getMessage());
    }


    /**
     * 检查是否存在联合商户
     * @param enterpriseName
     * @return null失败
     */
    private EnterpriseDTO checkEnterpriseByEnterpriseName(String enterpriseName) {
        ServiceResponse<EnterpriseDTO> response = enterpriseApiService.getEnterpriseByName(enterpriseName);
        if (response.isSuccess()) {
            EnterpriseDTO enterpriseDTO = response.getResult();
            if (enterpriseDTO != null) {
                if (enterpriseDTO.getEnterpriseId().equals(UserDetailUtils.getUserDetail().getEnterpriseId())) {
                    return null;
                }
                return enterpriseDTO;
            }
        }
        return null;
    }

    /**
     * 查询联合商户
     * @param ownEnterpriseId
     * @param unionEnterpriseId
     * @return
     */
    private UnionEnterpriseAuthDTO getUnionEnterpriseAuth(Integer ownEnterpriseId, Integer unionEnterpriseId) {
        ServiceResponse<UnionEnterpriseAuthDTO> unionResponse = unionEnterpriseAuthApiService
                .getUnionEnterpriseAuth(ownEnterpriseId, unionEnterpriseId);
        if (unionResponse.isSuccess()) {
            return unionResponse.getResult();
        }
        return null;
    }

    private UnionEnterpriseAuthDTO checkUnionEnterpriseAuth(Integer ownEnterpriseId, Integer unionEnterpriseId) {
        ServiceResponse<UnionEnterpriseAuthDTO> unionResponse = unionEnterpriseAuthApiService
                .getUnionEnterpriseAuth(ownEnterpriseId, unionEnterpriseId);
        if (!unionResponse.isSuccess()) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "联合记录不存在");
        }
        return unionResponse.getResult();
    }

    private EnterpriseDTO getEnterpriseById(Integer enterpriseId) {
        //查询联合商户信息
        ServiceResponse<EnterpriseDTO> enterpriseResponse = enterpriseApiService.getEnterpriseById(enterpriseId);
        if (!enterpriseResponse.isSuccess()) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "联合商户不存在");
        }
        return enterpriseResponse.getResult();
    }

    private Map<Long, ApplicationDTO> getAppMap() {
        ServiceResponse<List<ApplicationDTO>> appListResponse = applicationApiService.listApplicationAndServeProp();
        if (appListResponse.isSuccess()) {
            List<ApplicationDTO> list = appListResponse.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                return list.stream().collect(Collectors.toMap(e -> e.getApplicationId(), e -> e));
            }
        }
        return new HashMap<>();
    }

    /**
     * 联合商户应用策略
     * @param ownEnterpriseId
     * @param unionEnterpriseId
     * @return List<UnionAppResourceListVO> result
     */
    private RestResponse getAppStrategy(Integer ownEnterpriseId, Integer unionEnterpriseId) {
        ServiceResponse<List<UnionEnterpriseAuthAppDTO>> response = unionEnterpriseAuthApiService
                .listUnionEnterpriseAuthApp(ownEnterpriseId, unionEnterpriseId);
        if (response.isSuccess()) {
            List<UnionEnterpriseAuthAppDTO> list = response.getResult();

            //应用数据，为了获取名称
            Map<Long, ApplicationDTO> appMap = getAppMap();

            UnionEnterpriseAuthDTO old = getUnionEnterpriseAuth(ownEnterpriseId, unionEnterpriseId);
            if (old == null) {
                return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "联合商户不存在");
            }

            //用户域数据
            Map<Integer, Object> areaMap = new HashMap<>();
            RestResponse userArea = channelResource(old.getResourceGroupId(), unionEnterpriseId, ownEnterpriseId);
            if ("0000".equals(userArea.getCode())) {
                areaMap = (Map<Integer, Object>) userArea.getResult();
            }

            //组装数据
            List<UnionAppResourceListVO> result = new ArrayList<>(list.size());

            for (UnionEnterpriseAuthAppDTO temp : list) {
                UnionAppResourceListVO vo = new UnionAppResourceListVO();
                vo.setUnionEnterpriseAuthAppId(temp.getUnionEnterpriseAuthAppId());
                vo.setStrategyName(temp.getStrategyName());
                vo.setAppAuthStatus(temp.getAppAuthStatus());
                //应用数据
                List<UnionAppVO> appList = new ArrayList<>();
                String appIdTemp = temp.getAppId();
                if (StringUtils.isNotBlank(appIdTemp)) {
                    String[] appIdTempArr = appIdTemp.split(",");
                    for (String appId : appIdTempArr) {
                        UnionAppVO appVO = new UnionAppVO();
                        if (StringUtils.isBlank(appId)) {
                            continue;
                        }
                        appVO.setApplicationId(Long.valueOf(appId));
                        ApplicationDTO app = appMap.get(Long.valueOf(appId));
                        if (app != null) {
                            appVO.setIconUrl(app.getIconUrl());
                            appVO.setName(app.getName());
                        }
                        appList.add(appVO);
                    }
                }
                vo.setAppList(appList);

                //用户域
                String authData = temp.getAuthData();
                if (StringUtils.isNotBlank(authData)) {
                    Map<String, List<String>> authDataArr = new HashMap<>();
                    authDataArr = JSON.parseObject(authData, authDataArr.getClass());

                    vo.setMemberArea(UserAeaUtils.getUserArea(CuTypeEnum.MCU.getType(), areaMap, authDataArr));
                    vo.setAppletArea(UserAeaUtils.getUserArea(CuTypeEnum.ACU.getType(), areaMap, authDataArr));
                    vo.setServiceArea(UserAeaUtils.getUserArea(CuTypeEnum.SCU.getType(), areaMap, authDataArr));
                }

                result.add(vo);
            }
            return RestResponse.success(result);
        }
        return RestResponse.failure(response.getCode(), response.getMessage());
    }

    private void setUserAreaData(Integer ownEnterpriseId, OwnEnterpriseResourceDetailVO vo) {
        String customUserData = vo.getCustomUserData();
        if (StringUtils.isNotBlank(customUserData)) {
            List<ResourceGroupCustomUserDTO> customUserList = JSON.parseArray(customUserData, ResourceGroupCustomUserDTO.class);

            List<OwnerMemberCardDTO> memberCardList = memberCardApiService.getAll(ownEnterpriseId).getResult();
            List<AppletsConfigSimpleOutputDTO> appletList = appletsConfigApiService.listValidAppletConfigAppId(ownEnterpriseId)
                    .getResult();
            List<ServiceConfigSimpleOutputDTO> serviceList = serviceConfigApiService
                    .listValidServiceConfigAppId(ownEnterpriseId).getResult();

            Map<String, String> cuNameMap = new HashMap<>();
            if (CollectionUtils.isNotEmpty(memberCardList)) {
                for (OwnerMemberCardDTO member : memberCardList) {
                    cuNameMap.put(member.getCardConfigId().toString(), member.getCardName());
                }
            }
            if (CollectionUtils.isNotEmpty(appletList)) {
                for (AppletsConfigSimpleOutputDTO temp : appletList) {
                    cuNameMap.put(temp.getAppletConfigId().toString(), temp.getAppletConfigName());
                }
            }
            if (CollectionUtils.isNotEmpty(serviceList)) {
                for (ServiceConfigSimpleOutputDTO temp : serviceList) {
                    cuNameMap.put(temp.getServiceConfigId().toString(), temp.getServiceConfigName());
                }
            }
            for (ResourceGroupCustomUserDTO temp : customUserList) {
                StringBuilder nameTemp = new StringBuilder();
                String[] areaValueArr = temp.getAreaValue().split(",");
                for (String area : areaValueArr) {
                    String cuName = cuNameMap.get(area);
                    if (StringUtils.isNotBlank(cuName)) {
                        nameTemp.append(cuName).append("/");
                    }
                }
                temp.setAreaValue(nameTemp.toString());
            }
            vo.setCustomUserList(customUserList);
        }
    }

    private void setWeimobName(Integer enterpriseId, OwnEnterpriseResourceDetailVO vo) {
        ServiceResponse<List<WmStoreDTO>> listResponse = wmStoreApiService.listWmStore(enterpriseId);
        if (listResponse.isSuccess()) {
            String weimobStoreId = vo.getWeimobStoreId();
            if (StringUtils.isNotBlank(weimobStoreId)) {
                String[] weimobStoreIdArr = weimobStoreId.split(",");

                Map<String, String> map = new HashMap<>();
                List<WmStoreDTO> list = listResponse.getResult();
                if (CollectionUtils.isNotEmpty(list)) {
                    map = list.stream().collect(Collectors.toMap(e -> e.getWmMallStoreId().toString(), e -> e.getWmPidName()));
                }

                List<String> result = new ArrayList<>();
                for (String temp : weimobStoreIdArr) {
                    String name = map.get(temp);
                    if (name != null) {
                        result.add(name);
                    }
                }
                vo.setWeimobName(result);
            }
        }
    }

    private void setSmsName(Integer enterpriseId, OwnEnterpriseResourceDetailVO vo) {
        ServiceResponse<List<EnterpriseSmsSignDTO>> response = smsApiService
                .listSignByEnterprise(enterpriseId, null);
        if (response.isSuccess()) {
            String smsId = vo.getSmsId();
            if (StringUtils.isNotBlank(smsId)) {
                String[] smsIdArr = smsId.split(",");

                Map<String, String> map = new HashMap<>();
                List<EnterpriseSmsSignDTO> list = response.getResult();
                if (CollectionUtils.isNotEmpty(list)) {
                    map = list.stream().collect(Collectors.toMap(e -> e.getSignId().toString(), e -> e.getSmsSignText()));
                }

                List<String> result = new ArrayList<>();
                for (String temp : smsIdArr) {
                    String name = map.get(temp);
                    if (name != null) {
                        result.add(name);
                    }
                }
                vo.setSmsName(result);
            }
        }
    }

}
