package com.gic.enterprise.web.controller.wm;

import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.servlet.http.HttpServletResponse;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.annotation.HeaderSignIgnore;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.OperationResultUtils;
import com.gic.enterprise.base.UserInfo;
import com.gic.enterprise.constant.Constants;
import com.gic.enterprise.constant.MallModeEnum;
import com.gic.enterprise.constant.WmStoreConfigTypeEnum;
import com.gic.enterprise.dto.WmStoreDTO;
import com.gic.enterprise.dto.wm.WmStoreConfigDTO;
import com.gic.enterprise.dto.wm.WmStoreCouponConfigDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.qo.wm.CouponConfigQO;
import com.gic.enterprise.service.WmStoreApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.enterprise.web.utils.DomainUtils;
import com.gic.enterprise.web.vo.MemberCardVO;
import com.gic.enterprise.web.vo.wm.*;
import com.gic.goods.pump.api.service.WeiMobGoodsService;
import com.gic.marketing.api.dto.coupon.CouponDTO;
import com.gic.marketing.api.service.coupon.MemberCouponApiService;
import com.gic.member.config.api.enums.OrderPointCalculationPartyEnum;
import com.gic.member.config.api.service.MemberCardApiService;
import com.gic.member.config.api.service.OrderPointCalculationPartyApiService;
import com.gic.open.api.dto.EnterpriseServeInfoDTO;
import com.gic.open.api.service.EnterpriseLicenseApiService;
import com.gic.weimob.api.service.WeimobBaseApiService;
import com.gic.weimob.api.service.WeimobMemberBaseApiService;
import com.gic.weimob.api.service.WeimobMerchantApiService;
import com.gic.weimob.api.service.WeimobOrderApiService;
/**
 *
 * @ClassName: WmMallStoreController
 * @Description:
 * @author taogs
 * @date 2020/9/11 16:30
 */
@RestController
@RequestMapping("/wm-mall-store")
public class WmMallStoreController {
    private final static Logger LOGGER = LogManager.getLogger(WmMallStoreController.class);

    @Autowired
    private WmStoreApiService wmStoreApiService;
    @Autowired
    private MemberCardApiService memberCardApiService;
    @Autowired
    private WeimobMerchantApiService weimobMerchantService;
    @Autowired
    private MemberCouponApiService memberCouponApiService;
    @Autowired
    private WeimobMemberBaseApiService weimobMemberService;
    @Autowired
    private WeimobBaseApiService weimobBaseService;
    @Autowired
    private WeiMobGoodsService weiMobGoodsService;
    @Autowired
    private OrderPointCalculationPartyApiService orderPointCalculationPartyApiService;
    @Autowired
    private WeimobOrderApiService weimobOrderService;
    @Autowired
    private EnterpriseLicenseApiService enterpriseLicenseApiService;

    @RequestMapping("/save-wm-store")
    public RestResponse save(WmStoreDTO dto) {
        UserInfo userInfo = UserDetailUtils.getUserDetail().getUserInfo();
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        dto.setEnterpriseId(enterpriseId);
        ServiceResponse<Integer> result = wmStoreApiService.saveWmStore(dto);
        if (result.isSuccess()) {
            //创建微盟gic关联
            ServiceResponse<Void> saveStoreInfo = weimobMerchantService
                    .saveStoreInfo(result.getResult(), dto.getEnterpriseId(),
                            dto.getWmMainAccount(), Long.valueOf(dto.getWmPid()), Long.valueOf(dto.getWmStoreId()),
                            MallModeEnum.mapToWm(dto.getMallMode()));
            LOGGER.info("微盟商城店铺-微盟结果:{}", JSON.toJSONString(saveStoreInfo));
            ServiceResponse<Void> response = orderPointCalculationPartyApiService
                    .saveOrderPointCalculationParty(enterpriseId, Long.valueOf(result.getResult()),
                            dto.getWmPidName(), userInfo.getUserId(), userInfo.getUserName(), OrderPointCalculationPartyEnum.WEIMO.getCode());
            LOGGER.info("微盟商城店铺-会员组初始化结果:{}", JSON.toJSONString(response));
        }
        return OperationResultUtils.operationResult(result,
                OperationResultUtils.LOG_NEW + "微盟商城店铺", dto.getWmPidName());
    }

    @RequestMapping("/edit-wm-store")
    public RestResponse edit(WmStoreDTO dto) {
        ServiceResponse<Void> result = wmStoreApiService.modifyWmStore(dto);
        if (result.isSuccess()) {
            //微盟gic关联
            ServiceResponse<Void> saveStoreInfo = weimobMerchantService
                    .saveStoreInfo(dto.getWmMallStoreId(), dto.getEnterpriseId(),
                            dto.getWmMainAccount(), Long.valueOf(dto.getWmPid()), Long.valueOf(dto.getWmStoreId()),
                            MallModeEnum.mapToWm(dto.getMallMode()));
            LOGGER.info("微盟商城店铺编辑-微盟结果:{}", JSON.toJSONString(saveStoreInfo));
        }
        return OperationResultUtils.operationResult(result,
                OperationResultUtils.LOG_EDIT + "微盟商城店铺", dto.getWmPidName());
    }

    /**
     * 店铺详情
     * @param wmMallStoreId
     * @return
     */
    @RequestMapping("/get-wm-store-detail")
    public RestResponse getDetail(Integer wmMallStoreId) {
        return ResultControllerUtils.commonResultOne(wmStoreApiService.getWmStoreByWmMallStoreId(wmMallStoreId),
                WmStoreDetailVO.class);
    }

    /**
     * 店铺列表
     * @return
     */
    @RequestMapping("/list-wm-store")
    public RestResponse listWmStore() {
        ServiceResponse<List<WmStoreDTO>> listResponse = wmStoreApiService.listWmStore(UserDetailUtils.getUserDetail().getEnterpriseId());
        if (listResponse.isSuccess()) {
            List<WmStoreDTO> list = listResponse.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                List<WmStoreVO> voList = list.stream()
                        .map(e -> {
                            WmStoreVO vo = EntityUtil.changeEntityNew(WmStoreVO.class, e);
                            if (hasCouponSet(vo.getWmMallStoreId())) {
                                vo.setHasBindCoupon(1);
                            }
                            ServiceResponse<Set<String>> configSet = wmStoreApiService.getWmStoreConfigKey(e.getWmMallStoreId());
                            if (configSet.isSuccess()) {
                                Set<String> set = configSet.getResult();
                                boolean isAllConfig = Stream.of(WmStoreConfigTypeEnum.values())
                                        .allMatch(configType -> set.contains(String.valueOf(configType.getCode())));
                                if (isAllConfig){
                                    vo.setHasCompleteConfig(1);
                                }
                                //还未配置的项目
                                vo.setHasNotConfig(Stream.of(WmStoreConfigTypeEnum.values())
                                        .filter(configItem -> !set.contains(String.valueOf(configItem.getCode())))
                                        .map(configItem -> configItem.getCode())
                                        .collect(Collectors.toList()));
                            }
                            return vo;
                        })
                        .collect(Collectors.toList());
                return RestResponse.success(voList);
            }

        }
        return RestResponse.failure(listResponse.getCode(), listResponse.getMessage());
    }

    /**
     * 获取授权链接
     * @Title: authWmStore 
     * @Description:
     * @author guojuxing
     * @param wmMallStoreId 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("/get-auth-url")
    public RestResponse getAuthUrl(Integer wmMallStoreId) {
        //
        ServiceResponse<String> response = weimobMerchantService.getAuthUrl(wmMallStoreId);
        if (response.isSuccess()) {
            String authUrl = response.getResult();
            String mainHost = DomainUtils.getCurrentDomain() + "/gic-enterprise-web/wm-mall-store/auth-callback";

            return RestResponse.success(authUrl + "&redirect_uri=" + mainHost);
        } else {
            return RestResponse.failure(response.getCode(), response.getMessage());
        }
    }

    /**
     * 同步会员等级到微盟
     * @Title: syncMemberGradeToWm 
     * @Description:
     * @author guojuxing
     * @param memberCardId
     * @param wmMallStoreId 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("/sync-member-grade-to-wm")
    public RestResponse syncMemberGradeToWm(String memberCardId, Integer wmMallStoreId) {
        ServiceResponse<Void> syncMemberCardResponse = weimobMerchantService
                .saveWmMallStoreIdAndMcuAreaId(wmMallStoreId, Long.valueOf(memberCardId));
        LOGGER.info("同步会员等级-会员卡结果:{}", JSON.toJSONString(syncMemberCardResponse));
        if (!syncMemberCardResponse.isSuccess()) {
            return RestResponse.failure(syncMemberCardResponse.getCode(), syncMemberCardResponse.getMessage());
        }
        ServiceResponse<Void> syncMemberGradeResponse = weimobMerchantService
                .saveOrUpdateCardGradeSetting(wmMallStoreId, Long.valueOf(memberCardId));
        if (!syncMemberGradeResponse.isSuccess()) {
            return RestResponse.failure(syncMemberGradeResponse.getCode(), syncMemberGradeResponse.getMessage());
        }
        LOGGER.info("同步会员等级-会员卡等级结果:{}", JSON.toJSONString(syncMemberGradeResponse));
        return RestResponse.success();
    }

    /**
     * 获取会员卡等级已同步接口
     * @Title: listMemberGrade 
     * @Description:
     * @author guojuxing
     * @param wmMallStoreId 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("/list-member-grade")
    public RestResponse listMemberGrade(Integer wmMallStoreId) {
        return ResultControllerUtils.commonResult(weimobMerchantService.getGradeList(wmMallStoreId), MemberGradeVO.class);
    }

    /**
     * 卡券错误日志列表
     * @param gicCouponId
     * @param sendType 非必填，null时取全部，1 为推送卡券,2 核销卡券
     * @param currentPage
     * @param pageSize
     * @param searchStr
     * @return
     */
    @RequestMapping("/list-coupon-log")
    public RestResponse listCouponLog(String gicCouponId, Integer sendType,
                                      @RequestParam(defaultValue = "1") int currentPage,
                                      @RequestParam(defaultValue = "20") int pageSize, String searchStr) {
        return ResultControllerUtils.commonPageResult(weimobMerchantService
                .getCouponLogs(Long.valueOf(gicCouponId), sendType, currentPage, pageSize, searchStr), WeimobCouponErLogVO.class);
    }

    @RequestMapping("/license")
    public RestResponse license() {
        ServiceResponse<EnterpriseServeInfoDTO> info = enterpriseLicenseApiService
                .getServeInfo(UserDetailUtils.getUserDetail().getEnterpriseId(), "weimob");
        return ResultControllerUtils.commonResult(info);
    }

    /**
     * 删除卡券错误日志
     * @param couponErLogIds 卡券错误日志ID，英文逗号给开
     * @return
     */
    @RequestMapping("delete-coupon-error-log")
    public RestResponse deleteCouponErrorLog(String couponErLogIds) {
        String[] ids = couponErLogIds.split(",");
        List<Long> result = new ArrayList<>(ids.length);
        for (String id : ids) {
            result.add(Long.valueOf(id));
        }
        return ResultControllerUtils.commonResult(weimobMerchantService.deleteCouponErrorlog(result));
    }

    /**
     * 重新发送卡券
     * @param mcuId
     * @param couponId
     * @param couponCode
     * @param couponLogId
     * @return
     */
    @RequestMapping("send-coupon-reset")
    public RestResponse sendCouponReset(String mcuId, String couponId, String couponCode, String couponLogId) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        return ResultControllerUtils.commonResult(weimobMemberService.sendCoupon(enterpriseId, Long.valueOf(mcuId),
                Long.valueOf(couponId), Long.valueOf(couponLogId), couponCode));
    }


    /**
     * 获取错误订单日志
     * @Title: getWmOrderErrorLog 
     * @Description: 
     * @author guojuxing 
     * @param wmMallStoreId
     * @param currentPage
     * @param pageSize
     * @param searchStr 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("get-wm-order-error-log")
    public RestResponse getWmOrderErrorLog(Integer wmMallStoreId, 
                                           @RequestParam(defaultValue = "1") int currentPage,
                                           @RequestParam(defaultValue = "20") int pageSize,
                                           String searchStr) {
        return ResultControllerUtils.commonPageResult(weimobOrderService
                .getErrWmOrderLog(wmMallStoreId, currentPage, pageSize, searchStr), WeimobOrderErLogVO.class);
    }

    /**
     * 批量处理订单错误日志
     * @param errLogIds
     * @param type (0--"不处理";1--"重新同步 ")
     * @return
     */
    @RequestMapping("delete-wm-order-error-log")
    public RestResponse deleteWmOrderErrorLog(String errLogIds, Integer type) {
        if (StringUtils.isBlank(errLogIds)) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数为空");
        }
        String[] errLogArr = errLogIds.split(",");
        List<Long> errLogList = new ArrayList<>(errLogArr.length);
        for (String errLog : errLogArr) {
            if (StringUtils.isBlank(errLog)) {
                continue;
            }
            errLogList.add(Long.valueOf(errLog));
        }
        return ResultControllerUtils.commonResult(weimobOrderService
                .dealErrOrderBatch(errLogList, type));
    }

    /**
     * 核销卡券
     * @param mcuId
     * @param couponCode
     * @param couponLogId
     * @return
     */
    @RequestMapping("consume-coupon")
    public RestResponse consumeCoupon(String mcuId, String couponCode, String couponLogId, Integer changeType) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        return ResultControllerUtils.commonResult(weimobMemberService.consumeCoupon(enterpriseId, Long.valueOf(mcuId),
                Long.valueOf(couponLogId), couponCode, changeType));
    }

    /**
     * 商品属性
     * @param wmMallStoreId
     * @return
     */
    @RequestMapping("get-goods-property")
    public RestResponse getGoodsPropertyList(Integer wmMallStoreId) {
        return ResultControllerUtils.commonResult(weimobMerchantService
                .getGoodsPropertyList(wmMallStoreId), WeimobGoodsPropertyVO.class);
    }

    /**
     * 获取微盟卡券列表
     * @Title: listWmCoupon 
     * @Description:
     * @author guojuxing
     * @param wmMallStoreId 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("/list-wm-coupon")
    public RestResponse listWmCoupon(Integer wmMallStoreId) {
        ServiceResponse<WmStoreDTO> wmStoreResponse = wmStoreApiService.getWmStoreByWmMallStoreId(wmMallStoreId);
        if (wmStoreResponse.isSuccess()) {
            return ResultControllerUtils.commonResult(weimobMerchantService
                    .getWeimobCouponTemplate(Long.valueOf(wmStoreResponse.getResult().getWmPid())), WmCouponVO.class);
        }
        return RestResponse.failure(wmStoreResponse.getCode(), wmStoreResponse.getMessage());
    }

    /**
     * 查询已经配置的会员卡
     * @param wmMallStoreId
     * @return
     */
    @RequestMapping("/get-has-config-member-card")
    public RestResponse getHasConfigMemberCard(Integer wmMallStoreId) {
        ServiceResponse<Long> result = weimobMerchantService.getMcuAreaId(wmMallStoreId);
        if (result.isSuccess()) {
            if (result.getResult() == null) {
                return RestResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "未配置会员卡，数据为空");
            }
            return RestResponse.success(result.getResult().toString());
        }
        return RestResponse.failure(result.getCode(), result.getMessage());
    }

    /**
     * 同步商品信息
     * @param wmMallStoreId
     * @return
     */
    @RequestMapping("/sync-goods")
    public RestResponse syncGoods(Integer wmMallStoreId) {
        ServiceResponse<WmStoreDTO> response = wmStoreApiService.getWmStoreByWmMallStoreId(wmMallStoreId);
        if (response.isSuccess()) {
            WmStoreDTO wmStoreDTO = response.getResult();
            ServiceResponse<Void> goodsResponse = weiMobGoodsService
                    .saveEnterpriseIdAndPid(UserDetailUtils.getUserDetail().getEnterpriseId(), Long.valueOf(wmStoreDTO.getWmPid()));
            return ResultControllerUtils.commonResult(goodsResponse);
        }
        return RestResponse.failure(response.getCode(), response.getMessage());
    }

    /**
     * gic卡券列表
     * @param wmMallStoreId
     * @param cardName
     * @return
     */
    @RequestMapping("/list-gic-coupon")
    public RestResponse listGicCoupon(Integer wmMallStoreId, String cardName) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        //获取已配置的卡券ID
        ServiceResponse<Long> mcuIdResponse = weimobMerchantService.getMcuAreaId(wmMallStoreId);
        if (!mcuIdResponse.isSuccess()) {
            return RestResponse.failure(mcuIdResponse.getCode(), mcuIdResponse.getMessage());
        }
        LOGGER.info("gic卡券查询参数:{}-{}-{}", enterpriseId, mcuIdResponse.getResult(), cardName);
        ServiceResponse<List<CouponDTO>> gicCouponResponse = memberCouponApiService
                .listGicCoupon(enterpriseId, mcuIdResponse.getResult(), cardName);
        return ResultControllerUtils.commonResult(gicCouponResponse, CouponVO.class);
    }

    /**
     * 授权回调
     * @param code
     */
    @RequestMapping("auth-callback")
    @HeaderSignIgnore
    public RestResponse authCallBack(HttpServletResponse response, String code, Integer state) throws IOException {
        try {
            ServiceResponse<Void> authWmResponse = weimobMerchantService.weimobStoreAuth(state, code);
            LOGGER.info("微盟中间件返回结果：{}", JSON.toJSONString(authWmResponse));
            if (authWmResponse.isSuccess()) {
                //更新gic侧店铺授权状态
                ServiceResponse<String> wmMallStoreResponse = wmStoreApiService.authWmStore(state);
                if (!wmMallStoreResponse.isSuccess()) {
                    LOGGER.warn("企业侧微盟店铺授权结果:{}", JSON.toJSONString(wmMallStoreResponse));
                }
                //重定向
                response.setStatus(302);
                response.setHeader("location", DomainUtils.getCurrentDomain() + "/weimo/auth-result.html");
                return RestResponse.success();
            }
            return RestResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), authWmResponse.getMessage());
        } catch (Exception e) {
            LOGGER.warn("微盟授权返回结果:{}", e.getMessage(), e);
            return RestResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), e.getMessage());
        }
    }

    @RequestMapping("/list-mall-mode")
    public RestResponse listMallMode() {
        Map<String, String> result = new HashMap<>(16);
        for (MallModeEnum modeEnum : MallModeEnum.values()) {
            result.put(String.valueOf(modeEnum.getCode()), modeEnum.getMsg());
        }
        return RestResponse.success(result);
    }

    /**
     * 会员卡配置-会员卡列表
     * @Title: listMemberCard 
     * @Description:
     * @author guojuxing  
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("/list-member-card")
    public RestResponse listMemberCard() {
        return ResultControllerUtils.commonResult(memberCardApiService.getAll(UserDetailUtils.getUserDetail().getEnterpriseId()), MemberCardVO.class);
    }

    /**
     * 获取积分配置的开关
     * @Title: getIntegralConfig 
     * @Description:
     * @author guojuxing
     * @param wmMallStoreId 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("/get-integral-config")
    public RestResponse getIntegralConfig(Integer wmMallStoreId) {
        //获取已配置的卡券ID
        return ResultControllerUtils.commonResult(orderPointCalculationPartyApiService
                .isGicCalculateIntegral(UserDetailUtils.getUserDetail().getEnterpriseId(),
                        OrderPointCalculationPartyEnum.WEIMO.getCode(),
                        Long.valueOf(wmMallStoreId)));
    }

    /**
     * 编辑积分配置开关
     * @Title: editIntegralConfig 
     * @Description:
     * @author guojuxing
     * @param wmMallStoreId
     * @param integralRule gic或者erp）
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("/edit-integral-config")
    public RestResponse editIntegralConfig(Integer wmMallStoreId, String integralRule) {
        //1：gic计算 0:erp计算
        int state = 1;
        if (Constants.ERP.equals(integralRule)) {
            state = 0;
        }
        //获取已配置的卡券ID
        return ResultControllerUtils.commonResult(orderPointCalculationPartyApiService
                .updateIntegralRule(UserDetailUtils.getUserDetail().getEnterpriseId(),
                        OrderPointCalculationPartyEnum.WEIMO.getCode(),
                        Long.valueOf(wmMallStoreId),
                        state));
    }

    @RequestMapping("/list-wm-store-config")
    public RestResponse listWmStoreConfig() {
        Map<String, String> result = new HashMap<>(16);
        for (WmStoreConfigTypeEnum modeEnum : WmStoreConfigTypeEnum.values()) {
            if (modeEnum.isHasConfigData()) {
                result.put(String.valueOf(modeEnum.getCode()), modeEnum.getMessage());
            }
        }
        return RestResponse.success(result);
    }

    @RequestMapping("/save-or-update-wm-store-config")
    public RestResponse saveOrUpdateWmStoreConfig(WmStoreConfigDTO dto) {
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return OperationResultUtils
                .operationResult(wmStoreApiService.saveOrUpdateWmStoreConfig(dto),
                        "店铺配置"
                                + WmStoreConfigTypeEnum.getMessageByCode(dto.getStoreConfigType()),
                        "微盟商城");
    }

    /**
     * 获取店铺配置
     * @param wmMallStoreId
     * @param wmStoreConfigType
     * @return
     */
    @RequestMapping("/get-wm-store-config")
    public RestResponse getWmStoreConfig(Integer wmMallStoreId, Integer wmStoreConfigType) {
        return ResultControllerUtils
                .commonResult(wmStoreApiService.getWmStoreConfig(wmMallStoreId, wmStoreConfigType));
    }

    @RequestMapping("/get-wm-token")
    public RestResponse getWmToken(String pid) {
        return RestResponse.success(weimobBaseService.getWmStoreByPid(Long.valueOf(pid)));
    }

    /**
     * 绑定gic卡券和微盟卡券
     * @param dto
     * @return
     */
    @RequestMapping("/bind-coupon")
    public RestResponse bindCoupon(WmStoreCouponConfigDTO dto) {
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        Map<String, CouponDTO> gicCouponMap = getGicCoupon(dto.getWmMallStoreId());
        CouponDTO couponDTO = gicCouponMap.get(dto.getGicCouponId());
        if (couponDTO == null) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "gic卡券不存在");
        }
        //计算过期时间
        if (couponDTO.getCardEffectiveMode().intValue() == 0) {
            //固定日期，需要给一个过期时间
            dto.setGicCouponExpire(couponDTO.getExpireDate());
        }
        return OperationResultUtils
                .operationResult(wmStoreApiService.saveCouponConfig(dto),
                        "绑定卡券"
                                + dto.getGicCouponName() + ":" + dto.getWmCouponName(),
                        "微盟商城");
    }

    /**
     * gic卡券关联微盟卡券列表
     * @param params
     * @return
     */
    @RequestMapping("/page-coupon-config")
    public RestResponse pageCouponConfig(CouponConfigQO params) {
        return ResultControllerUtils
                .commonResult(wmStoreApiService.pageCouponConfig(params));
    }

    private Map<String, CouponDTO> getGicCoupon(Integer wmMallStoreId) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        //获取已配置的卡券ID
        ServiceResponse<Long> mcuIdResponse = weimobMerchantService.getMcuAreaId(wmMallStoreId);
        if (!mcuIdResponse.isSuccess()) {
            return new HashMap<>(2);
        }
        ServiceResponse<List<CouponDTO>> gicCouponResponse = memberCouponApiService
                .listGicCoupon(enterpriseId, mcuIdResponse.getResult(), "");
        if (gicCouponResponse.isSuccess()) {
            List<CouponDTO> list = gicCouponResponse.getResult();
            return list.stream().collect(Collectors.toMap(e -> e.getCouponId().toString(), e -> e));
        }
        return new HashMap<>(2);
    }

    private boolean hasCouponSet(Integer wmMallStoreId) {
        ServiceResponse<Page<WmStoreCouponConfigDTO>> result = wmStoreApiService
                .pageCouponConfig(new CouponConfigQO().setWmMallStoreId(wmMallStoreId));
        if (result.isSuccess()) {
            Page<WmStoreCouponConfigDTO> page = result.getResult();
            if (page != null && CollectionUtils.isNotEmpty(page.getResult())) {
                return true;
            }
        }
        return false;
    }
}
