package com.gic.enterprise.web.controller.wm;

import java.io.*;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.usermodel.*;
import org.redisson.api.RMap;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.service.UserApiService;
import com.gic.commons.annotation.HeaderSignIgnore;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.ExecutorPoolSingleton;
import com.gic.enterprise.constant.Constants;
import com.gic.enterprise.constant.MallModeEnum;
import com.gic.enterprise.dto.WmStoreDTO;
import com.gic.enterprise.dto.wm.WmStoreSyncLogDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.WmStoreApiService;
import com.gic.enterprise.service.WmStoreSyncLogApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.enterprise.web.vo.wm.WeimobStoreClerkLogVO;
import com.gic.enterprise.web.vo.wm.WmBranchVO;
import com.gic.redis.data.util.RedisUtil;
import com.gic.store.dto.*;
import com.gic.store.service.ClerkApiService;
import com.gic.store.service.StoreApiService;
import com.gic.store.service.StoreRegionApiService;
import com.gic.weimob.api.dto.WeimobPhysicalStoreDTO;
import com.gic.weimob.api.dto.WeimobStoreAccountDTO;
import com.gic.weimob.api.dto.WeimobStoreGicInfoDTO;
import com.gic.weimob.api.service.WeimobStoreSiteApiService;

/**
 * 同步门店to微盟
 * @ClassName: WmStoreSyncController 
 * @Description:  
 * @author guojuxing 
 * @date 2020/5/8 11:19 AM 
 */
@RestController
@RequestMapping("/wm-store-sync")
public class WmStoreSyncController {
    private static final Logger LOGGER = LogManager.getLogger(WmStoreSyncController.class);

    private static Map<String, String> BUSINESS_TIME_MAP = new HashMap<>(16);
    static {
        BUSINESS_TIME_MAP.put("1", "周一");
        BUSINESS_TIME_MAP.put("2", "周二");
        BUSINESS_TIME_MAP.put("3", "周三");
        BUSINESS_TIME_MAP.put("4", "周四");
        BUSINESS_TIME_MAP.put("5", "周五");
        BUSINESS_TIME_MAP.put("6", "周六");
        BUSINESS_TIME_MAP.put("7", "周日");
    }

    @Autowired
    private WmStoreApiService wmStoreApiService;
    @Autowired
    private WeimobStoreSiteApiService weimobStoreSiteService;
    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private StoreRegionApiService storeRegionApiService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private WmStoreSyncLogApiService wmStoreSyncLogApiService;
    @Autowired
    private ClerkApiService clerkApiService;


    private static final String REDIS_KEY = "enterprise:weimob4:init:";
    /**
     * 分店导入redis日志key
     */
    private static final String STORE_BRANCH_REDIS_KEY = "enterprise:weimob4:init:branch:";

    @RequestMapping("list-store")
    public RestResponse listStore(StoreSearchDTO storeSearchDTO, Integer wmMallStoreId) {
        storeSearchDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        storeSearchDTO.setStatus(1);
        //已完善
        storeSearchDTO.setCompleteStatus(1);
        ServiceResponse<List<WmStoreSyncLogDTO>> hasSyncStoreListResult = wmStoreSyncLogApiService.getStoreList(null, wmMallStoreId);
        //过滤已同步过的门店
        if (hasSyncStoreListResult.isSuccess()) {
            StringBuilder sb = new StringBuilder();
            hasSyncStoreListResult.getResult().forEach(e -> sb.append(e.getStoreId()).append(" "));
            storeSearchDTO.setStoreIdsOfNot(sb.toString());
        }
        return ResultControllerUtils.commonResult(storeApiService
                .listStore(storeSearchDTO, storeSearchDTO.getCurrentPage(), storeSearchDTO.getPageSize()));
    }

    /**
     * 查询分店
     * @param wmMallStoreId
     * @param storeId
     * @return
     */
    @RequestMapping("list-sub-by-store-code")
    public RestResponse listSubByStoreCode(Integer wmMallStoreId, Integer storeId) {
        StoreDTO storeDTO = getByStoreId(UserDetailUtils.getUserDetail().getEnterpriseId(), storeId);
        if (storeDTO == null) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店不存在");
        }
        ServiceResponse<List<WeimobStoreGicInfoDTO>> result = weimobStoreSiteService.listStoreId(wmMallStoreId, storeDTO.getStoreInfoId());
        if (result.isSuccess()) {
            List<WeimobStoreGicInfoDTO> subStoreList = result.getResult();
            if (CollectionUtils.isNotEmpty(subStoreList)) {
                StringBuilder sb = new StringBuilder();
                subStoreList.forEach(e -> {
                    sb.append(e.getStoreInfoId()).append(",");
                });
                List<StoreDTO> list = getStoreList(sb.toString(), true);
                Map<String, String> map = list
                        .stream()
                        .collect(Collectors.toMap(e -> e.getStoreInfoId().toString(), StoreDTO::getStoreName));

                List<WmBranchVO> voList = EntityUtil.changeEntityListNew(WmBranchVO.class, subStoreList);
                voList.forEach(vo -> vo.setStoreName(map.get(vo.getStoreInfoId().toString())));
                return RestResponse.success(voList);
            }
        }
        return RestResponse.success();
    }

    /**
     *
     * @param request
     * @param response
     * @throws IOException
     */
    @HeaderSignIgnore
    @RequestMapping("wm-store-import-template")
    public void wmStoreImportTemplate(HttpServletRequest request, HttpServletResponse response)
            throws IOException {
        ClassPathResource res = new ClassPathResource("exceltemplate/wm_store_import_template.xls");
        String userAgent = request.getHeader("User-Agent").toLowerCase();
        String fileName = "门店同步模板.xls";
        String firefox = "firefox";
        if (userAgent.contains(firefox)) {
            byte[] bytesName = fileName.getBytes("UTF-8");
            fileName = new String(bytesName, "ISO-8859-1");
            response.setHeader("content-disposition", "attachment;fileName=" + fileName);
        } else {
            response.setHeader("content-disposition", "attachment;fileName=" + URLEncoder.encode(fileName, "UTF-8"));
        }

        InputStream is = res.getInputStream();
        OutputStream out = response.getOutputStream();
        int bytes = 0;
        byte[] bufferOut = new byte[1024];
        while ((bytes = is.read(bufferOut)) != -1) {
            out.write(bufferOut, 0, bytes);
        }

        out.close();
        is.close();
    }

    /**
     * 总店-分店同步
     * @Title: syncHeadStore 
     * @Description:
     * @author guojuxing
     * @param wmMallStoreId 店铺逐渐ID
     * @param headStoreId 总店ID
     * @param storeIds  分店IDs
     * @param isBranch 是否是添加分店入口场景
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("sync-head-store")
    public RestResponse syncHeadStore(Integer wmMallStoreId, Integer headStoreId, String storeIds, Boolean isBranch) {
        if (!validWmMode(wmMallStoreId)) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(),
                    "非" + MallModeEnum.SMART_RETAIL.getMsg() + "不能同步门店");
        }

        if (headStoreId == null) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "总店参数为空");
        }

        if (isBranch == null) {
            isBranch = false;
        }

        if (!isBranch && getHasSyncStore(wmMallStoreId).contains(headStoreId.toString())) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "重复同步");
        }

        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        //查询总店
        ServiceResponse<StoreDTO> storeResponse = storeApiService.getStoreById(enterpriseId, headStoreId);
        if (!storeResponse.isSuccess()) {
            return RestResponse.failure(storeResponse.getCode(), storeResponse.getMessage());
        }
        //装载总店数据为微盟数据
        StoreDTO storeDTO = storeResponse.getResult();
        WeimobPhysicalStoreDTO weimobPhysicalStoreDTO = getWeimobPhsicalStore(storeDTO);
        if (weimobPhysicalStoreDTO == null) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "总店不存在或者未启用");
        }
        //分店
        List<WeimobPhysicalStoreDTO> subStoreList = new ArrayList<>();
        if (StringUtils.isNotBlank(storeIds)) {
            List<StoreDTO> storeList = getStoreList(storeIds, false);
            //循环装载微盟门店数据
            storeList.forEach(tempStore -> subStoreList.add(getWeimobPhsicalStore(tempStore)));
        }
        ServiceResponse<Void> wmResult = weimobStoreSiteService
                .addWeimobVirtualStore(wmMallStoreId, enterpriseId, weimobPhysicalStoreDTO, subStoreList);
        LOGGER.info("总店同步-微盟结果:{}", JSON.toJSONString(wmResult));
        if (wmResult.isSuccess()) {
            wmStoreSyncLogApiService.saveWmStoreSyncLog(Arrays.asList(headStoreId), wmMallStoreId, 1, enterpriseId);
            //同步导购
            List<Integer> storeIdList = new ArrayList<>();
            if (!isBranch) {
                storeIdList.add(headStoreId);
            }
            storeIdList.addAll(getListByStr(storeIds));
            clerkApiService.syncClerkToWeimob(wmMallStoreId, storeIdList, enterpriseId);
            return RestResponse.success();
        }
        return RestResponse.failure(wmResult.getCode(), wmResult.getMessage());
    }

    /**
     * 已同步的门店列表
     * @Title: listWmStoreLog 
     * @Description:
     * @author guojuxing
     * @param search 门店code/门店名称
     * @param type 1：总店 2：一对一
     * @param wmMallStoreId 店铺主键ID
     * @param currentPage
     * @param pageSize 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("list-wm-has-sync-store")
    public RestResponse listWmStoreLog(String search, Integer type, Integer wmMallStoreId,
                                 @RequestParam(defaultValue = "1") int currentPage, @RequestParam(defaultValue = "20") int pageSize) {
        return ResultControllerUtils.commonResult(wmStoreSyncLogApiService
                .pageWmStoreSyncLog(search, type, wmMallStoreId, currentPage, pageSize));
    }

    /**
     * 解除门店同步日志记录
     * @param storeId
     * @return
     */
    @RequestMapping("delete-logic")
    public RestResponse deleteLogic(Integer storeId, Integer wmMallStoreId) {
        if (storeId == null) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数为空");
        }
        StoreDTO storeDTO = getByStoreId(UserDetailUtils.getUserDetail().getEnterpriseId(), storeId);
        if (storeDTO == null) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "门店不存在");
        }

        wmStoreSyncLogApiService.deleteLogic(storeId, wmMallStoreId);

        ServiceResponse<Void> weimobResult = weimobStoreSiteService.deleteStoreRel(wmMallStoreId, storeDTO.getStoreInfoId());
        LOGGER.info("微盟中间件删除门店关联关系结果:{}", JSON.toJSONString(weimobResult));
        if (!weimobResult.isSuccess()) {
            return RestResponse.failure(weimobResult.getCode(), weimobResult.getMessage());
        }
        return RestResponse.success();
    }

    /**
     * 门店同步错误日志
     * @param wmMallStoreId 店铺ID
     * @param logType -- 非必填，null时取全部，STORE取门店,CLERK取导购
     * @param pageSize
     * @param currentPage
     * @param search
     * @return
     */
    @RequestMapping("list-store-clerk-error-log")
    public RestResponse listStoreClerkErrorLog(Integer wmMallStoreId, String logType,
                                               @RequestParam(defaultValue="20") int pageSize,
                                               @RequestParam(defaultValue="1")int currentPage, String search) {
        return ResultControllerUtils.commonPageResult(weimobStoreSiteService
                .getStoreClerkLog(wmMallStoreId, logType, currentPage, pageSize, search), WeimobStoreClerkLogVO.class);
    }

    /**
     * 导购重新同步
     * @param clerkIds 英文逗号隔开
     * @param wmMallStoreId
     * @return
     */
    @RequestMapping("/sync-clerk-to-wm-again")
    public RestResponse syncClerkToWmAgain(String clerkIds, Integer wmMallStoreId) {
        return ResultControllerUtils.commonResult(clerkApiService
                .syncClerkToWeimob(wmMallStoreId, clerkIds, UserDetailUtils.getUserDetail().getEnterpriseId()));
    }

    /**
     * 批量重新同步门店
     * @param storeInfoIds
     * @param wmMallStoreId
     * @return
     */
    @RequestMapping("/sync-store-to-wm-again")
    public RestResponse syncStoreToWmAgain(String storeInfoIds, Integer wmMallStoreId) {
        //区分是一对一还是总店
        List<StoreDTO> storeList = getStoreList(storeInfoIds, true);
        if (CollectionUtils.isNotEmpty(storeList)) {
            List<Integer> storeIdList = storeList.stream().map(e -> e.getStoreId()).collect(Collectors.toList());
            ServiceResponse<List<WmStoreSyncLogDTO>> wmStoreLogResult = wmStoreSyncLogApiService.listByStoreId(storeIdList, wmMallStoreId);
            if (wmStoreLogResult.isSuccess()) {
                List<WmStoreSyncLogDTO> wmStoreLogList = wmStoreLogResult.getResult();
                List<Integer> headStoreIdList = wmStoreLogList
                        .stream()
                        .filter(e -> e.getStoreType().intValue() == 1)
                        .map(e -> e.getStoreId())
                        .collect(Collectors.toList());
                //同步总店
                headStoreIdList.forEach(head -> {
                    LOGGER.info("批量重新同步门店(总店):{}", JSON.toJSONString(syncHeadStore(wmMallStoreId, head, null, false)));
                });
                StringBuilder storeIds = new StringBuilder();
                wmStoreLogList.stream().filter(e -> e.getStoreType().intValue() == 2)
                        .map(e -> e.getStoreId())
                        .forEach(storeId -> storeIds.append(storeId).append(","));
                LOGGER.info("批量重新同步门店(一对一):{}", JSON.toJSONString(weimobInitStore(wmMallStoreId, storeIds.toString())));
            }
        }
        return RestResponse.success();
    }

    /**
     * 门店错误日志-不处理
     * @Title: deleteStoreClerkErrorLog 
     * @Description:
     * @author guojuxing
     * @param ids
     * @return  com.gic.commons.webapi.reponse.RestResponse 
     */
    @RequestMapping("delete-store-clerk-error-log")
    @ResponseBody
    public RestResponse deleteStoreClerkErrorLog(String ids) {
        if (StringUtils.isNotBlank(ids)) {
            Arrays.stream(ids.split(",")).forEach( e -> weimobStoreSiteService.deleteStoreClerkErrorlog(Long.valueOf(e)));
        }
        return RestResponse.success();
    }

    /**
     * 一对一门店同步
     * @param wmMallStoreId
     * @param storeIds
     * @return
     */
    @RequestMapping("/weimob-init-store")
    public RestResponse weimobInitStore(Integer wmMallStoreId, String storeIds) {
        if (StringUtils.isBlank(storeIds)) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数为空");
        }
        if (!validWmMode(wmMallStoreId)) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(),
                    "非" + MallModeEnum.SMART_RETAIL.getMsg() + "不能同步门店");
        }
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();

        List<WeimobPhysicalStoreDTO> physicalStoreList = new ArrayList<>();
        List<StoreDTO> storeList = getStoreList(storeIds, false);
        storeList.forEach(store -> physicalStoreList.add(getWeimobPhsicalStore(store)));
        if (CollectionUtils.isEmpty(physicalStoreList)) {
            return RestResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "门店数据非启用");
        }
        ServiceResponse<List<Integer>> wmResult = weimobStoreSiteService
                .addWeimobPhysicalStoreBatch(wmMallStoreId, enterpriseId, physicalStoreList);
        LOGGER.info("实体门店批量到微盟结果：{}", JSON.toJSONString(wmResult));

        if (!wmResult.isSuccess()) {
            HashMap<Integer, String> map = JSON.parseObject(wmResult.getMessage(), HashMap.class);
            StringBuilder errorIds = new StringBuilder();
            Map<String, String> errorMessage = new HashMap<>(16);
            for (Map.Entry<Integer, String> entry : map.entrySet()) {
                errorIds.append(entry.getKey()).append(GlobalInfo.FLAG_COMMA);
                errorMessage.put(entry.getKey().toString(), entry.getValue());
            }

            this.wmStoreSyncLogApiService.saveWmStoreSyncLog(wmResult.getResult(),
                    wmMallStoreId, 2, enterpriseId);

            //导购同步 成功部分门店进行导购同步
            clerkApiService.syncClerkToWeimob(wmMallStoreId, wmResult.getResult(), enterpriseId);

            StringBuilder errorMessageResult = new StringBuilder();
            List<StoreDTO> hasSuccessStoreList = getStoreList(errorIds.toString(), true);
            hasSuccessStoreList.forEach(e -> errorMessageResult.append(e.getRegionId() + "-" + e.getStoreCode()).append(
                    errorMessage.get(e.getStoreInfoId().toString())));
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), errorMessageResult.toString());
        } else {
            this.wmStoreSyncLogApiService.saveWmStoreSyncLog(storeList.stream().map(e -> e.getStoreId()).collect(Collectors.toList()),
                    wmMallStoreId, 2, enterpriseId);
            //导购同步
            clerkApiService.syncClerkToWeimob(wmMallStoreId, getListByStr(storeIds), enterpriseId);
        }
        return RestResponse.success();
    }

    /**
     * 一对一门店导入同步日志
     * @return
     */
    @RequestMapping("/weimob-init-error-download-process")
    public RestResponse storeImportTemplateDownload() {
        return getExportProcess(REDIS_KEY);
    }

    /**
     * 分店导入同步日志
     * @return
     */
    @RequestMapping("/weimob-init-branch-error-download-process")
    public RestResponse storeBranchExportProcess() {
        return getExportProcess(STORE_BRANCH_REDIS_KEY);
    }

    /**
     * 一对一导入日志
     * @param request
     * @param response
     * @return
     * @throws Exception
     */
    @RequestMapping("weimob-init-error-download")
    @HeaderSignIgnore
    public RestResponse downloadError(HttpServletRequest request, HttpServletResponse response) throws Exception {
        return downloadExportLog(REDIS_KEY, request, response);
    }

    /**
     * 分店导入日志
     * @param request
     * @param response
     * @return
     * @throws Exception
     */
    @RequestMapping("weimob-init-branch-error-download")
    public RestResponse downloadErrorBranch(HttpServletRequest request, HttpServletResponse response) throws Exception {
        return downloadExportLog(STORE_BRANCH_REDIS_KEY, request, response);
    }

    /**
     * 总店-分店Excel导入
     * @param wmMallStoreId 店铺ID
     * @param headStoreId 总店ID
     * @param isBranch 是否是添加分店
     * @param file
     * @return
     * @throws IOException
     */
    @HeaderSignIgnore
    @RequestMapping("weimob-init-branch-import")
    public RestResponse importSettlementBranch(Integer wmMallStoreId, Integer headStoreId, boolean isBranch, MultipartFile file) throws IOException {
        if (file == null || file.isEmpty()) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "Excel文件为空");
        }
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();

        if (!validWmMode(wmMallStoreId)) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(),
                    "非" + MallModeEnum.SMART_RETAIL.getMsg() + "不能同步门店");
        }

        ServiceResponse<StoreDTO> headStoreResponse = storeApiService.getStoreById(enterpriseId, headStoreId);
        if (!headStoreResponse.isSuccess()) {
            return RestResponse.failure(headStoreResponse.getCode(), headStoreResponse.getMessage());
        }
        StoreDTO headStore = headStoreResponse.getResult();
        if (headStore == null || headStore.getStatus() == null || headStore.getStatus().intValue() != 1) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "总店不存在或者未启用");
        }

        InputStream in = file.getInputStream();
        List<Map<Integer, String>> data = readXlsOfBranch(in, 2);
        if (data.size() > 0 && data.get(0).size() != Constants.TWO) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "总列数与模板不一致");
        }
        //redis错误日志对象
        RedissonClient redisClient = RedisUtil.getRedisClient();
        RMap<String, String> rMap = redisClient.getMap(STORE_BRANCH_REDIS_KEY + enterpriseId);
        rMap.clear();
        if (CollectionUtils.isNotEmpty(data)) {
            rMap.put("门店同步总数", data.size() + "");
            //异步 同步给微盟
            ExecutorPoolSingleton.getInstance().executeTask(new MyTaskOfBranchStore(storeApiService, clerkApiService, wmStoreSyncLogApiService,
                    weimobStoreSiteService, enterpriseId, data, headStoreId, storeRegionApiService, wmMallStoreId, isBranch, false));
            return RestResponse.success();
        } else {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "文件格式错误");
        }
    }

    /**
     * 一对一Excel导入
     * @param wmMallStoreId
     * @param file
     * @return
     * @throws IOException
     */
    @HeaderSignIgnore
    @RequestMapping("weimob-init-import")
    public RestResponse importSettlement(Integer wmMallStoreId, MultipartFile file) throws IOException {
        if (file == null || file.isEmpty()) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "Excel文件为空");
        }
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();

        if (!validWmMode(wmMallStoreId)) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(),
                    "非" + MallModeEnum.SMART_RETAIL.getMsg() + "不能同步门店");
        }

        InputStream in = file.getInputStream();
        List<Map<Integer, String>> data = readXlsOfBranch(in, 2);
        if (data.size() > 0 && data.get(0).size() != Constants.TWO) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "总列数与模板不一致");
        }

        //redis错误日志对象
        RedissonClient redisClient = RedisUtil.getRedisClient();
        RMap<String, String> rMap = redisClient.getMap(REDIS_KEY + enterpriseId);
        rMap.clear();
        if (CollectionUtils.isNotEmpty(data)) {
            rMap.put("门店同步总数", data.size() + "");
            //异步 同步给微盟
            ExecutorPoolSingleton.getInstance().executeTask(new MyTaskOfBranchStore(storeApiService, clerkApiService, wmStoreSyncLogApiService,
                    weimobStoreSiteService, enterpriseId, data, null, storeRegionApiService, wmMallStoreId, false, true));
            return RestResponse.success();
        } else {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "文件格式错误");
        }
    }

    static class MyTaskOfBranchStore implements Runnable {
        private StoreApiService storeApiService;
        private ClerkApiService clerkApiService;
        private WmStoreSyncLogApiService wmStoreSyncLogApiService;
        private WeimobStoreSiteApiService weimobStoreSiteService;
        private Integer enterpriseId;
        private List<Map<Integer, String>> data;

        private Integer headerStoreId;
        private StoreRegionApiService storeRegionApiService;
        private Integer wmMallStoreId;
        private boolean isBranch;
        /**
         * 是否是一对一门店导入同步 是 否
         */
        private boolean oneAndOne;

        public MyTaskOfBranchStore(StoreApiService storeApiService, ClerkApiService clerkApiService,
                                   WmStoreSyncLogApiService wmStoreSyncLogApiService,
                                   WeimobStoreSiteApiService weimobStoreSiteService, Integer enterpriseId, List<Map<Integer, String>> data,
                                   Integer headerStoreId, StoreRegionApiService storeRegionApiService, Integer wmMallStoreId,
                                   boolean isBranch, boolean oneAndOne) {
            this.storeApiService = storeApiService;
            this.clerkApiService = clerkApiService;
            this.wmStoreSyncLogApiService = wmStoreSyncLogApiService;
            this.weimobStoreSiteService = weimobStoreSiteService;
            this.enterpriseId = enterpriseId;
            this.data = data;
            this.headerStoreId = headerStoreId;
            this.storeRegionApiService = storeRegionApiService;
            this.wmMallStoreId = wmMallStoreId;
            this.isBranch = isBranch;
            this.oneAndOne = oneAndOne;
        }

        @Override
        public void run() {
            try {
                if (oneAndOne) {
                    this.setOneAndOne();
                } else {
                    RedissonClient redisClient = RedisUtil.getRedisClient();
                    RMap<String, String> rMap = redisClient.getMap(STORE_BRANCH_REDIS_KEY + enterpriseId);
                    StoreDTO headStore = storeApiService.getStoreById(enterpriseId, headerStoreId).getResult();
                    WeimobPhysicalStoreDTO wmPhysicalStoreDTO = getWeimobPhsicalStore(headStore);
                    //标志位，如果成功，则需要把总店下面的导购也同步一下
                    int valid = 0;
                    for (Map<Integer, String> row : data) {
                        String regionName = row.get(0);
                        String storeCode = row.get(1);
                        StoreRegionDTO storeRegion = getStoreRegion(enterpriseId, regionName);
                        if (storeRegion == null) {
                            rMap.put(regionName + "-" + storeCode, "门店域不存在");
                            continue;
                        }
                        StoreDTO storeDTO = getStoreByRegionAndCode(enterpriseId, storeRegion.getRegionId(), storeCode);
                        if (storeDTO == null) {
                            rMap.put(regionName + "-" + storeCode, "门店code不存在");
                            continue;
                        }
                        List<WeimobPhysicalStoreDTO> list = new ArrayList<>(1);
                        list.add(getWeimobPhsicalStore(storeDTO));
                        ServiceResponse<Void> wmResult = this.weimobStoreSiteService
                                .addWeimobVirtualStore(wmMallStoreId, enterpriseId, wmPhysicalStoreDTO, list);
                        LOGGER.info("微盟同步总店-分店返回结果:{}", JSON.toJSONString(wmResult));
                        if (!wmResult.isSuccess()) {
                            rMap.put(regionName + "-" + storeCode, wmResult.getMessage());
                        } else {
                            rMap.put(regionName + "-" + storeCode, "成功");
                            ++valid;
                            List<Integer> tempStoreId = new ArrayList<>(1);
                            if (valid == 1 && !isBranch) {
                                //只执行一次总店导购的同步
                                tempStoreId.add(headerStoreId);
                                //新增企业侧日志
                                this.wmStoreSyncLogApiService.saveWmStoreSyncLog(Arrays.asList(headerStoreId), wmMallStoreId, 1, enterpriseId);
                            }
                            tempStoreId.add(storeDTO.getStoreId());
                            // 导购同步
                            clerkApiService.syncClerkToWeimob(wmMallStoreId, tempStoreId, enterpriseId);
                        }
                    }
                }
            } catch (Exception e) {
                LOGGER.warn("微盟总店同步线程日志:{}", e.getMessage(), e);
            }
        }

        private void setOneAndOne(){
            RedissonClient redisClient = RedisUtil.getRedisClient();
            RMap<String, String> rMap = redisClient.getMap(REDIS_KEY + enterpriseId);
            for (Map<Integer, String> row : data) {
                String regionName = row.get(0);
                String storeCode = row.get(1);
                StoreRegionDTO storeRegion = getStoreRegion(enterpriseId, regionName);
                if (storeRegion == null) {
                    rMap.put(regionName + "-" + storeCode, "门店域不存在");
                    continue;
                }
                StoreDTO storeDTO = getStoreByRegionAndCode(enterpriseId, storeRegion.getRegionId(), storeCode);
                if (storeDTO == null) {
                    rMap.put(regionName + "-" + storeCode, "门店code不存在");
                    continue;
                }
                List<WeimobPhysicalStoreDTO> list = new ArrayList<>(1);
                WeimobPhysicalStoreDTO obj = getWeimobPhsicalStore(storeDTO);
                if (obj != null) {
                    ServiceResponse<Boolean> existStoreResponse = this.weimobStoreSiteService.existWeimobPhysicalStore(wmMallStoreId,
                            enterpriseId, storeDTO.getStoreInfoId());
                    if (existStoreResponse.isSuccess()) {
                        if (!existStoreResponse.getResult()) {
                            list.add(obj);
                        }
                    }
                }
                if (CollectionUtils.isEmpty(list)) {
                    //同步过的记录日志，不能再次同步
                    this.wmStoreSyncLogApiService.saveWmStoreSyncLog(Arrays.asList(storeDTO.getStoreId()),
                            wmMallStoreId, 2, enterpriseId);
                    // 导购同步
                    rMap.put(regionName + "-" + storeCode, "微盟侧已存在");
                    clerkApiService.syncClerkToWeimob(wmMallStoreId, Arrays.asList(storeDTO.getStoreId()), enterpriseId);
                } else {
                    LOGGER.info("实体门店同步到微盟的参数:{}", JSON.toJSONString(list));
                    ServiceResponse<List<Integer>> weimobResult = this.weimobStoreSiteService
                            .addWeimobPhysicalStoreBatch(wmMallStoreId, enterpriseId, list);
                    LOGGER.info("实体门店批量到微盟结果：{}", JSON.toJSONString(weimobResult));

                    if (!weimobResult.isSuccess()) {
                        HashMap<Integer, String> map = JSON.parseObject(weimobResult.getMessage(), HashMap.class);
                        StringBuilder errorIds = new StringBuilder();
                        Map<String, String> errorMessage = new HashMap<>(16);
                        for (Map.Entry<Integer, String> entry : map.entrySet()) {
                            errorIds.append(entry.getKey()).append(GlobalInfo.FLAG_COMMA);
                            errorMessage.put(entry.getKey().toString(), entry.getValue());
                        }
                        //因为是一条一条处理，所以失败了，就不会有成功的，不用 导购同步逻辑
                        List<StoreDTO> hasSuccessStoreList = getStoreList(errorIds.toString(), enterpriseId);
                        hasSuccessStoreList.forEach(e -> rMap.put(e.getRegionId() + "-" + e.getStoreCode(),
                                errorMessage.get(storeDTO.getStoreInfoId().toString())));
                    } else {
                        rMap.put(regionName + "-" + storeCode, "成功");
                        //同步过的记录日志，不能再次同步
                        this.wmStoreSyncLogApiService.saveWmStoreSyncLog(Arrays.asList(storeDTO.getStoreId()),
                                wmMallStoreId, 2, enterpriseId);
                        // 导购同步
                        clerkApiService.syncClerkToWeimob(wmMallStoreId, Arrays.asList(storeDTO.getStoreId()), enterpriseId);
                    }
                }

            }
        }

        private List<StoreDTO> getStoreList(String storeInfoIds, Integer enterpriseId) {
            String[] subStoreIdArr = storeInfoIds.split(",");
            //一次性查询分店列表数据
            //转为空格
            storeInfoIds = storeInfoIds.replaceAll(",", " ");
            StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
            storeSearchDTO.setStoreInfoIds(storeInfoIds);
            storeSearchDTO.setEnterpriseId(enterpriseId);
            ServiceResponse<Page<StoreDTO>> subStoreResponse = storeApiService.listStore(storeSearchDTO, 1, subStoreIdArr.length);
            if (subStoreResponse.isSuccess()) {
                Page<StoreDTO> page = subStoreResponse.getResult();
                if (page != null) {
                    List<StoreDTO> storeList = page.getResult();
                    return storeList;
                }
            }
            return Collections.emptyList();
        }

        private WeimobPhysicalStoreDTO getWeimobPhsicalStore(StoreDTO storeDTO) {
            if (storeDTO.getStatus().intValue() == 1) {
                WeimobPhysicalStoreDTO weimobPhysicalStoreDTO = new WeimobPhysicalStoreDTO();
                //用storeInfoId
                weimobPhysicalStoreDTO.setStoreInfoId(storeDTO.getStoreInfoId());
                weimobPhysicalStoreDTO.setStoreNumber(storeDTO.getStoreCode());
                weimobPhysicalStoreDTO.setStoreName(storeDTO.getStoreName());
                //经纬度
                if (CollectionUtils.isNotEmpty(storeDTO.getLocation())) {
                    weimobPhysicalStoreDTO.setLongitude(new BigDecimal(storeDTO.getLocation().get(0)));
                    weimobPhysicalStoreDTO.setLatitude(new BigDecimal(storeDTO.getLocation().get(1)));
                } else {
                    if (org.apache.commons.lang.StringUtils.isNotBlank(storeDTO.getLongitude())) {
                        weimobPhysicalStoreDTO.setLongitude(new BigDecimal(storeDTO.getLongitude()));
                    }
                    if (org.apache.commons.lang.StringUtils.isNotBlank(storeDTO.getLatitude())) {
                        weimobPhysicalStoreDTO.setLatitude(new BigDecimal(storeDTO.getLatitude()));
                    }
                }
                //省市区名称
                String fullArea = StringUtils.isNotBlank(storeDTO.getProvinces()) ?
                        storeDTO.getProvinces().replaceAll("//", "") : "";
                weimobPhysicalStoreDTO.setAddress(fullArea + storeDTO.getAddress());
                StringBuilder businessTime = new StringBuilder();
                SimpleDateFormat simpleDateFormat = new SimpleDateFormat("HH:mm");
                List<StoreBusinessTimeDTO> businessTimeList = storeDTO.getBusinessTimeList();
                if (CollectionUtils.isNotEmpty(businessTimeList)) {
                    businessTimeList.stream().map(e -> {
                        String businessHours = simpleDateFormat.format(e.getOpenTime()) + "-" + simpleDateFormat.format(e.getCloseTime());
                        return businessTime(e.getWeekday()) + ":" + businessHours;
                    }).forEach(e -> {
                        businessTime.append(e).append("；");
                    });
                }
                weimobPhysicalStoreDTO.setBussinessHours(businessTime.toString());

                WeimobStoreAccountDTO accountDTO = new WeimobStoreAccountDTO();
                ClerkDTO headClerk = getHeadClerk(storeDTO.getEnterpriseId(), storeDTO.getStoreId());
                if (headClerk != null) {
                    accountDTO.setManagerLoginAccount(headClerk.getPhoneNumber());
                    accountDTO.setManagerName(headClerk.getClerkName());
                }
                if (StringUtils.isNotBlank(storeDTO.getConactsPhone())) {
                    weimobPhysicalStoreDTO.setStoreTel(storeDTO.getConactsPhone());
                } else {
                    //店长的号码
                    if (headClerk != null) {
                        weimobPhysicalStoreDTO.setStoreTel(headClerk.getPhoneNumber());
                    }
                }
                weimobPhysicalStoreDTO.setAccountVo(accountDTO);
                return weimobPhysicalStoreDTO;
            }
            return null;
        }

        private static String businessTime(String week) {
            StringBuilder businessTime = new StringBuilder();
            Arrays.asList(week.split(","))
                    .stream()
                    .filter(e -> StringUtils.isNumeric(e))
                    .mapToInt(e -> Integer.parseInt(e))
                    .boxed()
                    .sorted()
                    .collect(Collectors.toList())
                    .forEach(e -> {
                        businessTime.append(BUSINESS_TIME_MAP.get(e.toString())).append("、");
                    });
            String result = businessTime.toString();
            return result.substring(0, result.length() - 1);
        }

        private ClerkDTO getHeadClerk(Integer enterpriseId, Integer storeId) {
            ServiceResponse<ClerkDTO> clerkResponse = clerkApiService.getHeadClerk(enterpriseId, storeId);
            if (clerkResponse.isSuccess()) {
                return clerkResponse.getResult();
            }
            return null;
        }

        private StoreDTO getStoreByRegionAndCode(Integer enterpriseId, Integer regionId, String storeCode) {
            ServiceResponse<StoreDTO> storeResponse = storeApiService
                    .getStoreContainBusinessTimeByStoreCode(enterpriseId, regionId, storeCode);
            if (storeResponse.isSuccess()) {
                return storeResponse.getResult();
            }
            return null;
        }

        private StoreRegionDTO getStoreRegion(Integer enterpriseId, String regionName) {
            ServiceResponse<List<StoreRegionDTO>> regionResponse = storeRegionApiService.listStoreRegion(enterpriseId, null);
            if (regionResponse.isSuccess()) {
                List<StoreRegionDTO> regionList = regionResponse.getResult();
                if (CollectionUtils.isNotEmpty(regionList)) {
                    if (regionList.size() == 1) {
                        //如果只有一条，则
                        return regionList.get(0);
                    } else {
                        List<StoreRegionDTO> temp = regionList.stream().filter(e -> e.getRegionName().equals(regionName))
                                .collect(Collectors.toList());
                        if (CollectionUtils.isNotEmpty(temp)) {
                            return temp.get(0);
                        }
                    }
                }
            }
            return null;
        }
    }

    private WeimobPhysicalStoreDTO getWeimobPhsicalStore(StoreDTO storeDTO) {
        if (storeDTO.getStatus().intValue() == 1) {
            WeimobPhysicalStoreDTO weimobPhysicalStoreDTO = new WeimobPhysicalStoreDTO();
            //用storeInfoId
            weimobPhysicalStoreDTO.setStoreInfoId(storeDTO.getStoreInfoId());
            weimobPhysicalStoreDTO.setStoreNumber(storeDTO.getStoreCode());
            weimobPhysicalStoreDTO.setStoreName(storeDTO.getStoreName());
            //经纬度
            if (CollectionUtils.isNotEmpty(storeDTO.getLocation())) {
                weimobPhysicalStoreDTO.setLongitude(new BigDecimal(storeDTO.getLocation().get(0)));
                weimobPhysicalStoreDTO.setLatitude(new BigDecimal(storeDTO.getLocation().get(1)));
            } else {
                if (org.apache.commons.lang.StringUtils.isNotBlank(storeDTO.getLongitude())) {
                    weimobPhysicalStoreDTO.setLongitude(new BigDecimal(storeDTO.getLongitude()));
                }
                if (org.apache.commons.lang.StringUtils.isNotBlank(storeDTO.getLatitude())) {
                    weimobPhysicalStoreDTO.setLatitude(new BigDecimal(storeDTO.getLatitude()));
                }
            }
            //省市区名称
            String fullArea = StringUtils.isNotBlank(storeDTO.getProvinces()) ?
                    storeDTO.getProvinces().replaceAll("//", "") : "";
            weimobPhysicalStoreDTO.setAddress(fullArea + storeDTO.getAddress());
            StringBuilder businessTime = new StringBuilder();
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("HH:mm");
            List<StoreBusinessTimeDTO> businessTimeList = storeDTO.getBusinessTimeList();
            if (CollectionUtils.isNotEmpty(businessTimeList)) {
                businessTimeList.stream().map(e -> {
                    String businessHours = simpleDateFormat.format(e.getOpenTime()) + "-" + simpleDateFormat.format(e.getCloseTime());
                    return businessTime(e.getWeekday()) + ":" + businessHours;
                }).forEach(e -> {
                    businessTime.append(e).append("；");
                });
            }
            weimobPhysicalStoreDTO.setBussinessHours(businessTime.toString());

            WeimobStoreAccountDTO accountDTO = new WeimobStoreAccountDTO();
            ClerkDTO headClerk = getHeadClerk(storeDTO.getEnterpriseId(), storeDTO.getStoreId());
            if (headClerk != null) {
                accountDTO.setManagerLoginAccount(headClerk.getPhoneNumber());
                accountDTO.setManagerName(headClerk.getClerkName());
            }
            if (StringUtils.isNotBlank(storeDTO.getConactsPhone())) {
                weimobPhysicalStoreDTO.setStoreTel(storeDTO.getConactsPhone());
            } else {
                //店长的号码
                if (headClerk != null) {
                    weimobPhysicalStoreDTO.setStoreTel(headClerk.getPhoneNumber());
                }
            }
            weimobPhysicalStoreDTO.setAccountVo(accountDTO);
            return weimobPhysicalStoreDTO;
        }
        return null;
    }

    private static String businessTime(String week) {
        StringBuilder businessTime = new StringBuilder();
        Arrays.asList(week.split(","))
                .stream()
                .filter(e -> StringUtils.isNumeric(e))
                .mapToInt(e -> Integer.parseInt(e))
                .boxed()
                .sorted()
                .collect(Collectors.toList())
                .forEach(e -> {
                    businessTime.append(BUSINESS_TIME_MAP.get(e.toString())).append("、");
                });
        String result = businessTime.toString();
        return result.substring(0, result.length() - 1);
    }

    private ClerkDTO getHeadClerk(Integer enterpriseId, Integer storeId) {
        ServiceResponse<ClerkDTO> clerkResponse = clerkApiService.getHeadClerk(enterpriseId, storeId);
        if (clerkResponse.isSuccess()) {
            return clerkResponse.getResult();
        }
        return null;
    }

    private List<StoreDTO> getStoreList(String storeIds, boolean isStoreInfoIds) {
        String[] subStoreIdArr = storeIds.split(",");
        //一次性查询分店列表数据
        StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
        //转为空格
        storeIds = storeIds.replaceAll(",", " ");
        if (isStoreInfoIds) {
            storeSearchDTO.setStoreInfoIds(storeIds);
        } else {
            storeSearchDTO.setStoreIds(storeIds);
        }
        storeSearchDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        ServiceResponse<Page<StoreDTO>> subStoreResponse = storeApiService.listStore(storeSearchDTO, 1, subStoreIdArr.length);
        if (subStoreResponse.isSuccess()) {
            Page<StoreDTO> page = subStoreResponse.getResult();
            if (page != null) {
                List<StoreDTO> storeList = page.getResult();
                return storeList;
            }
        }
        return Collections.emptyList();
    }

    /**
     * Excel导入进度查询
     * @param key
     * @return
     */
    private RestResponse getExportProcess(String key) {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        RedissonClient redisClient = RedisUtil.getRedisClient();
        RMap<String, String> rMap = redisClient.getMap(key + enterpriseId);
        if (rMap.size() > 0) {
            String total = rMap.get("门店同步总数");
            if (org.apache.commons.lang3.StringUtils.isBlank(total)) {
                return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "不存在同步任务");
            }
            Integer totalCount = Integer.parseInt(total);
            DecimalFormat df = new DecimalFormat("0.0000");
            return RestResponse.success(df.format((rMap.size() - 1) / totalCount));
        }
        return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "不存在同步任务");
    }

    private RestResponse downloadExportLog(String key, HttpServletRequest request, HttpServletResponse response) throws IOException {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        RedissonClient redisClient = RedisUtil.getRedisClient();
        RMap<String, String> rMap = redisClient.getMap(key + enterpriseId);
        String firefox = "firefox";
        if (MapUtils.isNotEmpty(rMap)) {
            String fileName = "门店同步日志信息.csv";
            List<String> titleList = Arrays.asList("门店code", "日志信息");
            BufferedWriter writer = null;
            try {
                String userAgent = request.getHeader("User-Agent").toLowerCase();
                if (userAgent.contains(firefox)) {
                    byte[] bytesName = fileName.getBytes("UTF-8");
                    fileName = new String(bytesName, "ISO-8859-1");
                    response.setHeader("content-disposition", "attachment;fileName=" + fileName);
                } else {
                    response.setHeader("content-disposition", "attachment;fileName=" + URLEncoder.encode(fileName, "UTF-8"));
                }

                writer = new BufferedWriter(new OutputStreamWriter(
                        response.getOutputStream(), "gbk"));
                for (String aTitleList : titleList) {
                    writer.append(aTitleList).append(",");
                }
                writer.newLine();
                for (Map.Entry<String, String> entry : rMap.entrySet()) {
                    //防止0被忽略
                    writer.append(entry.getKey()).append(",");
                    writer.append(entry.getValue()).append(",");

                    writer.newLine();
                }


                writer.flush();
            } catch (Exception e) {
                LOGGER.error("导出错误", e);
            } finally {
                if (writer != null) {
                    writer.close();
                }
            }
            return RestResponse.success();
        } else {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "没有同步日志");
        }
    }

    /**
     * 是否是智慧零售
     * @param wmMallStoreId
     * @return
     */
    public boolean validWmMode(Integer wmMallStoreId) {
        ServiceResponse<WmStoreDTO> wmStoreResponse = wmStoreApiService.getWmStoreByWmMallStoreId(wmMallStoreId);
        if (wmStoreResponse.isSuccess()) {
            WmStoreDTO wmStoreDTO = wmStoreResponse.getResult();
            //是否智慧零售模式
            Integer mode = wmStoreDTO.getMallMode();
            if (mode.intValue() == MallModeEnum.SMART_RETAIL.getCode()) {
                return true;
            }
        }
        return false;
    }

    public static List<Map<Integer, String>> readXlsOfBranch(InputStream in, int row) {
        List<Map<Integer, String>> result = new ArrayList<Map<Integer, String>>();
        try {
            Workbook workbook = WorkbookFactory.create(in);
            for (int numSheet = 0; numSheet < 1; numSheet++) {
                Sheet sheet = workbook.getSheetAt(numSheet);
                for (int rowNum = row - 1, rowLength = sheet.getPhysicalNumberOfRows(); rowNum <= rowLength; rowNum++) {
                    Row hssfRow = sheet.getRow(rowNum);
                    if (hssfRow == null) {
                        continue;
                    }
                    //门店标签专属，只有2列，写死
                    int minColix = 0;
                    int maxColix = 2;

                    Map<Integer, String> map = new HashMap<Integer, String>();
                    for (int colix = minColix; colix < maxColix; colix++) {
                        Cell hssfCell = hssfRow.getCell(colix);
                        if (hssfCell == null) {
                            map.put(colix, "");
                        } else {
                            if (hssfCell.getCellType() != Cell.CELL_TYPE_STRING) {
                                hssfCell.setCellType(Cell.CELL_TYPE_STRING);
                            }
                            map.put(colix, hssfCell.getStringCellValue());
                        }
                    }
                    result.add(map);
                }
            }
        } catch (IOException e) {
            LOGGER.warn("e", e);
        } catch (InvalidFormatException e) {
            LOGGER.warn("e", e);
        }
        return result;
    }

    private static List<Integer> getListByStr(String storeIds) {
        List<Integer> storeIdList = new ArrayList<>();
        if (StringUtils.isNotBlank(storeIds)) {
            String[] arr = storeIds.split(",");
            for(String s : arr){
                storeIdList.add(Integer.parseInt(s));
            }
        }
        return storeIdList;
    }

    private StoreDTO getByStoreId(Integer enterpriseId, Integer storeId) {
        ServiceResponse<StoreDTO> response = storeApiService.getStoreById(enterpriseId, storeId);
        if (response.isSuccess()) {
            return response.getResult();
        }
        return null;
    }

    private Set<String> getHasSyncStore(Integer wmMallStoreId) {
        ServiceResponse<List<WmStoreSyncLogDTO>> hasSyncStoreListResult = wmStoreSyncLogApiService.getStoreList(null, wmMallStoreId);
        if (hasSyncStoreListResult.isSuccess()) {
            return new HashSet<>(hasSyncStoreListResult.getResult()
                    .stream().map(e -> e.getStoreId().toString()).collect(Collectors.toList()));
        }
        return new HashSet<>(1);
    }
}
