package com.gic.operation.web.controller;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.constant.EnterpriseInitCodeEnum;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.dto.EnterpriseInitDTO;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.enterprise.service.EnterpriseInitApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.marketing.api.dto.sms.EnterpriseSmsSignDTO;
import com.gic.marketing.api.dto.sms.SmsApplicationDTO;
import com.gic.marketing.api.dto.sms.SmsGicChannelDTO;
import com.gic.marketing.api.service.sms.SmsOuterApiService;
import com.gic.operation.web.vo.EnterpriseSmsSignVO;
import com.gic.operation.web.vo.sms.SmsChannelVO;
import com.gic.operation.web.vo.sms.SmsTemplateVO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 *
 * @ClassName: EnterpriseSmsController
 * @Description:
 * @author taogs
 * @date 2020/9/17 16:24
 */
@RestController
@RequestMapping("/enterprise/sms")
public class EnterpriseSmsController {
    private static final Logger LOGGER = LogManager.getLogger(EnterpriseSmsController.class);

    @Autowired
    private SmsOuterApiService smsOuterApiService;
    @Autowired
    private EnterpriseInitApiService enterpriseInitApiService;
    @Autowired
    private EnterpriseApiService enterpriseApiService;

    @InitBinder
    public void initBinder(WebDataBinder binder) {
        DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        dateFormat.setLenient(true);
        binder.registerCustomEditor(Date.class, new CustomDateEditor(dateFormat, true));
    }

    @RequestMapping("/save-app")
    public RestResponse saveApp(Integer enterpriseId, String appid, String appkey,Integer channelType) {
        return ResultControllerUtils.commonResult(smsOuterApiService.saveEnterpriseSmsApp(enterpriseId, appid, appkey,channelType));
    }

    @RequestMapping("/list-sms-channel")
    public RestResponse listSmsChannel() {
        ServiceResponse<List<SmsGicChannelDTO>> resp = smsOuterApiService.listGicSmsChannel();
        LOGGER.info("测试数据 === 》 {}",JSON.toJSONString(resp));
        return ResultControllerUtils.commonResult(resp, SmsChannelVO.class);
    }

    // TODO 弃用
    /**
     * 设置优先使用配置
     * @param enterpriseId
     * @param channelType 短信通道类型(1:腾讯云, 2:大汉三通)
     * @return
     */
    @RequestMapping("/set-sms-use-default")
    public RestResponse setUseFirstSmsApp(Integer enterpriseId, Integer channelType) {
        return ResultControllerUtils.commonResult(smsOuterApiService.setUseFirstSmsApp(enterpriseId, channelType));
    }

    @RequestMapping("/get-sms-template-max-count")
    public RestResponse getSmsTemplateMaxCount(Integer enterpriseId) {
        return ResultControllerUtils.commonResult(smsOuterApiService.getSmsTempalteMaxCount(enterpriseId));
    }

    @RequestMapping("/get-sms-app-info")
    public RestResponse getAllSmsAppInfoByEid(Integer enterpriseId) {
        return ResultControllerUtils.commonResult(smsOuterApiService.getAllSmsAppInfoByEid(enterpriseId));
    }

    @Deprecated
    @RequestMapping("/get-app")
    public RestResponse getApp(Integer enterpriseId) {
        return ResultControllerUtils.commonResult(smsOuterApiService.getEnterpriseSmsApp(enterpriseId));
    }

    @RequestMapping("/list-sms")
    public RestResponse listSms(Integer enterpriseId, Page<EnterpriseSmsSignDTO> page) {
        return ResultControllerUtils.commonPageResult(smsOuterApiService.listSmsSignByEid(enterpriseId, page),
                EnterpriseSmsSignVO.class);
    }

    /**
     * @Title: addSms 
     * @Description: 
      * @author guojuxing 
     * @param enterpriseId
     * @param tencentSignId 腾讯云短信签名id
     * @param smsSignText 短信签名
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/add-sms")
    public RestResponse addSms(Integer enterpriseId, String tencentSignId, String smsSignText) {
        ServiceResponse<Void> smsResult = smsOuterApiService.saveEnterpriseSmsSign(enterpriseId, tencentSignId, smsSignText);
        if (smsResult.isSuccess()) {
            //新增之前判断是否已经初始化完毕
            ServiceResponse<EnterpriseInitDTO> result = enterpriseInitApiService
                    .getByConfigCode(EnterpriseInitCodeEnum.SHORT_MESSAGE_SERVICE_CONFIG.getCode(), enterpriseId);
            if (result.isSuccess()) {
                EnterpriseInitDTO enterpriseInitDTO = result.getResult();
                if (enterpriseInitDTO.getIsInitConfig().intValue() == 0) {
                    //如果是未配置，则需要判断
                    if (isSmsConfig(enterpriseId)) {
                        //短信业务配置初始化
                        enterpriseInitApiService.shortMessageServiceConfig(enterpriseId);
                    }
                }
            }
        }
        return EnterpriseRestResponse.failure(smsResult);
    }

    @RequestMapping("/get-sms")
    public RestResponse getSms(Long signId, Integer enterpriseId) {
        return ResultControllerUtils.commonResultOne(smsOuterApiService.getSmsSignByid(enterpriseId, signId), EnterpriseSmsSignVO.class);
    }

    @RequestMapping("/edit-sms")
    public RestResponse editSms(Integer enterpriseId, Long signId, String tencentSignId, String smsSignText) {
        return ResultControllerUtils
                .commonResult(smsOuterApiService.editEnterpriseSmsSign(enterpriseId, signId, tencentSignId, smsSignText));
    }

    @RequestMapping("/delete-sms")
    public RestResponse deleteSms(Long signId, Integer enterpriseId) {
        ServiceResponse<Void> smsResult = smsOuterApiService.delEnterpriseSmsSign(enterpriseId, signId);
        if (smsResult.isSuccess()) {
            ServiceResponse<EnterpriseDTO> enterpriseResult = enterpriseApiService.getEnterpriseById(enterpriseId);
            if (enterpriseResult.isSuccess()) {
                //商户未初始化，才能回退短信业务配置
                if (enterpriseResult.getResult().getIsInitComplete().intValue() == 0) {
                    ServiceResponse<EnterpriseInitDTO> result = enterpriseInitApiService.getByConfigCode(
                            EnterpriseInitCodeEnum.SHORT_MESSAGE_SERVICE_CONFIG.getCode(), enterpriseId);
                    if (result.isSuccess()) {
                        EnterpriseInitDTO enterpriseInitDTO = result.getResult();
                        if (enterpriseInitDTO.getIsInitConfig().intValue() == 1) {
                            //如果是已配置，则需要判断
                            if (!isSmsConfig(enterpriseId)) {
                                //短信业务配置初始化回退
                                enterpriseInitApiService.unInitByConfigCode(
                                        EnterpriseInitCodeEnum.SHORT_MESSAGE_SERVICE_CONFIG.getCode(), enterpriseId);
                            }
                        }
                    }
                }
            }
        }
        return EnterpriseRestResponse.failure(smsResult);
    }


    /**
     *
     * @param enterpriseId
     * @param channelType 短信通道类型(1:腾讯云, 2:大汉三通)
     * @param search 模板名称/ID
     * @param type 类型: 0普通 1营销
     * @param beginTime
     * @param endTime
     * @param page
     * @return
     */
    @RequestMapping("/list-sms-channel-template")
    public RestResponse channelSmsTemplatePage(Integer enterpriseId, Integer channelType,
                                               String search, Integer type, Date beginTime, Date endTime,
                                               Page<Void> page) {
        if (channelType == null) {
            //默认腾讯云通道
            channelType = 1;
        }
        return ResultControllerUtils
                .commonPageResult(smsOuterApiService.channelSmsTemplatePage(enterpriseId, channelType, search, type, beginTime, endTime, page), SmsTemplateVO.class);
    }

    @RequestMapping("/get-sms-channel-template-number")
    public RestResponse countTemplateByChannel(Integer enterpriseId, Integer channelType) {
        if (channelType == null) {
            //默认腾讯云通道
            channelType = 1;
        }
        return ResultControllerUtils
                .commonResult(smsOuterApiService.countTemplateByChannel(enterpriseId, channelType));
    }

    @RequestMapping("/sync-sms-channel-template")
    public RestResponse checkTemplateStatus(Integer enterpriseId, Integer channelType, boolean checkAll,
                                               Long channelTemplateId) {
        if (channelType == null) {
            //默认腾讯云通道
            channelType = 1;
        }
        return ResultControllerUtils
                .commonResult(smsOuterApiService.checkTemplateStatus(enterpriseId, channelType, checkAll, channelTemplateId));
    }

    @RequestMapping("/init-sms-channel-template")
    public RestResponse initAddChannelTemplate(Integer enterpriseId, Integer channelType) {
        if (channelType == null) {
            //默认腾讯云通道
            channelType = 1;
        }
        ServiceResponse<Integer> result = smsOuterApiService.initAddChannelTemplate(enterpriseId, channelType);
        if (result.isSuccess()) {
            return RestResponse.success();
        }
        RestResponse response = RestResponse.failure(result.getCode(), result.getMessage());
        response.setResult(result.getResult());
        return response;
    }

    /**
     * 是否符合短信业务配置条件
     * @param enterpriseId
     * @return
     */
    private boolean isSmsConfig(Integer enterpriseId) {
        //appID和Appkey判断是否已经配置
        ServiceResponse<SmsApplicationDTO> smsAppResult = smsOuterApiService.getEnterpriseSmsApp(enterpriseId);
        if (smsAppResult.isSuccess()) {
            if (smsAppResult.getResult() == null) {
                return false;
            }
        } else {
            return false;
        }

        Page<EnterpriseSmsSignDTO> page = new Page<>();
        page.setCurrentPage(1);
        page.setPageSize(Integer.MAX_VALUE);
        ServiceResponse<Page<EnterpriseSmsSignDTO>> result = smsOuterApiService.listSmsSignByEid(enterpriseId, page);
        if (result.isSuccess()) {
            Page<EnterpriseSmsSignDTO> smsSignPage = result.getResult();
            if (smsSignPage != null) {
                List<EnterpriseSmsSignDTO> list = smsSignPage.getResult();
                if (CollectionUtils.isNotEmpty(list)) {
                    //有一条数据就符合条件
                    return true;
                }
            }
        }
        return false;
    }
}
