package com.gic.operation.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.dto.LinkDTO;
import com.gic.enterprise.qo.LinkQO;
import com.gic.enterprise.service.LinkApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Arrays;
import java.util.List;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-11-25 19:41
 */
@RestController
public class LinkController {

    @Autowired
    private LinkApiService linkApiService;

    @RequestMapping("/save-link-type")
    public RestResponse saveLinkType(String name, Integer type) {
        ServiceResponse<Void> serviceResponse = linkApiService.saveLinkType(name, type);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/update-link-type")
    public RestResponse updateLinkType(Integer linkId, String name, Integer type) {
        ServiceResponse<Void> serviceResponse = linkApiService.updateLinkType(linkId, name, type);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/save-link-child-menu")
    public RestResponse saveLinkChildMenu(@Validated(LinkQO.LinkSaveValid.class) LinkQO linkQO) {
        ServiceResponse<Void> serviceResponse = linkApiService.saveLinkChildMenu(EntityUtil.changeEntityByJSON(LinkDTO.class, linkQO));
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/update-link-child-menu")
    public RestResponse updateLinkChildMenu(@Validated({LinkQO.LinkSaveValid.class, LinkQO.LinkUpdateValid.class})LinkQO linkQO) {
        ServiceResponse<Void> serviceResponse = linkApiService.updateLinkChildMenu(EntityUtil.changeEntityByJSON(LinkDTO.class, linkQO));
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/del-link")
    public RestResponse delLink(Integer[] linkIds) {
        ServiceResponse<Void> serviceResponse = linkApiService.delLink(Arrays.asList(linkIds));
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/list-link-type")
    public RestResponse listLinkType() {
        ServiceResponse<List<LinkDTO>> serviceResponse = linkApiService.listLinkType();
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/list-link-menu")
    public RestResponse listLinkMenu(LinkQO linkQO) {
        ServiceResponse<Page<LinkDTO>> serviceResponse = linkApiService.listLinkMenu(linkQO);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/get-link")
    public RestResponse getLink(Integer linkId) {
        ServiceResponse<LinkDTO> serviceResponse = linkApiService.getLink(linkId);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/edit-link-enterprise")
    public RestResponse editLinkEnterprise(Integer linkId, Integer enterpriseType, Integer[] enterpriseIdList) {
        ServiceResponse<Void> serviceResponse = linkApiService.editLinkEnterprise(linkId, enterpriseType, Arrays.asList(enterpriseIdList));
        return ResultControllerUtils.commonResult(serviceResponse);
    }

}