package com.gic.operation.web.controller;

import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

import com.ctrip.framework.apollo.Config;
import com.ctrip.framework.apollo.ConfigService;
import com.gic.redis.data.util.RedisUtil;
import com.gic.sso.utils.LoginGicSsoUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.MenuDTO;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.MenuApiService;
import com.gic.auth.service.OperationUserApiService;
import com.gic.auth.service.UserApiService;
import com.gic.authcenter.security.core.dto.UserDetailsVO;
import com.gic.authcenter.security.core.util.UserUtils;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.base.EnterpriseInfo;
import com.gic.enterprise.base.MenuInfo;
import com.gic.enterprise.base.UserInfo;
import com.gic.enterprise.constants.Constants;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetail;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.open.api.service.market.MarketUserApiService;
/**
 *
 * @ClassName: LoginController
 * @Description:
 * @author taogs
 * @date 2020/9/17 16:25
 */
@RestController
public class LoginController {
    private static final Logger LOGGER = LogManager.getLogger(LoginController.class);

    @Autowired
    private OperationUserApiService operationUserApiService;
    @Autowired
    private EnterpriseApiService enterpriseApiService;
    @Autowired
    private MenuApiService menuApiService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private MarketUserApiService marketUserApiService;

    private static final String MENU_LOCAL_CACHE_KEY = "auth:menu:list:all";

    /**
     * 商户绑定服务市场
     * @Title: bind 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @param nationCode
     * @param phone
     * @param password 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("bind-enterprise-to-service-market")
    public RestResponse bind(Integer enterpriseId, String nationCode, String phone, String password)
            throws IOException {
        return ResultControllerUtils.commonResult(
                marketUserApiService.bindGicEnterprise(Long.valueOf(enterpriseId), nationCode.replace("+", ""), phone, password));
    }

    /**
     * 单点登录到服务市场首页
     * @Title: ssoToMarketService 
     * @Description:
      * @author guojuxing
     * @param enterpriseId
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("sso-to-market-service")
    public RestResponse ssoToMarketService(Integer enterpriseId) throws IOException {
        ServiceResponse<String> result = marketUserApiService.bindGicEnterprise(Long.valueOf(enterpriseId), null, null,
                null);
        if (result.isSuccess()) {
            Config config = ConfigService.getConfig("COMMON.4.0-gic-properties");
            String url = config.getProperty("service_host", "").replace("/gic", "");
            return RestResponse.success(url + "/market/gic" + "?loginKey=" + result.getResult()
                    + "&enterpriseId=" + enterpriseId);
        }
        return EnterpriseRestResponse.failure(result);
    }

    @RequestMapping("login-gic")
    public RestResponse login(Integer enterpriseId, String redirectUrl) throws IOException {
        return RestResponse.success(LoginGicSsoUtils.sso(enterpriseId, redirectUrl));
    }

    /**
     * gic平台的菜单数据塞本地缓存
     */
    private void setAllMenuToLocaleCache() {
        ServiceResponse<List<MenuDTO>> response = menuApiService.getAllMenu();
        if (response.isSuccess()) {
            List<MenuDTO> list = response.getResult();
            //gic平台的菜单数据塞本地缓存
            RedisUtil.setLocalCache(MENU_LOCAL_CACHE_KEY, EntityUtil.changeEntityListNew(MenuInfo.class, list), null);
        }
    }
}
