package com.gic.operation.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.authcenter.security.core.util.UserUtils;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.dto.PushMessageDTO;
import com.gic.enterprise.dto.PushTypeDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.PushMessageApiService;
import com.gic.operation.web.qo.PageQO;
import com.gic.operation.web.qo.PushMessageQO;
import com.gic.operation.web.vo.PushTypeVO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
/**
 *
 * @ClassName: PushMessageController
 * @Description:
 * @author taogs
 * @date 2020/9/17 16:26
 */
@RestController
public class PushMessageController {
    private static final Logger log = LogManager.getLogger(PushMessageController.class);

    @Autowired
    private PushMessageApiService pushMessageApiService;

    @RequestMapping("page-push-classify")
    public RestResponse pagePushType(String search, PageQO pageQO){
        ServiceResponse<Page<PushTypeDTO>> page = this.pushMessageApiService.pagePushType(search, pageQO.getCurrentPage(), pageQO.getPageSize());
        Page<PushTypeVO> pushTypePage = new Page<>(pageQO.getCurrentPage(), pageQO.getPageSize());
        if(page.isSuccess()){
            Page<PushTypeDTO> result = page.getResult();
            if(CollectionUtils.isNotEmpty(result.getResult())){
                List<PushTypeVO> list = new ArrayList<>();
                for(PushTypeDTO dto : result.getResult()){
                    PushTypeVO vo = EntityUtil.changeEntityByJSON(PushTypeVO.class, dto);
                    ServiceResponse<Page<PushMessageDTO>> response = this.pushMessageApiService.pagePushMessage(null, null, null, null, 1, 3, dto.getId());
                    vo.setMessageList(response.getResult().getResult());
                    list.add(vo);
                }
                pushTypePage.setResult(list);
                pushTypePage.setTotalCount(page.getResult().getTotalCount());
            }
        }
        return RestResponse.success(pushTypePage);
    }

    @RequestMapping("save-push-classify")
    public RestResponse savePushType(Integer classifyId, String classifyName, String classifyDesc){
        PushTypeDTO pushTypeDTO = new PushTypeDTO();
        pushTypeDTO.setId(classifyId);
        pushTypeDTO.setClassifyName(classifyName);
        pushTypeDTO.setClassifyDesc(classifyDesc);
        ServiceResponse<Integer> response = this.pushMessageApiService.savePushType(pushTypeDTO);
        return RestResponse.success(response.getResult());
    }

    @RequestMapping("push-classify-detail")
    public RestResponse pushTypeDetail(Integer classifyId){
        ServiceResponse<PushTypeDTO> response = this.pushMessageApiService.getPushTypeById(classifyId);
        return RestResponse.success(response.getResult());
    }

    @RequestMapping("delete-push-classify")
    public RestResponse deletePushType(Integer classifyId){
        ServiceResponse<Integer> response = this.pushMessageApiService.deletePushType(classifyId);
        return RestResponse.success(response.getResult());
    }

    @RequestMapping("page-push-message")
    public RestResponse pagePushMessage(String search, Integer pushStatus, Integer classifyId, String startTime, String endTime, PageQO pageQO){
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        ServiceResponse<Page<PushMessageDTO>> response = null;
        try {
            response = this.pushMessageApiService.pagePushMessage(search, pushStatus,
                    StringUtils.isBlank(startTime) ? null : simpleDateFormat.parse(startTime+" 00:00:00"), StringUtils.isBlank(endTime) ? null : simpleDateFormat.parse(endTime+" 23:59:59"), pageQO.getCurrentPage(), pageQO.getPageSize(), classifyId);
        } catch (ParseException e) {
            log.warn("e", e);
        }
        return RestResponse.success(response.getResult());
    }

    @RequestMapping("save-push-message")
    public RestResponse savePushMessage(@RequestBody PushMessageQO pushMessage){
        PushMessageDTO pushMessageDTO = EntityUtil.changeEntityByJSON(PushMessageDTO.class, pushMessage);
        pushMessageDTO.setCreator(UserUtils.getUser().getId());
        pushMessageDTO.setCreatorname(UserUtils.getUser().getLoginName());
        if(StringUtils.isNotBlank(pushMessage.getPushTime())){
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            try {
                pushMessageDTO.setPushTime(dateFormat.parse(pushMessage.getPushTime().replace("T", " ").replace(".000+0000", "")));
            } catch (ParseException e) {
                log.warn("异常", e);
                return RestResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "日期转换错误");
            }
        }
        ServiceResponse<Integer> response = this.pushMessageApiService.savePushMessage(pushMessageDTO, pushMessage.getSaveType());
        if(response.isSuccess()){
            return RestResponse.success(response.getResult());
        }
        return RestResponse.failure(response.getCode(), response.getMessage());
    }

    @RequestMapping("push-message-detail")
    public RestResponse pushMessageDetail(Integer messageId){
        ServiceResponse<PushMessageDTO> response = this.pushMessageApiService.getPushMessageById(messageId);
        return RestResponse.success(response.getResult());
    }

    @RequestMapping("delete-push-message")
    public RestResponse deletePushMessage(Integer messageId){
        ServiceResponse<Integer> response = this.pushMessageApiService.deletePushMessage(messageId);
        return RestResponse.success(response.getResult());
    }

    @RequestMapping("recall-message")
    public RestResponse recallMessage(Integer messageId){
        ServiceResponse<Void> response = this.pushMessageApiService.recallMessage(messageId);
        return RestResponse.success(response.getResult());
    }
}
