package com.gic.operation.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.authcenter.security.core.util.UserUtils;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.dto.TableSettingDTO;
import com.gic.enterprise.dto.TableSettingFieldDTO;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.service.TableSettingApiService;
import com.gic.operation.web.qo.TableSettingFieldQO;
import com.gic.operation.web.qo.TableSettingQO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.List;

@Controller
public class TableSettingController {
    @Autowired
    private TableSettingApiService tableSettingApiService;

    @RequestMapping("save-table-setting")
    @ResponseBody
    public RestResponse saveTableSetting(TableSettingQO param){
        TableSettingDTO tableSettingDTO = EntityUtil.changeEntityByJSON(TableSettingDTO.class, param);
        tableSettingDTO.setCreatetor(UserUtils.getUser().getId().toString());
        tableSettingDTO.setCreatetorName(UserUtils.getUser().getUsername());
        ServiceResponse<String> response = tableSettingApiService.saveTableSetting(tableSettingDTO);
        if(response.isSuccess()){
            return RestResponse.success();
        }
        return EnterpriseRestResponse.failure(response);
    }

    @RequestMapping("page-table-setting")
    @ResponseBody
    public RestResponse pageTableSetting(String search, @RequestParam(defaultValue = "1") Integer pageNum, @RequestParam(defaultValue = "20") Integer pageSize){
        Page<TableSettingDTO> result = tableSettingApiService.pageTableSetting(search, pageNum, pageSize).getResult();
        return RestResponse.success(result);
    }

    @RequestMapping("get-table-setting")
    @ResponseBody
    public RestResponse getTableSetting(Integer tableId){
        TableSettingDTO result = tableSettingApiService.getTableSetting(tableId).getResult();
        return RestResponse.success(result);
    }

    @RequestMapping("del-table-setting")
    @ResponseBody
    public RestResponse delTableSetting(Integer tableId){
        ServiceResponse<Void> response = tableSettingApiService.delTableSetting(tableId);
        return RestResponse.success();
    }

    @RequestMapping("add-table-field")
    @ResponseBody
    public RestResponse addTableField(TableSettingFieldQO qo){
        TableSettingFieldDTO tableSettingFieldDTO = EntityUtil.changeEntityByJSON(TableSettingFieldDTO.class, qo);
        ServiceResponse<Void> response = tableSettingApiService.saveTableField(tableSettingFieldDTO);
        if(response.isSuccess()){
            return RestResponse.success();
        }
        return EnterpriseRestResponse.failure(response);
    }

    @RequestMapping("list-table-field")
    @ResponseBody
    public RestResponse listTableField(Integer tableId){
        List<TableSettingFieldDTO> result = tableSettingApiService.listTableField(tableId).getResult();
        return RestResponse.success(result);
    }

    @RequestMapping("del-table-field")
    @ResponseBody
    public RestResponse delTableField(Integer fielId){
        tableSettingApiService.delField(fielId);
        return RestResponse.success();
    }
}
